#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fims.tcl,v 2.43 2004/06/17 23:04:31 vltsccm Exp $"       
#
# who       when      what
# --------  --------  ----------------------------------------------
# W.Hummel   27.Oct 97   copied from version 0.9
# W.Hummel   15.Nov 97   permanently progressing 
# W.Hummel   12.May 98   use with skycat V2.0.13 
# W.Hummel   25.May 98   V0.9.5
# W.Hummel   25.May 98   V0.9.6, SCT-2.0.15
# W.Hummel   21.Jul 98   allow to move the mask out of the frame 
# W.Hummel   22.Jul 98   allow to load a mask, moved oput of the frame 
# W.Hummel   30.Jul 98   bug in LSS, new modes IMS,PIMS,HIT 
# W.Hummel   sep/oct98   FORS1 Commissioning at paranal
# W.Hummel   17.Nov 98   Preparing for FORS1 COMM2 
# wgaessle   14/01/99    puts -nonewline for p_*
#                        asking for slit 10
#		         - search for wgaessle for the bigger changes -
# W.Hummel   15/01/99    - FORS optical correction applied, 
#                          if fits image from FORS
#                        - IPOL dither now more accurate 22.73'' and 11.36''
# W.Hummel   15/01/99    - slit 10 is drawn if applied
# W.Hummel   15/01/99    - first modifications for FORS2 
# wgaessle   14/01/99    added INS.FOCF.* to *foc files
# W.Hummel   18/01/99    changed default TSF name 
# W.Hummel   19/01/99    collimator no longer hardcoded but read from isf
# W.Hummel   20/01/99    grism,filter,lmin,lmax configured via load-mask
# W.Hummel   22/01/99    FORS1,FOSR2 dont interfere anymore
# W.Hummel   07/04/99    preparation for keyword change WID --> WIDTH     
# W.Hummel   14/04/99    start with echelle spectroscopy = ECHS
# W.Hummel   27/07/99    version 1.6 goes with skycat 2.4.3 !!!!!!!!
# W.Hummel   30/08/99    MOD for position angle corrected
#                        save lss reference target and slit target exchanged
# W.Hummel   01/09/99    curved MXU slit
# aaguayo    12/12/00    Added info_dialog about not allowed CURVED & CIRCLE 
#			 reference slits
# aaguayo    12/12/00    shape="none" not supported for MXU mode -> widget disabled  
# aaguayo    12/12/00    expiration date checking re-activated 
# aaguayo    12/12/00    expiration date checking put at the begining of a working
# aaguayo    14/12/00    expiration date checking... feature disabled again 
# aaguayo    14/12/00    INS.FIMS.NAME changed to filename_skycatversion_fimsversion
# mkiekebu   29/12/00    Added dialog info for GRIS_600RI
# aaguayo    14/03/01    fims 2.28 - (use of SKYCAT_VER variable) 
# mkiekebu   31/03/01    removed bug (last modification)
# aaguayo    05/05/01    modified save_TPX and load_TPX methods to avoid refstars 
#                        in the forbidden zone
# aaguayo    07/05/01    modified load_mxu_TPX to avoid positioning of Slits outside the 
#			 focal field
# mkiekebu   20/07/01    FORS 2 CCD Upgrade
# mkiekebu   30/10/01    Fix problem with merged image (bonsai effect)
# mkiekebu   03/11/01    Added fixed reference slit for MXU masks
# mkiekebu   07/11/01    fixed rotation of wlength limits lines
# mkiekebu   13/12/01    removed some print out
# mkiekebu   24/03/02    look for CHIP.ID instead of CHIP.NAME in ablage
# aaguayo    07/06/01    Removed changes in save_TPX, load_TPX and load_mxu_TPX
#                        that came up from v2.31 to go back to v 2.29
# mkiekebu 2002/10/05    Restored changes on load_mxu_TPX removed previous version
# aaguayo    05/12/02    PPRS7840 - changed Y pos offset of GRIS_600RI to 272.4 pixel
# aaguayo    05/12/02    PPRS7840 - changed Y pos offset of GRIS_1400V to 111.4 pixel
# aaguayo    05/12/02    PPRS7840 - commented workaround 13. Sept 2000 WH (FORS1 dist)
# aaguayo    05/12/02    PPRS7840 - commented plot section about 1000 Pixel area for mxu
# aaguayo    05/12/02    PPRS7840 - De-Select mxu slit when clicking 2-butt on label
# aaguayo    28/10/03    PPRS10127 - Edit button removed while saving
# aaguayo    28/10/03    PPRS10127 - refresh of the ref-slit plots after toggling labels on/off 
# aaguayo    28/10/03    PPRS10127 - Included 'SIMPLE' option 
# aaguayo    28/10/03    PPRS10127 - distortion factors for FORS1 read from the fits headers 
# aaguayo    28/10/03    PPRS10127 - Max number of mxu slits updated to 1000 
# tsz/pbg  2003/11/13    PPRS-10127 - Remove slit if out of range  
# pbaksai  2004/06/12    PPRS-13135 - FORS1 moved from UT1 to UT2
#                                   - FORS2 moved from UT4 to UT1
#                                   - changed focscale_UTx values
# mkiekebu 2004-06-14    Fixed problem with dialog windows (RH9.0)
# mkiekebu 2004-06-17    Accept float for grism wavelenght according new ISF config

#*******************************************************************************
#   NAME             
#   FORS - Tcl class           
# 
#   NAMESPACE
#       fors
#
#   SYNOPSIS         
#       FORS my_mask $g_can $g_ima [options]
#       
#   PARENT CLASS
#       none 
#
#   DESCRIPTION      
#       [incr]tcl/tk class containing full graphical handling
#       of all VLT-FORS1/2 instrumental modes.
#       This class is plugged in to the ESO SkyCat Tool. 
#       Following modes are supported: 
#       FORS1           FORS2
#       ----------------------
#       MOS             MOS 
#       IMG             IMG
#       LSS             LSS
#       HITS            HITS
#       IPOL            MXU
#       PMOS            ECHS
#       ----------------------
#       PMOS: Polarimetric MOS (only even MOS slit blades in use)
#       IMG : Imaging (obscure bright targets with MOS slit blades)
#       IPOL (SR) : Polarimetric IMG (only even slit blades can be used)
#       IPOL (HR) imaging polarimetry, a particular mask 
#       LSS : Long slit spectroscopy, a particular mask 
#       HITS : Hight time resolution spectroscopy,a the LSS mask with a decker
#       MXU : mask exchange unit (FORS2 only) create slits
#       ECHS : echelle spectroscopy, MOS 10 and MOS 9 at a fixed position 
#       There are also 2 fims internal submodes
#       IMS : imaging spectroscopy as a submode of MOS
#             = slitless spectroscopy     
#       PIMS: polarimetric imaging spectroscopy as a submode of PMOS 
#             = slitless spectropolarimetry
#
#       Some Methods of this class call the AstroCat class of SkyCat
#       There is also one external binary (AP) for automatic mask 
#       adjustment. 
#       Only one instance of the class FORS is allowed. 
#       Start with FORSInit. Methods for basic motions 
#       like shift rotate and move slit blades available
#
#   OPTIONS
#       -modus
#           instrumental modi can be one of
#           mos pmos img ipol lss hits
#       -rotangle
#           rotation step in degrees/click, default is 5
#       -width
#           the pixel width of the underlying frame,
#           default is [$g_ima width]
#       -height
#           the pixel height of the underlying frame,
#           default is [$g_ima height]
#       -label
#           flag to show the labels of the mask
#       -fors_frame
#           flag to use image scale from the fits header keyword CDELT1
#           option to apply optical corr to MOS slit positions
#           option to apply optical corr to reference target 
#       -pics      
#           flag to center slit on target, use PickObject 
#       -collimat
#           instrument part : COLL_SR+1 or COLL_HR+2
#       -graytype
#           layout option for tranparency, default 00
#       -slit_width
#           instrument part : MOS slit width in arcsec, default 1 
#       -slit_length
#           layout option : MOS slit length
#       -long_width 
#           instrument part : slit width for LSS and HITS, can be 
#           0.3 0.4 0.5 0.7 1.0 1.6 2.0 ( 2.5 LSS only)
#       -filter 
#           instrument part, default none 
#       -grism 
#           instrument part, default none
#       -colorset 
#           layout option, color of the mask, default Red/Yellow
#       -file_name 
#           project filename 
#
#   PUBLIC METHODS
#       adjust_slit_length {} 
#          handles virtual MOS slit lengths
#       reset_mask {} 
#          called by the rest mask button
#          call init_mask_world2 
#       update_and_replot {} 
#          calls update_scale and plot
#       demo {} 
#          called by demo button
#       init_mask_world2 {} 
#          most important, most complex and most sensistive Method 
#          determines the initial mask position and the complete 
#          layout of the mask 
#       update_mask {} 
#          rescales mask when the 'Z;oom or 'z'oom button is pressed
#       plot_mask {} 
#          plot the mask 
#       help {} 
#           maintenance  
#       AutoPos { kind } 
#           automatic positioning 
#       update_scale {} 
#           called when the zoom button is pushed 
#       load_TPX { filename } 
#           calls either load_mos_TPX or load_img_TPX 
#       save_TPX { } 
#           calls either save_mos_TPX, save_img_TPX or save_lss_TPX
#       wave_config_mask { two } 
#           called by wave_length to assign L_MIN and L_MAX
#       set_wcs { in } 
#           frameless mode
#       new_wcs { } 
#           frameless mode
#       wave_length { } 
#           EntryWidget for user defined wavelength range
#       select_filename  { } 
#           get user defined project name 
#       test_optic_distor_FORS { } 
#           maintenance 
#
#   PUBLIC DATA MEMBERS
#       rotangle    5
#       width       200
#       height      200
#       collimat    none
#       graytype    0
#       modus       mos
#       label       1
#       fors_frame  1  
#       pics        0  
#
#       slit_width  1
#       slit_length org
#       colorset    Red/Yellow
#       long_width  0.5
#       filter      none      
#       grism       none
#
#       lochost  $env(HOST)
#       isf_dir  $env(ISF_DIR)
#       log_dir  $env(LOG_DIR)
#       set_dir  $env(SET_DIR)
#      prep_dir $env(PREP_DIR)
#       int_dir    $env(FIMSROOT)
#
#   PROTECTED METHODS
#
#   PROTECTED DATA MEMBERS
#
#   PRIVATE METHODS
#       init {} 
#       init_instrument {{ccd "FORS-MIT"} 
#            define special parameters according CCD ship
#            FORS-SITE: Actual ccd
#            FORS-MIT : New mosaic CCD (2)
#
#       load_instrument {} 
#            read the FORS.isf and configures the class. 
#            e.g. reads the currently available grisms
#       part_load_foc {fileId line_org} 
#            load focal field dimensions from FORS1.isf
#       part_load_mos {fileId line_org} 
#            load MOS mask dimensions from FORS1.isf
#       part_load_pol {fileId line_org} 
#            load IPOL mask dimensions from FORS1.isf
#       part_load_lss {fileId line_org} 
#            load LSS mask dimensions from FORS1.isf
#       part_load_hits {fileId line_org} 
#            load HITS mask dimensions from FORS1.isf
#       part_load_pinh {fileId line_org} 
#            load positiions and size of the pinholes from FORS1.isf
#       load_ccd_window {}
#            load CCD (master and slave) dimensions from FORS2.isf
#       load_fieldstop {}
#            load field stop dimensions
#       wave_limit {} 
#            start popip window to specify wavelength reagion
#       load_gris_filt_disp{ } 
#            load dispersion table from FORS1.isf 
#       load_mos_TPX { filename } 
#            load target acquisition file for MOS and PMOS, 
#            plot and reset the mask
#            filename is <mode>.<ec>.<projectname>.fims 
#       load_img_TPX { filename } 
#            load target acquisition file for IMG and IPOL, 
#            plot and reset the mask
#            filename is <mode>.<ec>.<projectname>.fims 
#       DDMMSS2deg { ddmmss } 
#            conversion routine 
#       erase {} 
#            delete objects on canvas via tags 
#       get_cat {} 
#            get the currently active catalogue
#       get_ec { mode } 
#           scans the setup file directory and gets new exposure counter
#       get_fits_name { } 
#           initial project_name from fits file name 
#       save_mos_TPX {filename} 
#           save target acquisition file for MOS and PMOS mode 
#       save_img_TPX { filename } 
#           save target acquisition file for IMG and IPOL mode 
#       extra_lss { a b } 
#           handles reference target and slit target for LSS 
#       extra_pick { b } 
#           called by center_slit to assign variables
#       save_lss_TPX { filename } 
#           save target acquisition file for IMG and IPOL mode 
#       bonsai_matrix {} 
#           extrapolate WCS to areas out of the underlying fits frame, that 
#           the mask can also be plotted outside the fits frame 
#       reset_slits {} 
#           called by rotate to deselect all slits
#           we also account for OptiCorr and DiffRefrac in HITS
#       mark { w xs ys } 
#           called when clicked on the green circle to store current mask 
#           position 
#       plot_tcs_label {} 
#           plot the labels with telescope pointing on the canvas
#       shift { w xs ys } 
#           shift the whole mask, called when B1 move 
#       calc_zemax { my x y } 
#           apply correction of optical distortion of FORS1 to MOS slits
#       calc_zemax_ref { a d } 
#           apply correction of optical distortion of FORS1
#           to reference targets
#       calc_diff_refrac { my a d }
#           apply optical correction due to differential refraction
#       calc_oc { x_mm x_pix my a2 d2 a3 d3 } 
#           calculate empirical optical distortion curve 
#           (empirical distortion model no longer used)
#       move_slit { w sx sy {on 1} {click 1} } 
#           move one single slit in MOS and PMOS
#           (MOS,PMOS)
#       move_slit_blade { w sx sy but } 
#           move either MOS blade A or MOS blade B to cursor position
#           (IMG, IPOL-SR, MOS-slitless, PMOS-slitless)
#       rotate { w x y dir} 
#           rotate the whole mask 
#       calc_dither { c frac } 
#           gets the offset for IPOL
#       dither_pol { w c frac } 
#           handles the offset for IPOL
#       bind_frame { } 
#           bindings for green, white and gray circle 
#       bind_mos { } 
#           bindings for MOS and PMOS slit blades pairs 
#       bind_img { } 
#           bindings for IMG and IPOL slit blades 
#       bind_ipol { } 
#           bindings for IPOL mask 
#       center_slit { }  
#           center slit via PickObject
#       catalog { a_c } 
#           compare slit position with local catalog 
#       calc_ctom { } 
#           canvas to focal field mm conversion 
#       select_cat { sc } 
#           if more than one catalog is in use
#       od_F { x y } 
#           empirical optical correction, no longer used 
#       p0 { y } 
#           empirical optical correction, no longer used 
#       p1 { y } 
#           empirical optical correction, no longer used 
#       p2 { y } 
#           empirical optical correction, no longer used 
#       p3 { y } 
#           empirical optical correction, no longer used 
#       p4 { y } 
#           empirical optical correction, no longer used 
#       ablage_SR { r } 
#           theoretical optical distortion 
#       ablage_HR { r } 
#           theoretical optical distortion 
#       test_ablage {  } 
#           maintenance 
#
#   PRIVATE DATA MEMBERS
#       f_v s_v           // fims and skycat release number
#       lfId              // logfile id 
#       m_image,m_canvas  //  local name for canvas and image
#       canvscale         // scale (zoom) of the world/canvas
#       EQX               // equinox 
#       s_slit 4          // boundary errors in slits_lengths  [mm]
#       world_x, world_y  // the actual position of the mask in world units
#       wx,wy             // the initial position of the mask in world units
#       mos               // the cordinates of the MOS mask, a big matrix
#       A_disp            // the grism-filter-dispersion array     
#       good              // index of current filter-grism set     
#       targw             // the world coordinates of the 19 positioned slits
#       abs_ang_rad 0.0   // absolute angle of the mask in rad with respect
#                         // to the WCS                                        
#       x_ccd 300         // size of the ccd
#       ccx, ccy          // the  center of the mask (TCS position on canvas)
#       fr_x,fr_y         // initial canvas positon of mask = canvas center
#       worldscale 1.7    // scale pixel/''
#       oldpos            // old position for shifting
#       speclen 50.0      // the length of the spectrum on the CCD in arcsec
#       specmin 50.0
#       specmax 50.0
#       L_MIN,L_MAX       // user defined wavelength range
#                         // FORS configuration constants
#       XSRMAX,YSRMAX,XHRMAX,YHRMAX,XSRMIN,YSRMIN,XHRMIN,YHRMIN
#       mosslitmin,mosslitmax
#       htslitxmin,htslitymin,htslitxmax,htslitymax
#       longslitxmin,longslitymin,longslitxmax,longslitymax
#       polarimslitxmin,polarimslitymin,polarimslitxmax,polarimslitymax
#       long_number
#       mask
#       slit_coll
#       gris_range
#       filt_range
#       osfi_range
#       exposure_counter 1
#       trapa gray00      // transparanecy of slit blades string
#                         // VLT-U1 focal scale in arcsec/mm
#                         // there is rho=0.5280 theta -1.1288E-6 theta*theta
#                         // we use 1/0.5280 [arcsec/mm]
#                         // x [mm] *$focscale = y [arcsec]  (mtoas)
#       focscale 1.893939394
#                         // scale on the CCD  [mm/arcsec]
#                         // y [arcsec] * CCDscale = x [mm] (astom)
#                         // this can be modified:
#                         // take the Focus value from FORS.isf
#                         // nominal values
#       CCDscale_standard 0.2 // arcsec/pixel 
#       CCDscale_highres  0.1 // arcsec/pixel
#       PIX              0.024 // pixel size in mm 
#       atd 2.7777778E-4  // numeric arcmin_to_degree
#       trigo 1
#       ctom              //  numeric canvas_to_mm
#       color1 red        // fill colors
#       color2 yellow
#       color3 red        // outline colors 
#       color4 yellow
#                         // LSS reference and slit object
#       lss_ref_ra, lss_ref_de, lss_sli_ra, lss_sli_de
#                         // fits scale 
#       fits_x_scale, fits_y_scale
#       WP                // angle between WCS and pixel coords
#       man_WP            // interactive input of WP
#       CSO               // coord-system-orientation = 1/-1
#       CRO               // signum(CROTA2) needed for init_mask
#       PAM               // position angle of the mask 
#       tcs_pam           // = -PAM for tcs_label 
#       mask_initialized  // set by init_mask_world2
#       mountangle        // no comment                 
#
#   FILES
#       fims.tcl SkyCat_plugin.tcl SkySearch_plugin.tcl 
#
#   ENVIRONMENT
#       ESO-RTD (Real-Time-Display), 
#       ESO-DMD-SkyCat Tool
#       fims calls Methods of the original 
#       ESO-SkyCat code, which are modified for 
#       fims purposes:
#       RtdImageTrans: Methods rotate, update_trans and flip
#       AstroCat:      Method remove_FORS
#
#   COMMANDS
#       FORSInit { g_can g_ima menubar modus control su} 
#
#   RETURN VALUES
#
#   CAUTIONS
#       Use FIMS V1.2.9 only with skycat V>2.1.3
#
#   EXAMPLES
#
#   SEE ALSO
#
#
#         1) The Focal Field Orientation of FORS 
#
#            This diagram shows the focal field of FORS1/2 
#            together with the orientation of several coordinate
#            systems, like pixel (x,y), WCS (RA,DEC) and the fims
#            internal indicees of the large mask matrix 
#            mos(slitnumber,cornerindex)
#
#            Focal field as viewed from bottom to top
#            Field as it appears in the final fits frame 
#            and in FIMS 
#   
#
#
#         mos(0,a)          ^           mos(0,b)
#         (white circle)    |                           
#                           |
#                       del,DEC>0
#            Instrument Connection Point (ICP)       
#                  IPOL-mask/MXU-mask 
#                       North (ff)
#                        MOS 1
#                     y<0 (YSRMIN)
#                           |
#                           |     P.A.
#                           |    /
#                           |---/
#                           |  /
#                           | /
#          East  (ff)       |/        West (ff)   
#    <------Mos-A-----------+---------Mos-B--------->
#           x<0 (XSRMIN)    |         x>0 (XSRMAX)
#          (blue spectrum)  |         (red spectrum) 
#          alp,RA>0         |         alp,RA<0
#                           |
#                           |
#                        del,DEC<0
#                      (coll-drive)
#                        LSS-mask
#                       South (ff)
#                         MOS 19
#                     y>0 (YSRMAX)
#                           |
#                           |
#         mos(0,d)          V            mos(0,c)
#        (gray circle=
#        CCD-reference pixel)
#        pixel=1,1
#
#
#        2) orientation of the internal mask array
#
#         focfield A = mos(0,a) = (-x,+y)  (N,E) (white circle)
#                  B = mos(0,b) = (+x,+y)  (N,W)
#                  C = mos(0,c) = (+x,-y)  (S,W)
#                  D = mos(0,d) = (-x,-y)  (S,E) (gray circle 
#                                                 = CCD reference pixel)
#
#
#        3) orientation of the wavelength borders 
#                                                     
#        If 'label'-button == on, the user defined wavelength 
#        borders are drawn on the canvas. 
#        CCD frames  STANDARD RESOLUTION, high resolution 
#        e.g. mos(0,gr) is the high res mask SW wavelength border
#
#                            (-y,N)   
#                            (slit 1) 
#                A      Ar          Br       B    
#                +------+-----------+--------+
#                |e     |er         |fr      |f
#                |      |           |        |
#                |      |           |        |
#                |      |           |        |
#        (-x,E)  |      |           |        | (x,W, MOS B)
#        (MOS A) |      |           |        |
#                |      |           |        |
#                |      |           |        |
#                |h     |hr         |gr      |g
#                +------+-----------+--------+
#                D      Dr          Cr       C 
#
#                          (y,S, slit 19)
#
#        e.g. mos(0,br) is the north red wavelength border line 
#        for standard resolution collimator modus
#        wavelength labels at Cr,Dr and hr,gr
#
#        4) orientation of the individual slit blades
#                                                      
#        coords of a mos slit blade  mos($i,ax) ...
#                          
#            a              b        c               d
#            ---------------+        +----------------
#                           |  N,-y  |
#           (A, East,-x)    |        | (B, West, +x)
#                           |  S,+y  |
#            ---------------+        +----------------
#            e              f        g               h
#
#        e.g. mos(7,h) is the north east corner of slit 7
#
#        5)  coords of a long-slit and ipol stripe
#                                                  
#                        
#             a d               
#             +-+                   
#             | |          a                   d
#             | |          +-------------------+
#             | |  LSS     |                   |  IPOL
#             | |          +-------------------+
#             | |          e                   h
#             +-+       
#             e h 
#
#          LSS/HITS orientation position angle=0
#          --------------------------------
#          9 long slits in north-south direction
#          and two pinholes :  P=big pinhole, p=small pinhole
#          The big pinhole becomes visible in HITS 
#          together with the 0.3'' HIT slit. 
#          The small pinhile becomes visible together with 
#          the 0.4'' HIT slit. The 0.4'' HT slit is not supported
#          pinhole is not supported (neither by P2PP nor by FIMS)
#     
#     
#                              north          pixel 2000,2000
#                    +-------------------------+
#                    |  1.6   0.5   0.4   1.3  | <-slit width in arcsec
#                    |     \    |   |    /     |
#        internal    |      |   |   |   |      |           +-----> x
#        slit        |    | | | | | | | | |    |           |
#        number----> |    9 7 5 3 1 2 4 6 8    |           |
#             east   |    | | | |P|p| | | |    |  west    y|
#                    |    | | | | | | | | |    |           V
#                    |    | | | | | | | | |    |
#                    |    |   |   |   |   |    |
#                    |   /    /   |   \    \   |
#                    |2.5  1.0   0.3   0.7  2.0| <-slit width in arcsec 
#                    +-------------------------+
#            pixel 1,1
#     
#     
#             long slit decker
#             ----------------     
#                              north          pixel 2000,2000
#                    +-------------------------+
#                    |       +-+               |                        
#                    |       | | <-- LSS       |
#                    |       | |               |           +-----> x
#                    |       | |               |           |
#                    |       | |               |           |
#                    |       | |               |  west    y|
#                    |       | |               |           V
#                    |       | |               |
#                    |       | |  +-+          |
#                    |       | |  | | <-- HITS |
#                    |       +-+  +-+          |                        
#                    +-------------------------+
#            pixel 1,1
#
#                    The x-distance between HITS decker and LSS decker 
#                    is half the distance between two LSS slits.
#                    If HITS decker enables 0.3'' slit, the LSS decker
#                    will enble the big pinhole 
#     
#
#        6) miscellaneous: orientation of the FORS focal field
#           with respect to other coord systems 
#                                               
#                               (Tantal)
#                                   s
#        geographic :  (Pacific)  w   e (Andes)
#                                   n
#                             (Antofagasta)
#
#    
#                              ICP/IPOL
#         instrument :      MOS-A   MOS-B
#                              COLL/LSS 
#                                                
#                                #1
#        mask units :       MOS-A   MOS-B
#                                #19
#       
#                                (-y)   
#        cartes. focal field   (-x) (x) 
#                                 (y)
#    
#                                 n
#        sky in foc field        e w
#        ----------------         s
#    
#                              del/DEC>0
#        aequat coords    alp/RA>0    alp/RA<0
#                              del/DEC<0   
#
#
#   BUGS
#       none known 
#
#-------------------------------------------------------------------------------
#
#23456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567890
#


itk::usual FORS {}

# create a class for the application

namespace eval fors {

class FORS {

#------------------------
constructor {mc mi menu args } {
#------------------------
     global env
     set m_canvas $mc; set m_image $mi   
     set m_mb $menu ;                     # external menu bar

     set f_v [eval fims_version] ;        # fims version
     set s_v [eval skycat_version] ;      # skycat version
     set tcl_precision 18

    #Init CCD parameters
   
    #init_ccd_params "FORS-MIT"
    if {"[$this cget -instrument]" == "FORS2"} {
        init_ccd_params "FORS-MIT"
    } else {
	init_ccd_params "FORS-SITE"
    }

    # a very important act:
    # the instrument name (FORS1 - FORS2) must be known before the 
    # 'load_instrument'  Method, since we have to load either 
    # FORS1.isf or FORS2.isf to configure the instrument  
    # we call 1/2 from the menu button 
    #
    set instrument [string toupper [lindex [split $menu .] 3] ]

    puts stdout "\nNEW MODE -->SkyCat V$s_v -->  FIMS V$f_v for $instrument"


    # open a log-file to write some info
    # the instrument modus is not known yet
    #
    set log_file $log_dir
    append log_file "/startup.log"
    set lfId [open $log_file w]

       # Maintenance: If FORS1 is moved from UNIT -> UNIT3 
       # edit the '1' branch    
       switch -exact -- [string index [$this cget -instrument] 4] {
          1 { set focscale $focscale_UT2 
              puts stdout "FORS1 at UT2" 
            }
          2 { set focscale $focscale_UT1
              puts stdout "FORS2 at UT1" 
            }
      default { puts stdout "Error in telescope focal length" }
           }

     if {[string compare $s_v $env(SKYCAT_VER)]} {
             $m_mb entryconfigure 6 -state disabled
           info_dialog "SkyCat Version Error:\n\
                        \n\
                        You cannot \[Save\] !" .skycat1

             # but not for me (haha)
             if {[string compare [exec whoami] hummel]==0} { 
                $m_mb entryconfigure 6 -state normal } 
             if {[string compare [exec whoami] fors]==0} { 
                $m_mb entryconfigure 6 -state normal } 
             if {[string compare [exec whoami] observer]==0} { 
                $m_mb entryconfigure 6 -state normal } 
             }

 
    # load FORS1.isf or FORS2.isf
    # configure grism requires dispersion table 
    # 
     [code $this load_instrument ]

    # from here on collimat (either COLL_SR+1 or COLL_SR+6) 
    # known  

    # ------------------------
        eval configure $args
    # ------------------------
    #   now grism, filter known

    # we set expolicilety the collimator from co_range
    # $this configure -collimat [lindex [split $co_range " "] 0]
    # we highlight collimator button

    $m_mb.layout.sh invoke 0
    
    # if reference pixel is outside the image 
    # this can happen when using midas: EXTRACT/IMA
    # mask is centered to frame (pixel) center
    #
    # default telecope pointing == mask center 
    # == FITS header WCS reference pixel for WCS
    #
    set center 0
    set fr_x [$m_image fits get CRPIX1]
    set fr_y [$m_image fits get CRPIX2]
   
    # puts "reference  pixel = $fr_x $fr_y"
    if {$fr_x > [$m_image fits get NAXIS1] || 
        $fr_y > [$m_image fits get NAXIS2]} {
        set center 1
        set fr_x [expr $width/2]
        set fr_y [expr $height/2]
        puts stdout "Warning: reference pixel outside frame: $fr_x $fr_y"
        }

    # read the equinox (2000 or 1950) from fits frame
    # read WCS at reference pixel 
    #
    set EQX [lindex [$m_image wcsset] 8 ]
    set world_x [$m_image fits get CRVAL1]
    set world_y [$m_image fits get CRVAL2]

    # frameless mode, at least no image is loaded
    if {$world_x=="" || $world_y==""} {
       set wx [lindex [$m_image wcsset] 0]
       set wy [lindex [$m_image wcsset] 1]
       if [catch { $m_image convert coords $wx $wy "wcs $EQX" \
                   world_x world_y "deg $EQX" } result ] {
                  info_dialog "No Frame Loaded\n\No Catalog Loaded" .skycat1
                  set world_x 100; set world_y 100
                 } 
       }

    if {$center} {
	puts "FLAG center"
       # reference pixel outside frame, take frame center
       $m_image convert coords $fr_x $fr_y image world_x world_y "deg $EQX"
       puts stdout "Warning: WCS reference pixel outside frame : \
       recenter to     $world_x  $world_y"
       }

     set wx $world_x; set wy $world_y

     # this statement is questionable 
     # reference pixel only used in the zemax Method 
     set ccx $fr_x;   set ccy $fr_y  

     # create an instance of the reference target class
     #
     fors::REF_TARG C_ref_targ $m_canvas $m_image -eqx $EQX


     # the FIMS internal exposure counter will be part of the
     # setup files .p_targ .p_focf and .fims
     #
     set exposure_counter [get_ec [$this cget -modus] ]

     # the initial projectname is taken from the fits filename
     #
     $this configure -file_name [ get_fits_name ] 

     # initial mask scale taken from fits header if fits from FORS1 itself
     #
     # for the status panel 
     set sp_instrume [$m_image fits get INSTRUME]
     set sp_origin   [$m_image fits get ORIGIN]
     if {[string match *FORS* $sp_instrume ]} {
        # reset external radio button 
	$this configure -fors_frame 1
        # $m_mb.layout.ms invoke 1 

        if {[string compare ESO $sp_origin] } {
           puts stdout "Warning: FITS file observed with FORS but processed by [$m_image fits get ORIGIN]"
           }

        } else {
	$this configure -fors_frame 0
        #$m_mb.layout.ms invoke 0
        }



     # Copy the current image scale to mxu class
     #
     if {[string compare [$this cget -modus] mxu]==0} {
        fors::MXU_SLIT::fc $focscale
        fors::MXU_SLIT::label [$this cget -label]
        } 

     # mxu  : has internal mmfocalfield <-> canvas conversion
     # echs : requires explicitly mm <->canvas conversion
     # create an instance of the coord conversion class
     #  badpixelmap requires conversion for all modi 
     fors::CONV_MC C_mc -eqx $EQX

     # create an instance of the coord conversion class
     # WCS - canvas coords
     fors::CONV_WC C_wc -m_image $m_image -eqx $EQX

     # create a mask at the initial WCS position with default parameters
     #
     [code $this init_mask_world2 ]

     # plot the mask on the canvas 
     #

     [code $this plot_mask]


     # start the basic bindings : move and rotate the mask 
     #
     [code $this bind_frame]

     # update canvas scale
     #
     [code $this update_scale]

     if {[$this cget -badpixelmap]} { 
         fors::CONV_MC::coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
             $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
             $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
        update_BPM }


     # Note : from 0..NOG  GRISM
     #        from 19..16 XGRISM
     # bad habbit, hard coded
     if {[string match "*echs*" [$this cget -modus]]} {
        for {set BUT 0} {$BUT <= 19} {incr BUT 1} {
             set a [$m_mb.layout.gr entrycget $BUT -label]
             # disable normal, enable XGRIS
             if {[string match "*.*" $a] && ![string match "*110I*" $a]} {
             $m_mb.layout.gr entryconfigure $BUT  -state normal 
             # puts stdout "normal $a"
             } else {
             $m_mb.layout.gr entryconfigure $BUT  -state disabled
             # puts stdout "disable $a"
             }
             }

        # find the menu button number for 'none.none.none'
        # invoke 'none.none.none' for default 
           foreach BUT { 19 18 17 16} {
                 set a [$m_mb.layout.gr entrycget $BUT -label]
                  if {[string match "none.none.none" $a]} {
                     $m_mb.layout.gr invoke $BUT
                     }
                  }

        } else {

        # its not echs
           # lss mos hits mxu 
           # disable the XGRIS combinations 
        for {set BUT 0} {$BUT <= 19} {incr BUT 1} {
             set a [$m_mb.layout.gr entrycget $BUT -label]
             # disable normal, enable XGRIS
             if {[string match "*.*" $a]} {
             $m_mb.layout.gr entryconfigure $BUT  -state disabled 
             # puts stdout "disable $a"
             } else {
             $m_mb.layout.gr entryconfigure $BUT  -state normal
             # puts stdout "normal $a"
             }
             }
       
        # for MXU-echs EGRIS permitted
      # if {[string compare mxu [$this cget -modus]]==0} {
      # foreach BUT {19 18 17 16} {
      #          set a [$m_mb.layout.gr entrycget $BUT -label]
      #           if {![string match "*110I*" $a]} {
      #              $m_mb.layout.gr entryconfigure $BUT  -state normal
      #              }
      #      }
      #     }

        #find the menu button number for 'none' entry
        # default is 'none'
        set BUT 0
        while {[string compare  "none" [$m_mb.layout.gr entrycget $BUT -label]]} {
               # puts stdout "[$m_mb.layout.gr entrycget $BUT -value]"
               incr BUT 1
              }
        # puts stdout "$BUT = [$m_mb.layout.gr entrycget $BUT -value]"
        $m_mb.layout.gr invoke $BUT
        }

     # open a  new log file 
     open_log 

     # expiration checking in the begining of a session
     # if [expire_date] warn

     if {[string match "*mxu*" [$this cget -modus]]} { 
        load_mxu_refslit
        }
     puts stdout "mask initialized"
     set last_action "Enter [ string toupper [$this cget -modus] ]"
     catch {SP update}
     }



     #---------------
     destructor {
     #---------------
	 global imageName

     # delete drawings from the canvas 
     erase 
     # close the log file 
     puts $lfId ">>> Quit Mode <<"
     close $lfId
     delete object C_ref_targ
     delete object C_wc
     delete object C_mc 

    if {$pics}     {$m_mb.layout invoke 15}
    if {$graytype} {$m_mb.layout invoke 12}

    # handle grism types for FORS2 modes 
    foreach BUT { 0 1 2 3 4 5 6 7 8 9 10 11 12 13} {
            $m_mb.layout.gr entryconfigure $BUT  -state normal  
            }

     # delete all instances of MXU slit 
     if {[string compare [$this cget -modus] mxu]==0} {
        set noms [fors::MXU_SLIT::nos]
        if {$noms>0} {
           for {set i 1} {$i <= $noms} { incr i 1} { delete object C_mxus$i }
           }
        }

      set last_action "Quit [ string toupper [$this cget -modus] ]"

      catch {SP update}
      }

     #========================
     private method init {} {
     #========================
     }

     #=================================================
     private method init_ccd_params { {ccd "FORS-MIT"}} {
	 if {"$ccd" == "FORS-SITE"} {
	     set PIX 0.024 
             set CCDscale_standard 0.2
	     set ccdscale_highres  0.1
	 } elseif {"$ccd" == "FORS-MIT"} {
	     set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"]
             if {$binx == 1} {
		 set PIX 0.015
		 set CCDscale_standard 0.125062
                 set CCDscale_highres 0.0625313
	     } else {
	         set PIX 0.030
	         set CCDscale_standard 0.250124
	         set CCDscale_highres  0.1250626
		
	     }
	 }
       
	 
     }
     #=================================================

     #==================================
     private method load_instrument {} {
     #==================================
     # Open the instrument summary file, reads available 
     # configuration and set relevant variables. 
     # Open either FORS1.isf or FORS2.isf in the 
     # $ISF_DIR directory 
     puts "Executing load_instrument ..."
     puts stdout "load [$this cget -instrument].isf"
     #+++++++++++++++++++++++++++++++++++++++++++++++++++
     set myfile "$isf_dir/[$this cget -instrument].isf" 
     #+++++++++++++++++++++++++++++++++++++++++++++++++++
     set keyword VALUE
     set keyw2   RANGE

     set filt_range   "";  # filter range 
     set gris_range   "";  # grism range (normal dispersion)
     set htslit_range "";  # this does not exist in FORS2.isf 
     set slit_range   "";  # long slit range 
     set ph_range     "";  # pinhole range 
     set co_range     "";  # collimator range 
     set egris_range  "";  # echelle grism range 
     set xgris_range  "";  # cross disperser grism range 
     set fwhm_range   "";  # FWHM of filtrange 
    set lamcent_range "";  # lam0 of filtrange 

     if {[file exists $myfile] & [file readable $myfile]} {
        set fileId [open $myfile r ]
        puts $lfId "FORS::load_instrument  $fileId"
        # as long as there are lines in the file
        while {[gets $fileId line_org] >= 0} {
              if {[string match #* $line_org]} {
                 # ignore comment lines 
                 # puts stdout $line_org
               } else {

      # ---------------------------
      # PART-1 load mask dimensions 
      # ---------------------------
         if {[string match *$keyword* $line_org]==1} {
             switch -glob -- $line_org {
                   *MOSSLIT* { part_load_mos  $fileId $line_org }
                  *LONGSLIT* { part_load_lss  $fileId $line_org }
                    *HTSLIT* { part_load_hits $fileId $line_org }
               *POLARIMSLIT* { part_load_pol  $fileId $line_org }
                  *FOCFIELD* { part_load_foc  $fileId $line_org }
                      *PINH* { part_load_pinh $fileId $line_org }		      
                           }
           }

          # ---------------------------
          # PART-2 load setup ranges  
          #                   ------
          # ---------------------------

          if {[string match *$keyw2* $line_org]==1} {
             switch -glob -- $line_org {
            *FILT* { set filt_range [lrange [split $line_org \" ] 1 1]
                     set ls [expr [string length $filt_range] -1]
                     set filt_range [string range $filt_range 1 $ls]
                     set filt_range [split $filt_range  " "] 
                     set filt_range [string trimright $filt_range "\}" ] 
                                       set dummy "\{"
                     set filt_range [string trimright $filt_range "\\" ] 
                     set filt_range [ldelete_all $filt_range ""] 
                     }
            *FWHM* { set fwhm_range [lrange [split $line_org \" ] 1 1]
                     set ls [expr [string length $fwhm_range] -1]
                     set fwhm_range [string range $fwhm_range 1 $ls]
                     set fwhm_range [split $fwhm_range  " "] 
                     set fwhm_range [string trimright $fwhm_range "\}" ] 
                                       set dummy "\{"
                     set fwhm_range [string trimright $fwhm_range "\\" ] 
                     set fwhm_range [ldelete_all $fwhm_range ""] }
         *LAMCENT* { set lamcent_range [lrange [split $line_org \" ] 1 1]
                     set ls [expr [string length $lamcent_range] -1]
                     set lamcent_range [string range $lamcent_range 1 $ls]
                     set lamcent_range [split $lamcent_range  " "] 
                     set lamcent_range [string trimright $lamcent_range "\}" ] 
                                       set dummy "\{"
                     set lamcent_range [string trimright $lamcent_range "\\" ] 
                     set lamcent_range [ldelete_all $lamcent_range ""] }
            EGRIS* { set egris_range [lrange [split $line_org \" ] 1 1]
                     set ls [expr [string length $egris_range] -1]
                     set egris_range [string range $egris_range 1 $ls]
                     set egris_range [split $egris_range  " "] 
                     set egris_range [string trimright $egris_range "\}" ] 
                                       set dummy "\{"
                     set egris_range [string trimright $egris_range "\\" ] 
                     set egris_range [ldelete_all $egris_range ""] }
            XGRIS* { set xgris_range [lrange [split $line_org \" ] 1 1]
                     set ls [expr [string length $xgris_range] -1]
                     set xgris_range [string range $xgris_range 1 $ls]
                     set xgris_range [split $xgris_range  " "] 
                     set xgris_range [string trimright $xgris_range "\}" ] 
                                       set dummy "\{"
                     set xgris_range [string trimright $xgris_range "\\" ] 
                     set xgris_range [ldelete_all $xgris_range ""] }
             GRIS* { set gris_range [lrange [split $line_org \" ] 1 1]
                     set ls [expr [string length $gris_range] -1]
                     set gris_range [string range $gris_range 1 $ls]
                     set gris_range [split $gris_range  " "] 
                     set gris_range [string trimright $gris_range "\}" ] 
                                       set dummy "\{"
                     set gris_range [string trimright $gris_range "\\" ] 
                     set gris_range [ldelete_all $gris_range ""] }
          *HTSLIT* { set htslit_range [lrange [split $line_org \" ] 1 1]
                     set ls [expr [string length $htslit_range] -1]
                     set htslit_range [string range $htslit_range 1 $ls]
                     set htslit_range [split $htslit_range  " "] 
                     set htslit_range [string trimright $htslit_range "\}" ] 
                                       set dummy "\{"
                     set htslit_range [string trimright $htslit_range "\\" ] 
                     set htslit_range [ldelete $htslit_range ""] }
            *SLIT* { set slit_range [lrange [split $line_org \" ] 1 1]
                     set ls [expr [string length $slit_range] -1]
                     set slit_range [string range $slit_range 1 $ls]
                     set slit_range [split $slit_range  " "] 
                     set slit_range [string trimright $slit_range "\}" ] 
                                       set dummy "\{"
                     set slit_range [string trimright $slit_range "\\" ] 
                     set slit_range [ldelete $slit_range ""] }	      
            *PINH* { set ph_range [lrange [split $line_org \" ] 1 1]
                     set ls [expr [string length $ph_range] -1]
                     set ph_range [string range $ph_range 1 $ls]
                     set ph_range [split $ph_range  " "] 
                     set ph_range [string trimright $ph_range "\}" ] 
                                       set dummy "\{"
                     set ph_range [string trimright $ph_range "\\" ] 
                     set ph_range [ldelete $ph_range ""] }
            *COLL* { set co_range [lrange [split $line_org \" ] 1 1]
                     set ls [expr [string length $co_range] -1]
                     set co_range [string range $co_range 1 $ls]
                     set co_range [split $co_range  " "] 
                     set co_range [string trimright $co_range "\}" ] 
                                       set dummy "\{"
                     set co_range [string trimright $co_range "\\" ] 
                     set co_range [ldelete $co_range ""] }
                           }
              }
           }
         }
     # puts $lfId "closed:  $fileId"
     close $fileId
     } else {
     puts $lfId "Warning:$myfile: does not exist or has no read permission"
    puts stdout "Warning:$myfile: does not exist or has no read permission"
     }

          # ---------------------------
          # PART-3 load dispersion array   
          # ---------------------------
         load_gris_filt_disp 

   # puts stdout "  filt_range = $filt_range"
   # puts stdout "  fwhm_range = $fwhm_range"
   # puts stdout "lamcentrange = $lamcent_range"
   # puts stdout "  osfi_range = $osfi_range"
   # puts stdout "  gris_range = $gris_range"
   # puts stdout "  slit_range = $slit_range"
   # puts stdout "htslit_range = $htslit_range"
   # puts stdout "    ph_range = $ph_range"
   # puts stdout "    co_range = $co_range\n"

   puts $lfId "  filt_range = $filt_range"
   puts $lfId "  fwhm_range = $fwhm_range"
   puts $lfId "lamcentrange = $lamcent_range"

  set a [expr [llength $filt_range] -1 ]
  # last filter 'none' has not entries in fwhm and lamcent
  for {set i 0} {$i < $a} {incr i 1} {
      puts $lfId [format "%2d %18s %8.3f %8.3f" \
      $i [lindex $filt_range $i] \
      [lindex $lamcent_range $i] [lindex $fwhm_range $i] ]
      }

   puts $lfId "  osfi_range = $osfi_range"
   puts $lfId " Xgris_range = $xgris_range"
   puts $lfId " Egris_range = $egris_range"
   puts $lfId "  slit_range = $slit_range"
   puts $lfId "htslit_range = $htslit_range"
   puts $lfId "    ph_range = $ph_range"
   puts $lfId "    co_range = $co_range"

   # configure the menu buttons according to FORS1/2.isf 
   #
   set num 0;          # collimator 
   #                     ----------
   foreach i $co_range {
   $m_mb.layout.sh entryconfigure $num  -label $i \
          -variable co -value $i
    incr num 1
        }

   set num 0;            # order separation filter 
   #                       -----------------------
   # for FORS2 we must fill menu items 10 by 10 

   foreach i $filt_range {
   
   if {$num<=9 && $num>=0} {
   $m_mb.layout.fi entryconfigure $num -label $i \
          -variable fi -value $i
   }

   if {$num>=10 && $num <=19} {
   $m_mb.layout.fi.m2 entryconfigure [expr $num-10]  -label $i \
          -variable fi -value $i
   }

   if {$num>=20 && $num <=29} {
   $m_mb.layout.fi.m3 entryconfigure [expr $num-20]  -label $i \
          -variable fi -value $i
   }

   if {$num>=30 && $num <=39} {
   $m_mb.layout.fi.m4 entryconfigure [expr $num-30]  -label $i \
          -variable fi -value $i
   }

   if {$num>=40 && $num <=49} {
   $m_mb.layout.fi.m5 entryconfigure [expr $num-40]  -label $i \
          -variable fi -value $i
   }

   incr num 1
   }



   set num 0;            # grism 
   #                       -----
   foreach i $gris_range {
   $m_mb.layout.gr entryconfigure $num  -label $i \
          -variable gr -value $i
    incr num 1
        }

   # there are num grisms
   set NOG $num

   # the menu button names for the cross disperser grism,
   # high disperser grism and order separation filter 
   # combination in echs is still hardcoded
   #
   if {[string compare [$this cget -instrument] FORS2]==0} {

   # set echelle_range \
   #  "XGRIS_600B+92.EGRIS_110B+98.none \
   #   XGRIS_600V+90.EGRIS_110I+99.GG435+81 \
   #   XGRIS_300I+91.EGRIS_110I+99.OG590+32 \
   #   none.none.none";

   set num 19; # we name XGRIS from the uppermost menu entry

   foreach i $echelle_range {
   $m_mb.layout.gr entryconfigure $num  -label $i \
          -variable gr -value $i
    incr num -1
        }


    # disable the red EGRis, its not yet delivered 
    incr num +2 
    $m_mb.layout.gr entryconfigure $num -state disabled
    incr num +1 
    $m_mb.layout.gr entryconfigure $num -state disabled
    }

   # ---------------------------
   # PART-4 load ccd dimensions   
   # ---------------------------
   
   load_ccd_window

   # ----------------------------------
   # PART-5 load field stop dimensions
   # ----------------------------------
  
   load_fieldstop

   # ----------------------------------
   # PART-6 load mxu reference slit 
   # ----------------------------------
  
   load_mxu_refslit_info
   
}

# ==============================================
private method part_load_foc {fileId line_org} {
# ==============================================
# The current line, just read from FORS1/2.isf contains 
# the keyword 'FOCFIELD'. Hence we read the ISF further on 
# and set the focal field ranges 
 
set blank " "
# line_org is the first line containing FOCFIELD  
# the FOCFIELD in FORS1/2.isf has two
# values per line -> string operations 
# puts stdout "FOC: $line_org"
              
for {set i 1} {$i <5} { incr i 1} {
    # read three more lines
    set line [split $line_org \" ]
    set pattern [lrange $line 1 1];            # e.g. -11.1914 104.249
    set keyword [lrange $line 0 0];            # e.g. FOCFIELDXRANGESR.VALUE
    set keyword [split $keyword \. ];          # e.g. FOCFIELDXRANGESR VALUE
    set keyword [lrange $keyword 0 0];         # e.g. FOCFIELDXRANGESR
    set keyword [string range $keyword 2 [string length $keyword]]
    # puts stdout ">>$keyword<<  >>$pattern<<"

    if {[string match *$blank* $pattern]==1} {
       # puts $lfId "blank >>$blank<< in >>$pattern<<"
       set a [string length $pattern] 
       incr a -2 
       # extract { and } 
       set subpat [string range $pattern 1 $a]
 
       set subpat [split $subpat $blank ]
       set part1 [lrange $subpat 0 0]
       set part2 [lrange $subpat 1 1]
       # puts $lfId ">>$part1<< >>$part2<<"
       if {[string match *XRANGESR* $keyword]==1} {
          set XSRMIN $part1; set XSRMAX $part2 
          if {$XSRMIN > $XSRMAX} {
             set dummy $XSRMIN 
             set XSRMIN $XSRMAX
             set XSRMAX $dummy
             }
          }
       if {[string match *YRANGESR* $keyword]==1} {
          set YSRMIN $part1; set YSRMAX $part2 
          if {$YSRMIN > $YSRMAX} {
             set dummy $YSRMIN 
             set YSRMIN $YSRMAX
             set YSRMAX $dummy
             }
          }
       if {[string match *XRANGEHR* $keyword]==1} {
          set XHRMIN $part1; set XHRMAX $part2 
          if {$XHRMIN > $XHRMAX} {
             set dummy $XHRMIN 
             set XHRMIN $XHRMAX
             set XHRMAX $dummy
             }
          }
       if {[string match *YRANGEHR* $keyword]==1} {
          set YHRMIN $part1; set YHRMAX $part2 
          if {$YHRMIN > $YHRMAX} {
             set dummy $YHRMIN 
             set YHRMIN $YHRMAX
             set YHRMAX $dummy
             }
          }
       # read three more lines       
       gets $fileId line_org
       }
     }
     # some more controlling 
     if { $XSRMIN > $XSRMAX | $YSRMIN > $YSRMAX } {
        puts stdout "Warning:$myfile: problems with SR" }
     if { $XHRMIN > $XHRMAX | $YHRMIN > $YHRMAX } {
        puts stdout "Warning:$myfile: problems with HR" }
     if { $XHRMAX > $XSRMAX } {
     puts stdout "Warning:$myfile: problems with MIN or MAX" }

     # XSRMIN-MAX are the optical focal field limits 
     # XBLADEA, XBLADEA are the park positions of the MOS
     #                  are the min/max  of INS.MOS.POS
     # The park positions must be lower that the limit switches.
     # The nominal park positions are +-123 mm
     #
     set XBLADEA -123.0; # MOSA (mountangle 0=east, 180=west) 
     set XBLADEB +123.0; # MOSB (mountangle 0=east, 180=west)

     # dont show the real dimension of MOS1 and MOS19 but only 
     # the frcational part which is inside the focal field 
     #
     #set mosslitmin(1)  $YSRMIN
     #set mosslitmax(19) $YSRMAX

     puts $lfId "XSRMIN $XSRMIN $XSRMAX $XHRMIN $XHRMAX"
     puts $lfId "YSRMIN $YSRMIN $YSRMAX $YHRMIN $YHRMAX"

   }
 

#===============================================
private method part_load_mos {fileId line_org} {
#===============================================
# Read the geometrical dimensions of the 19 MOS slit blade pairs
# from the instrument summary file 
set flag 1
set count 1
for {set i 1} { $i < 39 } { incr i 1} {
    set line [split $line_org \" ]
    set pattern [lrange $line 1 1];                 # e.g. -113.9
    set keyword [lrange $line 0 0];                 # e.g. MOSSLITYMIN1.VALUE
    set keyword [split $keyword \. ];               # e.g. MOSSLITYMIN1 VALUE
    set keyword [lrange $keyword 0 0];              # e.g. MOSSLITYMIN1
    set keyword [string range $keyword 2 [string length $keyword]]
    # puts stdout ">>$keyword<< >>$pattern<<"
    # read 18 + 19 more slit blade positions 
    # alternate flip/flop storing 
    if {$flag} {
       # puts $lfId "$flag ->"
       set flag 0
       # puts $lfId "$flag"
       set mosslitmin($count) $pattern
       # puts $lfId "$count $mosslitmin($count)"
       } else {
       # puts $lfId "$flag ->"
       set flag 1
       # puts $lfId "$flag"
       set mosslitmax($count) $pattern
       incr count
       }
   # read the next line 
   gets $fileId line_org
   }
# no further tests
}

# ==============================================
private method part_load_pol {fileId line_org} {
# ==============================================
# puts stdout "POL: $line_org"
for {set i 1} { $i < 37 } { incr i 1} {
    set line [split $line_org \" ]
    set pattern [lrange $line 1 1];            # e.g. -113.9
    set keyword [lrange $line 0 0];            # e.g. POLARIMSLITXMIN1.VALUE
    set keyword [split $keyword \. ];          # e.g. POLARIMSLITXMIN1 VALUE
    set keyword [lrange $keyword 0 0];         # e.g. POLARIMSLITXMIN1
    set keyword [string range $keyword 2 [string length $keyword]]
    # puts stdout "$keyword.VALUE  \" $pattern \" "
    set name [string tolower $keyword]
    set number [string range $name end end]
    set name [string range $name 0 [expr [string length $name]-2]]
    # puts stdout ">>$name<< >>$number<< $pattern"
    set cmd "set $name\($number\) $pattern"
    # puts stdout $cmd
    eval $cmd
    # read the next line
    gets $fileId line_org
    }
# no further verification
}


# ==============================================
private method part_load_lss {fileId line_org} {
# ==============================================
#  puts stdout "LSS: $line_org"
for {set i 1} { $i < 37 } { incr i 1} {
    set line [split $line_org \" ]
    set pattern [lrange $line 1 1];            # e.g. -113.9
    set keyword [lrange $line 0 0];            # e.g. LONGSLITXMIN1.VALUE
    set keyword [split $keyword \. ];          # e.g. LONGSLITXMIN1 VALUE
    set keyword [lrange $keyword 0 0];         # e.g. LONGSLITXMIN1
    set keyword [string range $keyword 2 [string length $keyword]]
    # puts stdout "$keyword.VALUE  \" $pattern \" "
    set name [string tolower $keyword]
    set number [string range $name end end]
    set name [string range $name 0 [expr [string length $name]-2]]
    # puts stdout ">>$name<< >>$number<< $pattern"  
    set cmd "set $name\($number\) $pattern"
    # puts stdout $cmd
    eval $cmd 
    # read the next line 
    gets $fileId line_org
    }
    # only for the 'all' option 
    set longslitxmin(0) $longslitxmin(9)
    set longslitxmax(0) $longslitxmax(9)
    set longslitymin(0) $longslitymin(1)
    set longslitymax(0) $longslitymax(1)
# no further verification
}

# ==============================================
private method part_load_hits {fileId line_org} {
# ==============================================
# puts stdout "HITS: $line_org"
for {set i 1} { $i < 33 } { incr i 1} {
    set line [split $line_org \" ]
    set pattern [lrange $line 1 1];            # e.g. -113.9
    set keyword [lrange $line 0 0];            # e.g. HTSLITXMIN1.VALUE
    set keyword [split $keyword \. ];          # e.g. HTSLITXMIN1 VALUE
    set keyword [lrange $keyword 0 0];         # e.g. HTSLITXMIN1
    set keyword [string range $keyword 2 [string length $keyword]]
    # puts stdout "$keyword.VALUE  \" $pattern \" "
    set name [string tolower $keyword]
    set number [string range $name end end]
    set name [string range $name 0 [expr [string length $name]-2]]
    # puts stdout ">>$name<< >>$number<< $pattern"  
    set cmd "set $name\($number\) $pattern"
    # puts stdout $cmd
    eval $cmd 
    # read the next line 
    gets $fileId line_org
    }
    # only for the 'all' option 
    set htslitxmin(0) $htslitxmin(8)
    set htslitxmax(0) $htslitxmax(8)
    set htslitymin(0) $htslitymin(1)
    set htslitymax(0) $htslitymax(1)
# no further verification
}

# ==============================================
private method load_mxu_refslit_info {} {
    # ==============================================
    set fors_isf_file "$isf_dir/[$this cget -instrument].isf"    
    if {"[$this cget -instrument]" == "FORS2"} {
	if {[file exists $fors_isf_file] & [file readable $fors_isf_file]} {
	    set fileId [open $fors_isf_file r ]    
	    set patternTP MXUREFTP*.RANGE*
	    set patternSX MXUREFSX*.RANGE*
	    set patternSY MXUREFSY*.RANGE*
	    set patternWD MXUREFWD*.RANGE*
	    set patternLE MXUREFWD*.RANGE*
	    set patternRO MXUREFRO*.RANGE*
	    while {[gets $fileId line_org] >= 0} {
		if {[string match #* $line_org]} {
		    #puts stdout $line_org
		} else {
		    if {[string match $patternTP $line_org]==1} {
			#puts stdout "line_org =>$line_org<"
			
			set line [split $line_org \" ]
			set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
			set pat [lindex $pat 0]
			#puts "pat: $pat LEN: [llength $pat]"
			set nf_refslit [llength $pat]
			for {set i 0} {$i < [llength $pat]} {incr i} {
			    set mxureftp($i) [lindex $pat $i]
			}
		    }
		    if {[string match $patternSX $line_org]==1} {
			#puts stdout "line_org =>$line_org<"
			
			set line [split $line_org \" ]
			set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
			set pat [lindex $pat 0]
			for {set i 0} {$i < [llength $pat]} {incr i} {
			    set mxurefsx($i) [lindex $pat $i]
			}
		    }
		     if {[string match $patternSY $line_org]==1} {
			#puts stdout "line_org =>$line_org<"
			
			set line [split $line_org \" ]
			set pat [lrange $line 1 1]
			set pat [lindex $pat 0]
			#puts "pat: $pat LEN: [llength $pat]"
			for {set i 0} {$i < [llength $pat]} {incr i} {
			    set mxurefsy($i) [lindex $pat $i]
			}
		    }
		    if {[string match $patternWD $line_org]==1} {
			#puts stdout "line_org =>$line_org<"
			
			set line [split $line_org \" ]
			set pat [lrange $line 1 1]
			set pat [lindex $pat 0]
			#puts "pat: $pat LEN: [llength $pat]"
			for {set i 0} {$i < [llength $pat]} {incr i} {
			    set mxurefwd($i) [lindex $pat $i]
			}
		    }
		    if {[string match $patternLE $line_org]==1} {
			#puts stdout "line_org =>$line_org<"
			
			set line [split $line_org \" ]
			set pat [lrange $line 1 1]
			set pat [lindex $pat 0]
			#puts "pat: $pat LEN: [llength $pat]"
			for {set i 0} {$i < [llength $pat]} {incr i} {
			    set mxurefle($i) [lindex $pat $i]
			}
		    }
		    if {[string match $patternRO $line_org]==1} {
			#puts stdout "line_org =>$line_org<"
			
			set line [split $line_org \" ]
			set pat [lrange $line 1 1]
			set pat [lindex $pat 0]
			#puts "pat: $pat LEN: [llength $pat]"

			for {set i 0} {$i < [llength $pat]} {incr i} {
			    set mxurefro($i) [lindex $pat $i]
			}
		    } 
		}
	    }      	     
	}         
    }
}

# ==============================================
private method part_load_pinh {fileId line_org} {
# ==============================================
   set debug 0
   if {$debug} { puts stdout "PINH: $line_org" }
for {set i 1} { $i < 5 } { incr i 1} {
    set line [split $line_org \" ]
    set pattern [lrange $line 1 1];            # e.g. -113.9
    set keyword [lrange $line 0 0];            # e.g. LONGSLITXMIN1.VALUE
    set keyword [split $keyword \. ];          # e.g. LONGSLITXMIN1 VALUE
    set keyword [lrange $keyword 0 0];         # e.g. LONGSLITXMIN1
    set keyword [string range $keyword 2 [string length $keyword]]
    if {$debug} { puts stdout "$keyword.VALUE  \" $pattern \" " }
    set name [string tolower $keyword]
    set number [string range $name end end]
    set name [string range $name 0 [expr [string length $name]-2]]
    if {$debug} { puts stdout ">>$name<< >>$number<< $pattern" }
    set cmd "set $name\($number\) $pattern"
    if {$debug} { puts stdout $cmd }
    eval $cmd
    # read the next line
    gets $fileId line_org
    }
    # only for the long_number=1,2 
    # pinholes hardcoded
    # big pinhole
    set htslitxmin(9) [expr $pinhxcent(1)-0.95/$focscale]
    set htslitxmax(9) [expr $pinhxcent(1)+0.95/$focscale]
    set htslitymin(9) [expr $pinhycent(1)-0.95/$focscale]
    set htslitymax(9) [expr $pinhycent(1)+0.95/$focscale]
    # small pinhole
    set htslitxmin(0) [expr $pinhxcent(2)-0.02/$focscale]
    set htslitxmax(0) [expr $pinhxcent(2)+0.02/$focscale]
    set htslitymin(0) [expr $pinhycent(2)-0.02/$focscale]
    set htslitymax(0) [expr $pinhycent(2)+0.02/$focscale]
# no further verification
}


    # ==================================
    private method wave_limit {} {
    # ==================================
    # set the wavelength limits of the spectral range 
    # 'good' is the index of the dispersion table A_disp
    # is was calculated by public variable filter and grism
    set ins_lmin $A_disp($good,min)
    set ins_lmax $A_disp($good,max)
    set ins_lamc $A_disp($good,cen)
    set ins_disp $A_disp($good,disp)

    # 'ins_lmax' and 'ins_lmin' are the wavelength limits 
    # from the grism-filter combination, it is called the 
    # instrumental wavelength range.  
    # it is much larger than the CCD area 
    # L_MIN and L_MAX determine the user-defined wavelength region
    # The maximum wavelength range which can be 
    # matched by the CCD is c_min c_max 

     set id00 [ format \
       ">> %5.1f < %5.1f < %5.1f < %5.1f < %5.1f\n" \
        $ins_lmin $L_MIN $ins_lamc $L_MAX $ins_lmax ]
     # sorting order 
     if {$ins_lmax<$ins_lmin} { set dum1 $ins_lmax;
                                set ins_lmax $ins_lmin;
                                set ins_lmin $dum1 }

     # if L_MAX < L_MIN exchange  
     if {$L_MAX < $L_MIN} { set dum1 $L_MAX;
                       set L_MAX $L_MIN;
                       set L_MIN $L_MAX }

     if {$L_MIN<$ins_lmin*0.5} { set L_MIN $ins_lmin}
     if {$L_MAX>$ins_lmax*1.5} { set L_MAX $ins_lmax}

     # since the grism occurs in the optical path after the coll,
     # the dispersion on the CCD is the same in both coll. modes, but
     # 1) from SR->HR use the half slit width in order to retain
     #    the spectr. resolution
     # 2) the dispersion, projected into the MOS focal plane 
     #    (thats the blue lines we draw)
     #    is different in both coll modes

     # from nm (wavelength) to mm_focalplane (USerDefined)
     set speclen [expr abs($L_MAX-$L_MIN)/$ins_disp/$focscale]
     set speclen [expr $speclen/$PIX*$CCDscale_standard]

     # from nm (wavelength) to mm_focalplane (INStrumental) 
     set ins_len [expr abs($ins_lmax-$ins_lmin)/$ins_disp/$focscale]
     set ins_len [expr $ins_len/$PIX*$CCDscale_standard]

     # 
     # used as distances > 0
     set specmin [expr ($ins_lamc-$L_MIN)/$ins_disp/$focscale]
     set specmin [expr $specmin/$PIX*$CCDscale_standard]

     set specmax [expr ($L_MAX-$ins_lamc)/$ins_disp/$focscale]
     set specmax [expr $specmax/$PIX*$CCDscale_standard]

     set ccdm_specmin [expr ($ins_lamc-$L_MIN)/$ins_disp/$focscale]
     set ccdm_specmin [expr $ccdm_specmin/$PIX*$CCDscale_standard]

     set ccdm_specmax [expr ($L_MAX-$ins_lamc)/$ins_disp/$focscale]
     set ccdm_specmax [expr $ccdm_specmax/$PIX*$CCDscale_standard]

     # spec_len now in mm
     set id0 [ format \
       ">> %5.1f < %5.1f < %5.1f < %5.1f < %5.1f %s %s\n" \
        $ins_lmin $L_MIN $ins_lamc $L_MAX $ins_lmax [$this cget -grism] \
        [$this cget -filter] ]

     set id0a [ format  \
      ">>DISP %5.1f A/mmCD %5.1f A/pix %5.1f %5.1f A/'' %5.1f %5.1f A/mmfc\n" \
       [expr $ins_disp*10.0] \
        [expr $ins_disp*10.0*$PIX] \
        [expr $ins_disp*10.0*$PIX/$CCDscale_standard] \
        [expr $ins_disp*10.0*$PIX/$CCDscale_highres] \
        [expr $ins_disp*10.0*$PIX/$CCDscale_standard/$focscale] \
        [expr $ins_disp*10.0*$PIX/$CCDscale_highres/$focscale]  ]

     set id1 [ format \
       ">>BLU %6.1f A = %6.1f pixel = %6.1f '' = %6.3f ' =  %8.3f mm(fp)\n" \
       [expr ($ins_lamc-$L_MIN)*10.0] \
       [expr ($ins_lamc-$L_MIN)/$ins_disp/$PIX] \
       [expr ($ins_lamc-$L_MIN)/$ins_disp/$PIX*$CCDscale_standard]  \
       [expr ($ins_lamc-$L_MIN)/$ins_disp/$PIX*$CCDscale_standard/60.0] \
       $specmin ]

     set id2 [format \
       ">>RED %6.1f A = %6.1f pixel = %6.1f '' = %6.3f ' =  %8.3f mm(fp)\n" \
        [expr ($L_MAX-$ins_lamc)*10.0] \
        [expr ($L_MAX-$ins_lamc)/$ins_disp/$PIX] \
        [expr ($L_MAX-$ins_lamc)/$ins_disp/$PIX*$CCDscale_standard] \
        [expr ($L_MAX-$ins_lamc)/$ins_disp/$PIX*$CCDscale_standard/60.0] \
        $specmax ]

     set id3 [ format \
       ">>USD %6.1f A = %6.1f pixel = %6.1f '' = %6.3f ' =  %8.3f mm(fp)\n" \
       [expr ($L_MAX-$L_MIN)*10.0] \
       [expr ($L_MAX-$L_MIN)/$ins_disp/$PIX]  \
       [expr ($L_MAX-$L_MIN)/$ins_disp/$PIX*$CCDscale_standard] \
       [expr ($L_MAX-$L_MIN)/$ins_disp/$PIX*$CCDscale_standard/60.0] \
       [expr ($specmax + $specmin)] ]

     set id4 [ format \
       ">>SPE %6.1f A = %6.1f pixel = %6.1f '' = %6.3f ' =  %8.3f mm(fp)\n" \
       [expr ($ins_lmax-$ins_lmin)*10.0] \
       [expr ($ins_lmax-$ins_lmin)/$ins_disp/$PIX] \
       [expr ($ins_lmax-$ins_lmin)/$ins_disp/$PIX*$CCDscale_standard] \
       [expr ($ins_lmax-$ins_lmin)/$ins_disp/$PIX*$CCDscale_standard/60.0] \
       $ins_len ]


    puts "INS_DISP: $ins_disp"
    puts "FOCSCALE: $focscale"
    puts "PIX $PIX"
    puts "CCDscale_standard $CCDscale_standard"
    puts "CCDscale_highres $CCDscale_highres"

    set id5 [ format ">>Field Stop total range = %6.1f A (SR) = %6.1f A (HR)\n" \
    [expr ($XSRMAX-$XSRMIN)*$focscale*$PIX/$CCDscale_standard*$ins_disp*10.0] \
    [expr ($XHRMAX-$XHRMIN)*$focscale*$PIX/$CCDscale_highres*$ins_disp*10.0]  ]


    #- FORS2 have a new CCD -#
     if {"[$this cget -instrument]" == "FORS2"} {	 
        puts stdout "FORS2 CCD Frame"
        set id6 [ format ">>CCD total range = %6.1f A (SR) = %6.1f A (HR)\n" \
            [expr ($CCDMASTER_XSRMAX-$CCDMASTER_XSRMIN)*$focscale*$PIX/$CCDscale_standard*$ins_disp*10.0] \
            [expr ($CCDMASTER_XHRMAX-$CCDMASTER_XHRMIN)*$focscale*$PIX/$CCDscale_highres*$ins_disp*10.0]  ]
     } else {
	puts stdout "FORS2 CCD Frame"
	set id6 [ format ">>CCD total range = %6.1f A (SR) = %6.1f A (HR)\n" \
            [expr ($XSRMAX-$XSRMIN)*$focscale*$PIX/$CCDscale_standard*$ins_disp*10.0] \
            [expr ($XHRMAX-$XHRMIN)*$focscale*$PIX/$CCDscale_highres*$ins_disp*10.0]  ] 
     }
     
     set mess " $id00 $id0 $id0a $id1 $id2 $id3 $id4 $id5 $id6"
     puts stdout $mess

}
    # reading the CCD dimensions for Master and Slave
    # ==================================
    private method load_ccd_window { } {
    # ==================================
                   
    set fors_isf_file "$isf_dir/[$this cget -instrument].isf"    
    if {"[$this cget -instrument]" == "FORS2"} {
    if {[file exists $fors_isf_file] & [file readable $fors_isf_file]} {
       set fileId [open $fors_isf_file r ]       
       set patternMasterXSR  CCD*.MASTER_XSR*.RANGE*
       set patternSlaveXSR   CCD*.SLAVE_XSR*.RANGE*
       set patternMasterYSR  CCD*.MASTER_YSR*.RANGE*
       set patternSlaveYSR   CCD*.SLAVE_YSR*.RANGE*
       set patternGapSR      CCD*.GAP_SR*
       set patternMasterXHR  CCD*.MASTER_XHR*.RANGE*
       set patternSlaveXHR   CCD*.SLAVE_XHR*.RANGE*
       set patternMasterYHR  CCD*.MASTER_YHR*.RANGE*
       set patternSlaveYHR   CCD*.SLAVE_YHR*.RANGE*
       set patternGapHR      CCD*.GAP_HR* 
       
       while {[gets $fileId line_org] >= 0} {
        if {[string match #* $line_org]} {
          #puts stdout $line_org
          } else {

         if {[string match $patternMasterXSR $line_org]==1} {
             puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            #puts stdout "Master size line=>$pat<"
	    set pat [lindex $pat 0]
            set CCDMASTER_XSRMIN [lindex $pat 0]
            set CCDMASTER_XSRMAX [lindex $pat 1]
	    
	    puts $lfId "Master SR XMIN: $CCDMASTER_XSRMIN"
	    puts $lfId "Master SR XMAX: $CCDMASTER_XSRMAX"	    	     
	    }
	 
         if {[string match $patternSlaveXSR $line_org]==1} {
             puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            #puts stdout "Slave size line=>$pat<"
	    set pat [lindex $pat 0]
            set CCDSLAVE_XSRMIN [lindex $pat 0]
            set CCDSLAVE_XSRMAX [lindex $pat 1]
	    
	    puts $lfId "Slave SR XMIN: $CCDSLAVE_XSRMIN"
	    puts $lfId "Slave SR XMAX: $CCDSLAVE_XSRMAX"	
	    }

	 if {[string match $patternMasterYSR $line_org]==1} {
             puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            #puts stdout "Master size line=>$pat<"
	    set pat [lindex $pat 0]
            set CCDMASTER_YSRMIN [lindex $pat 0]
            set CCDMASTER_YSRMAX [lindex $pat 1]
	    
	    puts $lfId "Master SR YMIN: $CCDMASTER_YSRMIN"
	    puts $lfId "Master SR YMAX: $CCDMASTER_YSRMAX"	    	     
	    }
	 
         if {[string match $patternSlaveYSR $line_org]==1} {
             puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            #puts stdout "Slave size line=>$pat<"
	    set pat [lindex $pat 0]
            set CCDSLAVE_YSRMIN [lindex $pat 0]
            set CCDSLAVE_YSRMAX [lindex $pat 1]
	    
	    puts $lfId "Slave SR YMIN: $CCDSLAVE_YSRMIN"
	    puts $lfId "Slave SR YMAX: $CCDSLAVE_YSRMAX"	
	    }

	 if {[string match $patternGapSR $line_org]==1} {
             puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            # puts stdout "Gap size line =>$pat<"
            set CCDGAP_SR [lindex $pat 0]
	    puts $lfId "CCD SR Gap: $CCDGAP_SR"
	    }

	  if {[string match $patternMasterXHR $line_org]==1} {
             puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            #puts stdout "Master size line=>$pat<"
	    set pat [lindex $pat 0]
            set CCDMASTER_XHRMIN [lindex $pat 0]
            set CCDMASTER_XHRMAX [lindex $pat 1]
	    
	    puts $lfId "Master HR XMIN: $CCDMASTER_XHRMIN"
	    puts $lfId "Master HR XMAX: $CCDMASTER_XHRMAX"	    	     
	    }
	 
         if {[string match $patternSlaveXHR $line_org]==1} {
             puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            #puts stdout "Slave size line=>$pat<"
	    set pat [lindex $pat 0]
            set CCDSLAVE_XHRMIN [lindex $pat 0]
            set CCDSLAVE_XHRMAX [lindex $pat 1]
	    
	    puts $lfId "Slave HR XMIN: $CCDSLAVE_XHRMIN"
	    puts $lfId "Slave HR XMAX: $CCDSLAVE_XHRMAX"	
	    }

	 if {[string match $patternMasterYHR $line_org]==1} {
             puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            #puts stdout "Master size line=>$pat<"
	    set pat [lindex $pat 0]
            set CCDMASTER_YHRMIN [lindex $pat 0]
            set CCDMASTER_YHRMAX [lindex $pat 1]
	    
	    puts $lfId "Master HR YMIN: $CCDMASTER_YHRMIN"
	    puts $lfId "Master HR YMAX: $CCDMASTER_YHRMAX"	    	     
	    }
	 
         if {[string match $patternSlaveYHR $line_org]==1} {
             puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            #puts stdout "Slave size line=>$pat<"
	    set pat [lindex $pat 0]
            set CCDSLAVE_YHRMIN [lindex $pat 0]
            set CCDSLAVE_YHRMAX [lindex $pat 1]
	    
	    puts $lfId "Slave HR YMIN: $CCDSLAVE_YHRMIN"
	    puts $lfId "Slave HR YMAX: $CCDSLAVE_YHRMAX"	
	    }

	  
	 if {[string match $patternGapHR $line_org]==1} {
             puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            # puts stdout "Gap size line =>$pat<"
            set CCDGAP_HR [lindex $pat 0]
	    puts $lfId "CCD HR Gap: $CCDGAP_HR"
	    }
	  }	  
        }
       close $fileId        
      }        
  } else {
     set CCDMASTER_XSRMIN $XSRMIN
     set CCDMASTER_XSRMAX $XSRMAX
     set CCDMASTER_YSRMIN $YSRMIN
     set CCDMASTER_YSRMAX $YSRMAX
     
     set CCDMASTER_XHRMIN $XHRMIN
     set CCDMASTER_XHRMAX $XHRMAX
     set CCDMASTER_YHRMIN $YHRMIN
     set CCDMASTER_YHRMAX $YHRMAX

     set CCDSLAVE_XSRMIN  $XSRMIN
     set CCDSLAVE_XSRMAX  $XHRMAX
     set CCDSLAVE_XHRMIN  $XHRMIN
     set CCDSLAVE_XHRMAX  $XHRMAX

     set CCDSLAVE_YSRMIN  $YSRMIN
     set CCDSLAVE_YSRMAX  $YHRMAX
     set CCDSLAVE_YHRMIN  $YHRMIN
     set CCDSLAVE_YHRMAX  $YHRMAX
    
  }
 }

    # reading the CCD dimensions for Master and Slave
    # ==================================
    private method load_fieldstop { } {
    # ==================================
                   
    set fors_isf_file "$isf_dir/[$this cget -instrument].isf"    
    if {"[$this cget -instrument]" == "FORS2"} {
    if {[file exists $fors_isf_file] & [file readable $fors_isf_file]} {
       set fileId [open $fors_isf_file r ]       
       set patternX  FIELDXRANGE*.VALUE*
       set patternY  FIELDYRANGE*.VALUE*
       
       while {[gets $fileId line_org] >= 0} {
        if {[string match #* $line_org]} {
          #puts stdout $line_org
          } else {

         if {[string match $patternX $line_org]==1} {
             puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            #puts stdout "Master size line=>$pat<"
	    set pat [lindex $pat 0]
            set FIELDSTOP_XMIN [lindex $pat 0]
            set FIELDSTOP_XMAX [lindex $pat 1]
	    
	    puts $lfId "FIELD STOP XMIN: $FIELDSTOP_XMIN"
	    puts $lfId "FIELD STOP XMAX: $FIELDSTOP_XMAX"	    	     
	    }

          if {[string match $patternY $line_org]==1} {
             puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            #puts stdout "Master size line=>$pat<"
	    set pat [lindex $pat 0]
            set FIELDSTOP_YMIN [lindex $pat 0]
            set FIELDSTOP_YMAX [lindex $pat 1]
	    
	    puts $lfId "FIELD STOP YMIN: $FIELDSTOP_YMIN"
	    puts $lfId "FIELD STOP YMAX: $FIELDSTOP_YMAX"	    	     
	    }
	 
	}
        }
       close $fileId        
      }        
  } else {
     set FIELDSTOP_XMIN $XSRMIN
     set FIELDSTOP_XMAX $XSRMAX
     set FIELDSTOP_YMIN $YSRMIN
     set FIELDSTOP_YMAX $YSRMAX             
  }
 }
    # ==================================
    private method load_gris_filt_disp { } {
    # ==================================
    # reading the grism-filter dispersion array
    # from the instrument summnary file  
    # this table is only for GRIS and FILT, not for 
    # XGRIS and EGRIS (echs)
    # open FORS1/2.isf once more again
    set debug 0

    puts "executing load_gris_filt_disp ..."
    set disp_string ""
    set osfi_range ""
    set echelle_range ""
    # ++++++++++++++++++++++++++++++++++++++++
    set grisfile "$isf_dir/[$this cget -instrument].isf"
    # ++++++++++++++++++++++++++++++++++++++++
    if {[file exists $grisfile] & [file readable $grisfile]} {
       set fileId [open $grisfile r ]
       puts $lfId "FORS::load grism :  $fileId"
       # set search pattern, avoid "#GRIS1.FILT7..." 
       set pattern1  GRIS*.FILT*
       set pattern2  EX.COMB*
       # puts stdout $pattern
       # as long there are lines in the file
       set prio_flag 0
       # as long as there are lines in the file 
       set counter 0
       while {[gets $fileId line_org] >= 0} {
        if {[string match #* $line_org]} {
          #puts stdout $line_org
          } else {

         if {[string match $pattern1 $line_org]==1} {
            # puts stdout "line_org =>$line_org<"
            # extract ""
            set line [split $line_org \" ]
            # puts stdout "line =>$line<"
            set pat [lrange $line 1 1]; # e.g. none 5 463 300 700
            # puts stdout "pat =>$pat<"
            set pat [lindex $pat 0]
            # puts stdout "pat =>$pat<"
            lassign $pat l_filt l_disp l_cen l_min l_max
            incr counter 1
            set A_disp($counter,filter) $l_filt
            set A_disp($counter,disp) $l_disp
            set A_disp($counter,cen)  $l_cen
            set A_disp($counter,min) $l_min 
            set A_disp($counter,max) $l_max

            set pat [lindex $line 0]
            set b [split $pat \. ]
            set b [lindex $b 0]
            set b [string trimleft $b "GRIS"]
            set pat [split $gris_range " " ]
            # puts stdout ">$pat<"
            set pat [lindex $pat [expr $b-1] ]
            set A_disp($counter,grism) $pat
            set A_disp(0) $counter
            append disp_string [format "%-2d   "  $counter ]
            append disp_string [format "%-16s   "  $A_disp($counter,grism)  ]
            append disp_string [format "%-16s   "  $A_disp($counter,filter) ] 
            append disp_string [format "%6.2f   " $A_disp($counter,disp) ]
            append disp_string [format "%4f   "   $A_disp($counter,cen) ]
            append disp_string [format "%4f   "   $A_disp($counter,min) ]
            append disp_string [format "%4f   \n" $A_disp($counter,max) ]
            # append order separation filter to filter range in case it is not yet
            # in it, what is this ? 
            if {[string match *$A_disp($counter,filter)* $osfi_range]==0} {
               append osfi_range " $A_disp($counter,filter)"
               }
            } 

          if {[string match $pattern2 $line_org]==1} {
             # puts stdout $line_org
             set dummy [split $line_org " "]
             set dummy [ldelete_all $dummy ""]
             set dummy [lindex $dummy 1]
             set dummy [string trimright $dummy ";" ]
             set dummy [string trim $dummy \" ]
             append echelle_range "$dummy " 
             }

          }
         }
         close $fileId
         # add the none-none combination, used as default grism 
         incr counter 1
         set A_disp($counter,grism) none
         set A_disp($counter,filter) none
         set A_disp($counter,disp) 5.00
         set A_disp($counter,cen)  500
         set A_disp($counter,min) 500  
         set A_disp($counter,max) 500
         set A_disp(0) $counter
         append disp_string [format "%-2d   "  $counter ]
         append disp_string [format "%-16s   "  $A_disp($counter,grism)  ]
         append disp_string [format "%-16s   "  $A_disp($counter,filter) ] 
         append disp_string [format "%6.2f   " $A_disp($counter,disp) ]
         append disp_string [format "%4d   "   $A_disp($counter,cen) ]
         append disp_string [format "%4d   "   $A_disp($counter,min) ]
         append disp_string [format "%4d   \n" $A_disp($counter,max) ]

         if {$debug} {
         puts >$disp_string<"

         for {set i 1 } {$i <= $counter} { incr i 1} {
             puts stdout ">$A_disp($i,grism)< \
                          >$A_disp($i,filter)< \
                          >$A_disp($i,disp)< \
                          >$A_disp($i,cen)< \
                          >$A_disp($i,min)< \
                          >$A_disp($i,max)<"
              }
        }

         set osfi_range [string trimleft $osfi_range] 
         puts stdout "$counter gris-filt combinations read from \
                 [$this cget -instrument].isf\n"
         puts $lfId  "$counter gris-filt combinations read from \
                 [$this cget -instrument].isf\n"
         } else { 
         puts $lfId  "could not read $grisfile = \
                 [$this cget -instrument].isf\n"
         puts stdout "could not read $grisfile = \
                [$this cget -instrument].isf\n"
         }

      # for {set i 1} {$i <= $A_disp(0) } {incr i 1} {
      #     set a [ format " %8s %8s %6.1f %6.1f %6.1f %6.1f" \
      #         $A_disp($i,grism) $A_disp($i,filter) \
      #         $A_disp($i,disp) $A_disp($i,cen) \
      #         $A_disp($i,min) $A_disp($i,max) ]
      #     puts stdout $a
      #     } 
     }

    # ==================================
    private method load_mos_TPX { filename } {
    # ==================================
    # open setupfile (e.g.  'mos4.<ec>.<pn>.fims') and 
    # read the setup file, configure values and plot the 
    # mask at the proper position. 
    # slit positions are given in wcs 
    # filename = ""     : use default filename=auto_pos (used by AutoPos) 
    # filename = 0      : from filename_dialog widget 
    # filename = string : take this 

    # initialize the scales for coord transformation 
    #
    # puts stdout "load_mos_TPX: >$wx< >$wy<"
    C_wc coeff_wc $wx $wy
    # test if this is necessary 
    if {[string compare [$this cget -modus] echs]==0} {
    fors::CONV_MC::coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                      $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                      $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc) 
       }

   # --- open filename ---------
   #
  set myfile $set_dir
  if {$filename==""} {
     # no filename means load result from AutoPos, written to
     # .fims/LOG/(p)mos(1/2).<ec>.autopos.fims
     # we load auto_pos from LOG dir, to be independent from user-defined
     # set_dir
     set myfile $log_dir
     append myfile \
     "/[$this cget -modus][string index [$this cget -instrument] 4].$exposure_counter.auto_pos.fims"
     } else { 
     if {$filename} { append myfile "/$filename"
      } else {
      global .fs
      catch { destroy .fs  }
      set myfile [filename_dialog $set_dir \
        "[$this cget -modus][string index [$this cget -instrument] 4].*.fims" ]
      }
     }

     set keyword TEL
     set keyw2   INS
     set f_gris 0; set f_filt 0; set f_slit_width 0; set f_coll 0
     set f_lmin 0;  
     set lty ""; set epo_flag 0; set equ_flag 0

     # ------  read TCS keywords --------
     # step 1 : load opt1.targ, read only TEL pointing 
     #          shift and rotate according to these values
     if {[file exists $myfile] & [file readable $myfile]} {
        set fileId [open $myfile r ]
        puts $lfId "FORS::load_TARG  $fileId"
        # puts stdout "FORS::load_TARG  $fileId"
        # as long as there are lines in the file
        while {[gets $fileId line_org] >= 0} {

        if {[string match #* $line_org]} {
          # prints the comments to the term
          puts stdout $line_org
          } else {
          if {[string match *$keyword* $line_org]==1} {
             # puts stdout $line_org
             # TEL 
             switch -glob -- $line_org {
                   *ALPHA* { set lwx [lindex $line_org 1] }
                   *DELTA* { set lwy [lindex $line_org 1] }
                 *ROT.OFF* { set lPA [lindex $line_org 1] }
                 *EPOCH*   { set EQX [lindex $line_org 1]; set epo_flag 1 }
                 *EQUINOX* { set EQX [lindex $line_org 1]; set equ_flag 1 }
                 *TYPE*    { set lty [lindex $line_org 1] }
                           }
              }


          # --- by the way read SEQ.PREP keywords ---------
          if {[string match *SEQ.PREP* $line_org]==1} {

          if {[string match *COLL* $line_org]} {
                 # the collimator  
                 puts stdout "SEQ.PREP.COLL = [lindex $line_org 1]"
                 set l_coll [lindex $line_org 1] 
                 set f_coll 1
              }
          if {[string match *GRIS* $line_org]} {
                 # the grism   
                 puts stdout "SEQ.PREP.GRISM = [lindex $line_org 1]"
                 set l_gris [lindex $line_org 1] 
                 set f_gris 1
              }
          if {[string match *LMIN* $line_org]} {
                 # the user defined wavelength region  
                 puts stdout "SEQ.PREP.LMIN = [lindex $line_org 1]"
                 set L_MIN  [lindex $line_org 1] 
                 set f_lmin 1
              }
          if {[string match *LMAX* $line_org]} {
                 # the user defined wavelength region  
                 puts stdout "SEQ.PREP.LMAX = [lindex $line_org 1]"
                 set L_MAX [lindex $line_org 1] 
                 set f_lmin 1
              }
          if {[string match *FILT* $line_org]} {
                 # the os filter   
                 puts stdout "SEQ.PREP.FILT = [lindex $line_org 1]"
                 set l_filt [lindex $line_org 1] 
                 set f_filt 1
              }
          if {[string match *MTALI* $line_org]} {
                 # the 'alignment with mos slit 10' option   
                 puts stdout "SEQ.PREP.MTALI = [lindex $line_org 1]"
                 C_ref_targ configure -mos10ali [lindex $line_org 1] 

                 # for statuspanel
                 set sp_mtali [lindex $line_org 1]
                 catch {SP update} 
              }
          if {[string match *ECHS9* $line_org]} {
                 # the echs9 flag    
                 puts stdout "SEQ.PREP.ECHS9 = [lindex $line_org 1]"
                 set l_echs9 [lindex $line_org 1] 
              }
            }

          }
          }
     close $fileId
       } else {
       puts stdout "Problems with $myfile"
       }
      
      # some tests for backfward compatibility
      if {$lty == ""} { puts stdout "->->-> Warning TEL.TARG.TYPE not defined" }
      if {!$equ_flag} { puts stdout "->->-> Warning TEL.TARG.EQUINOX not found" }
      if {$epo_flag}  { puts stdout "->->-> Warning TEL.TARG.EQUINOX defined via TEL.TARG.EPOCH = $EQX" }


     # The file is closed now 
     # set local position angle (mask rotation)
     set lPA [expr $lPA * -1]
     set lPA [expr fmod($lPA,360) ]

     set mask_initialized 0; # assume mask is not yet reset  
     # this is for only one reset_mask when more than one 
     # instrument config has been changed 

     # configure instrument due to SEQ.PREP keywords if nessecary 
     #
     if {$f_lmin} { wave_limit }

     # individual slit width from .fims used
     # default MOS slit width not overwritten
     # 19/2/99 WH


     if {[string match echs [$this cget -modus] ] } {
        # echs_flag = current status, l_echs9 = just loaded
        #  puts stdout "fims echs9=$echs_flag, load_echs9=$l_echs9"
        if {$echs_flag != $l_echs9} {
           flip_echs
        #  puts stdout "fims echs9=$echs_flag, load_echs9=$l_echs9"
           }
        }

     if {$f_gris} { 
        if {[string compare [$this cget -grism] $l_gris]} {
           $m_mb.layout.gr invoke *$l_gris* 
           }
         }

     if {$f_filt} { 
        set app2 ""
        # change only if new filt != old filt  
        # puts stdout ">[$this cget -filter]< >$l_filt<"
        if {[string compare [$this cget -filter] $l_filt]} {
           # entryconfigure which submenue ? 
           # note we have layout.fi, fi.m2, fi.m3, fi,m4 fi.m5
           # first search the submenue (ekelhaft)
           # then configure 
           foreach append {"" .m2 .m3 .m4 .m5} {
              for {set i 0} {$i <= 9} {incr i 1} {
                  set local [$m_mb.layout.fi$append entrycget $i -label]
                  if {[string compare $local $l_filt]==0} {
                     set app2 $append
                     }   
                  }
              }   
           $m_mb.layout.fi$app2 invoke *$l_filt*
           }
        }

     if {$f_coll} { 
        # change only if new coll != old coll  
        if {[string compare [$this cget -collimat] $l_coll]} {
           $m_mb.layout.sh invoke *$l_coll* 
           }
        }


     # if mask is not yet initilized by the 'invoke' commands
     if {$mask_initialized==0} { reset_mask }

     # ---------  shift and rotate the mask -----------
     $m_image convert coords $mos(0,x0) $mos(0,y0) canvas mx my screen 
     # puts stdout "canvas $mos(0,x0) $mos(0,y0) screen $mx $my"
     mark $m_canvas $mx $my
     # puts stdout "Mark_Mask at screen $mx $my" 
     # $m_image convert coords [expr 15.0 *[DDMMSS2deg $lwx] ]\
     #       [DDMMSS2deg $lwy] "deg $EQX"  nwx nwy screen 
     set a1 [expr 15.0 *[DDMMSS2deg $lwx] ]
     set d1 [DDMMSS2deg $lwy]
     # puts stdout [format "deg $EQX: %10.3f %10.3f" $a1 $d1] 
     # wcs to canvas via bonsai matrix 
        set dum [C_wc w2c $a1 $d1]
        set cx [lindex $dum 0]
        set cy [lindex $dum 1]
     # puts stdout "canvas $cx $cy"
     $m_image convert coords $cx $cy canvas nwx nwy screen 
     # puts stdout "load_mos_TPX : Shift Mask"
     shift $m_canvas  $nwx $nwy 
     # puts stdout "load_mos_TPX : Shift Mask to canvas $cx $cy"

     # rotate mask 
     # in reset_mask abs_ang_rad is set to 0
     # puts stdout "rotangle=$lPA"
     $this configure -rotangle [ expr $lPA * $CSO ]
     set abs_ang_rad 0
     set PAM 0 
     $this rotate $m_canvas $mos(0,x0) $mos(0,y0) 1

     $this configure -rotangle 5
     # abs_ang_rad back in rad again 
     # puts stdout "load_mos_TPX : Rotate Mask $lPA"

    # -----------------------------------------
    # step 2: load the same file again, read INS 
    #         and adjust mos-slits 
    # puts stdout "load_mos_TPX : Move Slitlets"
    # reset targets and reference targets 
    for {set i 1} { $i <= 40 } { incr i 1} {
        set targw($i,f) 0
        }

     if {[file exists $myfile] & [file readable $myfile]} {
        set fileId [open $myfile r ]
        puts $lfId "FORS::load_TARG  $fileId"
        # as long as there are lines in the file
        while {[gets $fileId line_org] >= 0} {
             # puts stdout $line_org
          if {[string match *$keyw2* $line_org]==1} {
             # INS 
             switch -glob -- $line_org {
               *ALPHA* { set lwx [lindex $line_org 1] 
                   set key [lindex $line_org 0]
                   set key [split $key "."] 
                   set key [lindex $key 1]
                   set key [string trimleft $key "TARG"] 
                   set targw($key,a) [expr 15.0 * [ DDMMSS2deg $lwx] ] 
                   # reference targets have no names 
                   if {$key>30} { set targw($key,f) 1 }
                   # puts stdout "------A-$key-$targw($key,a)-"
                   }
               *DELTA* { set lwy [lindex $line_org 1] 
                   set key [lindex $line_org 0]
                   set key [split $key "."] 
                   set key [lindex $key 1]
                   set key [string trimleft $key "TARG"] 
                   set targw($key,d) [DDMMSS2deg $lwy] 
                   # reference targets have no names 
                   if {$key>30} { set targw($key,f) 1 }
                   # puts stdout "------D-$key-$targw($key,d)-"
                   }
               *WID* { set lwd [lindex $line_org 1] 
                   set key [lindex $line_org 0]
                   set key [split $key "."] 
                   set key [lindex $key 1]
                   set key [string trimleft $key "TARG"] 
                   set mos($key,b) $lwd
                   # for ECHS, slit 9 should have the same 
                   # slit width as slit 10 
                   set l_key $key
                   }
                *NAME* { set lna [lindex $line_org 1] 
                   set key [lindex $line_org 0]
                   set key [split $key "."] 
                   set key [lindex $key 1]
                   set key [string trimleft $key "TARG"] 
                   set targw($key,f) $lna
                   # puts stdout "------N-$key-$targw($key,f)-"
                   }
                 *MOS* { # this keyword should be read before 
                   # INS.TARG, here only verification 
                   # buts since it appears after INS.TARG
                   # and we do not want to read the file a third time
                   # we use it only for verification reasons
                   set lta [lindex $line_org 1]
                   set key [lindex $line_org 0]
                   set key [split $key "."] 
                   set key [lindex $key 1]
                   set key [string trimleft $key "MOS"] 
                   if {$targw($key,f) == 0} {
                            puts stdout "Warning: opt1.targ:  \
                            INS.MOS >><< INS.TARG"
                             }
                   if {$key != $lta} {
                            puts stdout "Warning: opt1.targ:   \
                            INS.MOS >> << INS.MOS"
                            }
                   # puts stdout "------M-$key-$lta-"
                    }
                  default { # puts stdout "------X->$line_org<" }
                 }
              }
          }
     close $fileId


     set s_c 0
     set result {ok}
     # for {set i 1} { $i < 20} { incr i 1} {
     #     puts stdout [format "%2d %10s %10.3f %10.3f" \
     #     $i $targw($i,f) $targw($i,a) $targw($i,d)]
     #     }

     # --------- reference targets 
     C_ref_targ reset
     for {set i 31} { $i <= 40} { incr i 1} {
        if {$targw($i,f)} {
        catch { #puts stdout "reference target : $i  $targw($i,a) $targw($i,d)" 
                $m_image convert coords $targw($i,a) $targw($i,d) "deg $EQX" \
                           x y "wcs $EQX"
                 C_ref_targ put $x $y
                 C_ref_targ plot $mos(0,r); # delete all and plot all
               }
            }
         }

     if {[C_ref_targ nort]} { puts stdout "[C_ref_targ nort] Reference Targets found"
                      } else {
                      puts stdout "->->-> Warning No Reference Targets found"
                      }

     # C_ref_targ show
     # --------- reference targets end

     if {[string compare [$this cget -modus] echs]==0} {
        # for echs we make the slit width common to the last 
        # specified slit width (= mos 10 in any case)
        if [catch { $m_mb.layout.sw invoke *$lwd* } result ] {
              puts stdout "$result \n Slit_Width Error $lsw" 
              $this configure -slit_width $lwd } else {
        # puts stdout "$m_mb.layout.sw invoke *$lwd*"
        }
       }


     # targw locally as deg, but when move_slit is called it is 
     # in wcs again
     for {set i 1} { $i < 20} { incr i 1} {
      if { $targw($i,f) != 0} {
      # the move_slit requires screen coords
      $m_image convert coords $targw($i,a) $targw($i,d) "deg $EQX" mx my screen
      puts stdout [format "%2d-->%s (%11.7f,%11.7f)(deg $EQX) \
                 (%8.3f,%8.3f)(screen) "\
                 $i $targw($i,f) $targw($i,a) $targw($i,d) $mx $my ]
       set targw($i,f) 0
                 #                       select,  not manually 
       set a [$this move_slit $m_canvas $mx $my 1 0]
       # puts stdout "result=>$a<"
       if { $a != ""} {
          incr s_c
          # puts stdout $s_c
          }
          }
       }
    puts stdout "+----------------------------+"
    puts stdout [ format "%2d MOS slits positioned" $s_c ]
    puts stdout "+----------------------------+"
    update_and_replot
    # puts "after  update_and_replot"
    }

   set last [string length $myfile] 
   set first [expr [string last "/" $myfile]  +1]
   set last_load_fn [string range $myfile $first [expr $last - 6] ]
   
   $this configure -file_name [lindex [split $last_load_fn "."] 2]
      
   catch {SP update }
   }



    # ==================================
    private method load_img_TPX { filename } {
    # ==================================
    # Open setup file, move and rotate mask and 
    # position slity blades according to the setup 
    # reading a img1.<ec>.<pn>.fims file 
    # IMG : MOS slit blades in mm focal plane 
    # LSS : 
    # HITS:
    # More information in load_mos_TPX  
    # puts stdout "load_img_TPX >$filename<"

    C_wc coeff_wc $wx $wy

     set myfile $set_dir
     # puts stdout "load_img_TPX >$filename<"

     if {$filename==""} {
     append myfile "/A"
     append myfile [$this cget -modus]
     append myfile $exposure_counter
     append myfile ".targ"
     } else {
     if {$filename} { append myfile "/$filename" 
      } else { 
      global .fs
      catch { destroy .fs }
      set myfile [filename_dialog $set_dir \
        "[$this cget -modus][string index [$this cget -instrument] 4].*.fims" ]
      }
     }
     # puts stdout "load_img_TPX: $myfile" 
     set keyword TEL
     set keyw2   INS
     set f_gris 0; set f_filt 0; set f_slit_width 0; set f_coll 0
     set f_lmin 0;  
     set lty ""; set epo_flag 0; set equ_flag 0
     
     # -----------------------------------------------------
     # step 1 : load opt1.targ, read only TEL pointing 
     #          shift and rotate according to these values
     # -----------------------------------------------------
     if {[file exists $myfile] & [file readable $myfile]} {
        set fileId [open $myfile r ]
        # puts stdout "FORS::load_TARG  $fileId"
        puts $lfId "FORS::load_TARG  $fileId"
        # as long as there are lines in the file
        while {[gets $fileId line_org] >= 0} {
        if {[string match #* $line_org]} {
          puts stdout $line_org
          } else {
          if {[string match *$keyword* $line_org]==1} {
             # puts stdout $line_org
             # TEL 
             switch -glob -- $line_org {
                   *ALPHA* { set lwx [lindex $line_org 1] }
                   *DELTA* { set lwy [lindex $line_org 1] }
                 *ROT.OFF* { set lPA [lindex $line_org 1] }
                 *EPOCH*   { set EQX [lindex $line_org 1]; set epo_flag 1 }
                 *EQUINOX* { set EQX [lindex $line_org 1]; set equ_flag 1 }
                 *TYPE*    { set lty [lindex $line_org 1] }
                 }
             }

          if {[string match *$keyw2* $line_org]==1} {
             # puts stdout $line_org
             # INS (SLIT or PINH)
             switch -glob -- $line_org {

               *PINH* { # in case it is  lss or hits 
                        # configure slit width before reset_mask
                        set key "INS.SLIT.NAID       lSlit0_3arcsec"
                        set key [split $key " "] 
                        set ll [expr [llength $key] -1]
         # puts stdout "[$this cget -modus] [lindex $key 0] [lindex $key $ll]"
         # config_mask "[$this cget -modus] [lindex $key 0] [lindex $key $ll]"
           $m_mb.layout.ht invoke *0.3*
                        }

               *SLIT* { set key $line_org 
                        set key [split $key " "] 
                        set ll [expr [llength $key] -1]
                        set l_slit [lindex $key $ll]
                        # puts stdout ">$l_slit<"
                         switch -exact -- $l_slit {
                           lSlit0_3arcsec { set sl 0.3 }
                          htSlit0_4arcsec -
                           lSlit0_4arcsec { set sl 0.4 }
                          htSlit0_5arcsec -
                           lSlit0_5arcsec { set sl 0.5 }
                          htSlit0_7arcsec -
                           lSlit0_7arcsec { set sl 0.7 }
                          htSlit1_0arcsec -
                           lSlit1_0arcsec { set sl 1.0 }
                          htSlit1_3arcsec -
                           lSlit1_3arcsec { set sl 1.3 }
                          htSlit1_6arcsec -
                           lSlit1_6arcsec { set sl 1.6 }
                          htSlit2_0arcsec -
                           lSlit2_0arcsec { set sl 2.0 }
                           lSlit2_5arcsec { set sl 2.5 }
                                  default { set sl 1.0 }
                                   }
                        # puts stdout ">$l_slit<  >$sl<"
                        # puts stdout ">[$this cget -long_width]<  >$sl<"
                        if {[string compare [$this cget -long_width] $sl]} {
                           if {[string index $l_slit 0]=="h"} {
                            # puts stdout "$m_mb.layout.ht invoke *$sl*"
                            $m_mb.layout.ht invoke *$sl*
                            # $this configure -long_width  $sl
                            # puts stdout "$m_mb.layout.ht invoke *$sl*"
                            } else {
                            # puts stdout "$m_mb.layout.ls invoke *$sl*"
                            $m_mb.layout.ls invoke *$sl*
                            # $this configure -long_width  $sl
                            # puts stdout "$m_mb.layout.ls invoke *$sl*"
                            } 
                            }
                        }
                    }
              }
           # pinhole slit end
 
            

          # step 2 -------------------------------------
          # --- by the way read SEQ.PREP keywords ---------
          if {[string match *SEQ.PREP* $line_org]==1} {
             
          if {[string match *WID* $line_org]==1} {
                 # the common slit_width
                 # puts stdout "WID = [lindex $line_org 1]"
                 set l_slit_width [lindex $line_org 1]
                 set f_slit_width 1
                }
          if {[string match *COLL* $line_org]==1} {
                 # the collimator
                 # puts stdout "SEQ.PREP.COLL = [lindex $line_org 1]"
                 set l_coll [lindex $line_org 1]
                 set f_coll 1
              }
          if {[string match *GRIS* $line_org]} {
                 # the grism
                 # puts stdout "SEQ.PREP.GRISM = [lindex $line_org 1]"
                 set l_gris [lindex $line_org 1]
                 set f_gris 1
              }
          if {[string match *LMIN* $line_org]} {
                 # the user defined wavelength region
                 # puts stdout "SEQ.PREP.LMIN = [lindex $line_org 1]"
                 set L_MIN  [lindex $line_org 1]
                 set f_lmin 1
              }
          if {[string match *LMAX* $line_org]} {
                 # the user defined wavelength region
                 # puts stdout "SEQ.PREP.LMAX = [lindex $line_org 1]"
                 set L_MAX [lindex $line_org 1]
                 set f_lmin 1
              }
          if {[string match *FILT* $line_org]} {
                 # the os filter
                 # puts stdout "SEQ.PREP.FILT = [lindex $line_org 1]"
                 set l_filt [lindex $line_org 1]
                 set f_filt 1
              }
          }
         }
          }
     close $fileId

       }

      # some tests for backfward compatibility
      if {$lty == ""} { puts stdout "->->-> Warning TEL.TARG.TYPE not defined" }
      if {!$equ_flag} { puts stdout "->->-> Warning TEL.TARG.EQUINOX not found" }
      if {$epo_flag}  { puts stdout "->->-> Warning TEL.TARG.EQUINOX defined via TEL.TARG.EPOCH = $EQX" }


      # rotator angle = - position angle of mask
      set lPA [expr $lPA* -1.0]
      set lPA [expr fmod($lPA,360) ]

      # puts stdout [ format  "load_img_TPX : HHMMSS.xxx %13.5f ->  \
      #       %10.6f deg"  $lwx [expr 15.0 *[ DDMMSS2deg  $lwx]  ]]
      # puts stdout [ format  "load_img_TPX : DDMMSS.xxx %13.5f ->  \
      #        %10.6f deg"  $lwy [ DDMMSS2deg  $lwy] ]
      # puts stdout [ format  "load_img_TPX : P.A. (M-W) %13.5f deg"  $lPA ]


     set mask_initialized 0; # assume mask not yet reset  
     # this is for only one reset_mask when more than one 
     # instrument config has been changed 

     # 5.11.98 I see this becomes very circumstancy
     #  --the SkyCat_plugin routines must become part of class FORS 
     #  --standard highres must become COLL_SR+1 ...
     #  --read COLL_SR+1 GRIS_... from FORS1/2.isf
     #  --
     # configure instrument due to SEQ.PREP keywords if nessecary 
     if {$f_lmin} { wave_limit }

     if {$f_slit_width} { 
        if {[string compare [$this cget -slit_width] $l_slit_width]} {
           # puts stdout "$m_mb.layout.sw invoke *$l_slit_width*"
           $m_mb.layout.sw invoke *$l_slit_width*
           # note the button will not change
           # $this configure -slit_width $l_slit_width
           # puts stdout "$m_mb.layout.sw invoke *$l_slit_width*"
            }
            }

     if {$f_gris} { 
        if {[string compare [$this cget -grism] $l_gris]} {
           # puts stdout "$m_mb.layout.gr invoke *$l_gris*"
           $m_mb.layout.gr invoke *$l_gris*
           # $this configure -grism  $l_gris
           # puts stdout "$m_mb.layout.gr invoke *$l_gris*"
           }
           }

     if {$f_filt} {
        set app2 ""
        # change only if new filt != old filt
        # puts stdout ">[$this cget -filter]< >$l_filt<"
        if {[string compare [$this cget -filter] $l_filt]} {
           # entryconfigure which submenue ?
           # note we have layout.fi, fi.m2, fi.m3, fi,m4 fi.m5
           # first search the submenue (ekelhaft)
           # then configure
           foreach append {"" .m2 .m3 .m4 .m5} {
              for {set i 0} {$i <= 9} {incr i 1} {
                  set local [$m_mb.layout.fi$append entrycget $i -label]
                  if {[string compare $local $l_filt]==0} {
                     set app2 $append
                     }
                  }
              }
           $m_mb.layout.fi$app2 invoke *$l_filt*
           }
        }


     if {$f_coll} { 
        # change only if different
        if {[string compare [$this cget -collimat] $l_coll]} {
        # puts stdout "$m_mb.layout.sh invoke *$l_coll*"
           $m_mb.layout.sh invoke *$l_coll*
        # puts stdout "$m_mb.layout.sh invoke *$l_coll*"
           }
           }


     if {$mask_initialized==0} { 
     # puts stdout "load_mos_TPX : Reset Mask"
     reset_mask }
     # set mask_initialized again 


     $m_image convert coords $mos(0,x0) $mos(0,y0) canvas mx my screen
     # puts stdout "canvas $mos(0,x0) $mos(0,y0) screen $mx $my"
     mark $m_canvas $mx $my
     # puts stdout "Mark_Mask at screen $mx $my"
     # $m_image convert coords [expr 15.0 *[DDMMSS2deg $lwx] ]\
     #       [DDMMSS2deg $lwy] "deg $EQX"  nwx nwy screen
     set a1 [expr 15.0 *[DDMMSS2deg $lwx] ]
     set d1 [DDMMSS2deg $lwy]
     # puts stdout [format "deg $EQX: %10.3f %10.3f" $a1 $d1]
     # wcs to canvas via bonsai matrix
        set dum [C_wc w2c $a1 $d1]
        set cx [lindex $dum 0]
        set cy [lindex $dum 1]
     # puts stdout "canvas $cx $cy"
     $m_image convert coords $cx $cy canvas nwx nwy screen
     # puts stdout "load_mos_TPX : Shift Mask"
     shift $m_canvas  $nwx $nwy
     # puts stdout "load_mos_TPX : Shift Mask to canvas $cx $cy"

     # rotate mask 
     # in reset_mask abs_ang_rad is set to 0
     # puts stdout "rotangle=$lPA"
     $this configure -rotangle [ expr $lPA * $CSO ]
     set abs_ang_rad 0
     set PAM 0 
     $this rotate $m_canvas $mos(0,x0) $mos(0,y0) 1

     $this configure -rotangle 5
     # abs_ang_rad back in rad again 
     # puts stdout "load_img_TPX : Rotate Mask $lPA"

    # we expect the file coinatins either 
    # INS.MOS{i}.POS and INS.MOS{i}.WID keywords for img, ipol modes
    # or INS.SLIT.NAID (lss,hits) or INS.PINH.NAID (hits) keywords


    # mask is now at the right place with the right position angle 
    # --------------------------------
    # step 3: load the same file again, read INS.MOS{i}.POS and INS.MOS{i}.WID 
    #         and adjust mos-slits 

    # what is this ? 
    for {set i 31} { $i<=40} { incr i 1} {
        set targw($i,f) 0
        }
 
     if {[file exists $myfile] & [file readable $myfile]} {
        set fileId [open $myfile r ]
        puts $lfId "FORS::load_TARG  $fileId"
        # as long as there are lines in the file
        while {[gets $fileId line_org] >= 0} {
             # puts stdout $line_org
          if {[string match *$keyw2* $line_org]==1} {
             # INS 
             switch -glob -- $line_org {

               *POS* { set lpo [lindex $line_org 1] 
                   # IMG  slit center in mm 
                   set key [lindex $line_org 0]
                   set key [split $key "."] 
                   set key [lindex $key 1]
                   set key [string trimleft $key "MOS"] 
                   # puts stdout "------P-$key-$lpo mm-"
                   set t(po,$key) $lpo 
                   }

               *WID* { set lwi [lindex $line_org 1] 
                   # slit width in mm 
                   set key [lindex $line_org 0]
                   set key [split $key "."] 
                   set key [lindex $key 1]
                   set key [string trimleft $key "MOS"] 
                   # puts stdout "------W-$key-$lwi mm-"
                   set t(wi,$key) $lwi 
                   }

               *SET* { set lwt [lindex $line_org 1] 
                   # this is for the blades on non-default positions 
                   set key [lindex $line_org 0]
                   set key [split $key "."] 
                   set key [lindex $key 1]
                   set key [string trimleft $key "MOS"] 
                   set targw($key,f) 1
                   # puts stdout "------T-$key-$targw($key,f)-"
                   }

               *ALPHA* { set lwx [lindex $line_org 1] 
                   # this is for the reference targets
                   set key [lindex $line_org 0]
                   set key [split $key "."] 
                   set key [lindex $key 1]
                   set key [string trimleft $key "TARG"] 
                   set targw($key,a) [expr 15.0 * [ DDMMSS2deg $lwx] ] 
                   set targw($key,f) 1
                   # puts stdout "------A-$key-$targw($key,a)-"
                   }

               *DELTA* { set lwy [lindex $line_org 1] 
                   # this is for the reference targets
                   set key [lindex $line_org 0]
                   set key [split $key "."] 
                   set key [lindex $key 1]
                   set key [string trimleft $key "TARG"] 
                   set targw($key,d) [DDMMSS2deg $lwy] 
                   set targw($key,f) 1
                   # puts stdout "------D-$key-$targw($key,d)-"
                   }

                  default { # puts stdout "------X-" }
                 }

              }
          }
     close $fileId
          

     # --------- reference targets 
     C_ref_targ reset

        # normal case 
        # IMG, IPOL, IMS, PIMS, HITS, LSS2, LSS1 
        for {set i 31} { $i <= 40} { incr i 1} {
            if {$targw($i,f)} {
            catch { $m_image convert coords $targw($i,a) $targw($i,d) "deg $EQX" \
                               x y "wcs $EQX"
                    C_ref_targ put $x $y
                    C_ref_targ plot $mos(0,r); # delete all and plot all
                    # puts stdout "reference target : $i  $x $y $EQX"
                  }
               }
            }

     if {[C_ref_targ nort]} { puts stdout "[C_ref_targ nort] Reference Targets found"
                      } else {
                      puts stdout "->->-> Warning No Reference Targets found"
                      }
     # --------- reference targets end
    
     # puts stdout "reftargs end" 

     # lets calculate the new MOS slit blade pos.
     # (POS-WID/2)/(XSRMAX-XSRMIN)=(f-e)/(h-e)=(b-a)/(d-a)
     # and so on
     set fux [ expr $XBLADEB-$XBLADEA ]


     if {[string compare img [$this cget -modus] ]==0} {

     if { $mos(10,xh)==$mos(10,xe) } {
     # north to right or left
     for {set i 1} { $i < 20} { incr i 1} {
         set mos($i,yf) [expr ($t(po,$i)-$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,yh)-$mos($i,ye)) +$mos($i,ye) ]
         set mos($i,yb) [expr ($t(po,$i)-$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,yd)-$mos($i,ya)) +$mos($i,ya) ]
         set mos($i,yg) [expr ($t(po,$i)+$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,yh)-$mos($i,ye)) +$mos($i,ye) ]
         set mos($i,yc) [expr ($t(po,$i)+$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,yd)-$mos($i,ya)) +$mos($i,ya) ]
         }
     } elseif { $mos(10,yh)==$mos(10,ye) } {
     for {set i 1} { $i < 20} { incr i 1} {
         set mos($i,xf) [expr ($t(po,$i)-$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,xh)-$mos($i,xe)) +$mos($i,xe) ]
         set mos($i,xb) [expr ($t(po,$i)-$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,xd)-$mos($i,xa)) +$mos($i,xa) ]
         set mos($i,xg) [expr ($t(po,$i)+$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,xh)-$mos($i,xe)) +$mos($i,xe) ]
         set mos($i,xc) [expr ($t(po,$i)+$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,xd)-$mos($i,xa)) +$mos($i,xa) ]
         }
     } else {
     for {set i 1} { $i < 20} { incr i 1} {
         set mos($i,xf) [expr ($t(po,$i)-$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,xh)-$mos($i,xe)) +$mos($i,xe) ]
         set mos($i,yf) [expr ($t(po,$i)-$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,yh)-$mos($i,ye)) +$mos($i,ye) ]
         set mos($i,xb) [expr ($t(po,$i)-$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,xd)-$mos($i,xa)) +$mos($i,xa) ]
         set mos($i,yb) [expr ($t(po,$i)-$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,yd)-$mos($i,ya)) +$mos($i,ya) ]
         set mos($i,xg) [expr ($t(po,$i)+$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,xh)-$mos($i,xe)) +$mos($i,xe) ]
         set mos($i,yg) [expr ($t(po,$i)+$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,yh)-$mos($i,ye)) +$mos($i,ye) ]
         set mos($i,xc) [expr ($t(po,$i)+$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,xd)-$mos($i,xa)) +$mos($i,xa) ]
         set mos($i,yc) [expr ($t(po,$i)+$t(wi,$i)/2.0-$XBLADEA)/$fux \
             *($mos($i,yd)-$mos($i,ya)) +$mos($i,ya) ]
         }
    }

    }
    update_and_replot
    }

   set last [string length $myfile] 
   set first [expr [string last "/" $myfile]  +1]
   set last_load_fn [string range $myfile $first [expr $last - 6] ]

   $this configure -file_name [lindex [split $last_load_fn "."] 2]


   catch {SP update }
   }

   private method load_mxu_refslit {} {
       if {"[$this cget -instrument]" == "FORS1"} {return}
     
       
 for {set i 0} { $i < $nf_refslit} {incr i} {
       
       set ff [list $XSRMIN $XSRMAX]
       if {[string match *HR* [$this cget -collimat] ]} {
           set ff [list $XHRMIN $XHRMAX]
       }
    
       fors::MXU_SLIT::set_mmff 1

	   
       set noms [fors::MXU_SLIT::nos]
       set a [fors::MXU_SLIT C_mxus[incr noms 1] \
                     $m_canvas \
                     $m_image  \
                     $mxurefsx($i) $mxurefsy($i) \
		     -ff $ff \
                     -eqx $EQX \
                     -target_name "refslit" \
                     -slit_width  $mxurefwd($i) \
                     -slit_length $mxurefle($i) \
                     -pos_ang     $mxurefro($i)  \
		     -ref_slit    1 \
                     -shape $mxureftp($i) \
                     -ON 1 \
                     -child $this ]
   }
      set nf_refslit  [fors::MXU_SLIT::nos]
      fors::MXU_SLIT::set_mmff 0
}
  

    # ==================================
    private method load_mxu_fad { filename } {
    # ==================================
    # reading a mxu2.<MaskId-900000>.standard.fad file 
    # the input file must be a fors aperture definition file 
    # like $FIMSROOT/lib/fors/mxu2.01.standard.fad 
    # such files define one of the 100 MXU standard masks 
    # permanently located on Paranal. 
    # these masks need not to be manufactured, they are already there
    # 
    puts stdout "In load_mxu_fad"
    set a [split $filename "/"]
    set b [lindex $a [expr [llength $a ] -1] ]
    set c [split $b "."] 
    set d [string trimleft [lindex $c 1] 0]
    set fn_maskid [expr 900000 + $d ]
    puts stdout "orig filename=$b"
    puts stdout "mode         =[lindex $c 0]"
    puts stdout "mask Id index=[lindex $c 1]"
    puts stdout "mask Id      =$fn_maskid"
    puts stdout "orig format  =[lindex $c 2]"
    puts stdout "orig .ext    =[lindex $c 3]\n\n"

     if {[file exists $filename] & [file readable $filename]} {
        set fileId [open $filename r ]
        puts $lfId "FORS::load_mxu_FAD  $fileId"
        puts stdout "FORS::load_mxu_FAD  $fileId"
        # set some flags
        set i_flag 0;  # first line must be 'FORS2'
        set lcounter 0; # line counter after 'FORS2'

        # adapt collimator, used to create mxu slit 
        set ff [list $XSRMIN $XSRMAX]
        if {[string match *HR* [$this cget -collimat] ]} {
        set ff [list $XHRMIN $XHRMAX] }

        reset_mask 
  
        # this is the flag which determines 
        # science mask or standard mask mode
        # sky mask or technical mask mode 

        fors::MXU_SLIT::set_mmff 1


        # extent the functionality of the playground
        # center/slit now for the whole mask
        # $m_canvas addtag ali withtag mxu

        # as long as there are lines in the file
        while {[gets $fileId line_org] >= 0} {

        if {[string match #* $line_org]} {

          # this is an optional comment line 
          # do nothing 
          puts stdout "comment> $line_org"

          } else {

          # puts stdout ">$i_flag< >$lcounter< >>>$line_org"

          if {$i_flag} {
                # we count lines after FORS2 string
                incr lcounter 1 

             switch -exact -- $lcounter {
                 1 { 
                   # we expect the checksum 
                   # this checksum will be used as the 
                   # the program id for standard masks   
                   set checksum $line_org
                   puts stdout "checksum=$checksum"
                   }
                 2 {
                   # we expect real mask id
                   set maskid $line_org 
                   puts stdout "maskid  =$maskid"
                   if {$maskid != $fn_maskid } {
                       puts stdout "MASK_NAID inconsistency"
                       } else {
                       fors::MXU_SLIT::set_MASK_NAID "$checksum $maskid"
                       puts stdout "maskid =[fors::MXU_SLIT::get_MASK_NAID]"
                       }
                   }
                 3 {
                   # we expect number of STRAIGHT slits
                   set straight $line_org
                   puts stdout "number of STRAIGHT=$straight"
                   }
                 4 {
                   # we expect number of CURVED slits
                   set curved  $line_org
                   puts stdout "number of CURVED  =$curved"
                   }
                 5 {
                   # we expect number of CIRCLE holes 
                   set circle $line_org
                   puts stdout "number of CIRCLE  =$circle"
                   }
           default { 
                     # now we get the apertures 
                     # we make life easy. Instead of counting 
                     # the slit numbers we decide via the number of 
                     # parameters in line_org 
                     #       3 parameters => CIRCLE
                     #  4 or 5 parameters => STRAIGHT
                     # 5 to 12 parameters => CURVED 
                     set np [llength [ldelete_all [split $line_org " "] ""] ]
                     puts stdout "linecounter=[expr $lcounter-5] nofpar=$np" 

                     switch -exact -- $np {
                         0 { # empty line 
                             puts stdout "empty line, do nothing"
                           }
                         3 { 
                           # this must be a circle aperture
                           $this configure -mxu_shape CIRCLE   
                           set b [ldelete_all [split $line_org " "] ""] 
                           set lx [lindex $b 0]
                           set ly [lindex $b 1]
                           set ld [lindex $b 2]
                           set lle $ld; set lro 0.0
     puts stdout "CIRCLE with x,y=($lx $ly) a,b=($ld,$lle) mm" 
                           }
                         4 -
                         5 { 
                           # this must be a box aperture 
                           $this configure -mxu_shape STRAIGHT
                           set b [ldelete_all [split $line_org " "] ""] 
                           set lx [lindex $b 0]
                           set ly [lindex $b 1]
                           set ld [lindex $b 2]
                           set lle [lindex $b 3]
                           set lro [lindex $b 4]
     puts stdout "STRAIGHT with x,y=($lx,$ly) l,w=($ld,$lle) >$lro< mm" 
                           }
                         6 -
                         7 -
                         8 -
                         9 -
                        10 -
                        11 -
                        12 { 
                           # this must be a curved aperture 
                           # $this configure -mxu_shape CURVED   
     puts stdout "CURVED   with x,y=($lx,$ly) l,w=($ld,$lle) >$lro< mm" 
                           }
                     default { puts stdout " wrong format in $filename" }
                          }


   if {$np > 2} {
      # if it was not an empty line 
      # if the number of parameters was at least 3

      set noms [fors::MXU_SLIT::nos]
      # puts stdout "noms= $noms"
      puts stdout "$lx $ly $EQX $ld $lle $lro $ff [$this cget -mxu_shape]"
                            
     if {$noms==$mnomxus} {
        info_dialog "Error: Cannot handle more than\n $mnomxus MXU slits" .skycat1
        return
        }

               # puts stdout "set a ...>[expr $noms +1]<"
               set a [fors::MXU_SLIT C_mxus[incr noms 1] \
                     $m_canvas \
                     $m_image  \
                     $lx $ly \
                     -ff $ff \
                     -eqx $EQX \
                     -target_name aperture \
                     -slit_width  $ld \
                     -slit_length $lle \
                     -pos_ang     $lro  \		    
                     -shape [$this cget -mxu_shape] \
                     -ON 1 \
                     -child $this ]

               # puts stdout "..a"
               set noms [fors::MXU_SLIT::nos]
                   # C_mxus$noms status
                   # end of: must be an aperture definition
           }
                   }
                   }

                } else {

                if {[string match *FORS2* $line_org]} {
                   set i_flag 1
                   puts stdout "we found the FORS2 string" 
                   }

                 }  
               }
          }
        }
     close $fileId

   
   update_and_replot
 
   set last [string length $filename] 
   set first [expr [string last "/" $filename]  +1]
   set last_load_fn [string range $filename $first [expr $last - 6] ]

   $this configure -file_name [lindex [split $last_load_fn "."] 2]

   catch {SP update }
      }



    # ==================================
    private method load_mxu_TPX { filename } {
    # ==================================
    # reading a mxu2.<ec><pn>.fims file 
    # the input file must be a setup file like 'mxu2.12.project.fims'
    # slits are given in wcs 

    set debug 0 

    # test if this is necessary 
    # 
    C_wc coeff_wc $wx $wy
       fors::MXU_SLIT::coeff $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                       $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                       $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc) 

   # --- open filename ---------
     set myfile $set_dir
     if {$debug} { puts stdout "load_mxu_TPX >$filename<" }
     if {$filename==""} {
	 # no filename means load result from AutoPos, written to
	 # .fims/SET/(p)mos(1/2).<ec>.autopos.fims
	 # we load autopos from LOG dir 
	 set myfile $log_dir
	 append myfile \
		 "/[$this cget -modus][string index [$this cget -instrument] 4].$exposure_counter.auto_pos.fims"
     } else {
	 if {$filename} { 
	     append myfile "/$filename" 
	 } else { 
	     # filename is zero, call dialog 
	     global .fs
	     catch { destroy .fs  }
	     set myfile [filename_dialog $set_dir \
		     "[$this cget -modus][string index [$this cget -instrument] 4].*.fims" ]
	 }	 
     }

     puts stdout "load_mxu_TPX: $myfile"

     if {$debug} { puts stdout "load_mxu_TPX: $myfile"  }

     # if file is a fors2 standard-mask aperture definition file (FAD)
     # b must be 'fad'
     set a [lindex [split $myfile "/"] [expr [llength [split $myfile "/" ] ] -1] ]
     set b [lindex [split $a "."] [expr [llength [split $a "."] ] -1 ] ]
     if {[string compare $b fad]==0} { 
        load_mxu_fad $myfile
        return
        }

     set keyword TEL
     set keyw2   INS
     set f_gris 0; set f_filt 0; set f_slit_width 0; set f_coll 0
     set f_lmin 0;  
     set L_prop M_
     set L_mid 900000
     set lty ""; set epo_flag 0; set equ_flag 0


     # ------  read TCS keywords --------
     # step 1 : load opt1.targ, read only TEL pointing 
     #          shift and rotate according to these values
     if {[file exists $myfile] & [file readable $myfile]} {
	 set fileId [open $myfile r ]
	 puts $lfId "FORS::load_TARG  $fileId"
	 # puts stdout "FORS::load_TARG  $fileId"
	 # as long as there are lines in the file
	 while {[gets $fileId line_org] >= 0} {

	     if {[string match #* $line_org]} {
		 if {$debug} { puts stdout $line_org }
	     } else {
		 if {[string match *$keyword* $line_org]==1} {
		     # puts stdout $line_org
		     # TEL 
		     switch -glob -- $line_org {
			 *ALPHA* { set lwx [lindex $line_org 1] }
			 *DELTA* { set lwy [lindex $line_org 1] }
			 *ROT.OFF* { set lPA [lindex $line_org 1] }
			 *EPOCH*   { set EQX [lindex $line_org 1]; set epo_flag 1 }
			 *EQUINOX* { set EQX [lindex $line_org 1]; set equ_flag 1 }
			 *TYPE*    { set lty [lindex $line_org 1] }
		     }
		 }

		 # --- by the way read SEQ.PREP keywords ---------
		 if {[string match *SEQ.PREP* $line_org]==1} {

		     if {[string match *COLL* $line_org]} {
			 # the collimator  
			 if {$debug} { puts stdout "SEQ.PREP.COLL = [lindex $line_org 1]" }
			 set l_coll [lindex $line_org 1] 
			 set f_coll 1
		     }
		     if {[string match *GRIS* $line_org]} {
			 # the grism   
			 if {$debug} { puts stdout "SEQ.PREP.GRISM = [lindex $line_org 1]" }
			 set l_gris [lindex $line_org 1] 
			 set f_gris 1
		     }
		     if {[string match *LMIN* $line_org]} {
			 # the user defined wavelength region  
			 if {$debug} { puts stdout "SEQ.PREP.LMIN = [lindex $line_org 1]" }
			 set L_MIN  [lindex $line_org 1] 
			 set f_lmin 1
		     }
		     if {[string match *LMAX* $line_org]} {
			 # the user defined wavelength region  
			 if {$debug} { puts stdout "SEQ.PREP.LMAX = [lindex $line_org 1]" }
			 set L_MAX [lindex $line_org 1] 
			 set f_lmin 1
		     }
		     if {[string match *FILT* $line_org]} {
			 # the os filter   
			 if {$debug} { puts stdout "SEQ.PREP.FILT = [lindex $line_org 1]" }
			 set l_filt [lindex $line_org 1] 
			 set f_filt 1
		     }
		     if {[string match *MASK_NA* $line_org]} {
			 # the eso allocated proposal id       
			 if {$debug} { puts stdout "SEQ.PREP.MASK_NA = [lindex $line_org 1]" }
			 set L_prop [lindex $line_org 1] 
		     }
		     if {[string match *MASK_ID* $line_org]} {
			 # the mask id is either 900000 for science (sky) masks 
			 # or 900001 <= id <= 900100 for technical masks
			 # science masks will get their id at paranal            
			 if {$debug} { puts stdout "SEQ.PREP.MASK_ID = [lindex $line_org 1]" }
			 set L_mid [lindex $line_org 1] 
			 fors::MXU_SLIT::set_MASK_NAID [list  $L_prop $L_mid]
			 # puts stdout "> [fors::MXU_SLIT::get_MASK_NAID]<"
			 # we apply standard mask mode a liitle bit later 
			 # after reset_mask  and after every slit has been created
			 # look for 'L_mid' 
		     }
		 }

	     }
	 }
	 close $fileId
     }
      
      # some tests for backfward compatibility
      if {$lty == ""} { puts stdout "->->-> Warning TEL.TARG.TYPE not defined" }
      if {!$equ_flag} { puts stdout "->->-> Warning TEL.TARG.EQUINOX not found" }
      if {$epo_flag}  { puts stdout "->->-> Warning TEL.TARG.EQUINOX defined via TEL.TARG.EPOCH = $EQX" }


     # puts stdout "file closed" 
     set lPA [expr $lPA * -1]
     set lPA [expr fmod($lPA,360) ]

      # puts stdout [ format  "load_mos_TPX : HHMMSS.xxx %13.5f ->  \
      #       %10.6f deg"  $lwx [expr 15.0 *[ DDMMSS2deg  $lwx]  ]]
      # puts stdout [ format  "load_mos_TPX : DDMMSS.xxx %13.5f ->  \
      #        %10.6f deg"  $lwy [ DDMMSS2deg  $lwy] ]
      # puts stdout [ format  "load_mos_TPX : P.A. (M-W) %13.5f deg"  $lPA ]

     set mask_initialized 0; # assume mask not yet reset  
     # this is for only one reset_mask when more than one 
     # instrument config has been changed 

     # 5.11.98 I see this becomes very circumstancy
     #  --the SkyCat_plugin routines must become part of class FORS 
     #  --standard highres must become COLL_SR+1 ...
     #  --read COLL_SR+1 GRIS_... from FORS1/2.isf
     #  --
     # configure instrument due to SEQ.PREP keywords if nessecary 
     if {$f_lmin} { wave_limit }


     if {$f_gris} { 
	 # if the grism defined in the .fims file 
	 # is different from the current one 
	 if {[string compare [$this cget -grism] $l_gris]} {
	     if {$debug} { puts stdout "$m_mb.layout.gr invoke *$l_gris*" }
	     $m_mb.layout.gr invoke *$l_gris* 
	     # $this configure -grism  $l_gris 
	     if {$debug} { puts stdout "$m_mb.layout.gr invoke *$l_gris*" }
	 }
     }

     if {$f_filt} {
	 set app2 ""
	 # change only if new filt != old filt
	 # puts stdout ">[$this cget -filter]< >$l_filt<"
	 if {[string compare [$this cget -filter] $l_filt]} {
	     # entryconfigure which submenue ?
	     # note we have layout.fi, fi.m2, fi.m3, fi,m4 fi.m5
	     # first search the submenue (ekelhaft)
	     # then configure
	     foreach append {"" .m2 .m3 .m4 .m5} {
		 for {set i 0} {$i <= 9} {incr i 1} {
		     set local [$m_mb.layout.fi$append entrycget $i -label]
		     if {[string compare $local $l_filt]==0} {
			 set app2 $append
                     }
		 }
	     }
	     $m_mb.layout.fi$app2 invoke *$l_filt*
           }
       }

     if {$f_coll} { 
	 # change only if different 
	 if {[string compare [$this cget -collimat] $l_coll]} {
	     if {$debug} { puts stdout "$m_mb.layout.sh invoke *$l_coll*" }
	     $m_mb.layout.sh invoke *$l_coll* 
	     if {$debug} { puts stdout "$m_mb.layout.sh invoke *$l_coll*" }
	 }
     }

      set ff [list $XSRMIN $XSRMAX]
      if {[string match *HR* [$this cget -collimat] ]} {
	  set ff [list $XHRMIN $XHRMAX] 
      }

     if {$mask_initialized==0} { 
	 if {$debug} { puts stdout "load_mos_TPX : Reset Mask" }
	 reset_mask
     }

     # set mask_initialized again 

     # again, since reset does reset MASK_NAID
     fors::MXU_SLIT::set_MASK_NAID [list  $L_prop $L_mid]
     # puts stdout "> [fors::MXU_SLIT::get_MASK_NAID]<"

     $m_image convert coords $mos(0,x0) $mos(0,y0) canvas mx my screen 
     # puts stdout "canvas $mos(0,x0) $mos(0,y0) screen $mx $my"
     mark $m_canvas $mx $my
     # puts stdout "Mark_Mask at screen $mx $my" 
     # $m_image convert coords [expr 15.0 *[DDMMSS2deg $lwx] ]\
     #       [DDMMSS2deg $lwy] "deg $EQX"  nwx nwy screen 
     set a1 [expr 15.0 *[DDMMSS2deg $lwx] ]
     set d1 [DDMMSS2deg $lwy]
     # puts stdout [format "deg $EQX: %10.3f %10.3f" $a1 $d1] 
     # wcs to canvas via bonsai matrix 
        set dum [C_wc w2c $a1 $d1]
        set cx [lindex $dum 0]
        set cy [lindex $dum 1]
     # puts stdout "canvas $cx $cy"
     $m_image convert coords $cx $cy canvas nwx nwy screen 
     # puts stdout "load_mos_TPX : Shift Mask"
     shift $m_canvas  $nwx $nwy 
     # puts stdout "load_mos_TPX : Shift Mask to canvas $cx $cy"

     # rotate mask 
     # in reset_mask abs_ang_rad is set to 0
     # puts stdout "rotangle=$lPA"
     $this configure -rotangle [ expr $lPA * $CSO ]
     set abs_ang_rad 0
     set PAM 0 
     $this rotate $m_canvas $mos(0,x0) $mos(0,y0) 1

     $this configure -rotangle 5
     # abs_ang_rad back in rad again 
     # puts stdout "load_mos_TPX : Rotate Mask $lPA"

    set lshap ""; set lwd 0.0; set lle 0.0; set lro 0.0

    set loc_x1 ""; set loc_x2 ""; set loc_x3 ""
    set loc_y1 ""; set loc_y2 ""; set loc_y3 ""
    set loc_REF  0
    # ------------------------------------------
    # step 2: load the same file again, read INS 
    #         and create mxu slits  
    # ------------------------------------------
    puts stdout "load_mxu_TPX : create mxu slits"
    # reset targets and reference targets 
    # check this:
    for {set i 1} { $i <= 40 } { incr i 1} {
        set targw($i,f) 0
        }

     if {[file exists $myfile] & [file readable $myfile]} {
        set fileId [open $myfile r ]
        puts $lfId "FORS::load_TARG  $fileId"
        # as long as there are lines in the file
        while {[gets $fileId line_org] >= 0} {
          if {$debug} { puts stdout $line_org; # list the .fims file  }
          if {[string match *$keyw2* $line_org]==1} {
            # INS 

            switch -glob -- $line_org {

     *NUMBER* { set s_c [lindex $line_org 1] 
                  if {$debug} { puts stdout "$s_c MXU slits in file" }
                }

     *ALPHA* { set lwx [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_RA [expr 15.0 * [ DDMMSS2deg $lwx] ] 
               if {$debug} { puts stdout "------A-$key-$loc_RA-" }
               # science slit numbers start with 101 
               # reference targets have no names 
               if {$key<100} { 
                  # reference target 
                  set targw($key,f) 1 
                  set targw($key,a) $loc_RA  
                  }
               }
           *DELTA* { set lwy [lindex $line_org 1] 
	      
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_DEC [DDMMSS2deg $lwy] 
               if {$debug} { puts stdout "------D-$key-$loc_DEC-" }
               # reference targets have no names 
               if {$key<100} { 
                  # reference target
                  set targw($key,f) 1 
                  set targw($key,d) $loc_DEC 
                  } else {

               # we assume that 'DELTA' is the last keyword for the 
               # slit read from the .fims file 
               # we can therefore create the slit here and now 
               # the next line read from the '.fims' will contain 
               # information for the next (=another) MXU slit 
               $m_image convert coords $loc_RA $loc_DEC "deg $EQX" x y canvas
               $m_image convert coords $loc_RA $loc_DEC "deg $EQX" px py image
               set noms [fors::MXU_SLIT::nos]

      # puts stdout ">$lshap< >$lwd< >$lle< >$lro<"
      # puts stdout ">$loc_x1< >$loc_y1< >$loc_x2< >$loc_y2< >$loc_x3< >$loc_y3<"

     # this is necessary for calc_zemax
     set targw(1,f) 1
     set targw(1,x) $px
     set targw(1,y) $py
     $m_image convert coords $px $py image targw(1,a) targw(1,d) "wcs $EQX"

     if {$noms==$mnomxus} {
        info_dialog "Error: Cannot handle more than\n $mnomxus MXU slits" .skycat1
        return
        }

      #
      # Checking that slit is well positioned (inside the focal field)

      set inside_mask_flag 0; # default outside the focal field
      set a [$m_canvas find overlapping \
	    [expr $x-1] [expr $y-1] \
	    [expr $x+1] [expr $y+1] ]


      foreach c $a {
	 set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
	 if {[string compare [lindex $local_tags 0] A]==0} {
		set inside_mask_flag 1
	 }
      }      


	if {$inside_mask_flag} {
	 #  puts stdout "inside $loc_name"
	} else {
	   #info_dialog "The mask $loc_name is outside!!! \n This will skipped !!"
	   puts stdout "The mask $loc_name is outside!!! \n This will skipped !!"
	   continue
 	}

	if {$loc_REF == 0} {
               set a [fors::MXU_SLIT C_mxus[incr noms 1] \
                     $m_canvas \
                     $m_image  \
                     $x $y \
                     -eqx $EQX \
                     -target_name $loc_name \
                     -slit_width  $lwd \
                     -slit_length $lle \
                     -pos_ang     $lro  \
		     -ref_slit    $loc_REF \
                     -shape $lshap \
                     -ff $ff \
                     -ON 1 \
                     -child $this ]


               if {[string match *CURVED* $lshap]} {
                  C_mxus$noms move_curve \
                  [list $loc_x1 $loc_y1 $loc_x2 $loc_y2 $loc_x3 $loc_y3]
                  }
	      }
      # for the next slit 
      set loc_REF 0

      switch -exact -- $lshap { 
      STRAIGHT -
      CIRCLE {
           # optical correction, we just use targw 1
           calc_zemax 1 $px $py 
           C_mxus$noms put_OC  [list $targw(1,ca) $targw(1,cd)]
           # apply correction of differential refraction (10.5.99)
           # input slit position, corrected for optical distortion
             if {$dire_flag} {calc_diff_refrac 1 $targw(1,ca) $targw(1,cd)}
             C_mxus$noms put_OC  [list $targw(1,ca) $targw(1,cd)]
           }
       CURVED {
         # correct the loaded bezier points for optical distortion
         update_bez_oc $noms

      #  set a [C_mxus$noms get_curve_ca]
      #  set targw(1,a) [C_mxus$noms get_RA]
      #  set targw(1,d) [C_mxus$noms get_DEC]

      #  $m_image convert coords [lindex $a 0] [lindex $a 1] canvas \
      #           targw(1,x) targw(1,y) image 
      #  calc_zemax 1 $targw(1,x) $targw(1,y)
      #  C_mxus$noms put_b_oc 1 [list $targw(1,cx) $targw(1,cy)]

      #  $m_image convert coords [lindex $a 2] [lindex $a 3] canvas \
      #           targw(1,x) targw(1,y) image 
      #  calc_zemax 1 $targw(1,x) $targw(1,y)
      #  C_mxus$noms put_b_oc 2 [list $targw(1,cx) $targw(1,cy)]

      #  $m_image convert coords [lindex $a 4] [lindex $a 5] canvas \
      #           targw(1,x) targw(1,y) image 
      #  calc_zemax 1 $targw(1,x) $targw(1,y)
      #  C_mxus$noms put_b_oc 3 [list $targw(1,cx) $targw(1,cy)]

      #  $m_image convert coords [lindex $a 6] [lindex $a 7] canvas \
      #           targw(1,x) targw(1,y) image 
      #  calc_zemax 1 $targw(1,x) $targw(1,y)
      #  C_mxus$noms put_b_oc 4 [list $targw(1,cx) $targw(1,cy)]

      #  $m_image convert coords [lindex $a 8] [lindex $a 9] canvas \
      #           targw(1,x) targw(1,y) image 
      #  calc_zemax 1 $targw(1,x) $targw(1,y)
      #  C_mxus$noms put_b_oc 5 [list $targw(1,cx) $targw(1,cy)]

      #  $m_image convert coords [lindex $a 10] [lindex $a 11] canvas \
      #           targw(1,x) targw(1,y) image 
      #  calc_zemax 1 $targw(1,x) $targw(1,y)
      #  C_mxus$noms put_b_oc 6 [list $targw(1,cx) $targw(1,cy)]

           # apply correction of differential refraction (10.5.99)
           # input slit position, corrected for optical distortion
           #  if {$dire_flag} {calc_diff_refrac 1 $targw(1,ca) $targw(1,cd)}
           #  C_mxus$noms put_OC  [list $targw(1,ca) $targw(1,cd)]
              }
	  }

             # C_mxus$noms status
      }
  }

           *WID* { set lwd [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_W $lwd
               if {$debug} { puts stdout "------W-$key-$loc_W-" }
               }
           *ROT* { set lro [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_RO $lro
               if {$debug} { puts stdout "------R-$key-$loc_RO-" }
               }
           *SHAPE* { set lshap [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_SHAP $lshap
               if {$debug} { puts stdout "------S-$key-$loc_SHAP-" }
               }
           *REF* { set lref [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               if {$lref} { set loc_REF 1 }
               if {$debug} { puts stdout "------R-$key-$loc_REF-" }
               }
          *X1* { set x1 [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_x1 $x1
               if {$debug} { puts stdout "------X-$key-$x1-" }
               }
          *X2* { set x2 [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_x2 $x2
               if {$debug} { puts stdout "------X-$key-$x2-" }
               }
          *X3* { set x3 [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_x3 $x3
               if {$debug} { puts stdout "------X-$key-$x3-" }
               }
          *Y1* { set y1 [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_y1 $y1
               if {$debug} { puts stdout "------Y-$key-$y1-" }
               }
          *Y2* { set y2 [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_y2 $y2
               if {$debug} { puts stdout "------Y-$key-$y2" }
               }
          *Y3* { set y3 [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_y3 $y3
               if {$debug} { puts stdout "------Y-$key-$y3-" }
               }
           *LEN* { set lle [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_LEN $lle
               if {$debug} { puts stdout "------L-$key-$loc_LEN-" }
               }
            *NAME* { set lna [lindex $line_org 1] 
               set key [lindex $line_org 0]
               set key [split $key "."] 
               set key [lindex $key 1]
               set key [string trimleft $key "TARG"] 
               set loc_name $lna
               if {$debug} { puts stdout "------N-$key-$loc_name-" }
               }
              default { puts stdout "------X->$line_org" }
	  }
      }
  }
     close $fileId

     set s_c 0
     set result {ok}
     # for {set i 1} { $i < 20} { incr i 1} {
     #     puts stdout [format "%2d %10s %10.3f %10.3f" \
     #     $i $targw($i,f) $targw($i,a) $targw($i,d)]
     #     }

     # --------- reference targets 
     C_ref_targ reset
     for {set i 31} { $i <= 40} { incr i 1} {
        if {$targw($i,f)} {
        catch { #puts stdout "reference target : $i  $targw($i,a) $targw($i,d)" 
                $m_image convert coords $targw($i,a) $targw($i,d) "deg $EQX" \
                           x y "wcs $EQX"
                 C_ref_targ put $x $y
                 C_ref_targ plot $mos(0,r); # delete all and plot all
               }
            }
	}

     if {[C_ref_targ nort]} { puts stdout "[C_ref_targ nort] Reference Targets found"
                      } else {
                      puts stdout "->->-> Warning No Reference Targets found"
                      }

     # C_ref_targ show
     # --------- reference targets end

     # change to standard mask modus 
     # you loaded a science setup baes on a technical mask 
     if {$L_mid>900000 && $L_mid<=900100} { 
        fors::MXU_SLIT::set_mmff 1
        puts stdout "tech_mask = >[fors::MXU_SLIT::get_mmff]<"
       
        }


     # targw locally as deg, but when move_slit is called it is 
     # in wcs again
     # for {set i 1} { $i < 20} { incr i 1} {
     #  if { $targw($i,f) != 0} {
     #   # move_slit requires screen coords
     #   $m_image convert coords $targw($i,a) $targw($i,d) "deg $EQX" mx my screen
     #   puts stdout [format "%2d-->%s (%11.7f,%11.7f)(deg $EQX) \
     #             (%8.3f,%8.3f)(screen) "\
     #             $i $targw($i,f) $targw($i,a) $targw($i,d) $mx $my ]
     #   set targw($i,f) 0
     #             #                       select,  not manually 
     #   if { [$this move_slit $m_canvas $mx $my 1 0] != ""} {
     #      incr s_c
     #      # puts stdout $s_c
     #      }
     #      }
     #   }

    puts stdout "+----------------------------+"
    puts stdout [ format "%2d MXU slits positioned" [fors::MXU_SLIT::nos] ]
    puts stdout "+----------------------------+"
    update_and_replot
    }
   set last [string length $myfile] 
   set first [expr [string last "/" $myfile]  +1]
   set last_load_fn [string range $myfile $first [expr $last - 6] ]

   $this configure -file_name [lindex [split $last_load_fn "."] 2]

   catch {SP update }
}



#=====================================
private method DDMMSS2deg { ddmmss } {
#=====================================
     # RA,DEC format conversion 
     # from  ddmmss.xxx to dd.xxx
     # like -703514.123 (means -70 deg 35 arcmin 14.123 arcsec)
     # to -70.5872563 
     # this routine works also for hour angles
     # but the factor of 15 must be corrected outside this proc
     # this proc is very primitive, it works char by char 

     # split by floating point
     set a [split $ddmmss "." ]
     set a2 .[lindex $a 1]
     set a2 [expr $a2/3600]

     set a1 [lindex $a 0]
     set l [string length $a1]

     set a11 [string index $a1 [expr $l-1] ]
     set a12 [string index $a1 [expr $l-2] ]
     set a11 [expr ($a11 + 10.0*$a12) / 3600.0]

     set a21 [string index $a1 [expr $l-3] ]
     set a22 [string index $a1 [expr $l-4] ]
     set a21 [expr ($a21 + 10.0*$a22) / 60.0]

     set a31 [string index $a1 [expr $l-5] ]
     set a32 [string index $a1 [expr $l-6] ]
     set a31 [expr ($a31 + 10.0*$a32) ]

     set  a31 [expr $a31 + $a21 + $a11 + $a2]
     if {[string compare [string index $ddmmss 0] "-"] ==0} { 
        set a31 [expr $a31 * -1 ] 
        }
     return $a31
     }

#=============================
private method F2G { value } {
#=============================            
     # format conversion for floating point 
     # from  xxx.yyy to xxxyyy to be used in MXU 
     # mode for gerber (rs274x) format  
     # X33Y33
     set value [format "%3.3f" $value]
     set value [split $value "."]
     set value "[lindex $value 0][lindex $value 1]"
     # puts stdout ">$value<"
     return $value
     }


     #========================
     public  method adjust_slit_length {} {
     #========================
     # reduce the mos slit length by s_slit in mm
     # call this after load_instrument but before init_mask_world2
     # transpose slit_length from '' to mm and adjust mosslit...
     # dont call this when -slit_length = real 
     if {[string compare [$this cget -modus] "mos"]==0} {
     if {[string compare $slit_length "real"]} {
        # mos, not real 
     for { set i 2} { $i <19} { incr i 1} {
         set delta [expr ($mosslitmax($i) - $mosslitmin($i)  \
            -$slit_length/$focscale) /2]
         set mosslitmin($i) [expr $mosslitmin($i)+$delta]
         set mosslitmax($i) [expr $mosslitmax($i)-$delta]
         }
         # special handling of the rim blades 
         set delta [expr ($mosslitmax(1) - $mosslitmin(1)  \
            -($slit_length-12.0)/$focscale) /2]
         set mosslitmin(1)  [expr $mosslitmin(1)+$delta]
         set mosslitmax(1)  [expr $mosslitmax(1)-$delta]
         set delta [expr ($mosslitmax(19) - $mosslitmin(19)  \
            -($slit_length-12.0)/$focscale) /2]
         set mosslitmin(19) [expr $mosslitmin(19)+$delta]
         set mosslitmax(19) [expr $mosslitmax(19)-$delta]
      } else {
        puts stdout "adjust_slit_length: Error 'real' cannot be handeled" 
        }
     } else {
     # we assume this can only be modus pmos
     if {[string compare $slit_length "real"]} {
        # pmos, not real 
     for { set i 2} { $i <20} { incr i 2} {
         set delta [expr ($mosslitmax($i) - $mosslitmin($i)  \
            -$slit_length/$focscale) /2]
         set mosslitmin($i) [expr $mosslitmin($i)+$delta]
         set mosslitmax($i) [expr $mosslitmax($i)-$delta]
         }
         # the rim blades are closed, no special handling 
      } else { 
        puts stdout "adjust_slit_length: Error 'real' cannot be handeled"
        }
      }
     }

     #========================
     private method erase {} {
     #========================
     $m_canvas delete $this -tag A
     $m_canvas delete $this -tag D
     $m_canvas delete $this -tag REFTARG
     }

     #==================================
     private method set_scale { flag } {
     #==================================
     # new philosophy since 13/9/1999
     # the focscale (''/mm_focalplane) is only dependent on the VLT unit
     # $focscale is a constant for FORS1 (and FORS2) as long as 
     # the instrument stays at the VLT UnitX.
     # Moving the FORS camera (due to different filters,  temperature and 
     # seasonal air pressure at Paranal) changes the (''/pixel) scale, 
     # Hence changes the $CCDscale_standard and $CCDscale_highres 

     if {$flag} { 
        # take the ''/pixel scale from the FoRS fits image,
        # since the nominal scale might not be appropriate
        # the real scale is dependent on the instrument
        # temperature and the BB filter
        set fisc [lindex [$m_image wcsset] 2]
	if {"$fisc" == ""} {set fisc 0} 
        # puts stdout ">>>$fisc<<<"
        set frac_s [expr $fisc / $CCDscale_standard ]
        set frac_h [expr $fisc / $CCDscale_highres  ]
        if {abs($frac_s-1.0)<abs($frac_h-1.0)} {
           # its the scale of the standard resolution collimator
           set frac $frac_s
           } else {
           # its the scale of the highres collimator
           set frac $frac_h
           }

       # we have to think about this 
       #set CCDscale_highres  [expr $CCDscale_highres  * $frac]
       #set CCDscale_standard [expr $CCDscale_standard * $frac]

       # focscale is dependent on the telescope unit
       # Maintenence: If FORS1 is moved from UNIT -> UNIT4 
       # edit the '1' branch    
       switch -exact -- [string index [$this cget -instrument] 4] {
          1 { set focscale $focscale_UT2 
              puts stdout "FORS1 at UT2" 
            }
          2 { set focscale $focscale_UT1 
              puts stdout "FORS2 at UT1" 
            }
      default { puts stdout "Error in telescope focal length" }
           }
          
       # set focscale [expr $focscale * $frac]
     if {[string compare [$this cget -modus] mxu]==0} {
       fors::MXU_SLIT::fc $focscale
       }
       # puts stdout [format \
       #      "FITS SCALE : %8f''/p = %8e o/p \
       #                    %8f''/p = %8e o/p \
       #                    %8.6f''/mm ff %e" \
       #             $CCDscale_standard [expr $CCDscale_standard/3600.0] \
       #             $CCDscale_highres  [expr $CCDscale_highres/3600.0] \
       #             $focscale $frac]
       } else {
       # Fits image not from FORS
       # use the nominal values of ''/pixel 
       # there is no nominal value of ''/mm
       # focscale is dependent on the telescope unit
       # the CCD_scale... is not so important, since it 
       # does only manage the wavelength borders

       # Maintenence: If FORS1 is moved from UNIT -> UNIT4 
       # edit the '1' branch    
       switch -exact -- [string index [$this cget -instrument] 4] {
          1 { set focscale $focscale_UT2 
              set CCDscale_highres  0.1 
              set CCDscale_standard 0.2 
              puts stdout "FORS1 at UT2" 
            }
          2 {
	     set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"]
             if {$binx == 1} {
		 set PIX 0.015
		 set CCDscale_standard 0.125062
                 set CCDscale_highres 0.0625313
	     } else {
	         set PIX 0.030
	         set CCDscale_standard 0.250124
	         set CCDscale_highres  0.1250626
		
	     }
	      set focscale $focscale_UT1              
              puts stdout "FORS2 at UT1" 
            }
      default { puts stdout "Error in telescope focal length" }
           }

     if {[string compare [$this cget -modus] mxu]==0} {
       fors::MXU_SLIT::fc $focscale
       }
       puts stdout [format \
            "NOMI SCALE : %8f''/p = %8e o/p \
                          %8f''/p = %8e o/p \
                          %8.6f''/mm ff" \
                   $CCDscale_standard [expr $CCDscale_standard/3600.0] \
                   $CCDscale_highres  [expr $CCDscale_highres/3600.0] \
                   $focscale]
       }
    }


     #=============================
     private method update_mask_na {} {
     #=============================
             set na M[format "%03d" $exposure_counter][$this cget -file_name]
             set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
             catch { fors::MXU_SLIT::set_MASK_NAID "$na [lindex $PM 1]" }
             catch {SP update} 
             }

     #=============================
     private method update_mask_id {} {
     #=============================
             set old_id [exec date -u +%H%M%S]
             set l [string length $old_id]
             set id "9"
             for { set i 1 } { $i < $l } { incr i 1 } {
                 append id [string index $old_id $i]
                 }

             if {$id < 900100} { 
                # oh, its accidently a (forbidden) technical mask number
                set id [expr $id + 1000]
                }

             set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
             catch { fors::MXU_SLIT::set_MASK_NAID "[lindex $PM 0] $id" }
             catch {SP update} 
             }

     #=============================
     private method update_mask_naid {} {
     #=============================
        # this is called when a project name is specified 
        set na M[format "%03d" $exposure_counter][$this cget -file_name]


        set old_id [exec date -u +%H%M%S]
        set l [string length $old_id]
        set id "9"
        for { set i 1 } { $i < $l } { incr i 1 } {
            append id [string index $old_id $i]
            }

        if {$id < 900100} { 
           # oh, its accidently a (forbidden) technical mask number
           set id [expr $id + 1000]
           }


        catch { fors::MXU_SLIT::set_MASK_NAID "$na $id" }
        catch {SP update} 
        }

     #=============================
     public  method reset_mask {} {
     #=============================
     # reset the mask
     # telescope pointing in [canvas]
     # puts $lfId "Reset_Mask:  m_image=>>$m_image<< \

     # special event for lss1 period 64 
     # remove this for period 65 
     if {[string compare [$this cget -modus] "lss"] == 0} {
        if {[string index [$this cget -instrument] 4]==1} {
           C_ref_targ reset
          }
       }

     # update mask value
     # update the mask mode
     if {[string compare [$this cget -modus] "ipol"] == 0} {
        if {[string match *HR* [$this cget -collimat] ]} {
           set mask pol 
           # modus=ipol mask=pol 
           } else { 
           set mask mos 
           # modus=ipol mask=mos
           }
           }

     # in case a new image is loadad
     # mask center x_ff=0,y_ff=0 is center of frame 
     set world_x [$m_image fits get CRVAL1] 
     set world_y [$m_image fits get CRVAL2]

    # frameless mode
    # maybe better a string length operation 
    if { $world_x == "" || $world_y == "" } {
       set wx [lindex [$m_image wcsset] 0]
       set wy [lindex [$m_image wcsset] 1]
       $m_image convert coords $wx $wy "wcs $EQX" world_x world_y "deg $EQX"
       }

     # write Warning if FORS image is processed by reduction software
     # for status panel 
     set sp_instrume [$m_image fits get INSTRUME]
     set sp_origin   [$m_image fits get ORIGIN]

  #  if {[string match *FORS* $sp_instrume ]} {
  #  if {[string compare ESO $sp_origin  ]} {
  #     puts stdout "Warning: FITS file observed with $sp_instrume\
  #                  but processed by $sp_origin"
  #     }
  #     }

     set wx $world_x; set wy $world_y
     set ccx [$m_image fits get CRPIX1]
     set ccy [$m_image fits get CRPIX2]
     # puts stdout [ format "reset_mask : CCD REF_PIX %10e %10e %7.2f %7.2f" \
     #        $wx $wy $ccx $ccy] 
     
     # delete all instances of MXU slit 
     if {[string compare [$this cget -modus] mxu]==0} {
        set noms [fors::MXU_SLIT::nos]
        # puts stdout "old number of mxu slits = $noms"
        if {$noms>0} {
        for {set i 1} {$i <= $noms} { incr i 1} { 
            delete object C_mxus$i 
            }
         }
        set noms [fors::MXU_SLIT::nos]
        # puts stdout "new number of mxu slits = $noms"
        fors::MXU_SLIT::set_mmff 0

        }

     [code $this init_mask_world2 ]


     catch {$m_canvas delete A}
     catch {$m_canvas delete D}
     plot_mask 

     # call echs_pos and echs targ after masdk had been 
     # plotted
     if {[string compare [$this cget -modus] echs]==0} {
        echs_pos
        echs_targ
        }

     # badpixelmap
     if {$badpixelmap} {
         fors::CONV_MC::coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
             $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
             $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
        update_BPM
        }

     if {[string match "*mxu*" [$this cget -modus]]} { 	 
        load_mxu_refslit
        }
     set last_action "Reset Mode"
 
     catch {SP update} 
     }


     #=============================
     public  method update_and_replot {} {
     #=============================
     # do not reset the mask but replot the mask 
     # at the current position with current configuration 
     # options but with different image zoom, flip option ...
     # and plot_mask it 
     # scratch the whole old mask and plot_mask  
          
     update_color
     update_scale
     update_mask
     catch {$m_canvas delete A}
     catch {$m_canvas delete D}
     # badpixelmap
     if {$badpixelmap} {
         fors::CONV_MC::coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
             $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
             $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
        update_BPM
        }
     return [plot_mask]
     }

     #=============================
     public  method demo {} {
     #=============================
     # A short demo, but should be updated
     # reset the mask 
     # there must be already a mask 

        set w .skycat1.dm
        catch {destroy $w}
        set dw [DialogWidget $w \
           -text "Demo ? \n  1-5 min" \
           -bitmap warning \
           -default -1 \
           -buttons {START STOP} ]
        if { [$dw activate] } return 

     puts stdout "Demo Start ..."

      if {[string match echs [$this cget -modus] ] } {
             # $this configure -grism  XGRIS_600B+92.EGRIS_110B+98.FILT_465_250+82
             $m_mb.layout.gr invoke 10
             # puts stdout ">grims>>>>"
          }


     # delete everything and reset current mask 
     catch {$m_canvas delete A}
     reset_mask

     puts stdout "after rseet"

     # FORS1 f2=0, FORS2 f2=1
     # if {$f2}
     set f2 [expr [string index [$this cget -instrument] 4] -1]



     $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x y screen
     # move slits needs screen coords

     # first step move slit blades in MOS and IMG 

     
     puts stdout "Demo 1 all : set collimator"
     # --------------------------------------
     if {$f2} {
              $this configure -collimat COLL_SR+6 
              } else {
              $this configure -collimat COLL_SR+1 
              }

     puts stdout "after collimator" 

     puts stdout "Demo 2 set grism" 
     # -----------------------------
     switch -exact -- [$this cget -modus] {
     mos -
    pmos -
     lss -
     hits -
     mxu {
     if {$f2} {
              $this configure -grism GRIS_600B+22 
              $this configure -filter none 
              wave_config_mask [list 360.0 450.0]
              } else {
              $this configure -grism GRIS_600B+12 
              $this configure -filter none 
              wave_config_mask [list 360.0 450.0]
              }
         }
      echs { # was already set before reset_mask
           }
      }

     switch -exact -- [$this cget -modus] {
    pmos -
     mos {
         puts stdout "Demo 1 mos : move slits"
         # ----------------------------------
     for {set i 1} { $i < 25} { incr i 1} {
         set xn [expr $x + [random 250]]
         set yn [expr $y + [random 250]]
         catch {move_slit $m_canvas $xn $yn 1 0}; update; after 1
         set xn [expr $x - [random 250]]
         set yn [expr $y - [random 250]]
         catch {move_slit $m_canvas $xn $yn 1 0}; update; after 1
         set xn [expr $x + [random 250]]
         set yn [expr $y - [random 250]]
         catch {move_slit $m_canvas $xn $yn 1 0}; update; after 1
         set xn [expr $x - [random 250]]
         set yn [expr $y + [random 250]]
         catch {move_slit $m_canvas $xn $yn 1 0}; update; after 1
         }
     }
    mxu {
         puts stdout "Demo 1 mxu : create slits"
         # ----------------------------------
         $this configure -slit_length 6
         $this configure -slit_width 2 
         $this configure -mxu_shape STRAIGHT 

     for {set i 1} { $i < 60} { incr i 1} {
         set xn [expr $x + [random 210]]
         set yn [expr $y + [random 250]]
         $m_image convert coords $xn $yn screen  xc yc canvas
         set a [$m_canvas find overlapping \
               [expr $xc-1] [expr $yc-10] \
               [expr $xc+1] [expr $yc+10] ]

         set go 1
         foreach c $a {
          set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
          if {[string match *mm* $local_tags] } {
             # position occupied
             set go 0 }
            } 

         if {$go } { catch { mxu_create_slit $m_canvas $xn $yn 0} 
            update; after 1 }
         }

     for {set i 1} { $i < 60} { incr i 1} {
         set xn [expr $x + [random 210]]
         set yn [expr $y - [random 250]]
         $m_image convert coords $xn $yn screen  xc yc canvas
         set a [$m_canvas find overlapping \
               [expr $xc-1] [expr $yc-10] \
               [expr $xc+1] [expr $yc+10] ]

         set go 1
         foreach c $a {
          set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
          if {[string match *mm* $local_tags] } {
             # position occupied
             set go 0 }
            } 

         if {$go } { catch { mxu_create_slit $m_canvas $xn $yn 0} 
            update; after 1 }
         }
         $this configure -mxu_shape CIRCLE

     for {set i 1} { $i < 80} { incr i 1} {
         set xn [expr $x - [random 210]]
         set yn [expr $y - [random 250]]
         $m_image convert coords $xn $yn screen  xc yc canvas
         set a [$m_canvas find overlapping \
               [expr $xc-1] [expr $yc-1] \
               [expr $xc+1] [expr $yc+1] ]

         set go 1
         foreach c $a {
          set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
          if {[string match *mm* $local_tags] } {
             # position occupied
             set go 0 }
            } 

         if {$go } { catch { mxu_create_slit $m_canvas $xn $yn 0} 
            update; after 1 }
         }

     for {set i 1} { $i < 20} { incr i 1} {
         set xn [expr $x - [random 210]]
         set yn [expr $y + [random 250]]
         $m_image convert coords $xn $yn screen  xc yc canvas
         set a [$m_canvas find overlapping \
               [expr $xc-1] [expr $yc-1] \
               [expr $xc+1] [expr $yc+1] ]

         set go 1
         foreach c $a {
          set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
          if {[string match *mm* $local_tags] } {
             # position occupied
             set go 0 }
            } 

         if {$go } { catch { mxu_create_slit $m_canvas $xn $yn 0} 
            update; after 1 }
         }
        }
    ipol -
    img {
         puts stdout "Demo 1 img : move slits"
         # ----------------------------------- 
     for {set i 1} { $i < 60} { incr i 1} {
         set xn [expr $x + [random 500]]
         set yn [expr $y + [random 500]]
         catch {move_slit_blade $m_canvas $xn $yn [random 2] }
         update; after 10
         set xn [expr $x - [random 250]]
         set yn [expr $y - [random 250]]
         catch {move_slit_blade $m_canvas $xn $yn [random 2] }
         update; after 10
         }
     }

    hits -
    lss -
    mxu -
    echs -
    default {
            puts stdout "Demo 1 [$this cget -modus]: no mos slits move"
            }
    } 



     puts stdout "Demo 2 all : colors and tranparency:"
      #-------------------------------------------------

      $this configure -label 1;                   # 10
       $this configure -graytype 0 ;              # 10

     switch -exact -- [$this cget -modus] {
    pmos -
     img -
     echs -
     mos {
     foreach col {Red/Yellow Black/White Violet/Orange \
                  Gray Green/Red Bright Dark Red/Yellow} {
         $this configure -colorset $col

         if {[string match echs [$this cget -modus] ] } { flip_echs }

         update_and_replot; update; after 1000 
     # graytype 
       $this configure -graytype 1 ;              # 11
       update_and_replot; update; after 1000 
       $this configure -label 0 ;                 # 01   
       update_and_replot; update; after 1000 
       $this configure -graytype 0 ;                 # 00   
       update_and_replot; update; after 1000 
       $this configure -label 1;                    # 10  
       update_and_replot; update; after 1000 
      }
     }

    default {
       # all modi 
     foreach col {Red/Yellow Black/White Violet/Orange \
                  Gray Green/Red Bright Dark Red/Yellow} {
         $this configure -colorset $col
         update_and_replot; update; after 1000 
       $this configure -label 0 
       update_and_replot; update; after 1000 
       $this configure -label 1 
       update_and_replot; update; after 1000 
     }
            }
     }


if {[string match echs [$this cget -modus] ] } { flip_echs }
        
     
# ---------------------------------
#  SLIT WIDTH
# ---------------------------------

     switch -exact -- [$this cget -modus] {
     pmos -
      echs -
      mxu -
      mos {
          puts stdout "Demo 3 mos : slit widths"
          # ------------------------------------
     set xn [expr $x + 20]; set yn $y 
     # MOS slit_width only in MOS or PMOS mode 
     #puts stdout "Slit Width"

     for {set i 51} {$i >= 1} {incr i -5 } {
         # puts stdout "slit width = [expr (1.0*$i)]''"
         $this configure -slit_width  [expr (1.0*$i)] 
         update_and_replot; 
         # move_slit $m_canvas $xn $yn 1
         update; after 1
         }
     for {set i 1} {$i <= 10} {incr i 1 } {
         # puts stdout "slit width = [expr (0.1*$i)]''"
         $this configure -slit_width  [expr (0.1*$i)] 
         # update_and_replot; 
         # move_slit $m_canvas $xn $yn 1 0
         update; after 1
         }
      $this configure -slit_width  5 
      update_and_replot; 
      #puts stdout "..end"
      

       if {[string compare echs [$this cget -modus] ]} {
          # no slit length demo for ECHS

          puts stdout "Demo 3 mos : slit length"
           # -----------------------------------
     # slit_length  only in MOS mode 
     for {set i 20} {$i >= 5} {incr i -5} {
         $this configure -slit_length $i       
         adjust_slit_length; reset_mask; update; after 1 
         }
     for {set i 5} {$i <=20}  {incr i 5} {
         $this configure -slit_length $i       
         adjust_slit_length; reset_mask; update; after 1 
         }
         $this configure -slit_length 22       
         adjust_slit_length; reset_mask; 

         }
 
       }

     lss {
          puts stdout "Demo 3 lss : slit width"
          # ------------------------------------
         
   foreach i {all 2.5 1.6 1.0 0.5 0.3 0.4 0.7 1.3 2.0 all 2.5} {
       $this configure -long_width $i       
       switch -exact -- [$this cget -long_width] {
          all {set long_number 0 }
          0.3 {set long_number 1 }
          0.4 {set long_number 2 }
          0.5 {set long_number 3 }
          0.7 {set long_number 4 }
          1.0 {set long_number 5 }
          1.3 {set long_number 6 }
          1.6 {set long_number 7 }
          2.0 {set long_number 8 }
          2.5 {set long_number 9 }
        default {puts stdout "Warning: LSS slit" }
           }
         # puts stdout "$long_number $i [$this cget -long_width] "
         reset_mask; update; after 2000 
         }
     }

    hits {

         puts stdout "Demo 3 hits: slit width"
         # ------------------------------------
   foreach i {all 1.6 1.0 0.5 PH 0.7 1.3 2.0 all PH} {
       $this configure -long_width $i       
       switch -exact -- [$this cget -long_width] {
          all {set long_number 0 }
          0.3 {set long_number 1 }
          0.4 {set long_number 2 }
          0.5 {set long_number 3 }
          0.7 {set long_number 4 }
          1.0 {set long_number 5 }
          1.3 {set long_number 6 }
          1.6 {set long_number 7 }
          2.0 {set long_number 8 }
        default {puts stdout "Warning: HITS slit" }
           }
         # puts stdout "$long_number $i [$this cget -long_width] "
         reset_mask; update; after 2000 
         }
         }

     img {
         puts stdout "Demo 3 img : slit width TBD"
         # ---------------------------------------
         # move_slit_blade $w sx sy 1
         }
     ipol {
          if {[string match *SR* [$this cget -collimat] ]} { 
             puts stdout "Demo 3 ipol : slit width TBD"
             # ------------------------------------------
             }
          } 
     mxu -
    default {
            puts stdout "Demo 3 [$this cget -modus]: no mos slits width"
            }
      }



     puts stdout "Demo 4 all : rotate mask "
     # rotate mask 

     if {[string match img  [$this cget -modus]]} { reset_mask }
     if {[string match ipol [$this cget -modus]]} { reset_mask }


     # rotate mask 
     $this configure -rotangle 5
     for {set i 1} { $i < 14} { incr i 1} {
         $this rotate $m_canvas $mos(0,x0) $mos(0,y0) -1 
         update; after 1
         }
     $this configure -rotangle 1
     for {set i 1} { $i < 20 } { incr i 1} {
         $this rotate $m_canvas $mos(0,x0) $mos(0,y0) 1 
         update; after 1
         }
     $this configure -rotangle 5


     puts stdout "Demo 5 all : move mask "
     #  move mask
     $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x y screen
     mark $m_canvas $x $y
     for {set i 1} { $i <90} { incr i 4} {
         set a [expr 1.0 * $i]
         update; after 1
         set xn [expr $x + 40*cos($a*0.0174532)]
         set yn [expr $y + 40*sin($a*0.0174532)]
         catch {$this shift $m_canvas $xn $yn}
         }
      $this shift $m_canvas $x $y
      update_and_replot


     puts stdout "Demo 6 all : change collimator "
     # change collimator
     if {[string compare "pmos" [$this cget -modus]]} {

     for {set i 1} { $i < 3} { incr i 1} {

         $this configure -collimat [lindex [split $co_range " "] 1] 
         switch -exact -- [$this cget -modus] {
          mxu -
          hits -
          echs -
          lss -
          mos { update_and_replot; update; after 1000 }
         pmos -                       
          img -
         ipol { reset_mask; update_and_replot; update; after 1000 }
        default { puts stdout "Warning: modus"}
             }

         $this configure -collimat [lindex [split $co_range " "] 0] 
         switch -exact -- [$this cget -modus] {
          mxu -
          hits -
          echs -
          lss -
          mos { update_and_replot; update; after 1000 }
         pmos -
          img - 
         ipol { reset_mask; update_and_replot; update; after 1000 }
        default { puts stdout "Warning: modus"}
             }



         }

     #puts stdout "ok"
     } 

     $this configure -graytype 0 
     update; after 1000 

     puts stdout "Demo finished"
     return 0
     }  

     #============================
     public method load_TPX { filename } {
     #============================
     # the central mewthod to load a .fims file
     # if in MOS or PMOS the -slit_width == open, 
     # load an ims.  fims file  
     # puts stdout "load_TPX >$filename< [$this cget -modus]"
     switch -exact -- [$this cget -modus] {
         mos { 
             if {[string compare [$this cget -slit_width] open]==0} {
                load_img_TPX $filename
                } else {
                load_mos_TPX $filename
                }
             }
        pmos { 
             if {[string compare [$this cget -slit_width] open]==0} {
                load_img_TPX $filename
                } else {
                load_mos_TPX $filename
                }
             }
         img { load_img_TPX $filename }
        ipol { load_img_TPX $filename }
         lss { load_img_TPX $filename }
        hits { load_img_TPX $filename }
         mxu { load_mxu_TPX $filename }
        echs { load_mos_TPX $filename }
            }

     # for statuspanel
     set last_action "Load SetUp"
     set sp_nort [C_ref_targ nort];  # number of reference targets
     catch {SP update}

     #Verify reference_targets
     set fail_flag [edit_ref_targ verify_load]
     if {$fail_flag} {
     	 info_dialog "There are reference targets in the forbidden zone!!!." .skycat1
        }
     }

     #============================
     private method  get_cat {} {
     #============================
     # gets alll AstroCat widgets, send the 
     # list to select_cat and returns the selected 
     # catalog 
     set test1 [itcl::find objects ]
     set a [split $test1 " " ]
     set b [lsearch -glob $a *AstroCat*]
     set a [lindex $a $b]
     set b [split $a "::" ]
     set a [llength $b]
     set b [lindex $b [expr $a -1] ]
     set a [split $b "."]
     set gc ".[lindex $a 1].[lindex $a 2]"

     set gc [itcl::find objects -class ::skycat::SkySearch]
     set gc [split $gc " " ]
     set ll [llength $gc]

     if {$ll>1} { set gc [select_cat $gc] }
     return $gc
     }

     #============================
     private method  get_ec { mode } {
     #============================
     # get exposure counter 
     # scan setupfile directory with mode filter
     # set ec to the preceeding value 
     set local_dir "$set_dir/"
     if [ catch { set a [exec ls -1 $local_dir | grep $mode ] } result ] {
        puts "$result\nNo setup files found ; set expo-counter 1"
        set exposure_counter 1
        } else {
          set a [split $a \n ]
          set exposure_counter 0
          for {set i 0} {$i < [llength $a]} {incr i 1} {
              # puts stdout ">[lindex $a $i]<"
              set part [split [lindex $a $i] . ] 
              for {set j 0} { $j < [llength $part]} {incr j 1} {
                  set test [lindex $part $j]
                  if {[scan $test {%d} nr]} { 
                     if {$nr>$exposure_counter} { 
                        set exposure_counter $nr 
                     }
                  }
               }
           }
           incr exposure_counter 1
           }
      return $exposure_counter
      }

     #============================
     private method get_fits_name { } {
     #============================
     # get the 6 first chars of the fits file 
     # as an initial project name 
     set name [$m_image cget -file]
     if {$name==""} { set name "no_frame" }
     set a [split $name "/" ];                      # split by directory 
     set a [lindex $a [expr [llength $a] - 1 ] ];   # get last 
     set a [split $a "."];                          # split by dot
     set a [lindex $a 0 ];                          # get first
     }

     #============================
     public method save_TPX { } {
     #============================
     # central function for saving a fims setup file 
     # For multi object spectroscopic modes MOS, PMOS and MXU 
     # a catlog can be selected. slit positions will be compared
     # with catalog positons and the catalog target name will be stored 
     # together with the slit position information

     # initial file  name without extension : 'mos1.12.project'

     set filename [$this cget -modus][string index  [$this cget -instrument] 4].$exposure_counter.[$this cget -file_name]
     set M [ string toupper [$this cget -modus] ]
      

   # for mxu check first the number of reference slits 
   if {[string match mxu [$this cget -modus] ]} {
      # just calculate the number of reference slits
      set noms [fors::MXU_SLIT::nos]
      if {$noms>0} { set sp_rsc 0
         for {set i 1} {$i <= $noms} { incr i 1} {       
             if {[C_mxus$i cget -ON]} {
             if {[C_mxus$i cget -ref_slit]} { incr sp_rsc 1
             } } }
      # for statuspanel           
      catch {SP update}
      if {$sp_rsc<3} {
         info_dialog "Less than 3 MXU Reference Slits found\n\
                     Verificatin Failed !\n\
                     Setup not Saved !\n\n\
                     Click on the Slit Number Label\n\
                     (e.g. #4) close to the Slit to select\n\
                     as Reference Slit.\n\
                     LeftMouseBut=ON, MidMouseBut=OFF" .skycat1
         return
         } else {
#         info_dialog "$sp_rsc MXU Reference Slits found\n\
#                     Verification Ok\n\
#                     Reason : $sp_rsc > 2"
	     puts "$sp_rsc MXU Reference Slits found - Verification Ok"
         }
      }
    }

     # for all modes, check the number of reference targtes 
     set text "Save  : $filename\n\n" 
     set fail_flag [edit_ref_targ verify]
     if {$fail_flag} {
	 append text "Verify: Failed ! \n\n\Press  : \[Cancel\]"
	 set deff 1
        } else {
	    append text "Verify: Ok \n\n\Press  : \[Ok\]"
	    set deff 0
        }

     set wid .skycat1.save
     catch {destroy $wid}
     set w [DialogWidget $wid \
         -text $text \
         -bitmap warning \
         -default $deff \
         -buttons { Ok Cancel } ]

     # AAG - PPRS 10127
     # Edit was removed because of potential problems in long 
     # filenames

     set result [$w activate] 

     # activate expiration_date checking
     # set warn_flag [expire_date]

     if {$warn_flag && $result==0} { warn }

     if {$result==1} return; # Cancel

     puts stdout "\n store acquisition in -->$set_dir/$filename"

     switch -exact -- [$this cget -modus] {
        echs -
         mos { 
             set sc [get_cat] 
             if {$sc == ""} {
                puts stdout "No catalog active" 
                set CatFileName no_catalog
                } else { 
                catch { catalog $sc }
             }
             if {[string compare [$this cget -slit_width] open]==0} {
                set filename \
                    ims[string index [$this cget -instrument] 4].$exposure_counter.[$this cget -file_name] 
                save_img_TPX $filename
                } else {
                save_mos_TPX $filename
                }
             }
        pmos {
             set sc [get_cat] 
             if {$sc == ""} {
                puts stdout "No catalog active" 
                set CatFileName no_catalog
                } else { 
                catch {catalog $sc }
             }
             if {[string compare [$this cget -slit_width] open]==0} {
                set filename \
                   pims[string index [$this cget -instrument] 4].$exposure_counter.[$this cget -file_name] 
                save_img_TPX $filename
                } else {
                save_mos_TPX $filename
                }
             }
         img { save_img_TPX $filename }
        ipol { save_img_TPX $filename }
         lss { 
             if {[string compare [$this cget -long_width] all]==0} {
                info_dialog "You cannot save with\nSlit width = all" .skycat1
                return
                }

             # CAUTION CAUTION CAUTION
             # -----------------------
             # 8.Jul.99 : lss2 will use new alignment
             #            lss1 will still use the old alignment
             # use new alignment for lss1 when FORS1-OS has been 
             # updated ( ?? april 2000 ?? )
             # if {[string index [$this cget -instrument] 4]==1} {
             #       # old alignment for FORS1 lss
             #       save_lss_TPX $filename 
             #       } else {
             #       # new alignment for FORS2 lss
             #       save_hits_TPX $filename
             #       }

             save_hits_TPX $filename
             }
        hits {
             if {[string compare [$this cget -long_width] all]==0} {
                info_dialog "You cannot save with\nSlit width = all" .skycat1
                return
                }
             save_hits_TPX $filename
             }
         mxu { 

	     # find catalog name 

             set sc [get_cat] 
             if {$sc == ""} {
                puts stdout "No catalog active" 
                set CatFileName no_catalog
                } else { 
                catch {catalog $sc }
             }


                # saving files for the mxu
                # --------------------------
                # we save 4 different formats
                # adf is the aperture definition file (no longer used)
                # msf is the machine slit file  (no longer used)
                # rs274x is the gerberX file  (to be included in OBs)
                # TPX creates the well known fims file (to be used by FIMS)
                #
                # focal field coords of FORS2 (X,Y) have to be translated to 
                # MMU working table coords. The Transfornation is 
                # X_mmu = Y_ff
                # Y_mmu = -X_ff
                # why ? 
                # 1)
                # looking down (as a photon) we see a left-hand xy-system 
                # in the FORS2 focalplane 
                # while the mmu working table used a right-hand system. 
                # therefore x -> -x  y -> y 
                # 2) 
                # for mechanical reasons it is more suitable to cut 
                # the mask from the invar sheet when rotated by 90 degr
                # therefore y -> x, x -> y 
                

		# standard masks have not been target of 
		# correction of the optical distortion 
                # we only change RA,DEC but not mmff values 
                #  

	        fors::MXU_SLIT::set_mmff 1
                if {[fors::MXU_SLIT::get_mmff]} {
                   # prepare standard mask to be saved
                   # the wcs of all slits (in mm) have to be determined

                   puts stdout "standard mask:: prepare save"

                   # update canvas--mmff conversion
		  
                   fors::MXU_SLIT::coeff $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                                         $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                                         $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc) 
                   set noms [fors::MXU_SLIT::nos]
                   if {$noms>0} {
                   for {set i 1} {$i <= $noms} { incr i 1} { 		      
                       set refOut [C_mxus$i M2W]; # x,y -> xc,yc -> ra,dec in mxu  
		       if {$refOut} {
			   puts stdout "AAG PPRS 10127 - OutOfRange $i. Continuing"
			   #TSZ remove slit:
			   C_mxus$i erase
			   C_mxus$i configure -ON 0
			   continue	   
		       } else {
			   puts stdout "AAG PPRS 10127 - InRange $i."
			   set targw(1,a) [C_mxus$i get_RA]
			   set targw(1,d) [C_mxus$i get_DEC]
			   
			   #puts "RA: $targw(1,a)"
			   #puts "DEL: $targw(1,d)"
			   #$m_image convert coords [lindex $dum 0] [lindex $dum 1]  "wcs $EQX" \
			       #         targw(1,x) targw(1,y) screen 
			   $m_image convert coords $targw(1,a) $targw(1,d) "wcs $EQX" \
			       targw(1,x) targw(1,y) image		       	 		      	  		     
			   
			   calc_zemax 1 $targw(1,x) $targw(1,y)
			   # targw(1,cx) and targw(1,cy) is not used
			   C_mxus$i put_OC  [list $targw(1,ca) $targw(1,cd)]
			   
			   # apply correction of differential refraction (10.5.99)
			   # input slit position, corrected for optical distortion
			   if {$dire_flag} {
			       calc_diff_refrac 1 $targw(1,ca) $targw(1,cd)
			       C_mxus$i put_OC  [list $targw(1,ca) $targw(1,cd)]
			   }
		       }
		   }
		   }
	       }
	     fors::MXU_SLIT::set_mmff 0
	     # special curved slit handling 

                prepare_mxu_for_save
               

                # puts stdout "->ADF begin"
                # save_mxu_ADF $filename
                # puts stdout "->ADF end MSF begin"
                # save_mxu_MSF $filename
                # puts stdout "->MSF end GBR begin"

                # gerber file format for MASKID > 900100 can be saved 
                # without maintenance password  

            # handle mask ID <> 900100

                # 15.54.2000 greetings to jason and the DMD 
                # agreement: FIMS must generate a 
                # INS.MASK.NA = Mdddcccccccc
                # INS.MASK.ID = 9ddddd
                # INS.MASK.NAID = $INS.MASK.NA+$INS.MAK.ID
                # with c = single car and d = single digit
                # for technical masks the old concept is retained


                # if technical mask take naid from mxu class
                # if sky mask create new naid 
                if {[fors::MXU_SLIT::get_mmff]} {
                set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
                puts stdout "SAVE_TPX: technical mask >$PM<"
                } else {
                update_mask_id
                set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
                puts stdout "SAVE_TPX: sky       mask >$PM<"
                }

                # new concept: the user cannot specify mask ids, 
                # only technical masks have default mask id 
                set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
                set a [lindex $PM 1]; 
                # must be 900000 (default sky mask) or 
                # 900001-900100 (technical mask) 

                    if {$a < 900101 && $a > 900000} {
                       # technical mask 
                       # if there is no ins.mask.naid, the mask 
                       # is just newly defined by the staff astronomer
                       # otherwise, if the ins.mask.naid is known 
                       # since it has been read from the fad, do not 
                       # create a gerber file since this mask already 
                       # exists on Paranal and is available

                       if {[string compare [lindex $PM 0] "create"]==0} {
                          # maintenenace branch, mask not yet produced 
                       set wahrheit [get_calib_passwd]
                       if {!$wahrheit} {
                           puts stdout "Password ERROR: NO Gerber file created"
                           } else {
                           # password ok, save gerber file of technical mask
                           save_mxu_RS274X $filename
                           puts stdout "gerber file saved with mask id "

                           save_mxu_RS274X_forPS  $filename
                           # and save it as a postscript file 
       if [catch { 
          [exec $int_dir/bin/gerber2eps.perl $log_dir/$filename ] 
          } result] {
        puts stdout "Gerb2PS: $result"
        set a [exec rm $log_dir/$filename]
        } else {
        # the perl script always gives a message
        }
                           }
                         } else {
                         # save a setup with a technical mask 
                         # no gerber file required
                         # no postscript file required
                         puts stdout "Save technical mask without Gerber file" 
                         }

                       } else { 
                       # science mask with default mask id 
                       # outside 900000 .. 900100
                       save_mxu_RS274X $filename
                       puts stdout "gerber file saved with mask id "
                       save_mxu_RS274X_forPS  $filename

                       # in case of problems with perl, go on !     

       if [catch { 
          [exec $int_dir/bin/gerber2eps.perl $log_dir/$filename ] 
          } result] {
        puts stdout "Gerb2PS: $result"
        set a [exec rm $log_dir/$filename]
        } else {
        # the perl script always gives a message
        }

        }
                  
                # TPX must be the last, since it 
                # increments the exposure counter

                save_mxu_TPX $filename
             }

            }

     set last_action "Save SetUp"

     set last_save_fn $filename
     catch { SP update }
     # open a new log file
     open_log 
     }

     #=============================
     private method update_bez_oc { i } {
     #=============================
      # copy bezier points out of mxu class
      # apply optical distortion (or not)
      # copy values back to mxu class

      set dummy [C_mxus$i get_curve_ca]
      set targw(1,a) [C_mxus$i get_RA]
      set targw(1,d) [C_mxus$i get_DEC]

      $m_image convert coords [lindex $dummy 0] [lindex $dummy 1] canvas \
               targw(1,x) targw(1,y) image
      calc_zemax 1 $targw(1,x) $targw(1,y)
      C_mxus$i put_b_oc 1 [list $targw(1,cx) $targw(1,cy)]

      $m_image convert coords [lindex $dummy 2] [lindex $dummy 3] canvas \
               targw(1,x) targw(1,y) image
      calc_zemax 1 $targw(1,x) $targw(1,y)
      C_mxus$i put_b_oc 2 [list $targw(1,cx) $targw(1,cy)]

      $m_image convert coords [lindex $dummy 4] [lindex $dummy 5] canvas \
               targw(1,x) targw(1,y) image
      calc_zemax 1 $targw(1,x) $targw(1,y)
      C_mxus$i put_b_oc 3 [list $targw(1,cx) $targw(1,cy)]

      $m_image convert coords [lindex $dummy 6] [lindex $dummy 7] canvas \
               targw(1,x) targw(1,y) image
      calc_zemax 1 $targw(1,x) $targw(1,y)
      C_mxus$i put_b_oc 4 [list $targw(1,cx) $targw(1,cy)]

      $m_image convert coords [lindex $dummy 8] [lindex $dummy 9] canvas \
               targw(1,x) targw(1,y) image
      calc_zemax 1 $targw(1,x) $targw(1,y)
      C_mxus$i put_b_oc 5 [list $targw(1,cx) $targw(1,cy)]

      $m_image convert coords [lindex $dummy 10] [lindex $dummy 11] canvas \
               targw(1,x) targw(1,y) image
      calc_zemax 1 $targw(1,x) $targw(1,y)
      C_mxus$i put_b_oc 6 [list $targw(1,cx) $targw(1,cy)]
    }

     #============================
     private method prepare_mxu_for_save {} {
     #============================
     #  update slits of type CURVED
     set noms [fors::MXU_SLIT::nos]
     if {$noms>0} {
     for {set i 1} {$i <= $noms} { incr i 1} { 
         if {[C_mxus$i cget -ON ]} {
         if {[string match *CURVED* [C_mxus$i cget -shape]]} { 

           # we determine the curved slitcenter, apply correction of 
           # of optical distortion and save correct values to mxu slit
           # this point has only meaning for DRS 

           set dummy [C_mxus$i update_curved_center]
           $m_image convert coords [lindex $dummy 0] [lindex $dummy 1] canvas \
                    targw(1,x) targw(1,y) image 
           set targw(1,a) [C_mxus$i get_RA]
           set targw(1,d) [C_mxus$i get_DEC]
           calc_zemax 1 $targw(1,x) $targw(1,y) 
           # targw(1,cx) and targw(1,cy) is not used 
           C_mxus$i put_OC  [list $targw(1,ca) $targw(1,cd)]
     
           update_bez_oc $i
           
           # update the three Bezier points 
           # get points in [canvas]
      #    set dummy [C_mxus$i get_curve_ca]
      #    set targw(1,a) [C_mxus$i get_RA]
      #    set targw(1,d) [C_mxus$i get_DEC]

      #    $m_image convert coords [lindex $dummy 0] [lindex $dummy 1] canvas \
      #             targw(1,x) targw(1,y) image
      #    calc_zemax 1 $targw(1,x) $targw(1,y) 
      #    C_mxus$i put_b_oc 1 [list $targw(1,cx) $targw(1,cy)]
      #  
      #    $m_image convert coords [lindex $dummy 2] [lindex $dummy 3] canvas \
      #             targw(1,x) targw(1,y) image
      #    calc_zemax 1 $targw(1,x) $targw(1,y) 
      #    C_mxus$i put_b_oc 2 [list $targw(1,cx) $targw(1,cy)]
      #    
      #    $m_image convert coords [lindex $dummy 4] [lindex $dummy 5] canvas \
      #             targw(1,x) targw(1,y) image
      #    calc_zemax 1 $targw(1,x) $targw(1,y) 
      #    C_mxus$i put_b_oc 3 [list $targw(1,cx) $targw(1,cy)]
      #    
      #    $m_image convert coords [lindex $dummy 6] [lindex $dummy 7] canvas \
      #             targw(1,x) targw(1,y) image
      #    calc_zemax 1 $targw(1,x) $targw(1,y) 
      #    C_mxus$i put_b_oc 4 [list $targw(1,cx) $targw(1,cy)]
      #    
      #    $m_image convert coords [lindex $dummy 8] [lindex $dummy 9] canvas \
      #             targw(1,x) targw(1,y) image
      #    calc_zemax 1 $targw(1,x) $targw(1,y) 
      #    C_mxus$i put_b_oc 5 [list $targw(1,cx) $targw(1,cy)]
      #    
      #    $m_image convert coords [lindex $dummy 10] [lindex $dummy 11] canvas \
      #             targw(1,x) targw(1,y) image
      #    calc_zemax 1 $targw(1,x) $targw(1,y) 
      #    C_mxus$i put_b_oc 6 [list $targw(1,cx) $targw(1,cy)]
          }
         }
        }
       } 
     }


     #============================
     private method save_mos_TPX {filename} {
     #============================
     # open the *.obx file 
     # saving current parameters of one exposure
     # modus either mos or pmos or echs 
     # we save 5 !! files 
     # <mode>.<expo_count>.<fits_name>.p_fims -> SET
     # <mode>.<expo_count>.<fits_name>.p_fims -> SET
     # <mode>.<expo_count>.<fits_name>.fims   -> SET
     # <mode>.<expo_count>.<fits_name>.targ   -> LOG
     # <mode>.<expo_count>.<fits_name>.focf   -> LOG
     # .fims contains all specified keywords, its the 
     #       file to be loaded when [Load Mask] 
     # .focf contains only the INS.FOC.SETUP keyword in one line 
     #       to be inserted in an obd file (e.g. via emacs)
     # .targ contains only the INS.TARG.SETUP keyword ...
     # .p_fof contains only the value of the INS.FOC.SETUP 
     #         keyword to be inserted to an obx file by p2pp
     # .p_targ contains only the value of the INS.TARG.SETUP keyword ...
     # 18/01/99 mos-> mos1 or mos2 
     # save results as one long string for the p_focf and p_targ
     set string_foc  ""
     set string_targ ""

     # apply optical correction if FITS frame from FORS
     #
     set corr_flag [$this cget -fors_frame]

     # Ask if slit 10 for alignment
     #
#    set slit_flag 0
#    set w [DialogWidget .corr \
#           -text "MOS 10 slit alignment : [C_ref_targ cget -mos10ali]" \
#           -bitmap warning \
#           -buttons {ON OFF} ]
#    if { [$w activate] } { # puts stdout "OFF"
#                           set slit_flag 0 
#                           C_ref_targ configure -mos10ali OFF
#                         } else { 
#                           # puts stdout "ON" 
#                           set slit_flag 1
#                           C_ref_targ configure -mos10ali ON 
#                         }

#    # for status panel
#    set sp_mtali [C_ref_targ cget -mos10ali]
#    catch {SP update}

     # ------------------------------------
     # 1 -- open setupfile and write header 
     #
     set mod "[$this cget -modus][string index [$this cget -instrument] 4]"
     if {$filename==""} {
         set myfile \
         "$set_dir/$mod.$exposure_counter.fims"
         } else {
         set myfile "$set_dir/$filename.fims"
         } 
     set fileId [open $myfile w ]
     puts $lfId "FORS::save_mos_TPX:  $fileId $myfile"
     puts $fileId "# created by SKYCAT-TOOL V$s_v -- FIMS-plugin V$f_v"
     puts $fileId "# orig filename  : $myfile"
     puts stdout  "# orig filename  : $myfile"
     puts $fileId "# user-date-host : [exec whoami] [exec date -u +%Y-%m-%dT%H:%M:%S] $lochost"
     puts $fileId "# fits file dir  : [ [.skycat1 component image] cget -file]"
     puts $fileId "# fits file name : [$this get_fits_name]"
     puts $fileId "# fits instrument: [$m_image fits get INSTRUME]"
     puts $fileId "# fits origin    : [$m_image fits get ORIGIN]" 
     puts $fileId "# apply optidist : [$this cget -fors_frame]" 
     puts $fileId "# fits object    : [$m_image fits get OBJECT]" 
     puts $fileId "# catalog name   : $CatFileName"
     puts $fileId "# target setup for [$this cget -instrument]-[string toupper $mod]"
     puts $fileId "# values in alp/del"
     puts $fileId "# --------------------------------------------"

     # ------------------------------------
     # 2 -- telescope pointing : _targ 

     $m_image convert coords $world_x $world_y "deg $EQX" \
        dummy_x dummy_y "wcs $EQX" 
     # puts stdout "$world_x $world_y $dummy_x $dummy_y"

   # particular Pipeline issue
   set last [string length $myfile] 
   set first [expr [string last "/" $myfile]  +1]
#   append string_foc  [format "INS.FIMS.NAME  %20s.p_focf " \
#	    [string range $myfile $first [expr $last - 6] ] ]
#   append string_targ [format "INS.FIMS.NAME  %20s.p_targ " \
#	    [string range $myfile $first [expr $last - 6] ] ]
   # WH 9.5.2000
#   append string_foc [format "INS.FIMS.NAME  %1s_%-1s_%-1s.p_focf  " \
#   $s_v [lindex $f_v 0] [string range $myfile $first [expr $last - 6] ] ]
#   append string_targ [format "INS.FIMS.NAME %1s_%-1s_%-1s.p_targ  " \
#   $s_v [lindex $f_v 0] [string range $myfile $first [expr $last - 6] ] ]

   append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s  " \
   [string range $myfile $first [expr $last - 6] ] $s_v [lindex $f_v 0] ]
   append string_targ [format "INS.FIMS.NAME %-1s.p_targ_%1s_%-1s  " \
   [string range $myfile $first [expr $last - 6] ] $s_v [lindex $f_v 0] ]


         # for the .fims 
         lassign [split $dummy_x :] d1 d2 d3
         puts $fileId [format "TEL.TARG.ALPHA    %02s%02s%06.3f" $d1 $d2 $d3 ]
         lassign [split $dummy_y :] d1 d2 d3
         puts $fileId [format "TEL.TARG.DELTA    %02s%02s%06.3f" $d1 $d2 $d3 ]

         if {$corr_flag && [string match echs [$this cget -modus] ]} {
         # used corrected pointing for ECHS on fors images
         lassign [split $targw(2,ca) :] d1 d2 d3
append string_targ  [format "  TEL.TARG.ALPHA    %02s%02s%06.3f" $d1 $d2 $d3 ]
append string_foc   [format "INS.FOCF.ALPHA  %02s%02s%06.3f" $d1 $d2 $d3 ]
         lassign [split $targw(2,cd) :] d1 d2 d3
append string_targ  [format "  TEL.TARG.DELTA    %02s%02s%06.3f" $d1 $d2 $d3 ]
append string_foc   [format "  INS.FOCF.DELTA  %02s%02s%06.3f" $d1 $d2 $d3 ]
         } else {
         # use real pointing for MOS 
         $m_image convert coords $world_x $world_y "deg $EQX" \
           dummy_x dummy_y "wcs $EQX"
         lassign [split $dummy_x :] d1 d2 d3
append string_targ  [format "  TEL.TARG.ALPHA    %02s%02s%06.3f" $d1 $d2 $d3 ]
append string_foc   [format "INS.FOCF.ALPHA  %02s%02s%06.3f" $d1 $d2 $d3 ]
         lassign [split $dummy_y :] d1 d2 d3
append string_targ  [format "  TEL.TARG.DELTA    %02s%02s%06.3f" $d1 $d2 $d3 ]
append string_foc   [format "  INS.FOCF.DELTA  %02s%02s%06.3f" $d1 $d2 $d3 ]
         }

       puts $fileId [format "TEL.ROT.OFFANGLE    %6.3f" [expr $PAM * -1]  ]
append string_targ  [format "  TEL.ROT.OFFANGLE  %6.3f" [expr $PAM * -1]  ]
append string_foc   [format "  INS.FOCF.OFFANGLE  %6.3f" [expr $PAM * -1]  ]
#        puts $fileId [format "TEL.TARG.EPOCH          %6.1f" $EQX ]
# append string_targ  [format "  TEL.TARG.EPOCH     %6.1f" $EQX ]
       puts $fileId [format "TEL.TARG.EQUINOX        %6.1f" $EQX ]
append string_targ  [format "  TEL.TARG.EQUINOX    %6.1f" $EQX ]
       puts $fileId [format "TEL.TARG.TYPE    COORDINATE" ]
append string_targ  [format "  TEL.TARG.TYPE    COORDINATE" ]

     # ------------------------------------
     # 3 -- MOS slit loop  
     #
     for { set i 1} {$i < 20 } { incr i 1 } {
         # targw($i,f) is either
         # 0 = not even moved 
         # 1 = moved, did not found entry in AstroCat widget
         # string = the target id of  AstroCat local_list

      # 3a target name 
      #
         if {[string compare $targw($i,f) "0"]==0} { # do nothing 
                 # slit is not defined
            } elseif {[string compare $targw($i,f) "1"]==0} { 
            puts $fileId [format "INS.TARG$i.NAME  %19s" NN  ]
       append string_foc [format "  INS.TARG$i.NAME  %19s" NN  ]
            } else {
            set targw($i,f) [string_exchange $targw($i,f) " " "_" ]
            puts $fileId [format "INS.TARG$i.NAME  %19s" $targw($i,f) ]
       append string_foc [format "  INS.TARG$i.NAME  %19s" $targw($i,f) ] 
            }



      # 3b MOS slit width  
      #
         if {[string compare $targw($i,f) "0"]} {
            # only f!=0 targets should contain valid a,d

         puts $fileId [format "INS.TARG$i.WID          %3.1f" $mos($i,b) ]
  append string_foc [format "  INS.TARG$i.WID  %3.1f" $mos($i,b) ]
# --------------------------------------------
# 08/04/99 INS.MOS$i.WID  -->  INS.MOS$i.WIDTH
# --------------------------------------------
#        puts $fileId [format "INS.TARG$i.WIDTH        %3.1f" $mos($i,b) ]
# append string_foc [format "  INS.TARG$i.WIDTH  %3.1f" $mos($i,b) ]

      # 3c target position inside the MOS slit  
      #
      # puts stdout "$targw($i,a) $targw($i,d) $targw($i,f)"
      # save uncorrected values to .fims file to avoid 
      # iterative optical correction when load/save/load/save ..
      lassign [split $targw($i,d) :] d1 d2 d3
      lassign [split $targw($i,a) :] a1 a2 a3
      # puts stdout  [format "INS.TARG$i.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
        puts $fileId [format "INS.TARG$i.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
      # puts stdout  [format "INS.TARG$i.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
        puts $fileId [format "INS.TARG$i.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

         if {$corr_flag} {
         lassign [split $targw($i,ca) :] a1 a2 a3
         lassign [split $targw($i,cd) :] d1 d2 d3
         }

    append string_foc [format "  INS.TARG$i.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
    append string_foc [format "  INS.TARG$i.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

      # save targets in CCD-pixel format
      # if {$corr_flag} {
      #  puts stdout  [format "INS.TARG$i.X          %05.3f" $targw($i,cx) ]
      #  puts $fileId [format "INS.TARG$i.X          %05.3f" $targw($i,cx) ]
      # } else {
      #  puts stdout  [format "INS.TARG$i.X          %05.3f" $targw($i,x) ]
      #  puts $fileId [format "INS.TARG$i.X          %05.3f" $targw($i,x) ]
      # }
      #  puts stdout  [format "INS.TARG$i.Y          %05.3f" $targw($i,y) ]
      #  puts $fileId [format "INS.TARG$i.Y          %05.3f" $targw($i,y) ]


 #     if {$corr_flag} {
 #       puts stdout [ format \
 #       "INS.TARG$i.ALPHA  %02s%02s%06.3f \
 #        INS.TARG$i.DELTA  %02s%02s%06.3f \
 #        INS.TARG$i.X %05.3f \
 #        INS.TARG$i.Y %05.3f" \
 #        $a1 $a2 $a3 $d1 $d2 $d3 $targw($i,cx) $targw($i,y) ]
 #      } else {
 #       puts stdout [ format \
 #       "INS.TARG$i.ALPHA  %02s%02s%06.3f \
 #        INS.TARG$i.DELTA  %02s%02s%06.3f \
 #        INS.TARG$i.X %05.3f \
 #        INS.TARG$i.Y %05.3f" \
 #        $a1 $a2 $a3 $d1 $d2 $d3 $targw($i,x) $targw($i,y) ]
 #      }



         }
          }

     # -------------------------
     #  4 --- Slit number association
     #
     for { set j 1} {$j < 20 } { incr j 1 } {
         # assign only slits, which have been moved
         if {[string compare $targw($j,f) "0"]} {
         puts $fileId [format "INS.MOS$j.TARG    %3d" $j ]
    append string_foc [format "  INS.MOS$j.TARG  %3d" $j ]
            }
         }



        # ----------------------------
        # reference objects are known 
        # ----------------------------
        # total numer of ref objects
        puts stdout "[C_ref_targ nort] Reference Objects selected"

     # Reference Objects already in correct coords
     # save reference objects to .fims setup file
     for { set j 1} {$j <= [C_ref_targ nort] } { incr j 1 } {
         set z [expr $j + 30]; # the INS.TARG$z. number

         lassign [split [C_ref_targ get $j a] :] a1 a2 a3
      #  puts stdout  [format "INS.TARG$z.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]
         puts $fileId [format "INS.TARG$z.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]

         lassign [split [C_ref_targ get $j d] :] d1 d2 d3
      #  puts stdout  [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
         puts $fileId [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

         if {$corr_flag} {
         # correct reference targets for zemax and diff refrac 
         set new_ref [calc_zemax_ref [C_ref_targ get $j a] \
                     [C_ref_targ get $j d] ]
         lassign [split $new_ref " " ] a0 d0
         puts stdout "new REF COORDS : >$a0<  >$d0<"

         # apply differential refraction 
         if {$dire_flag} {
            set new_ref [calc_diff_refrac 0 $a0 $d0]
            lassign [split $new_ref " " ] a0 d0
            puts stdout "new REF COORDS : >$a0<  >$d0<"
            }

         lassign [split $a0 :] a1 a2 a3
         lassign [split $d0 :] d1 d2 d3
         }
   append string_targ [format "  INS.TARG$z.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
   append string_targ [format "  INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
         }

     #--------------------
     # reference targets end
     #--------------------

# ----------------------------------
# 7 ----
# wgaessle 14/01/99 added the if command
     if {[C_ref_targ cget -mos10ali]} {
	 if {[C_ref_targ nort]>0} {
	     # if there is an alignment specify alignment slit
	     append string_targ "  INS.MOS10.POS 0.000   INS.MOS10.WID  0.5"
# --------------------------------------------
# 08/04/99 INS.MOS$i.WID  -->  INS.MOS$i.WIDTH
# --------------------------------------------
#            append string_targ "  INS.MOS10.POS 0.000   INS.MOS10.WIDTH  0.5"
	 }
     }
####### END of wgaessle's if ####################            

# ----------------------------
# 8 -- write seq.prep keywords 

     #  save also grism, OS filter and collimator 
     puts $fileId [format "SEQ.PREP.LMIN   %8.3f" $L_MIN ]
     puts $fileId [format "SEQ.PREP.LMAX   %8.3f" $L_MAX ]
     puts $fileId [format "SEQ.PREP.GRIS   %10s" [$this cget -grism] ]
     puts $fileId [format "SEQ.PREP.FILT   %10s" [$this cget -filter] ]
     puts $fileId [format "SEQ.PREP.MTALI  %10s" [C_ref_targ cget -mos10ali] ]
     puts $fileId [format "SEQ.PREP.COLL   %10s" [$this cget -collimat] ]
     if {[string match echs [$this cget -modus] ]} {
        puts $fileId [format "SEQ.PREP.ECHS9  %1d" $echs_flag ]
        } 
     append string_targ "  INS.COLL.NAID   [$this cget -collimat]"

     close $fileId

#-------------------------
# 9 -- save p_... files to be used by the P2PP FORS instrument package 
#


     # save the same stuff again as one string to a 
     # file to be called by p2pp  
     set myfile $set_dir/$filename.p_targ
     set fileId [open $myfile w ]
     puts -nonewline $fileId $string_targ
     close $fileId

     set myfile $set_dir/$filename.p_focf
     set fileId [open $myfile w ]
     puts -nonewline $fileId $string_foc
     close $fileId

     # save the same stuff again but in a more readable format
     # into the .fims/LOG directory (only for maintenance)

     set s2t [ldelete_all [split $string_targ " "]  ""]
     set number [llength $s2t]
     for {set i 0} {$i < $number} { incr i 2 } {
         set s2t [ lreplace $s2t $i $i [lrange $s2t $i [expr $i+1] ] ]
         }
     for {set i [expr $number-1] } {$i > 0} { incr i -2 } {
         set s2t [ lreplace $s2t $i $i ]
         }
     set string2_targ [join $s2t "\n" ]

     set myfile $log_dir/$filename.targ
     #                             ----
     set fileId [open $myfile w ]
     puts $fileId $string2_targ
     close $fileId

     set s2f [ldelete_all [split $string_foc " "]  ""]
     set number [llength $s2f]
     for {set i 0} {$i < $number} { incr i 2 } {
         set s2f [ lreplace $s2f $i $i [lrange $s2f $i [expr $i+1] ] ]
         }
     for {set i [expr $number-1] } {$i > 0} { incr i -2 } {
         set s2f [ lreplace $s2f $i $i ]
         }
     set string2_foc [join $s2f "\n" ]

     set myfile $log_dir/$filename.focf
     #                             ----
     set fileId [open $myfile w ]
     puts $fileId $string2_foc
     close $fileId


     incr exposure_counter 1

     }

     #============================
     private method save_img_TPX { filename } {
     #============================
     # open the *.obx file 
     # saving current parameters of one exposure
     # we save 5 !! files 
     # <mode>.<expo_count>.<fits_name>.fims -> SET
     # <mode>.<expo_count>.<fits_name>.targ -> LOG
     # <mode>.<expo_count>.<fits_name>.focf -> LOG
     # <mode>.<expo_count>.<fits_name>.p_targ -> SET
     # <mode>.<expo_count>.<fits_name>.p_forc -> SET
     # .fims contains all specified keywords, its the 
     #       file to be loaded when [Load Mask] 
     # .focf contains only the INS.FOC.SETUP keyword in one line 
     #       to be inserted in an obd file (e.g. via emacs)
     # .targ contains only the INS.TARG.SETUP keyword ...
     # .p_focf contains only the value of the INS.FOC.SETUP 
     #         keyword to be inserted to an obx file by p2pp
     # .p_targ contains only the value of the INS.TARG.SETUP keyword ...
     # if IPOL save also the offset file 
     # ipol.<ec>.<fits_name>.offs -> LOG 
     # 18.11.99 bug detected: INS.MASK.NAID polarimMask not saved to p_focf
     #          in IPOL-HR

     # save results as one long string
     set string_foc  ""
     set string_targ ""
     # empirical optical distortion of FORS1
     set corr_flag [$this cget -fors_frame]

     # modus either img or ipol
     set mod "[$this cget -modus][string index [$this cget -instrument] 4]"
     if {$filename==""} {
         set myfile "$set_dir/$mod.$exposure_counter.fims"
         } else {
         set myfile "$set_dir/$filename.fims"
         } 
     set fileId [open $myfile w ]

     # write comment block 
     puts $lfId "FORS::save_img_TPX:  $fileId $myfile"
     puts $fileId "# created by SKYCAT-TOOL V$s_v -- FIMS-plugin V$f_v"
     puts $fileId "# orig filename  : $myfile"
     puts stdout  "# orig filename  : $myfile"
     puts $fileId "# user-date-host : [exec whoami] [exec date -u +%Y-%m-%dT%H:%M:%S] $lochost"
     puts $fileId "# fits file dir  : [ [.skycat1 component image] cget -file]"
     puts $fileId "# fits file name : [$this get_fits_name]"
     puts $fileId "# fits instrument: [$m_image fits get INSTRUME]"
     puts $fileId "# fits origin    : [$m_image fits get ORIGIN]" 
     puts $fileId "# apply optidist : [$this cget -fors_frame]" 
     puts $fileId "# fits object    : [$m_image fits get OBJECT]" 
     puts $fileId "# catalog name   : $CatFileName"
     puts $fileId "# target setup for [$this cget -instrument]-[string toupper $mod]"
     puts $fileId "# values in mm of the focal field "
     puts $fileId "# --------------------------------------------"

   # particular Pipeline issue
   set last [string length $myfile] 
   set first [expr [string last "/" $myfile]  +1]
#    append string_foc [format "INS.FIMS.NAME  %20s.p_focf " \
#           [string range $myfile $first [expr $last - 6] ] ]
#  append string_targ [format "INS.FIMS.NAME  %20s.p_targ " \
#          [string range $myfile $first [expr $last - 6] ] ]
   # WH 9.5.2000
#   append string_foc [format "INS.FIMS.NAME  %1s_%-1s_%-1s.p_focf  " \
#   $s_v [lindex $f_v 0] [string range $myfile $first [expr $last - 6] ] ]
#   append string_targ [format "INS.FIMS.NAME  %1s_%-1s_%-1s.p_targ  " \
#   $s_v [lindex $f_v 0] [string range $myfile $first [expr $last - 6] ] ]

   append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s  " \
   [string range $myfile $first [expr $last - 6] ] $s_v [lindex $f_v 0] ]
   append string_targ [format "INS.FIMS.NAME  %-1s.p_targ_%1s_%-1s  " \
   [string range $myfile $first [expr $last - 6] ] $s_v [lindex $f_v 0] ]

     # TCS pointing 
         $m_image convert coords $world_x $world_y "deg $EQX" \
           dummy_x dummy_y "wcs $EQX"
         lassign [split $dummy_x :] d1 d2 d3
         puts $fileId [format "TEL.TARG.ALPHA    %02s%02s%06.3f" $d1 $d2 $d3 ]
         lassign [split $dummy_y :] d1 d2 d3
         puts $fileId [format "TEL.TARG.DELTA    %02s%02s%06.3f" $d1 $d2 $d3 ]

     # here a special feature for IPOL dither
     # when an offset of 1 MOS stripe width is choosen
     # -> 2 exposures 1 offsets
     # when an offset of 2/3 MOS stripe width is choosen
     # -> 3 exposures 2 offsets
     # change TEL keyword for *.targ (not for *.fims)
     if {[string compare [$this cget -modus] "ipol"]==0} {
        # save the offset file
        switch -exact -- $dith_num {
        1 {
          # one offset 
          # calculate the offset
          set result [calc_dither 1 1.0]
          set off_a [lindex $result 2]
          set off_d [lindex $result 3]
          }
        2 {
          # move the mask back and calculate the offset
          # the offset will be carried out two times
          # 
          set result [calc_dither -1 2.0/3.0]
          set off_a [ expr -1.0 * [lindex $result 2] ]
          set off_d [ expr -1.0 * [lindex $result 3] ]
          set a0 [lindex $result 0]; # new TEL
          set d0 [lindex $result 1]; # new TEL
          # overwrite default TEL 
          # $m_image convert coords $a0 $d0 "deg $EQX" \
          #  dummy_x dummy_y "wcs $EQX"
          }
      default { puts stdout :Warning IPOL OFFSET }
        }
        # this is no longer necessary since the templates use 
        # TEL.TARG.OFFSETY
        
     if {[string compare [$this cget -modus] "ipol"]==0} {
      if {[string match *HR*  [$this cget -collimat] ]} {
         # one could also asl for 'if {$mask==pol}
	 append string_foc "INS.MASK.NAID polarimMask" 
         }
	 }

        # open offset file and save offset
        #  set offfile "$log_dir/$filename.offs"
        #  set offId [open $offfile w ]
        #  puts $offId [ format \
        #  "SEQ.OFF.NUMBER  %1d  SEQ.OFF.ALPHA  %8.5f SEQ.OFF.DELTA  %8.5f" \
        #  $dith_num $off_a $off_d ]
        #  close $offId

 #      info_dialog  [format "IPOL offset (for P2PP):\n\n\
 #                    SEQ.OFF.NUMBER  %1d \n\
 #                    SEQ.OFF.ALPHA  %8.5fs\n\
 #                    SEQ.OFF.DELTA  %8.5f''\n\n\
 #                    written to\n\
 #                    %s/\n%s" \
 #                    $dith_num $off_a $off_d \
 #                    $log_dir $filename.offs ]
        }

         # take modified TEL in case of 2/3 offset IPOL 
         lassign [split $dummy_x :] d1 d2 d3
   append string_targ [format "TEL.TARG.ALPHA  %02s%02s%06.3f" $d1 $d2 $d3 ]
         lassign [split $dummy_y :] d1 d2 d3
   append string_targ [format "  TEL.TARG.DELTA  %02s%02s%06.3f" $d1 $d2 $d3 ]

        # the rest is  independent on IPOL 
       puts $fileId [format "TEL.ROT.OFFANGLE       %6.3f" [expr $PAM * -1]  ]
append string_targ  [format "  TEL.ROT.OFFANGLE     %6.3f" [expr $PAM * -1]  ]
#        puts $fileId [format "TEL.TARG.EPOCH          %6.1f" $EQX ]
# append string_targ  [format "  TEL.TARG.EPOCH     %6.1f" $EQX ]
       puts $fileId [format "TEL.TARG.EQUINOX        %6.1f" $EQX ]
append string_targ  [format "  TEL.TARG.EQUINOX    %6.1f" $EQX ]
       puts $fileId [format "TEL.TARG.TYPE    COORDINATE" ]
append string_targ  [format "  TEL.TARG.TYPE    COORDINATE" ]

         # this is not for IPOL hr and PIMS hr (mask=pol), but for
         # IMG hr/sr, IPOLsr IMS sh/hr PIMS sr
         if {[string compare $mask "pol"]} {
         set frac [expr $XBLADEA / ($XBLADEA-$XBLADEB) ]
     for { set i 1} {$i < 20 } { incr i 1 } {
         # calculate canvas back to mm 
         # ctom
         # update width 
         set xdum [expr $mos($i,xb) - $mos($i,xc)]
         set ydum [expr $mos($i,yb) - $mos($i,yc)]
         $m_image convert dist $xdum $ydum canvas x y "deg $EQX"
         set mos($i,b) [expr 3600.0*sqrt($x*$x + $y*$y) ]

         # update middel position 
         set px [expr $mos($i,xb) + ($mos($i,xc)-$mos($i,xb))/2.0 ]
         set py [expr $mos($i,yb) + ($mos($i,yc)-$mos($i,yb))/2.0 ]
                 # puts stdout "#$i- $mos($i,xb) < $px < $mos($i,xc) || \
                 #              #$i- $mos($i,yb) < $py < $mos($i,yc)"
         # $m_image convert coords $xdum $ydum canvas px py "deg $EQX"
         # x=zero position
                # puts stdout "$XSRMIN/$XSRMAX/$frac"
         set ax [expr $mos($i,xa)+($mos($i,xd)-$mos($i,xa))*$frac]
         set ay [expr $mos($i,ya)+($mos($i,yd)-$mos($i,ya))*$frac]
                # puts stdout "#$i- $mos($i,xa) < $ax < $mos($i,xd) || \
                #              #$i- $mos($i,ya) < $ay < $mos($i,yd)"
         $m_image convert dist [expr $px-$ax] [expr $py-$ay] canvas \
                      a d "deg $EQX" 
         set dist_arc [expr 3600.0 * sqrt ($a*$a +$d*$d) ]
      
         # since we are only dealing with distances, we still 
         # need the sign +-
         # positiv if c-axe < b-axis
         set cax [expr sqrt( ($mos($i,xc)-$ax)*($mos($i,xc)-$ax) + \
                             ($mos($i,yc)-$ay)*($mos($i,yc)-$ay) )]
         set bax [expr sqrt( ($mos($i,xb)-$ax)*($mos($i,xb)-$ax) + \
                             ($mos($i,yb)-$ay)*($mos($i,yb)-$ay) )]
         if {$bax<$cax} { set sign 1 } else { set sign (-1) }

         set dist_mm [expr $sign * $dist_arc / $focscale]
         if {$dist_mm < -105.0} {set dist_mm -117; set mos($i,b) \
             [ expr 6.0*$focscale] }
         if {$dist_mm >  105.0} {set dist_mm +117; set mos($i,b) \
            [ expr 6.0*$focscale] }

         puts $fileId [format "INS.MOS$i.POS         %06.3f" $dist_mm ]
   append string_foc  [format "  INS.MOS$i.POS   %06.3f" $dist_mm ]
         set dummy  [expr $mos($i,b)/$focscale]
         if {$dummy > 240.0} { set dummy 234}
         if {$dummy < 6.0  } { set dummy 6 }
         puts $fileId [format "INS.MOS$i.WID          %05.3f" $dummy  ]
    append string_foc [format "  INS.MOS$i.WID    %05.3f" $dummy  ]
         # puts stdout "save_img_TPX: POS=$dist_arc ''  POS=$dist_mm WID=$dummy"
# -----------------------------------------------------------
#   since 08/04/99 new DICD INS.MOS$i.WID --> INS.MOS$i.WIDTH
# -----------------------------------------------------------
#        puts $fileId [format "INS.MOS$i.WIDTH        %05.3f" $dummy  ]
#   append string_foc [format "  INS.MOS$i.WIDTH  %05.3f" $dummy  ]
#        puts stdout "save_img_TPX: POS=$dist_arc ''  POS=$dist_mm WIDTH=$dummy"

        # save if the blade has been moved
        # this is a good information when loading
        # the setup again
        if {$targw($i,f) != 0} {
           puts $fileId [format "INS.MOS$i.SET          %2d" $i  ]
           }

         }
          }


        # ----------------------------
        # reference objects are known 
        # ----------------------------
        # total numer of ref objects
        puts stdout "[C_ref_targ nort] Reference Objects selected"

     # Reference Objects already in correct coords
     # save reference objects to setup file
     for { set j 1} {$j <= [C_ref_targ nort] } { incr j 1 } {
         set z [expr $j + 30]; # the INS.TARG$z. number

         lassign [split [C_ref_targ get $j a] :] a1 a2 a3
      #  puts stdout  [format "INS.TARG$z.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]
         puts $fileId [format "INS.TARG$z.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]

         lassign [split [C_ref_targ get $j d] :] d1 d2 d3
      #  puts stdout  [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
         puts $fileId [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

         if {$corr_flag} {
         set new_ref [calc_zemax_ref [C_ref_targ get $j a] \
                     [C_ref_targ get $j d] ]
         lassign [split $new_ref " " ] a0 d0
         puts stdout "new REF COORDS : >$a0<  >$d0<"

         # apply differential refraction 
         if {$dire_flag} {
            set new_ref [calc_diff_refrac 0 $a0 $d0]
            lassign [split $new_ref " " ] a0 d0
            puts stdout "new REF COORDS : >$a0<  >$d0<"
            }

         lassign [split $a0 :] a1 a2 a3
         lassign [split $d0 :] d1 d2 d3
         }
   append string_targ [format "  INS.TARG$z.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
   append string_targ [format "  INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
         }

     #--------------------
     # reference targets end
     #--------------------
     # if {[C_ref_targ nort]>0} {
        # if there is an alignment specify alignment slit
   #    append string_targ "  INS.MOS10.POS 0.000   INS.MOS10.WID  0.5"
    #     } 

     #  save also grism, OS filter and collimator 
     # ims or pims 
     if {[string compare [$this cget -slit_width] open]==0} {
     puts $fileId [format "SEQ.PREP.LMIN   %8.3f" $L_MIN ]
     puts $fileId [format "SEQ.PREP.LMAX   %8.3f" $L_MAX ]
     puts $fileId [format "SEQ.PREP.GRIS   %10s" [$this cget -grism] ]
     puts $fileId [format "SEQ.PREP.FILT   %10s" [$this cget -filter] ]
       }
     puts $fileId [format "SEQ.PREP.COLL   [$this cget -collimat]" ]
     append string_targ "  INS.COLL.NAID   [$this cget -collimat]"

     close $fileId

     # save the same stuff again as one string to a
     # file to be called by p2pp
     set myfile $set_dir/$filename.p_targ
     set fileId [open $myfile w ]
     puts -nonewline $fileId $string_targ
     close $fileId

     set myfile $set_dir/$filename.p_focf
     set fileId [open $myfile w ]
     # special behaviour when saving an empty string to a file
     # the file  will have 1 char and P2PP will complain 
     # therefore dont copy empty string to file 
     if {[string length $string_foc]>0} {puts -nonewline $fileId $string_foc}
     close $fileId

     # save the same stuff again but in a more readable format
     # into the .fims/LOG directory (only for maintenance)

     set s2t [ldelete_all [split $string_targ " "]  ""]
     set number [llength $s2t]
     for {set i 0} {$i < $number} { incr i 2 } {
         set s2t [ lreplace $s2t $i $i [lrange $s2t $i [expr $i+1] ] ]
         }
     for {set i [expr $number-1] } {$i > 0} { incr i -2 } {
         set s2t [ lreplace $s2t $i $i ]
         }
     set string2_targ [join $s2t "\n" ]

     set myfile $log_dir/$filename.targ
     #                             ----
     set fileId [open $myfile w ]
     puts $fileId $string2_targ
     close $fileId

     set s2f [ldelete_all [split $string_foc " "]  ""]
     set number [llength $s2f]
     for {set i 0} {$i < $number} { incr i 2 } {
         set s2f [ lreplace $s2f $i $i [lrange $s2f $i [expr $i+1] ] ]
         }
     for {set i [expr $number-1] } {$i > 0} { incr i -2 } {
         set s2f [ lreplace $s2f $i $i ]
         }
     set string2_foc [join $s2f "\n" ]

     set myfile $log_dir/$filename.focf
     #                             ----
     set fileId [open $myfile w ]
     puts $fileId $string2_foc
     close $fileId


     incr exposure_counter 1
     }


     #============================
     private method save_mxu_TPX {filename} {
     #============================
     # open the *.obx file 
     # saving current parameters of one exposure
     # modus mxu 
     # some changes of the output files after 
     # VIC-ESO meeting in 12/99
     # we save 5 !! files 
     # <mode>.<expo_count>.<fits_name>.fims   -> SET
     # <mode>.<expo_count>.<fits_name>.targ   -> LOG
     # <mode>.<expo_count>.<fits_name>.focf   -> LOG
     # <mode>.<expo_count>.<fits_name>.p_targ -> SET (PAF header file)
     # <mode>.<expo_count>.<fits_name>.p_focf -> SET (PAF header file)  
     # .fims contains all specified keywords, its the 
     #       file to be loaded when [Load Mask] 
     # .focf contains only the INS.FOC.SETUP keyword in one line 
     #       to be inserted in an obd file (e.g. via emacs)
     # .targ contains only the INS.TARG.SETUP keyword ...
     #  and the INS.MSKREADY.NAID keyword 
     # .p_focf contains only the value of the INS.FOC.SETUP 
     #         keyword to be inserted to an obx file by p2pp
     #         and the INS.MASK.NAID
     # .p_targ contains only the value of the INS.TARG.SETUP keyword ...
     # 18/01/99 mos-> mos1 or mos2 
     # save results as one long string
     set string_foc  ""
     set string_targ ""

     # before we do anything we show the status of all mxu slits 
     puts stdout "\n\nmxu_save_TPX" 
     puts stdout "save_mxu_TPX: >[fors::MXU_SLIT::get_MASK_NAID]<"
     set noms [fors::MXU_SLIT::nos]
     if {$noms>0} {
     for {set i 1} {$i <= $noms} { incr i 1} { 
         if {[C_mxus$i cget -ON ]} {
    puts stdout [ format "#%2d %1d %14s %5.2f %5.2f %5.2f %8s %8s %8s %8s %8s" \
          $i \
          [C_mxus$i cget -ON         ] \
          [C_mxus$i cget -target_name] \
          [C_mxus$i cget -slit_width ] \
          [C_mxus$i cget -slit_length] \
          [C_mxus$i cget -pos_ang    ] \
          [C_mxus$i cget -shape      ] \
          [C_mxus$i get_RA           ] \
          [C_mxus$i get_DEC          ] \
          [C_mxus$i get_OC_RA        ] \
          [C_mxus$i get_OC_DEC       ] \
          ]
          }
          }
          }
 
     set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]

     # apply optical correction if fits-image from fors
     set corr_flag [$this cget -fors_frame]

     # <><><><><><><><><><><><><><><><><><><><><>
     # workaround 13. Sept 2000 WH
     # FORS1 images are aliens fro MXU 
     #if {[string compare [$m_image fits get INSTRUME] "FORS1"]==0} {
     #   set corr_flag 0 }
     # <><><><><><><><><><><><><><><><><><><><><>

     # open setupfile and write header 
     set mod "[$this cget -modus][string index [$this cget -instrument] 4]"
     if {$filename==""} {
         set myfile "$set_dir/$mod.$exposure_counter.fims"
         } else { set myfile "$set_dir/$filename.fims"} 


       
       # technical mask flag 
       set tm [fors::MXU_SLIT::get_mmff]
       # science mask flag 
       set sm [expr ! $tm]

       # puts stdout "tm = $tm, sm = $sm"

     set fileId [open $myfile w ];   # fims file 
     puts $lfId "FORS::save_mxu_TPX:  $fileId $myfile"; # log file 

   if {$sm} {

   # we copy the INS.FIMS.NAME again to find it as the first chars in KE
   # java-pippi 

   # particular Pipeline issue
   set last [string length $myfile] 
   set first [expr [string last "/" $myfile]  +1]
#  append string_foc [format "# p_focf = %20s.p_focf\n" \
#       [string range $myfile $first [expr $last - 6] ] ]
#  append string_targ [format "# p_targ = %20s.p_targ\n" \
#       [string range $myfile $first [expr $last - 6] ] ]
   # WH 9.5.2000
#   append string_foc [format "INS.FIMS.NAME  %1s_%-1s_%-1s.p_focf\n" \
#   $s_v [lindex $f_v 0] [string range $myfile $first [expr $last - 6] ] ]
#   append string_targ [format "INS.FIMS.NAME  %1s_%-1s_%-1s.p_targ\n" \
#   $s_v [lindex $f_v 0] [string range $myfile $first [expr $last - 6] ] ]

   append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s\n" \
   [string range $myfile $first [expr $last - 6] ] $s_v [lindex $f_v 0] ]
   append string_targ [format "INS.FIMS.NAME  %-1s.p_targ_%1s_%-1s\n" \
   [string range $myfile $first [expr $last - 6] ] $s_v [lindex $f_v 0] ]

   }

     # .fims header 
     # -------------

     set header ""
     append header "# created by SKYCAT-TOOL V$s_v -- FIMS-plugin V$f_v\n"
     append header "# orig filename  : $filename : $myfile\n"
     puts stdout  "# orig filename  : $myfile"
     append header "# user-date-host : [exec whoami] [exec date -u +%Y-%m-%dT%H:%M:%S] $lochost\n"
     append header "# fits file dir  : [ [.skycat1 component image] cget -file]\n"
     append header "# fits file name : [$this get_fits_name]\n"
     append header "# fits instrument: [$m_image fits get INSTRUME]\n"
     append header "# fits origin    : [$m_image fits get ORIGIN]\n" 
     append header "# apply optidist : [$this cget -fors_frame]\n" 
     append header "# fits object    : [$m_image fits get OBJECT]\n" 
     append header "# catalog name   : $CatFileName\n"
     append header "# INS.MASK.NA    : [lindex $PM 0]\n"
     append header "# INS.MASK.ID    : [lindex $PM 1]\n"
     append header "# INS.MASK.NAID  : [lindex $PM 0]+[lindex $PM 1]\n"
     append header "# target setup for \
                         [$this cget -instrument]-[string toupper $mod]\n"
     append header "# --------------------------------------------\n\n"

     puts $fileId $header
     
    if {$sm} {
     # PAF header for p_targ file
     # --------------------------------
     # the mxu2.p_targ differs from all other p_targ files of the 
     # other FORS1/2 instrumental modi
append string_targ "PAF.HDR.START;                            # Marks start of header\n"
append string_targ "PAF.TYPE          \"Instrument Setup\";   # Type of parameter file\n"
append string_targ "PAF.ID            \"FORS2_[$this cget -modus]\";   # Parameter file ID\n"
append string_targ "PAF.NAME          \"$filename.p_targ\";  # Parameter file NAME\n"
append string_targ "PAF.DESC          \"FORS MXU Mask Setup File\";  # Purpose\n"
append string_targ "PAF.CRTE.NAME     \"FIMS $f_v\";  # SW version date UT\n"
append string_targ "PAF.CRTE.DAYTIM   \"[exec date -u +%Y-%m-%dT%H:%M:%S]\";  # Date and time of creation\n"
append string_targ "PAF.LCHG.NAME     \"\";                     # Who did last change\n"
append string_targ "PAF.LCHG.DAYTIM   \"\";                     # Date and time of last change\n"
append string_targ "PAF.CHCK.NAME     \"\";                     # Appl. checking par. file\n"
append string_targ "PAF.CHCK.DAYTIM   \"\";                     # Date and time of lastcheck\n"
append string_targ "PAF.CHCK.CHECKSUM \"\";                     # Parameter file checksum\n"
append string_targ "PAF.HDR.END;                                # Marks end of header\n\n"

append string_targ $header 

     # PAF header for p_focf file
     # --------------------------------
     # the mxu2.p_focf differs from all other p_targ files of the 
     # other FORS1/2 instrumental modi
append string_foc "PAF.HDR.START;                            # Marks start of header\n"
append string_foc "PAF.TYPE          \"Instrument Setup\";   # Type of parameter file\n"
append string_foc "PAF.ID            \"FORS2_[$this cget -modus]\";   # Parameter file ID\n"
append string_foc "PAF.NAME          \"$filename.p_focf\";  # Parameter file NAME\n"
append string_foc "PAF.DESC          \"FORS MXU Mask Setup File\";  # Purpose\n"
append string_foc "PAF.CRTE.NAME     \"FIMS $f_v\";  # SW version date UT\n"
append string_foc "PAF.CRTE.DAYTIM   \"[exec date -u +%Y-%m-%dT%H:%M:%S]\";  # Date and time of creation\n"
append string_foc "PAF.LCHG.NAME     \"\";                     # Who did last change\n"
append string_foc "PAF.LCHG.DAYTIM   \"\";                     # Date and time of last change\n"
append string_foc "PAF.CHCK.NAME     \"\";                     # Appl. checking par. file\n"
append string_foc "PAF.CHCK.DAYTIM   \"\";                     # Date and time of lastcheck\n"
append string_foc "PAF.CHCK.CHECKSUM \"\";                     # Parameter file checksum\n"
append string_foc "PAF.HDR.END;                                # Marks end of header\n\n"

append string_foc $header 

} 
# end of science mask case 

     $m_image convert coords $world_x $world_y "deg $EQX" \
        dummy_x dummy_y "wcs $EQX" 
     # puts stdout "$world_x $world_y $dummy_x $dummy_y"

   # particular Pipeline issue
   set last [string length $myfile] 
   set first [expr [string last "/" $myfile]  +1]
#  append string_foc [format "INS.FIMS.NAME  %20s.p_focf " \
#       [string range $myfile $first [expr $last - 6] ] ]
#  append string_targ [format "INS.FIMS.NAME  %20s.p_targ " \
#       [string range $myfile $first [expr $last - 6] ] ]
   # WH 9.5.2000
#   append string_foc [format "INS.FIMS.NAME  %1s_%-1s_%-1s.p_focf  " \
#   $s_v [lindex $f_v 0] [string range $myfile $first [expr $last - 6] ] ]
#   append string_targ [format "INS.FIMS.NAME  %1s_%-1s_%-1s.p_targ " \
#   $s_v [lindex $f_v 0] [string range $myfile $first [expr $last - 6] ] ]

   append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s  " \
   [string range $myfile $first [expr $last - 6] ] $s_v [lindex $f_v 0] ]
   append string_targ [format "INS.FIMS.NAME  %-1s.p_targ_%1s_%-1s  " \
   [string range $myfile $first [expr $last - 6] ] $s_v [lindex $f_v 0] ]

   if {$sm} { append string_foc "\n"; append string_targ "\n" }
   # what does this mean ? technical mask p_focf p_targ consist of one string 
   # sky masks follow the PAF convention 

      #-------------------
      #  ------ TCS ------
      #-------------------

         lassign [split $dummy_x :] d1 d2 d3
         puts $fileId [format "TEL.TARG.ALPHA    %02s%02s%06.3f" $d1 $d2 $d3 ]
   append string_targ [format "TEL.TARG.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3 ]
   append string_foc  [format "INS.FOCF.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3 ]
   if {$sm} { append string_foc "\n"; append string_targ "\n" }
          lassign [split $dummy_y :] d1 d2 d3
         puts $fileId [format "TEL.TARG.DELTA  %02s%02s%06.3f" $d1 $d2 $d3 ]
   append string_targ [format "TEL.TARG.DELTA  %02s%02s%06.3f " $d1 $d2 $d3 ]
   append string_foc  [format "INS.FOCF.DELTA  %02s%02s%06.3f " $d1 $d2 $d3 ]
   if {$sm} { append string_foc "\n"; append string_targ "\n" }
       puts $fileId [format "TEL.ROT.OFFANGLE  %6.3f" [expr $PAM * -1]  ]
  append string_targ  [format "TEL.ROT.OFFANGLE   %6.3f " [expr $PAM * -1]  ]
  append string_foc   [format "INS.FOCF.OFFANGLE  %6.3f " [expr $PAM * -1]  ]
   if {$sm} { append string_foc "\n"; append string_targ "\n" }
#        puts $fileId [format "TEL.TARG.EPOCH          %6.1f" $EQX ]
# append string_targ  [format "TEL.TARG.EPOCH  %6.1f " $EQX ]
       puts $fileId [format "TEL.TARG.EQUINOX        %6.1f" $EQX ]
append string_targ  [format "TEL.TARG.EQUINOX    %6.1f" $EQX ]
   if {$sm} { append string_foc "\n"; append string_targ "\n" }
       puts $fileId [format "TEL.TARG.TYPE    COORDINATE" ]
append string_targ  [format "TEL.TARG.TYPE    COORDINATE" ]
   if {$sm} { append string_targ "\n" }

 # user-defined program ID and MASK.NAID
 #
 # if {[lindex $PM 1]>900000} 
 append string_targ "INS.MSKREADY.NAID   [lindex $PM 0]+[lindex $PM 1] "
 append string_foc  "INS.MASK.NAID       [lindex $PM 0]+[lindex $PM 1] "
   if {$sm} { append string_foc "\n"; append string_targ "\n" }
    
   if {$sm} { 
 append string_targ "TPL.FILE.DIRNAME    \$INS_ROOT/SYSTEM/COMMON/SETUPFILES/INS"
 append string_foc  "TPL.FILE.DIRNAME    \$INS_ROOT/SYSTEM/COMMON/SETUPFILES/INS"
   append string_foc "\n"; append string_targ "\n" 
     }


     # save number of slits 
     set noms [fors::MXU_SLIT::nos]
         set s_c 0; set sp_rsc 0;
     for {set i 1} {$i <= $noms} { incr i 1} { 
         if {[C_mxus$i cget -ON]} { 
	    incr s_c 1 
            if {[C_mxus$i cget -ref_slit]} { incr sp_rsc  1
            }
	    }
         }
     puts $fileId [format "INS.SLIT.NUMBER    %d" $s_c ]

     # if no reference slits are defined by the user, 
     # take the first (up to) ten 
     if {$sp_rsc ==0} {
        # no reference slit specified
        puts stdout "$sp_rsc reference slits specified by user"
        set r_c 0
        for {set i 1} {$i <= $noms} { incr i 1} { 
            if {$r_c<10} {
            if {[C_mxus$i cget -ON]} { 
            if {[string match *STRAIGHT* [C_mxus$i cget -shape] ]} {
               incr r_c 1
               C_mxus$i configure -ref_slit 1 
               }
            }
            }
         }
	set sp_rsc $r_c 
        puts stdout "$sp_rsc reference slits specified by FIMS"
        }

     catch {SP update}
     puts $fileId [format "INS.RSLIT.NUMBER    %d" $sp_rsc ]


      # -----------------
      # --- slit loop ---
      # -----------------

     set s_c 100;     # science slit counter offset ?  
     set r_c 1;     # reference slit counter offset
     if {$noms>0} {
     for {set i 1} {$i <= $noms} { incr i 1} { 

         # if slit is specified 
         if {[C_mxus$i cget -ON]} {
            incr s_c 1

           # -1----------- NAME 

           puts $fileId  [format "INS.TARG$s_c.NAME  %19s" \
                         [C_mxus$i cget -target_name] ]
           #puts stdout  [format "INS.TARG$s_c.NAME  %19s" \
           #             [C_mxus$i cget -target_name] ]
           # to appear later in the fits file header of the FORS2 image 
      #    append string_targ [format "INS.TARG$s_c.NAME  %19s\n" \
      #                  [C_mxus$i cget -target_name] ]
           append string_foc  [format "INS.TARG$s_c.NAME  %19s " \
                         [C_mxus$i cget -target_name] ]
           if {$sm} { append string_foc "\n"  }


           # -2----------- Reference slits 

           if {[C_mxus$i cget -ref_slit]} { 
	   # append string_targ [format "INS.SLIT%1d.REF %1d\n" $r_c $s_c]
	   append string_foc  [format "INS.SLIT%1d.REF %1d " $r_c $s_c]
           if {$sm} { append string_foc "\n"  }
	   puts $fileId       [format "INS.SLIT%1d.REF %1d" $r_c $s_c]
           incr r_c 1
           }

           # -3----------- SLIT DIMENSIONS  

           set cu_width  [C_mxus$i cget -slit_width ]
           set cu_length [C_mxus$i cget -slit_length]
           set cu_posang [C_mxus$i cget -pos_ang    ]
           set cu_shape  [C_mxus$i cget -shape      ]

         # puts stdout  [format "INS.TARG$s_c.SHAPE  %19s"  $cu_shape  ]
           puts $fileId [format "INS.TARG$s_c.SHAPE  %19s"  $cu_shape  ]
         # puts stdout  [format "INS.TARG$s_c.WID    %3.1f" $cu_width  ]
           puts $fileId [format "INS.TARG$s_c.WID    %3.1f" $cu_width  ]
# --------------------------------------------
# 08/04/99 INS.MOS$i.WID  -->  INS.MOS$i.WIDTH
# --------------------------------------------
#    puts stdout  [format "INS.TARG$s_c.WIDTH    %3.1f" $cu_width  ]
#    puts $fileId [format "INS.TARG$s_c.WIDTH    %3.1f" $cu_width  ]

#      puts stdout  [format "INS.TARG$s_c.LEN    %3.1f" $cu_length ]
       puts $fileId [format "INS.TARG$s_c.LEN    %3.1f" $cu_length ]


    # slit position angle only for STRAIGHT 
    if {[string match *STRAIGHT* $cu_shape]} {
#      puts stdout  [format "INS.TARG$s_c.ROT    %3.1f" $cu_posang ]
       puts $fileId [format "INS.TARG$s_c.ROT    %3.1f" $cu_posang ]
       }

   append string_foc  [format "INS.TARG$s_c.SHAPE  %19s "  $cu_shape]
           if {$sm} { append string_foc "\n"  }
   append string_foc  [format "INS.TARG$s_c.WID    %3.1f " $cu_width]
           if {$sm} { append string_foc "\n"  }
   append string_foc  [format "INS.TARG$s_c.LEN    %3.1f " $cu_length]
           if {$sm} { append string_foc "\n"  }
#  append string_targ [format "INS.TARG$s_c.SHAPE  %19s\n"  $cu_shape]
#  append string_targ [format "INS.TARG$s_c.WID    %3.1f\n" $cu_width]
#  append string_targ [format "INS.TARG$s_c.LEN    %3.1f\n" $cu_length]


# --------------------------------------------
# 08/04/99 INS.MOS$i.WID  -->  INS.MOS$i.WIDTH
# --------------------------------------------
# append string_targ [format "INS.TARG$s_c.WIDTH  %3.1f\n" $cu_width ]
# append string_foc  [format "INS.TARG$s_c.WIDTH  %3.1f\n" $cu_width ]

   # slit position angle only for STRAIGHT 
   if {[string match *STRAIGHT* $cu_shape]} {
#  append string_targ [format "INS.TARG$s_c.ROT    %3.1f\n" $cu_posang ]
   append string_foc  [format "INS.TARG$s_c.ROT    %3.1f " $cu_posang ]
           if {$sm} { append string_foc "\n"  }
   }

    # 3 bezier points for slit  shape = CURVED  
    if {[string match *CURVED* $cu_shape]} {
        set bez_list [C_mxus$i get_curve_mm ]
        # the middle slit curve 
        set x1 [expr ([lindex $bez_list 0] + [lindex $bez_list 6] )*0.5 ]
        set y1 [expr ([lindex $bez_list 1] + [lindex $bez_list 7] )*0.5 ]
        set x2 [expr ([lindex $bez_list 2] + [lindex $bez_list 8] )*0.5 ]
        set y2 [expr ([lindex $bez_list 3] + [lindex $bez_list 9] )*0.5 ]
        set x3 [expr ([lindex $bez_list 4] + [lindex $bez_list 10] )*0.5 ]
        set y3 [expr ([lindex $bez_list 5] + [lindex $bez_list 11] )*0.5 ]

        puts $fileId [format "INS.TARG$s_c.X1     %3.1f" $x1 ]
        puts $fileId [format "INS.TARG$s_c.Y1     %3.1f" $y1 ]
        puts $fileId [format "INS.TARG$s_c.X2     %3.1f" $x2 ]
        puts $fileId [format "INS.TARG$s_c.Y2     %3.1f" $y2 ]
        puts $fileId [format "INS.TARG$s_c.X3     %3.1f" $x3 ]
        puts $fileId [format "INS.TARG$s_c.Y3     %3.1f" $y3 ]

#       append string_targ [format "INS.TARG$s_c.X1     %3.1f\n" $x1 ]
#       append string_targ [format "INS.TARG$s_c.Y1     %3.1f\n" $y1 ]
#       append string_targ [format "INS.TARG$s_c.X2     %3.1f\n" $x2 ]
#       append string_targ [format "INS.TARG$s_c.Y2     %3.1f\n" $y2 ]
#       append string_targ [format "INS.TARG$s_c.X3     %3.1f\n" $x3 ]
#       append string_targ [format "INS.TARG$s_c.Y3     %3.1f\n" $y3 ]

        append string_foc  [format "INS.TARG$s_c.X1     %3.1f " $x1 ]
           if {$sm} { append string_foc "\n"  }
        append string_foc  [format "INS.TARG$s_c.Y1     %3.1f " $y1 ]
           if {$sm} { append string_foc "\n"  }
        append string_foc  [format "INS.TARG$s_c.X2     %3.1f " $x2 ]
           if {$sm} { append string_foc "\n"  }
        append string_foc  [format "INS.TARG$s_c.Y2     %3.1f " $y2 ]
           if {$sm} { append string_foc "\n"  }
        append string_foc  [format "INS.TARG$s_c.X3     %3.1f " $x3 ]
           if {$sm} { append string_foc "\n"  }
        append string_foc  [format "INS.TARG$s_c.Y3     %3.1f " $y3 ]
           if {$sm} { append string_foc "\n"  }
        }
     # curved slits will not be used for the alignment 


    # puts stdout "$targw($i,a) $targw($i,d) $targw($i,f)"
    # save uncorrected values to .fims file to avoid 
    # iterative optical correction when load/save/load/save ..
     lassign [split [C_mxus$i get_RA ] :] a1 a2 a3
     lassign [split [C_mxus$i get_DEC] :] d1 d2 d3
#    puts stdout [format "INS.TARG$s_c.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
     puts $fileId [format "INS.TARG$s_c.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
#    puts stdout  [format "INS.TARG$s_c.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
     puts $fileId [format "INS.TARG$s_c.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

         if {$corr_flag} {
         lassign [split [C_mxus$i get_OC_RA ]  :] a1 a2 a3
         lassign [split [C_mxus$i get_OC_DEC] :] d1 d2 d3
         }
#append string_targ [format "INS.TARG$s_c.ALPHA  %02s%02s%06.3f\n" $a1 $a2 $a3]
#append string_targ [format "INS.TARG$s_c.DELTA  %02s%02s%06.3f\n" $d1 $d2 $d3]
 append string_foc  [format "INS.TARG$s_c.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
           if {$sm} { append string_foc "\n"  }
 append string_foc  [format "INS.TARG$s_c.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
           if {$sm} { append string_foc "\n"  }


      # save targets in CCD-pixel format
      # if {$corr_flag} {
      #  puts stdout  [format "INS.TARG$s_c.X          %05.3f" $targw($i,cx) ]
      #  puts $fileId [format "INS.TARG$s_c.X          %05.3f" $targw($i,cx) ]
      # } else {
      #  puts stdout  [format "INS.TARG$s_c.X          %05.3f" $targw($i,x) ]
      #  puts $fileId [format "INS.TARG$s_c.X          %05.3f" $targw($i,x) ]
      # }
      #  puts stdout  [format "INS.TARG$s_c.Y          %05.3f" $targw($i,y) ]
      #  puts $fileId [format "INS.TARG$s_c.Y          %05.3f" $targw($i,y) ]


#      if {$corr_flag} {
#        puts stdout [ format \
#        "INS.TARG$i.ALPHA  %02s%02s%06.3f \
#         INS.TARG$i.DELTA  %02s%02s%06.3f \
#         INS.TARG$i.X %05.3f \
#         INS.TARG$i.Y %05.3f" \
#         $a1 $a2 $a3 $d1 $d2 $d3 $targw($i,cx) $targw($i,y) ]
#       } else {
#        puts stdout [ format \
#        "INS.TARG$i.ALPHA  %02s%02s%06.3f \
#         INS.TARG$i.DELTA  %02s%02s%06.3f \
#         INS.TARG$i.X %05.3f  \
#         INS.TARG$i.Y %05.3f" \
#         $a1 $a2 $a3 $d1 $d2 $d3 $targw($i,x) $targw($i,y) ]
#       }

      }

          }

        }




        puts stdout "[C_ref_targ nort] Reference Objects selected"

     # Reference Objects already in correct coords
     # save reference objects to setup file
     for { set j 1} {$j <= [C_ref_targ nort] } { incr j 1 } {
         set z [expr $j + 30]; # the INS.TARG$z. number

         lassign [split [C_ref_targ get $j a] :] a1 a2 a3
      #  puts stdout  [format "INS.TARG$z.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]
         puts $fileId [format "INS.TARG$z.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]

         lassign [split [C_ref_targ get $j d] :] d1 d2 d3
      #  puts stdout  [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
         puts $fileId [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

         if {$corr_flag} {
         set new_ref [calc_zemax_ref [C_ref_targ get $j a] \
                     [C_ref_targ get $j d] ]
         lassign [split $new_ref " " ] a0 d0
         puts stdout "new REF COORDS : >$a0<  >$d0<"

         # apply differential refraction 
         if {$dire_flag} {
            set new_ref [calc_diff_refrac 0 $a0 $d0]
            lassign [split $new_ref " " ] a0 d0
            puts stdout "new REF COORDS : >$a0<  >$d0<"
            }
 
         lassign [split $a0 :] a1 a2 a3
         lassign [split $d0 :] d1 d2 d3
         }
   append string_targ [format "INS.TARG$z.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
           if {$sm} { append string_targ "\n"  }
   append string_targ [format "INS.TARG$z.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
           if {$sm} { append string_targ "\n"  }
   # append string_foc  [format "INS.TARG$z.ALPHA  %02s%02s%06.3f\n" $a1 $a2 $a3]
   # append string_foc  [format "INS.TARG$z.DELTA  %02s%02s%06.3f\n" $d1 $d2 $d3]
         }

     #--------------------
     # reference targets end
     #--------------------


     #  save also grism, OS filter and collimator 
     puts $fileId [format "SEQ.PREP.LMIN   %8.3f" $L_MIN ]
     puts $fileId [format "SEQ.PREP.LMAX   %8.3f" $L_MAX ]
     puts $fileId [format "SEQ.PREP.GRIS   %10s" [$this cget -grism] ]
     puts $fileId [format "SEQ.PREP.FILT   %10s" [$this cget -filter] ]
     puts $fileId [format "SEQ.PREP.MASK_NA %10s" [lindex $PM 0] ]
     puts $fileId [format "SEQ.PREP.MASK_ID %10s" [lindex $PM 1] ]
     puts $fileId "SEQ.PREP.COLL   [$this cget -collimat]"
     append string_targ "INS.COLL.NAID   [$this cget -collimat]"

     close $fileId

     # save the same stuff again as one string to a 
     # file to be called by the P2PP tool   
     set myfile $set_dir/$filename.p_targ
     #                             ------
     set fileId [open $myfile w ]
     puts -nonewline $fileId $string_targ
     close $fileId

     set myfile $set_dir/$filename.p_focf
     #                             ------
     set fileId [open $myfile w ]
     puts -nonewline $fileId $string_foc
     close $fileId


     # save the same stuff again but in a more readable format 
     # into the .fims/LOG directory (only for maintenance) 

     # set s2t [ldelete_all [split $string_targ " "]  ""]
     # set number [llength $s2t]
     # for {set i 0} {$i < $number} { incr i 2 } {
     #     set s2t [ lreplace $s2t $i $i [lrange $s2t $i [expr $i+1] ] ]
     #     }
     # for {set i [expr $number-1] } {$i > 0} { incr i -2 } {
     #     set s2t [ lreplace $s2t $i $i ]
     #     }
     # set string2_targ [join $s2t "\n" ]

     # set myfile $log_dir/$filename.targ
     #                             ----
     # set fileId [open $myfile w ]
     # puts $fileId $string2_targ
     # close $fileId

     # set s2f [ldelete_all [split $string_foc " "]  ""]
     # set number [llength $s2f]
     # for {set i 0} {$i < $number} { incr i 2 } {
     #     set s2f [ lreplace $s2f $i $i [lrange $s2f $i [expr $i+1] ] ]
     #     }
     # for {set i [expr $number-1] } {$i > 0} { incr i -2 } {
     #     set s2f [ lreplace $s2f $i $i ]
     #     }
     # set string2_foc [join $s2f "\n" ]

     # set myfile $log_dir/$filename.focf
     #                             ----
     # set fileId [open $myfile w ]
     # puts $fileId $string2_foc
     # close $fileId

     incr exposure_counter 1

     # select_file $myfile 
     }




     #=======================================
     private method save_mxu_MSF {filename} {
     #=======================================
     # saving mask setup for MXU in MachineSlitFile format 
     set string_msf  ""
     # msf_header
     # see VLT-TRE-VIRG-1416-0057 05/02/99 Issue 1 Rev 1, page 33  
     # ---------------------------------------------------------
     set corr_flag [$this cget -fors_frame]

     # <><><><><><><><><><><><><><><><><><><><><>
     # workaround 13. Sept 2000 WH
     # FORS1 images are aliens fro MXU 
     #if {[string compare [$m_image fits get INSTRUME] "FORS1"]==0} {
     #   set corr_flag 0 }
     # <><><><><><><><><><><><><><><><><><><><><>

     set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]

     append string_msf [format "FORS2\n" ]
     append string_msf [format "0\n"]                               
     append string_msf [format "%6d\n" [lindex $PM 1] ] 

     # determine number of rectangular and curved slits 
     set STRA_sc 0;   # slit counter STRAIGHT 
     set CURV_sc 0;   # slit counter CURVED       
     set CIRC_sc 0;   # slit counter CIRCLE       
     set noms [fors::MXU_SLIT::nos]
     if {$noms>0} {
       for {set i 1} {$i <= $noms} { incr i 1} { 
         # if slit has is specified 
         if {[C_mxus$i cget -ON]} {
            set loc_shap [C_mxus$i cget -shape]
            
            # count the number of slits 
            switch -exact -- $loc_shap {
             STRAIGHT { incr STRA_sc 1 }
             CURVED   { incr CURV_sc 1 }
             CIRCLE   { incr CIRC_sc 1 }
                 }

            }
         }
     append string_msf [format "%d\n" $STRA_sc ]
     append string_msf [format "%d\n" $CURV_sc ]
     append string_msf [format "%d\n" $CIRC_sc ]

     if {$noms>0} {

        # first the straight slits 
        # -------------------------
        for {set i 1} {$i <= $noms} { incr i 1} { 
            if {[C_mxus$i cget -ON]} {
               if {[string match *STRAIGHT* [C_mxus$i cget -shape]]} { 

                    if {$corr_flag} {
                       # set slit [C_mxus$i get_oc_slit] 
                       set slit [C_mxus$i get_oc_slit_mmu] 
                       } else {
                       # set slit [C_mxus$i get_slit]
                       set slit [C_mxus$i get_slit_mmu]
                       }
               append string_msf [format "%9.3f   "  [lindex $slit 0] ]
               append string_msf [format "%9.3f   "  [lindex $slit 1] ]
               append string_msf [format "%9.3f   "  [lindex $slit 2] ]
               append string_msf [format "%9.3f   "  [lindex $slit 3] ]
               append string_msf [format "%9.3f\n"   [C_mxus$i cget -pos_ang] ]
                      }
                }
            }

        # now the curved  slits 
        # --------------------
        for {set i 1} {$i <= $noms} { incr i 1} { 
            if {[C_mxus$i cget -ON]} {
               if {[string match *CURVED* [C_mxus$i cget -shape]]} { 

                    if {$corr_flag} {
                       # set slit [C_mxus$i get_oc_slit] 
                       set slit [C_mxus$i get_oc_slit_mmu] 
                       } else {
                       # set slit [C_mxus$i get_slit]
                       set slit [C_mxus$i get_slit_mmu]
                       }
               append string_msf [format "%9.3f   "  [lindex $slit 0] ]
               append string_msf [format "%9.3f   \n"  [lindex $slit 1] ]
                      }
                }
            }

        # now the circle slits 
        # --------------------
        for {set i 1} {$i <= $noms} { incr i 1} { 
            if {[C_mxus$i cget -ON]} {
               if {[string match *CIRCLE* [C_mxus$i cget -shape]]} { 

                    if {$corr_flag} {
                       # set slit [C_mxus$i get_oc_slit] 
                       set slit [C_mxus$i get_oc_slit_mmu] 
                       } else {
                       # set slit [C_mxus$i get_slit]
                       set slit [C_mxus$i get_slit_mmu]
                       }
               append string_msf [format "%9.3f   "  [lindex $slit 0] ]
               append string_msf [format "%9.3f   "  [lindex $slit 1] ]
               append string_msf [format "%9.3f   \n"  [lindex $slit 2] ]
                      }
                }
            }


         }
        }


     #  SAVE  --- MSF ---
     set myfile $log_dir/$filename.fad 
     set fileId [open $myfile w ]
     puts -nonewline $fileId $string_msf
     close $fileId
     }

     #=======================================
     private method save_mxu_ADF {filename} {
     #=======================================
     # saving mask setup for MXU in ApertureDefinitionFile format 
     set string_adf  ""
     set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]

     # adf_header
     # see VLT-TRE-VIRG-1416-0058 02/07/98 Issu1 Rev 0, page 120
     # ---------------------------------------------------------
     set dump "user-date-host : [exec whoami] [exec date -u +%Y-%m-%dT%H:%M:%S] $lochost"
     # apply optical correction if fits-image from fors
     set corr_flag [$this cget -fors_frame]

     # <><><><><><><><><><><><><><><><><><><><><>
     # workaround 13. Sept 2000 WH
     # FORS1 images are aliens fro MXU 
     #if {[string compare [$m_image fits get INSTRUME] "FORS1"]==0} {
     #   set corr_flag 0 }
     # <><><><><><><><><><><><><><><><><><><><><>

     append string_adf [format "TYPE        MOS\n"]
     append string_adf [format "VERSION     1\n"]                               
     append string_adf [format "MASK NA     %30s\n" [lindex $PM 0]  ]
     append string_adf [format "NAME        %50s\n"  $dump ]  
     append string_adf [format "MASK ID     %6d\n" [lindex $PM 1] ]
     append string_adf [format "MASK NAID   %30s\n" \
            [lindex $PM 0]+$exposure_counter ]
     append string_adf [format "QUAD        %d\n" 0 ]
     append string_adf [format "PLATESCALE  %10.8f\n" $focscale ]
     append string_adf [format "REF NO      %d\n" 0 ]
     append string_adf [format "SLIT NO     %d\n" [fors::MXU_SLIT::nos] ]

      # ----------------
      # --- slit loop ---
      # -----------------
     set slit_c 0
     set noms [fors::MXU_SLIT::nos]
     if {$noms>0} {
        for {set i 1} {$i <= $noms} { incr i 1} { 
            # if slit has is specified 
            if {[C_mxus$i cget -ON]} {
               incr slit_c 1;   # count only the non-deleted slits 
               puts stdout  [format "INS.TARG$i.NAME  %19s" \
                            [C_mxus$i cget -target_name] ]
               append string_adf [format "SLIT%d ID          %d\n" $slit_c $i]
  append string_adf [format "SLIT%d NAME  %19s\n" $slit_c \
                            [C_mxus$i cget -target_name] ]
  append string_adf [format "SLIT%d TYPE  %19s\n" $slit_c STRAIGHT ]

               # slit dimensions and wcs   
               if {$corr_flag} {
                  # set slit [C_mxus$i get_oc_slit]
                  set slit [C_mxus$i get_oc_slit_mmu]
                  lassign [split [C_mxus$i get_OC_RA ] :] a1 a2 a3
                  lassign [split [C_mxus$i get_OC_DEC] :] d1 d2 d3
               } else {
                  # set slit [C_mxus$i get_slit]
                  set slit [C_mxus$i get_slit_mmu]
                  lassign [split [C_mxus$i get_RA ] :] a1 a2 a3
                  lassign [split [C_mxus$i get_DEC] :] d1 d2 d3
               }

 append string_adf [format "SLIT%d X      %5.3f\n" $slit_c [lindex $slit 0] ]
 append string_adf [format "SLIT%d Y      %5.3f\n" $slit_c [lindex $slit 1] ]
 append string_adf [format "SLIT%d DIMX   %5.3f\n" $slit_c [lindex $slit 2] ]
 append string_adf [format "SLIT%d DIMY   %5.3f\n" $slit_c [lindex $slit 3] ]
 append string_adf [format "SLIT%d OBJ NO %d\n" $slit_c 1        ]
 append string_adf [format "SLIT%d OBJ X  %5.3f\n" $slit_c [lindex $slit 0] ]
 append string_adf [format "SLIT%d OBJ Y  %5.3f\n" $slit_c [lindex $slit 1] ]
 append string_adf [format "SLIT%d OBJ RA %02s%02s%06.3f\n" \
                                           $slit_c $a1 $a2 $a3 ]
 append string_adf [format "SLIT%d OBJ DEC %02s%02s%06.3f\n" \
                                            $slit_c $d1 $d2 $d3 ]
 append string_adf [format "SLIT%d OBJ LWIDTH     %5.3f\n" $slit_c 0.815  ]
               }
            }
      }

     #  SAVE  --- ADF ---
     set myfile $log_dir/$filename.adf 
     set fileId [open $myfile w ]
     puts -nonewline $fileId $string_adf
     close $fileId
     }


     #============================
     private method save_mxu_RS274X {filename} {
     #============================
     # saving mask setup for MXU as Gerber format 
     # the gerber file has been testet and improved 
     # in June 1999 with the LPKF SW and Stencil laser 
     # cutting machine in Milano
     #
     # the empirical offset will be applied to the contour only 
     # not to the slit positions 

     # offset in focal field mm units 
     # 00/06/99 W.Hummel initial offset 0.000 0.000 mm
     # 23/07/99 W.Hummel   first offset 0.353 1.398 mm
     set FF_X 0.353; # empirical offset of the mask rim to coincide
                   # slits   
     set FF_Y 1.398; # empirical offset of the mask rim to coincide
                   # slits  

     set MM_X $FF_Y;               # offset in mm of the mmu working table 
     set MM_Y [expr -1.0 * $FF_X]; # offset in mm of the mmu working table 


     set string_g ""
     # we create two files, one for p2pp and for direct run
     set string_p ""; set string_g ""
     # with two different separators
     set S "?"; set C "\n"
     set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
     set noms [fors::MXU_SLIT::nos]

     # PAF header for .p_gbr file
     # --------------------------------

append string_g "PAF.HDR.START;                            # Marks start of header\n"
append string_g "PAF.TYPE          \"Mask Manufacuring Unit Executable\";   # Type of parameter file\n"
append string_g "PAF.ID            \"FORS2-[string toupper [$this cget -modus] ]\";   # Parameter file ID\n"
# append string_g "PAF.NAME          \"\$INS_ROOT/SYSTEM/COMMON/SETUPFILES/$filename.p_gbr\";  # Parameter file NAME\n"
append string_g "PAF.NAME          \"$filename.p_gbr\";  # Parameter file NAME\n"
append string_g "PAF.DESC          \"gerber file, executable for MMU\";  # Purpose\n"
append string_g "PAF.CRTE.NAME     \"FIMS $f_v\";                 # SW version date UT\n"
append string_g "PAF.CRTE.DAYTIM   \"[exec date -u +%Y-%m-%dT%H:%M:%S]\";  # Date and time of creation\n"
append string_g "PAF.LCHG.NAME     \"\";                     # Who did last change\n"
append string_g "PAF.LCHG.DAYTIM   \"\";                     # Date and time of last change\n"
append string_g "PAF.CHCK.NAME     \"\";                     # Appl. checking par. file\n"
append string_g "PAF.CHCK.DAYTIM   \"\";                     # Date and time of lastcheck\n"
append string_g "PAF.CHCK.CHECKSUM \"\";                     # Parameter file checksum\n"
append string_g "PAF.HDR.END;                                # Marks end of header\n\n"

append string_g "TPL.FILE.DIRNAME    \$INS_ROOT/SYSTEM/COMMON/SETUPFILES/INS\n"


# -------------
# info header 
# -------------

  set a "G04: created by SKYCAT-TOOL V$s_v -- FIMS-plugin V$f_v*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: user-date-host : [exec whoami] [exec date -u +%Y-%m-%dT%H:%M:%S] $lochost*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: orig filename  : $filename*"
  append string_g $a$C; append string_p $a$S;
     # puts stdout  "# orig filename  : $filename"
  set a "G04: fits file dir  : \
                     [ [.skycat1 component image] cget -file]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: fits file name : [$this get_fits_name]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: fits instrument: [$m_image fits get INSTRUME]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: fits origin    : [$m_image fits get ORIGIN]*" 
  append string_g $a$C; append string_p $a$S;
  set a "G04: apply optidist : [$this cget -fors_frame]*" 
  append string_g $a$C; append string_p $a$S;
  set a "G04: fits object    : [$m_image fits get OBJECT]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: catalog name   : $CatFileName*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: INS.MASK.NA    : [lindex $PM 0]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: INS.MASK.ID    : [lindex $PM 1]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: INS.MASK.NAID  : [lindex $PM 0]+[lindex $PM 1]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: RS-274X for    : [$this cget -instrument]-\
      [string toupper [$this cget -modus] ]*"
  append string_g $a$C; append string_p $a$S;
     set a "G04:  --------------------------------------------*"
  append string_g $a$C; append string_p $a$S;

# -------------
# gerberX setup 
# -------------

     set a "%IN FORS2_MXU_200x260_[lindex $PM 1]*%"
  append string_g $a$C; append string_p $a$S;
     set a "%ASAXBY*%"
  append string_g $a$C; append string_p $a$S;
     set a "%FSLAX33Y33*%"
  append string_g $a$C; append string_p $a$S;
     set a "%MOMM*%"
  append string_g $a$C; append string_p $a$S;
#    set a "%OFA0B0*%"
# append string_g $a$C; append string_p $a$S;
     set a "%SFA1.0B1.0*%"
  append string_g $a$C; append string_p $a$S;
     set a "%IOA0B0*%"
  append string_g $a$C; append string_p $a$S;
#    set a "%IPPOS*%"
# append string_g $a$C; append string_p $a$S;
     set a "%IR0*%"
  append string_g $a$C; append string_p $a$S;

# define apertures 

     set a "G04______________DEFINE APERTURES*"
  append string_g $a$C; append string_p $a$S;
     set a "%ADD10C,0.0*%"
  append string_g $a$C; append string_p $a$S;
     for {set i 1} {$i<=$noms} { incr i 1} {
        if { [C_mxus$i cget -ON] } {
#       [string length [C_mxus$i get_add_mmu] ] 
           set a [C_mxus$i get_add_mmu]
           # puts stdout $a 
  append string_g $a$C; append string_p $a$S;
           }
         }

# -------------
# slits 
# -------------

     set a "G04_______________SLIT_CUTTING*"
  append string_g $a$C; append string_p $a$S;
     set a "%LNSLITS*%"
  append string_g $a$C; append string_p $a$S;


     set a "G90*";      # once more absolute 
  append string_g $a$C; append string_p $a$S;
     set a "G54D10*";   # small aperture 
  append string_g $a$C; append string_p $a$S;
     set a "D02*";      # close shutter 
  append string_g $a$C; append string_p $a$S;

     # we ask all slits on their size (width, length) and
     # orientation (posang). Each rectangular slit with 
     # zero pos_angle will get its own aperture definition 
     set ad_counter 10
     set noms [fors::MXU_SLIT::nos]
     if {$noms>0} {
     for {set i 1} {$i <= $noms} { incr i 1} { 

         # if slit has is specified 
         if {[C_mxus$i cget -ON]} {

           set loc_shape [C_mxus$i cget -shape]

           switch -exact -- $loc_shape {
 
              STRAIGHT {
             # we have a non-deleted rectangular slit 
             if {[fors::MXU_SLIT::get_mmff]} {
              # technical mask production, 
              # do not use OC values 
             set slit [C_mxus$i get_slit_mmu]
             } else {
             set slit [C_mxus$i get_oc_slit_mmu]
             }

             set lw [F2G [lindex $slit 3] ]
             set ll [F2G [lindex $slit 2] ]; #slit dimensions 
             set lang [F2G [C_mxus$i cget -pos_ang] ]
               if {[string length [C_mxus$i get_add_mmu] ] } {

                  # slit not rotated, use flash command 
                  set a "G04________SLIT$i RECT $lw $ll $lang *"
  append string_g $a$C; append string_p $a$S;
                  set a G54D[expr $i+$ad_counter]* 
  append string_g $a$C; append string_p $a$S;
                  set a "X[F2G [lindex $slit 0]]"
                  append a "Y[F2G [lindex $slit 1]]D03*"
  append string_g $a$C; append string_p $a$S;

                  } else {

                  # slit rotated to parallelogram, use individual contour 
                  if {[fors::MXU_SLIT::get_mmff]} {
                     # technical mask production, 
                     # do not use OC values 
                    set slit_dim [C_mxus$i get_slit_contour_mmu]
                    } else {
                    set slit_dim [C_mxus$i get_oc_slit_contour_mmu]
                    }
                  # puts stdout ">$slit_dim<"
                  # convert 123.456789 to 123456 
              set a "G04________SLIT$i PARA $lw $ll $lang *"
  append string_g $a$C; append string_p $a$S;
              set a "G54D10*" 
  append string_g $a$C; append string_p $a$S;
              set a "X[F2G [lindex $slit_dim 0]]"
              puts stdout [lindex $slit_dim 1]
              append a "Y[F2G [lindex $slit_dim 1]]D02*"
  append string_g $a$C; append string_p $a$S;
              set a "X[F2G [lindex $slit_dim 2]]"
              append a "Y[F2G [lindex $slit_dim 3]]D01*"
  append string_g $a$C; append string_p $a$S;
              set a "X[F2G [lindex $slit_dim 4]]"
              append a "Y[F2G [lindex $slit_dim 5]]D01*"
  append string_g $a$C; append string_p $a$S;
              set a "X[F2G [lindex $slit_dim 6]]"
              append a "Y[F2G [lindex $slit_dim 7]]D01*"
  append string_g $a$C; append string_p $a$S;
              set a "X[F2G [lindex $slit_dim 0]]"
              append a "Y[F2G [lindex $slit_dim 1]]D01*"
  append string_g $a$C; append string_p $a$S;

                  }
               }
      CIRCLE { 
             # we have a non-deleted circle aperture
             if {[fors::MXU_SLIT::get_mmff]} {
              # technical mask production, 
              # do not use OC values 
             set slit [C_mxus$i get_slit_mmu]
             } else {
             set slit [C_mxus$i get_oc_slit_mmu]
             }

             set lw [F2G [lindex $slit 3] ]
             # use flash command 
             set a "G04________SLIT$i CIRC $lw *"
             append string_g $a$C; append string_p $a$S;
             set a G54D[expr $i+$ad_counter]* 
             append string_g $a$C; append string_p $a$S;
             set a "X[F2G [lindex $slit 0]]"
             append a "Y[F2G [lindex $slit 1]]D03*"
             append string_g $a$C; append string_p $a$S;
             }
      CURVED { 
             # we have a non-deleted curved slit 
             if {[fors::MXU_SLIT::get_mmff]} {
                puts stdout "Error: CURVED in standard mask !!!"
                } 
             set slit_dim [C_mxus$i get_b_oc_mmu]
             # puts stdout ">$slit_dim<"
             set lw [C_mxus$i cget -slit_width]
             set a "G04________SLIT$i CURV $lw *"
             append string_g $a$C; append string_p $a$S;
             set a "G54D10*" 
             append string_g $a$C; append string_p $a$S;
             #
             # index of the six bezier points
             #
             #    0,1 +----+ 6,7
             #         \    \
             #          \    \
             #           \    \
             #           /    /  2,3 +     + 8,9
             #      4,5 +----+ 10,11
             #          
             # flash the first point
             # convert 123.456789 to 123456 
             set a "X[F2G [lindex $slit_dim 0]]"
             append a "Y[F2G [lindex $slit_dim 1]]D02*"
             append string_g $a$C; append string_p $a$S;


             # number of polygon steps for a curved slit 
             set res 200

             #   1 pixel = 0.2 '' = 0.1 mm = 100 mu 
             # 200 pixel = 40''   = 20  mm = 20000 mu (maximum slit length)

             # red Bezier
             set x1 [lindex $slit_dim 0]
             set y1 [lindex $slit_dim 1]
             set x2 [lindex $slit_dim 2]
             set y2 [lindex $slit_dim 3]
             set x3 [lindex $slit_dim 4]
             set y3 [lindex $slit_dim 5]

             set x0 $x1; set y0 $y1
             for {set j 0} {$j <= $res} {incr j 1} {
                  set t [expr (1.0*$j) / (1.0*$res) ]
                  set xt [expr (1-$t)*(1-$t)*$x1+2*(1-$t)*$t*$x2+$t*$t*$x3]
                  set yt [expr (1-$t)*(1-$t)*$y1+2*(1-$t)*$t*$y2+$t*$t*$y3]

                  set a "X[F2G $xt]"
                  append a "Y[F2G $yt]D01*"
                  append string_g $a$C; append string_p $a$S;

                  set x0 $xt; set y0 $yt
                  }

             # blue Bezier
             # draw in opposite direction 
             set x1 [lindex $slit_dim 10]
             set y1 [lindex $slit_dim 11]
             set x2 [lindex $slit_dim 8]
             set y2 [lindex $slit_dim 9]
             set x3 [lindex $slit_dim 6]
             set y3 [lindex $slit_dim 7]

             # draw from red contour to blue contour
             set a "X[F2G $x1]"
             append a "Y[F2G $y1]D01*"
             append string_g $a$C; append string_p $a$S;
             
             set x0 $x1; set y0 $y1
             for {set j 0} {$j <= $res} {incr j 1} {
                  set t [expr (1.0*$j) / (1.0*$res) ]
                  set xt [expr (1-$t)*(1-$t)*$x1+2*(1-$t)*$t*$x2+$t*$t*$x3]
                  set yt [expr (1-$t)*(1-$t)*$y1+2*(1-$t)*$t*$y2+$t*$t*$y3]

                  set a "X[F2G $xt]"
                  append a "Y[F2G $yt]D01*"
                  append string_g $a$C; append string_p $a$S;

                  set x0 $xt; set y0 $yt
                  }

             # draw blue conour back to red contour and end
             set x1 [lindex $slit_dim 0]
             set y1 [lindex $slit_dim 1]
             set a "X[F2G $x1]"
             append a "Y[F2G $y1]D01*"
             append string_g $a$C; append string_p $a$S;
             }
              } 

             }
           }
         }


# -------------
# mask id 
# -------------
#
# W A R N I N G 
# ---------------
#
# This part of the code exists as a copy in the 
# m2g script to insert the mask id to science masks
# as part nof the science opartion on Paranal
# any change in this part of the code must be 
# mirrored in the m2g script 


     set maskid [lindex $PM 1]
      
     # new concept since 12/1999:
     # sky masks contain no mask id
     # technical masks contain mask id 

     # include MASD.ID only to technical masks
     # if {$maskid > 900000 && $maskid <= 900100} 
        
        if {$noms>1000} { puts stdout \
           "Error: number of apertures collides with maskid" 
           }

         # this is a technical mask
         # create gerber file with mask id 

     set a "G04______________MASK_ID_CUTTING $maskid*"
  append string_g $a$C; append string_p $a$S;
     # this defines a new layer for the LPKF SW
     # MASK ID will be cut faster
     set a "%LNBARCODE*%"
  append string_g $a$C; append string_p $a$S;

        # philosophy of mask id creation
        #
        #  --   this is a '8' consisting of 7 bars.
        # |  |  each bar is l x b large (length x width=breite).
        #  --   the full digit is 2s x 4s large, where 
        # |  |  2s is a little bit larger than l in order to 
        #  --   avoid overlapping bars. 
        #       You can change the total size of the MASKID digit 
        #       by parameter s. The digit is 2*s x 4*s mm large. 
        #       You can change the layout of the digit with 
        #       parameter l. 
        #       Note that l must be smaller than 2*s in any case. 


        # size of the digit
        set s  3.0;   # digit size = 2s x 4s 
        set l  4.0;   # length of a bar, l < 2s !!
        set b  [expr (2.0*$s-$l)/4.0]; # width of a bar
        set ss [expr $s*2.0]

        if {$l >= $ss} { puts stdout "ERROR: MASK_ID font size" }

        # for mmu working table switch l and b
        # set dummy $l; 
        # set l $b; 
        # set b $dummy

        puts -nonewline stdout "MASK_ID ="
    for {set k 0} {$k<=5} {incr k 1} {

        # central position of the digit in the focal field
        # set x0 [expr -75+$k*15]
        # set y0 -140
        # central position  on the mmu working table 
        set x0 -140 
        set y0 [expr 75-$k*15]


        # select the digit
        set ziffer [string index $maskid $k ]
        # puts stdout ">>$ziffer<<"

        # we paint digits directly in mmu working table coords

        switch -exact -- $ziffer {
        0 { # puts "$k th number is a 0" 
            puts -nonewline stdout  0
          set a "%AMZERO*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                         21,1,\$2,\$1,\$3+$ss,\$4,0*\
                         21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                         21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1000ZERO,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1000*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        1 { # puts stdout "$k th number is a 1" 
            puts -nonewline stdout  1
          set a "%AMONE*21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                        21,1,\$1,\$2,\$3-$s,\$4-$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1001ONE,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1001*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        2 { # puts stdout "$k th number is a 2" 
            puts -nonewline stdout  2
          set a "%AMTWO*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                        21,1,\$2,\$1,\$3+$ss,\$4,0*\
                        21,1,\$2,\$1,\$3,\$4,0*\
                        21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                        21,1,\$1,\$2,\$3+$s,\$4-$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1002TWO,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1002*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        3 { # puts stdout "$k th number is a 3" 
            puts -nonewline stdout  3
          set a "%AMTHREE*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                          21,1,\$2,\$1,\$3+$ss,\$4,0*\
                          21,1,\$2,\$1,\$3,\$4,0*\
                          21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                          21,1,\$1,\$2,\$3+$s,\$4-$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1003THREE,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1003*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        4 { # puts stdout "$k th number is a 4"
            puts -nonewline stdout  4
          set a "%AMFOUR*21,1,\$2,\$1,\$3,\$4,0*\
                         21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1004FOUR,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1004*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        5 { # puts stdout "$k th number is a 5" 
            puts -nonewline stdout  5
          set a "%AMFIVE*21,1,\$2,\$1,\$3,\$4,0*\
                         21,1,\$2,\$1,\$3+$ss,\$4,0*\
                         21,1,\$2,\$1,\$3-$ss,\$4,0*\
                         21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1005FIVE,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1005*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        6 { # puts stdout "$k th number is a 6" 
            puts -nonewline stdout  6
          set a "%AMSIX*21,1,\$2,\$1,\$3,\$4,0*\
                        21,1,\$2,\$1,\$3+$ss,\$4,0*\
                        21,1,\$2,\$1,\$3-$ss,\$4,0*\
                        21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                        21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                        21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1006SIX,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1006*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        7 { # puts stdout "$k th number is a 7" 
            puts -nonewline stdout  7
          set a "%AMSEVEN*21,1,\$2,\$1,\$3+$ss,\$4,0*\
                          21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                          21,1,\$1,\$2,\$3-$s,\$4-$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1007SEVEN,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1007*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        8 { # puts stdout "$k th number is a 8" 
            puts -nonewline stdout  8
          set a "%AMEIGHT*21,1,\$2,\$1,\$3,\$4,0*\
                          21,1,\$2,\$1,\$3+$ss,\$4,0*\
                          21,1,\$2,\$1,\$3-$ss,\$4,0*\
                          21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                          21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                          21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                          21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1008EIGHT,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1008*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        9 { # puts stdout "$k th number is a 9" 
            puts -nonewline stdout  9
          set a "%AMNINE*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                         21,1,\$2,\$1,\$3,\$4,0*\
                         21,1,\$2,\$1,\$3+$ss,\$4,0*\
                         21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4+$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4-$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1009NINE,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1009*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        A { puts stdout $k th number is a A, but is not yet supported" }
  default { puts stdout "something wrong " } 
         }
     }

    # end of : inbcluder mask id only for science masks 

     puts stdout "\n"

# -------------
# mask contour 
# -------------

# View A:
# You will hold it in your hand  in such a way
#
#  (94.9,126)    o     (-95,126)
#          +-----------+C               FF         MMU
#         / D           \
#       E+               +B              ^ Y          ^ X
#        |        _      |               |            |
#        |       |_      |o              |            |
#        |       |       |               +---X   <----+
#        |               |                        Y
#        |               |o
#        |---------------|    (-126)
#        | 987654        |
#       F+---------------+G  (-203)
#                        | 
#                        +A = starting point for cutting
#                           
# o = the three frame pins 
#
# View B: flip view A at the FG-axis
# You prepare it with fims in an orientation as viewed
# from bottom to top  
#
#                        +A = starting point 
#                        | 
# (-203)F+---------------+G  
#        |68L924         |
# (-126) |---------------|    (y=-126)
#        |    North      |o
#        |               |                X        Y
#   East |       |_      |West        +--->    <---+
#        |       |_      |            |            |
#        |               |o           |            |
#       E+               +B           V Y          V X
#         \ D   South   / 
#          +-----------+C                 FF         MMU
#  (-95,126)      o    (94.9,126)
# 
# 
#  View C: = view A rotated by 90deg
#  It will be cut on the MMU working table like this 
#
#      F                E        ^           +---->Y 
#      +--+-------------+        |Y          |
#      |_o|              \       |           |
#      |oo|               +D     |           |X
#      |  |    _ _        |      +---> X      
#      |  |     | |       |o                  
#      |  |               |                   
#      |  |               +C     MMU          FF  
#      |  |              /                    
#   A+-+--+-------------+B
#      G    o         o
#
# contours are hard-coded, fix-points at BC and AB
# x=94.9mm  means 0.2 mm space for thermal contraction of the frame
# focal field ends in -Y at C=-126mm but there is still about 75mm 
# space available in -Y for mask ID cutting 

     set a "G04______________CONTOUR_CUTTING*"
  append string_g $a$C; append string_p $a$S;
     set a "%LNCONTOUR*%"
  append string_g $a$C; append string_p $a$S;
     set a "G90*";                 # absolute
  append string_g $a$C; append string_p $a$S;
     set a "G54D10*";              # circle apertur 
  append string_g $a$C; append string_p $a$S;
     # round corners 
     # at B and E, we reduce the plate width by 2mm 
     # on 2cm in y 

    # in focal field coords 
    # apply FF_X, FF_Y 
#    set a "X-095000Y-164000D02*";            # A = starting point
#    set a "X-095000Y106000D01*";             # B 
#    set a "X-093000Y126000D01*";             # C
#    set a "X092900Y126000D01*";              # D
#    set a "X094900Y106000D01*";              # E
#    set a "X094900Y-162000D01*";             # F
#    set a "X-095000Y-162000D01*";            # G

    # in mmu working table coords, x=y, y=x 

    # original contours, no offset applied
#     set a "X-164000Y-095000D02*";            # A = starting point
#  append string_g $a$C; append string_p $a$S;
#     set a "X106000Y-095000D01*";             # B 
#  append string_g $a$C; append string_p $a$S;
#     set a "X126000Y-093000D01*";             # C
#  append string_g $a$C; append string_p $a$S;
#     set a "X126000Y092900D01*";            # D
#  append string_g $a$C; append string_p $a$S;
#     set a "X106000Y094900D01*";           # E
#  append string_g $a$C; append string_p $a$S;
#     set a "X-162000Y094900D01*";           # F
#  append string_g $a$C; append string_p $a$S;
#     set a "X-162000Y-095000D01*";            # G
#  append string_g $a$C; append string_p $a$S;

    # apply MM_X, MM_Y 
     set a "X[F2G [expr -164.000+$MM_X]]Y[F2G [expr -095.000+$MM_Y]]D02*";# A 
  append string_g $a$C; append string_p $a$S;
     set a "X[F2G [expr 106.000+$MM_X]]Y[F2G [expr -095.000+$MM_Y]]D01*"; # B 
  append string_g $a$C; append string_p $a$S;
     set a "X[F2G [expr 126.000+$MM_X]]Y[F2G [expr -093.000+$MM_Y]]D01*"; # C
  append string_g $a$C; append string_p $a$S;
     set a "X[F2G [expr 126.000+$MM_X]]Y[F2G [expr 092.900+$MM_Y]]D01*";  # D
  append string_g $a$C; append string_p $a$S;
     set a "X[F2G [expr 106.000+$MM_X]]Y[F2G [expr 094.900+$MM_Y]]D01*";  # E
  append string_g $a$C; append string_p $a$S;
     set a "X[F2G [expr -162.000+$MM_X]]Y[F2G [expr 094.900+$MM_Y]]D01*"; # F
  append string_g $a$C; append string_p $a$S;
     set a "X[F2G [expr -162.000+$MM_X]]Y[F2G [expr -095.000+$MM_Y]]D01*"; # G
  append string_g $a$C; append string_p $a$S;


     # end 
     set a "D02*"
  append string_g $a$C; append string_p $a$S;
     set a "M02*"
  append string_g $a$C; append string_p $a$S;

     #  SAVE  --- gerber format --- hitherto used for P2PP
     # set myfile $log_dir/$filename.p_gbr
     # set fileId [open $myfile w ]
     # puts -nonewline $fileId $string_p
     # close $fileId
     # we will close this file soon 

# later at Paranal use UNIX command 
# 
# tr -s '?' '\012' < mxu2.60.test_0.p_gbr > mxu2.60.test_0.gbr
# 
# to connvert '?' to CR again
# conversion to DOS readable files:
# insert a Ctrl-M, use Ctrl-V Ctrl-M in vi for the script: 
# sed -e 's/$/^M/' $1 > $2 

     #  SAVE  --- gerber format --- for LPKF-SW  
     set myfile $set_dir/$filename.p_gbr 
     set fileId [open $myfile w ]
     puts -nonewline $fileId $string_g
     close $fileId
     }




     #============================
     private method save_mxu_RS274X_forPS  {filename} {
     #============================
     # saving mask setup for MXU as Gerber format 
     # the gerber file has been testet and improved 
     # in June 1999 with the LPKF SW and Stencil laser 
     # cutting machine in Milano
     #
     # the empirical offset will be applied to the contour only 
     # not to the slit positions 

     # offset in focal field mm units 
     # 00/06/99 W.Hummel initial offset 0.000 0.000 mm
     # 23/07/99 W.Hummel   first offset 0.353 1.398 mm
     set FF_X 0.353; # empirical offset of the mask rim to coincide
                   # slits   
     set FF_Y 1.398; # empirical offset of the mask rim to coincide
                   # slits  

     # set MM_X $FF_Y;             # offset in mm of the mmu working table 
     # set MM_Y [expr -1.0 * $FF_X]; # offset in mm of the mmu working table 
     
     set MM_X $FF_X; set MM_Y $FF_Y; 

     set OFFX 140.0; set OFFY 200.0

     set MM_X [expr $MM_X + $OFFX]; # special offset to create gerberfiel with
                                    # all posotions x,y>0
     set MM_Y [expr $MM_Y + $OFFY]; # special offset to create gerberfiel with
                                    # all posotions x,y>0
     set string_g ""
     # we create two files, one for p2pp and for direct run
     set string_p ""; set string_g ""
     # with two different separators
     set S "?"; set C "\n"
     set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
     set noms [fors::MXU_SLIT::nos]

     # PAF header for .p_gbr file
     # --------------------------------

append string_g "G04: PAF.HDR.START;                            # Marks start of header\n"
append string_g "G04: PAF.TYPE          \"Mask Manufacuring Unit Executable\";   # Type of parameter file\n"
append string_g "G04: PAF.ID            \"FORS2-[string toupper [$this cget -modus] ]\";   # Parameter file ID\n"
append string_g "G04: PAF.NAME          \"\$INS_ROOT/SYSTEM/COMMON/SETUPFILES/$filename.p_gbr\";  # Parameter file NAME\n"
append string_g "G04: PAF.DESC          \"gerber file, executable for MMU\";  # Purpose\n"
append string_g "G04: PAF.CRTE.NAME     \"FIMS $f_v\";                 # SW version date UT\n"
append string_g "G04: PAF.CRTE.DAYTIM   \"[exec date -u +%Y-%m-%dT%H:%M:%S]\";  # Date and time of creation\n"
append string_g "G04: PAF.LCHG.NAME     \"\";                     # Who did last change\n"
append string_g "G04: PAF.LCHG.DAYTIM   \"\";                     # Date and time of last change\n"
append string_g "G04: PAF.CHCK.NAME     \"\";                     # Appl. checking par. file\n"
append string_g "G04: PAF.CHCK.DAYTIM   \"\";                     # Date and time of lastcheck\n"
append string_g "G04: PAF.CHCK.CHECKSUM \"\";                     # Parameter file checksum\n"
append string_g "G04: PAF.HDR.END;                                # Marks end of header\n\n"


# -------------
# info header 
# -------------

  set a "G04: created by SKYCAT-TOOL V$s_v -- FIMS-plugin V$f_v*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: user-date-host : [exec whoami] [exec date -u +%Y-%m-%dT%H:%M:%S] $lochost*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: orig filename  : $filename*"
  append string_g $a$C; append string_p $a$S;
     # puts stdout  "# orig filename  : $filename"
  set a "G04: fits file dir  : \
                     [ [.skycat1 component image] cget -file]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: fits file name : [$this get_fits_name]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: fits instrument: [$m_image fits get INSTRUME]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: fits origin    : [$m_image fits get ORIGIN]*" 
  append string_g $a$C; append string_p $a$S;
  set a "G04: apply optidist : [$this cget -fors_frame]*" 
  append string_g $a$C; append string_p $a$S;
  set a "G04: fits object    : [$m_image fits get OBJECT]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: catalog name   : $CatFileName*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: INS.MASK.NA    : [lindex $PM 0]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: INS.MASK.ID    : [lindex $PM 1]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: INS.MASK.NAID  : [lindex $PM 0]+[lindex $PM 1]*"
  append string_g $a$C; append string_p $a$S;
  set a "G04: RS-274X for    : [$this cget -instrument]-\
      [string toupper [$this cget -modus] ]*"
  append string_g $a$C; append string_p $a$S;
     set a "G04:  --------------------------------------------*"
  append string_g $a$C; append string_p $a$S;

# -------------
# gerberX setup 
# -------------

     set a "G04: %IN FORS2_MXU_200x260_[lindex $PM 1]*%"
  append string_g $a$C; append string_p $a$S;
     set a "G04: %ASAYBX*%"
  append string_g $a$C; append string_p $a$S;
     set a "%FSLAX33Y33*%"
  append string_g $a$C; append string_p $a$S;
     set a "%MOMM*%"
  append string_g $a$C; append string_p $a$S;
#    set a "%OFA0B0*%"
# append string_g $a$C; append string_p $a$S;
     set a "%SFA0.8B0.8*%"
  append string_g $a$C; append string_p $a$S;
     set a "G04: %IOA0B0*%"
  append string_g $a$C; append string_p $a$S;
#    set a "%IPPOS*%"
# append string_g $a$C; append string_p $a$S;
     set a "G04: %IR0*%"
  append string_g $a$C; append string_p $a$S;

# define apertures 

     set a "G04______________DEFINE APERTURES*"
  append string_g $a$C; append string_p $a$S;
     set a "%ADD10C,0.0*%"
  append string_g $a$C; append string_p $a$S;
     for {set i 1} {$i<=$noms} { incr i 1} {
        if { [C_mxus$i cget -ON] } {
#       [string length [C_mxus$i get_add] ] 
           set a [C_mxus$i get_add]
           # puts stdout $a 
  append string_g $a$C; append string_p $a$S;
           }
         }

# -------------
# slits 
# -------------

     set a "G04_______________SLIT_CUTTING*"
  append string_g $a$C; append string_p $a$S;
     set a "%LNSLITS*%"
  append string_g $a$C; append string_p $a$S;


     set a "G04: G90*";      # once more absolute 
  append string_g $a$C; append string_p $a$S;
     set a "G54D10*";   # small aperture 
  append string_g $a$C; append string_p $a$S;
     set a "D02*";      # close shutter 
  append string_g $a$C; append string_p $a$S;

     # we ask all slits on their size (width, length) and
     # orientation (posang). Each rectangular slit with 
     # zero pos_angle will get its own aperture definition 
     set ad_counter 10
     set noms [fors::MXU_SLIT::nos]
     if {$noms>0} {
     for {set i 1} {$i <= $noms} { incr i 1} { 

         # if slit has is specified 
         if {[C_mxus$i cget -ON]} {

           set loc_shape [C_mxus$i cget -shape]

           switch -exact -- $loc_shape {
 
              STRAIGHT {
             # we have a non-deleted rectangular slit 
             if {[fors::MXU_SLIT::get_mmff]} {
              # technical mask production, 
              # do not use OC values 
             set slit [C_mxus$i get_slit]
             } else {
             set slit [C_mxus$i get_oc_slit]
             }

             set lw [F2G [lindex $slit 3] ]
             set ll [F2G [lindex $slit 2] ]; #slit dimensions 
             set lang [F2G [C_mxus$i cget -pos_ang] ]
               # if {[string length [C_mxus$i get_add] ] } {

#                 # slit not rotated, use flash command 
#                 set a "G04________SLIT$i RECT $lw $ll $lang *"
# append string_g $a$C; append string_p $a$S;
#                 set a G54D[expr $i+$ad_counter]* 
# append string_g $a$C; append string_p $a$S;
#                 set a "X[F2G [expr [lindex $slit 0] + $OFFX ] ]"
#                 append a "Y[F2G [expr [lindex $slit 1] + $OFFY ] ]D03*"
# append string_g $a$C; append string_p $a$S;

                #   } else {

                  # slit rotated to parallelogram, use individual contour 
                  if {[fors::MXU_SLIT::get_mmff]} {
                     # technical mask production, 
                     # do not use OC values 
                    set slit_dim [C_mxus$i get_slit_contour]
                    } else {
                    set slit_dim [C_mxus$i get_oc_slit_contour]
                    }
                  # puts stdout ">$slit_dim<"
                  # convert 123.456789 to 123456 
              set a "G04________SLIT$i PARA $lw $ll $lang *"
  append string_g $a$C; append string_p $a$S;
              set a "G54D10*" 
  append string_g $a$C; append string_p $a$S;
              set a "X[F2G [expr [lindex $slit_dim 0] + $OFFX ] ]"
              # puts stdout [lindex $slit_dim 1]
              append a "Y[F2G [expr [lindex $slit_dim 1] + $OFFY ] ]D02*"
  append string_g $a$C; append string_p $a$S;
              set a "X[F2G [expr [lindex $slit_dim 2] + $OFFX ] ]"
              append a "Y[F2G [expr [lindex $slit_dim 3] + $OFFY ] ]D01*"
  append string_g $a$C; append string_p $a$S;
              set a "X[F2G [expr [lindex $slit_dim 4] + $OFFX ] ]"
              append a "Y[F2G [expr [lindex $slit_dim 5] + $OFFY ] ]D01*"
  append string_g $a$C; append string_p $a$S;
              set a "X[F2G [expr [lindex $slit_dim 6] + $OFFX ] ]"
              append a "Y[F2G [expr [lindex $slit_dim 7] + $OFFY ] ]D01*"
  append string_g $a$C; append string_p $a$S;
              set a "X[F2G [expr [lindex $slit_dim 0] + $OFFX ] ]"
              append a "Y[F2G [expr [lindex $slit_dim 1] + $OFFY ] ]D01*"
  append string_g $a$C; append string_p $a$S;

#                 }
               }
      CIRCLE { 
             # we have a non-deleted circle aperture
             if {[fors::MXU_SLIT::get_mmff]} {
              # technical mask production, 
              # do not use OC values 
             set slit [C_mxus$i get_slit]
             } else {
             set slit [C_mxus$i get_oc_slit]
             }

             set lw [F2G [lindex $slit 3] ]
             # use flash command 
             set a "G04________SLIT$i CIRC $lw *"
             append string_g $a$C; append string_p $a$S;
             set a G54D[expr $i+$ad_counter]* 
             append string_g $a$C; append string_p $a$S;
             set a "X[F2G [expr [lindex $slit 0] + $OFFX ] ]"
             append a "Y[F2G [expr [lindex $slit 1] + $OFFY ] ]D03*"
             append string_g $a$C; append string_p $a$S;
             }
      CURVED { 
             # we have a non-deleted curved slit 
             if {[fors::MXU_SLIT::get_mmff]} {
                puts stdout "Error: CURVED in standard mask !!!"
                } 
             set slit_dim [C_mxus$i get_b_oc]
             # puts stdout ">$slit_dim<"
             set lw [C_mxus$i cget -slit_width]
             set a "G04________SLIT$i CURV $lw *"
             append string_g $a$C; append string_p $a$S;
             set a "G54D10*" 
             append string_g $a$C; append string_p $a$S;
             #
             # index of the six bezier points
             #
             #    0,1 +----+ 6,7
             #         \    \
             #          \    \
             #           \    \
             #           /    /  2,3 +     + 8,9
             #      4,5 +----+ 10,11
             #          
             # flash the first point
             # convert 123.456789 to 123456 
             set a "X[F2G [expr [lindex $slit_dim 0] + $OFFX ] ]"
             append a "Y[F2G [expr [lindex $slit_dim 1] + $OFFY ] ]D02*"
             append string_g $a$C; append string_p $a$S;


             # number of polygon steps for a curved slit 
             set res 200

             #   1 pixel = 0.2 '' = 0.1 mm = 100 mu 
             # 200 pixel = 40''   = 20  mm = 20000 mu (maximum slit length)

             # red Bezier
             set x1 [lindex $slit_dim 0]
             set y1 [lindex $slit_dim 1]
             set x2 [lindex $slit_dim 2]
             set y2 [lindex $slit_dim 3]
             set x3 [lindex $slit_dim 4]
             set y3 [lindex $slit_dim 5]

             set x0 $x1; set y0 $y1
             for {set j 0} {$j <= $res} {incr j 1} {
                  set t [expr (1.0*$j) / (1.0*$res) ]
                  set xt [expr (1-$t)*(1-$t)*$x1+2*(1-$t)*$t*$x2+$t*$t*$x3]
                  set yt [expr (1-$t)*(1-$t)*$y1+2*(1-$t)*$t*$y2+$t*$t*$y3]

                  set a "X[F2G [expr $xt + $OFFX ] ]"
                  append a "Y[F2G [expr $yt + $OFFY ] ]D01*"
                  append string_g $a$C; append string_p $a$S;

                  set x0 $xt; set y0 $yt
                  }

             # blue Bezier
             # draw in opposite direction 
             set x1 [lindex $slit_dim 10]
             set y1 [lindex $slit_dim 11]
             set x2 [lindex $slit_dim 8]
             set y2 [lindex $slit_dim 9]
             set x3 [lindex $slit_dim 6]
             set y3 [lindex $slit_dim 7]

             # draw from red contour to blue contour
             set a "X[F2G [expr $x1 + $OFFX ] ]"
             append a "Y[F2G [expr $y1 + $OFFY ] ]D01*"
             append string_g $a$C; append string_p $a$S;
             
             set x0 $x1; set y0 $y1
             for {set j 0} {$j <= $res} {incr j 1} {
                  set t [expr (1.0*$j) / (1.0*$res) ]
                  set xt [expr (1-$t)*(1-$t)*$x1+2*(1-$t)*$t*$x2+$t*$t*$x3]
                  set yt [expr (1-$t)*(1-$t)*$y1+2*(1-$t)*$t*$y2+$t*$t*$y3]

                  set a "X[F2G [expr $xt + $OFFX ] ]"
                  append a "Y[F2G [expr $yt + $OFFY] ]D01*"
                  append string_g $a$C; append string_p $a$S;

                  set x0 $xt; set y0 $yt
                  }

             # draw blue conour back to red contour and end
             set x1 [lindex $slit_dim 0]
             set y1 [lindex $slit_dim 1]
             set a "X[F2G [expr $x1 + $OFFX ] ]"
             append a "Y[F2G [expr $y1 + $OFFY ] ]D01*"
             append string_g $a$C; append string_p $a$S;
             }
              } 

             }
           }
         }


# -------------
# mask id 
# -------------
#
# W A R N I N G 
# ---------------
#
# This part of the code exists as a copy in the 
# m2g script to insert the mask id to science masks
# as part nof the science opartion on Paranal
# any change in this part of the code must be 
# mirrored in the m2g script 

     set maskid [lindex $PM 1]
      
     # new concept since 12/1999:
     # sky masks contain no mask id
     # technical masks contain mask id 

     if {$maskid > 900000 && $maskid <= 900100} {
        
        if {$noms>1000} { puts stdout \
           "Error: number of apertures collides with maskid" 
           }

         # this is a technical mask
         # create gerber file with mask id 

     set a "G04______________MASK_ID_CUTTING $maskid*"
  append string_g $a$C; append string_p $a$S;
     # this defines a new layer for the LPKF SW
     # MASK ID will be cut faster
     set a "%LNBARCODE*%"
  append string_g $a$C; append string_p $a$S;

        # philosophy of mask id creation
        #
        #  --   this is a '8' consisting of 7 bars.
        # |  |  each bar is l x b large (length x width=breite).
        #  --   the full digit is 2s x 4s large, where 
        # |  |  2s is a little bit larger than l in order to 
        #  --   avoid overlapping bars. 
        #       You can change the total size of the MASKID digit 
        #       by parameter s. The digit is 2*s x 4*s mm large. 
        #       You can change the layout of the digit with 
        #       parameter l. 
        #       Note that l must be smaller than 2*s in any case. 


        # size of the digit
        set s  3.0;   # digit size = 2s x 4s 
        set l  4.0;   # length of a bar, l < 2s !!
        set b  [expr (2.0*$s-$l)/4.0]; # width of a bar
        set ss [expr $s*2.0]

        if {$l >= $ss} { puts stdout "ERROR: MASK_ID font size" }

        # for mmu working table switch l and b
        # set dummy $l; 
        # set l $b; 
        # set b $dummy

        puts -nonewline stdout "MASK_ID ="
    for {set k 0} {$k<=5} {incr k 1} {

        # central position of the digit in the focal field
        # set x0 [expr -75+$k*15]
        # set y0 -140
        # central position  on the mmu working table 
        set x0 [expr -140 + $OFFX ] 
        set y0 [expr 75-$k*15 + $OFFY ]


        # select the digit
        set ziffer [string index $maskid $k ]
        # puts stdout ">>$ziffer<<"

        # we paint digits directly in mmu working table coords

        switch -exact -- $ziffer {
        0 { # puts "$k th number is a 0" 
            puts -nonewline stdout  0
          set a "%AMZERO*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                         21,1,\$2,\$1,\$3+$ss,\$4,0*\
                         21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                         21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1000ZERO,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1000*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        1 { # puts stdout "$k th number is a 1" 
            puts -nonewline stdout  1
          set a "%AMONE*21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                        21,1,\$1,\$2,\$3-$s,\$4-$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1001ONE,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1001*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        2 { # puts stdout "$k th number is a 2" 
            puts -nonewline stdout  2
          set a "%AMTWO*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                        21,1,\$2,\$1,\$3+$ss,\$4,0*\
                        21,1,\$2,\$1,\$3,\$4,0*\
                        21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                        21,1,\$1,\$2,\$3+$s,\$4-$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1002TWO,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1002*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        3 { # puts stdout "$k th number is a 3" 
            puts -nonewline stdout  3
          set a "%AMTHREE*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                          21,1,\$2,\$1,\$3+$ss,\$4,0*\
                          21,1,\$2,\$1,\$3,\$4,0*\
                          21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                          21,1,\$1,\$2,\$3+$s,\$4-$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1003THREE,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1003*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        4 { # puts stdout "$k th number is a 4"
            puts -nonewline stdout  4
          set a "%AMFOUR*21,1,\$2,\$1,\$3,\$4,0*\
                         21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1004FOUR,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1004*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        5 { # puts stdout "$k th number is a 5" 
            puts -nonewline stdout  5
          set a "%AMFIVE*21,1,\$2,\$1,\$3,\$4,0*\
                         21,1,\$2,\$1,\$3+$ss,\$4,0*\
                         21,1,\$2,\$1,\$3-$ss,\$4,0*\
                         21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1005FIVE,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1005*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        6 { # puts stdout "$k th number is a 6" 
            puts -nonewline stdout  6
          set a "%AMSIX*21,1,\$2,\$1,\$3,\$4,0*\
                        21,1,\$2,\$1,\$3+$ss,\$4,0*\
                        21,1,\$2,\$1,\$3-$ss,\$4,0*\
                        21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                        21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                        21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1006SIX,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1006*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        7 { # puts stdout "$k th number is a 7" 
            puts -nonewline stdout  7
          set a "%AMSEVEN*21,1,\$2,\$1,\$3+$ss,\$4,0*\
                          21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                          21,1,\$1,\$2,\$3-$s,\$4-$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1007SEVEN,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1007*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        8 { # puts stdout "$k th number is a 8" 
            puts -nonewline stdout  8
          set a "%AMEIGHT*21,1,\$2,\$1,\$3,\$4,0*\
                          21,1,\$2,\$1,\$3+$ss,\$4,0*\
                          21,1,\$2,\$1,\$3-$ss,\$4,0*\
                          21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                          21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                          21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                          21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1008EIGHT,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1008*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        9 { # puts stdout "$k th number is a 9" 
            puts -nonewline stdout  9
          set a "%AMNINE*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                         21,1,\$2,\$1,\$3,\$4,0*\
                         21,1,\$2,\$1,\$3+$ss,\$4,0*\
                         21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4+$s,0*\
                         21,1,\$1,\$2,\$3+$s,\$4-$s,0*%"
          append string_g $a$C; append string_p $a$S;
          set a "%ADD1009NINE,$l\X$b\X0X0%"
          append string_g $a$C; append string_p $a$S;
          set a "G54D1009*"
          append string_g $a$C; append string_p $a$S;
          set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
          append string_g $a$C; append string_p $a$S;
          }
        A { puts stdout $k th number is a A, but is not yet supported" }
  default { puts stdout "something wrong " } 
         }
     }

    }

     puts stdout "\n"

# -------------
# mask contour 
# -------------

# View A:
# You will hold it in your hand  in such a way
#
#  (94.9,126)    o     (-95,126)
#          +-----------+C               FF         MMU
#         / D           \
#       E+               +B              ^ Y          ^ X
#        |        _      |               |            |
#        |       |_      |o              |            |
#        |       |       |               +---X   <----+
#        |               |                        Y
#        |               |o
#        |---------------|    (-126)
#        | 987654        |
#       F+---------------+G  (-203)
#                        | 
#                        +A = starting point for cutting
#                           
# o = the three frame pins 
#
# View B: flip view A at the FG-axis
# You prepare it with fims in an orientation as viewed
# from bottom to top  
#
#                        +A = starting point 
#                        | 
# (-203)F+---------------+G  
#        |68L924         |
# (-126) |---------------|    (y=-126)
#        |    North      |o
#        |               |                X        Y
#   East |       |_      |West        +--->    <---+
#        |       |_      |            |            |
#        |               |o           |            |
#       E+               +B           V Y          V X
#         \ D   South   / 
#          +-----------+C                 FF         MMU
#  (-95,126)      o    (94.9,126)
# 
# 
#  View C: = view A rotated by 90deg
#  It will be cut on the MMU working table like this 
#
#      F                E        ^           +---->Y 
#      +--+-------------+        |Y          |
#      |_o|              \       |           |
#      |oo|               +D     |           |X
#      |  |    _ _        |      +---> X      
#      |  |     | |       |o                  
#      |  |               |                   
#      |  |               +C     MMU          FF  
#      |  |              /                    
#   A+-+--+-------------+B
#      G    o         o
#
# contours are hard-coded, fix-points at BC and AB
# x=94.9mm  means 0.2 mm space for thermal contraction of the frame
# focal field ends in -Y at C=-126mm but there is still about 75mm 
# space available in -Y for mask ID cutting 

     set a "G04______________CONTOUR_CUTTING*"
  append string_g $a$C; append string_p $a$S;
     set a "%LNCONTOUR*%"
  append string_g $a$C; append string_p $a$S;
     set a "G04: G90*";                 # absolute
  append string_g $a$C; append string_p $a$S;
     set a "G54D10*";              # circle apertur 
  append string_g $a$C; append string_p $a$S;
     # round corners 
     # at B and E, we reduce the plate width by 2mm 
     # on 2cm in y 

    # in focal field coords 
    # apply FF_X, FF_Y 
    set a "X[F2G [expr 95.0+$MM_X]]Y[F2G [expr -164.0 + $MM_Y]]D02*";   # A = starting point
  append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr 95.0+$MM_X]]Y[F2G [expr  106.0 + $MM_Y]]D01*";   # B 
  append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr 93.0+$MM_X]]Y[F2G [expr  126.0 + $MM_Y]]D01*";   # C
  append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr  -92.9+$MM_X]]Y[F2G [expr  126.0 + $MM_Y]]D01*";   # D
  append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr  -94.9+$MM_X]]Y[F2G [expr  106.0 + $MM_Y]]D01*";   # E
  append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr  -94.9+$MM_X]]Y[F2G [expr -162.0 + $MM_Y]]D01*";   # F
  append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr 95.0+$MM_X]]Y[F2G [expr -162.0 + $MM_Y]]D01*";   # G
  append string_g $a$C; append string_p $a$S;

    # in mmu working table coords, x=y, y=x 

    # apply MM_X, MM_Y 
#    set a "X[F2G [expr -164.000+$MM_X]]Y[F2G [expr -095.000+$MM_Y]]D02*";# A 
# append string_g $a$C; append string_p $a$S;
#    set a "X[F2G [expr 106.000+$MM_X]]Y[F2G [expr -095.000+$MM_Y]]D01*"; # B 
# append string_g $a$C; append string_p $a$S;
#    set a "X[F2G [expr 126.000+$MM_X]]Y[F2G [expr -093.000+$MM_Y]]D01*"; # C
# append string_g $a$C; append string_p $a$S;
#    set a "X[F2G [expr 126.000+$MM_X]]Y[F2G [expr 092.900+$MM_Y]]D01*";  # D
# append string_g $a$C; append string_p $a$S;
#    set a "X[F2G [expr 106.000+$MM_X]]Y[F2G [expr 094.900+$MM_Y]]D01*";  # E
# append string_g $a$C; append string_p $a$S;
#    set a "X[F2G [expr -162.000+$MM_X]]Y[F2G [expr 094.900+$MM_Y]]D01*"; # F
# append string_g $a$C; append string_p $a$S;
#    set a "X[F2G [expr -162.000+$MM_X]]Y[F2G [expr -095.000+$MM_Y]]D01*"; # G
# append string_g $a$C; append string_p $a$S;


     # end 
     set a "D02*"
  append string_g $a$C; append string_p $a$S;
     set a "M02*"
  append string_g $a$C; append string_p $a$S;

     #  SAVE  --- gerber format --- hitherto used for P2PP
     # set myfile $log_dir/$filename.p_gbr
     # set fileId [open $myfile w ]
     # puts -nonewline $fileId $string_p
     # close $fileId
     # we will close this file soon 

# later at Paranal use UNIX command 
# 
# tr -s '?' '\012' < mxu2.60.test_0.p_gbr > mxu2.60.test_0.gbr
# 
# to connvert '?' to CR again
# conversion to DOS readable files:
# insert a Ctrl-M, use Ctrl-V Ctrl-M in vi for the script: 
# sed -e 's/$/^M/' $1 > $2 

     #  SAVE  --- gerber format --- for LPKF-SW  
     set myfile $log_dir/$filename 
     set fileId [open $myfile w ]
     puts -nonewline $fileId $string_g
     close $fileId
     }



     #============================
     private method extra_lss { a b } {
     #============================
     # split the result string of PickObject 
     # and copy values to globals for further
     # processing 
     set b [split $b " " ] 
     switch -exact -- $a {
        1 { set lss_sli_ra [lindex $b 0] 
            set lss_sli_de [lindex $b 1] 
          }
        2 { set lss_ref_ra [lindex $b 0] 
            set lss_ref_de [lindex $b 1] 
          }
        }
     }

     #================================
     private method extra_pick { b } {
     #================================
     # split the result string of PickObject
     # and copy values to globals for further
     # processing
     # puts stdout "in extra_pick output of PickObject =\n >>$b<<"
     set b [split $b " " ]
     # puts stdout "extra_pick : $b"
     if {[lindex $b 5]>0} {
        # successfull center gauss
        set lss_sli_ra [lindex $b 0]
        set lss_sli_de [lindex $b 1]
        # puts stdout "center slit ok"
        } else {
        # puts stdout "center slit ### failed ###"
        }
     }

     #=========================================
     private method save_lss_TPX { filename } {
     #=========================================
     # open the *.obx file
     # saving current parameters of one exposure
     # modus lss 
     # we save 5 !! files 
     # <mode>.<expo_count>.<fits_name>.fims
     # <mode>.<expo_count>.<fits_name>.targ
     # <mode>.<expo_count>.<fits_name>.focf
     # <mode>.<expo_count>.<fits_name>.p_fims
     # <mode>.<expo_count>.<fits_name>.p_fims
     # .fims contains all specified keywords, its the 
     #       file to be loaded when [Load Mask] 
     # .focf contains only the INS.FOC.SETUP keyword in one line 
     #       to be inserted in an obd file (e.g. via emacs)
     # .targ contains only the INS.TARG.SETUP keyword ...
     # .p_focf contains only the value of the INS.FOC.SETUP 
     #         keyword to be inserted to an obx file by p2pp
     # .p_targ conatins only the value of the INS.TARG.SETUP keyword ...

     # save results as one long string
     set string_foc  ""
     set string_targ ""
     

     # apply optical correction if fits-image from fors
     #
     set corr_flag [$this cget -fors_frame]

        C_ref_targ reset 
        set stop 1

        plot_ref_star_area 0
	

        info_dialog "Select the Slit Position (#1)" .skycat1
        EnterObject .eo \
           -title {Slit Position (#1)} \
           -image .skycat1.image \
           -labels {ra dec} \
           -command [code $this extra_lss 1 ] 
        tkwait window .eo

        C_ref_targ put $lss_sli_ra $lss_sli_de
        C_ref_targ show
        C_ref_targ plot $mos(0,r); # plot them all 

        info_dialog "Select the Reference Object (#2)" .skycat1
        EnterObject .eo \
           -title {Reference Object (#2)} \
           -image .skycat1.image \
           -labels {ra dec} \
           -command  [code $this extra_lss 2 ] 
        tkwait window .eo
   
        C_ref_targ put $lss_ref_ra $lss_ref_de
        C_ref_targ show
        C_ref_targ plot $mos(0,r); # plot them all 

        # delete cyan forbidden area
        $m_canvas delete rtz
        set reftarg_zone 0

        set stop 0

     set mod "[$this cget -modus][string index [$this cget -instrument] 4]"
     if {$filename==""} {
         set myfile "$set_dir/$mod.$exposure_counter.fims"
         } else {
         set myfile "$set_dir/$filename.fims"
         }
     set fileId [open $myfile w ]
     puts $lfId "FORS::save_lss_TPX:  $fileId $myfile"
     puts $fileId "# created by SKYCAT-TOOL V$s_v -- FIMS-plugin V$f_v"
     puts $fileId "# orig filename  : $myfile"
     puts stdout  "# orig filename  : $myfile"
     puts $fileId "# user-date-host : [exec whoami] [exec date -u +%Y-%m-%dT%H:%M:%S] $lochost"
     puts $fileId "# fits file dir  : [ [.skycat1 component image] cget -file]"
     puts $fileId "# fits file name : [$this get_fits_name]"
     puts $fileId "# fits instrument: [$m_image fits get INSTRUME]"
     puts $fileId "# fits origin    : [$m_image fits get ORIGIN]" 
     puts $fileId "# apply optidist : [$this cget -fors_frame]" 
     puts $fileId "# fits object    : [$m_image fits get OBJECT]" 
     puts $fileId "# catalog name   : $CatFileName"
     puts $fileId "# target setup for [$this cget -instrument]-[string toupper $mod]"
     puts $fileId "# 1=slit 2=reference target"
     puts $fileId "# --------------------------------------------"

     $m_image convert coords $world_x $world_y "deg $EQX" \
       dummy_x dummy_y "wcs $EQX"
     # puts stdout "$world_x $world_y $dummy_x $dummy_y"

   # particular Pipeline issue
   set last [string length $myfile] 
   set first [expr [string last "/" $myfile]  +1]
#  append string_foc [format "INS.FIMS.NAME  %20s.p_focf  " \
#    [string range $myfile $first [expr $last - 6] ] ]
#  append string_targ [format "INS.FIMS.NAME  %20s.p_targ " \
#    [string range $myfile $first [expr $last - 6] ] ]
   # WH 9.5.2000
#   append string_foc [format "INS.FIMS.NAME  %1s_%-1s_%-1s.p_focf  " \
#   $s_v [lindex $f_v 0] [string range $myfile $first [expr $last - 6] ] ]
#   append string_targ [format "INS.FIMS.NAME  %1s_%-1s_%-1s.p_targ  " \
#   $s_v [lindex $f_v 0] [string range $myfile $first [expr $last - 6] ] ]

   append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s  " \
   [string range $myfile $first [expr $last - 6] ] $s_v [lindex $f_v 0] ]
   append string_targ [format "INS.FIMS.NAME  %-1s.p_targ_%1s_%-1s  " \
   [string range $myfile $first [expr $last - 6] ] $s_v [lindex $f_v 0] ]

     #  TCS
     # ----
     lassign [split $dummy_x :] d1 d2 d3
     puts $fileId [format "TEL.TARG.ALPHA    %02s%02s%06.3f" $d1 $d2 $d3 ]
 append string_targ [format "TEL.TARG.ALPHA  %02s%02s%06.3f  " $d1 $d2 $d3 ]
     lassign [split $dummy_y :] d1 d2 d3
     puts $fileId [format "TEL.TARG.DELTA    %02s%02s%06.3f" $d1 $d2 $d3 ]
 append string_targ [format "  TEL.TARG.DELTA  %02s%02s%06.3f" $d1 $d2 $d3 ]
     puts $fileId [format "TEL.ROT.OFFANGLE       %6.3f" [expr $PAM * -1] ]
append string_targ  [format "  TEL.ROT.OFFANGLE  %6.3f" [expr $PAM * -1] ]
#      puts $fileId [format "TEL.TARG.EPOCH          %6.1f" $EQX ]
# append string_targ  [format "  TEL.TARG.EPOCH     %6.1f" $EQX ]
       puts $fileId [format "TEL.TARG.EQUINOX        %6.1f" $EQX ]
append string_targ  [format "  TEL.TARG.EQUINOX    %6.1f" $EQX ]
       puts $fileId [format "TEL.TARG.TYPE    COORDINATE" ]
append string_targ  [format "  TEL.TARG.TYPE    COORDINATE" ]

     # SLIT
     # ----
     set dummy [$this cget -long_width]
     set ins_long "lSlit"; 

     # pinhole only available under save_hits_TPX 
   append ins_long [string index $dummy 0] "_" [string index $dummy 2] "arcsec"
     puts  $fileId [format "INS.SLIT.NAID       %s" $ins_long ]
append string_foc [format "  INS.SLIT.NAID       %s" $ins_long ]

     # reference and slit target
     # -------------------------
     # puts stdout "$lss_sli_ra \n$lss_sli_de \n$lss_ref_ra \n$lss_ref_de"
     # TARGET 1 is SLIT OBJECT 
     lassign [split $lss_sli_ra :] a1 a2 a3
     # puts stdout  [format "INS.TARG1.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]
     puts $fileId [format "INS.TARG1.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]
     lassign [split $lss_sli_de :] d1 d2 d3
     # puts stdout  [format "INS.TARG1.DELTA   %02s%02s%06.3f" $d1 $d2 $d3 ]
     puts $fileId [format "INS.TARG1.DELTA   %02s%02s%06.3f" $d1 $d2 $d3 ]

         if {$corr_flag} {
         set new_ref [calc_zemax_ref $lss_sli_ra $lss_sli_de ]
         lassign [split $new_ref " " ] a0 d0
         puts stdout "new REF COORDS : >$a0<  >$d0<"

         # apply differential refraction 
         if {$dire_flag} {
            set new_ref [calc_diff_refrac 0 $a0 $d0]
            lassign [split $new_ref " " ] a0 d0
            puts stdout "new REF COORDS : >$a0<  >$d0<"
            }

         lassign [split $a0 :] a1 a2 a3
         lassign [split $d0 :] d1 d2 d3
         }
append string_targ [format "  INS.TARG1.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]
append string_targ [format "  INS.TARG1.DELTA   %02s%02s%06.3f" $d1 $d2 $d3 ]
         
   
     # TARGET 2 is REFERENCE OBJECT 
     lassign [split $lss_ref_ra :] a1 a2 a3
     # puts stdout  [format "INS.TARG2.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]
     puts $fileId [format "INS.TARG2.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]
     lassign [split $lss_ref_de :] d1 d2 d3
     # puts stdout  [format "INS.TARG2.DELTA   %02s%02s%06.3f" $d1 $d2 $d3 ]
     puts $fileId [format "INS.TARG2.DELTA   %02s%02s%06.3f" $d1 $d2 $d3 ]


         if {$corr_flag} {
         # apply optical correction to reference targets
         set new_ref [calc_zemax_ref $lss_ref_ra $lss_ref_de ]
         lassign [split $new_ref " " ] a0 d0
         puts stdout "new REF COORDS : >$a0<  >$d0<"

         # apply differential refraction 
         if {$dire_flag} { 
            set new_ref [calc_diff_refrac 0 $a0 $d0]
            lassign [split $new_ref " " ] a0 d0
            puts stdout "new REF COORDS : >$a0<  >$d0<"
            }

         lassign [split $a0 :] a1 a2 a3
         lassign [split $d0 :] d1 d2 d3
         }
append string_targ [format "  INS.TARG2.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]
append string_targ [format "  INS.TARG2.DELTA   %02s%02s%06.3f" $d1 $d2 $d3 ]
   
     #  save also grism, OS filter and collimator 
     # -------------------------------------------
     puts $fileId [format "SEQ.PREP.LMIN   %8.3f" $L_MIN ]
     puts $fileId [format "SEQ.PREP.LMAX   %8.3f" $L_MAX ]
     puts $fileId [format "SEQ.PREP.GRIS   %10s" [$this cget -grism] ]
     puts $fileId [format "SEQ.PREP.FILT   %10s" [$this cget -filter] ]
     puts $fileId "SEQ.PREP.COLL   [$this cget -collimat]" 
     append string_targ "  INS.COLL.NAID   [$this cget -collimat]" 


     close $fileId


     # save the same stuff again as one string to a
     # file to be called by p2pp
     set myfile $set_dir/$filename.p_targ
     set fileId [open $myfile w ]
     puts -nonewline $fileId $string_targ
     close $fileId

     set myfile $set_dir/$filename.p_focf
     set fileId [open $myfile w ]
     puts -nonewline $fileId $string_foc
     close $fileId

     # save the same stuff again but in a more readable format
     # into the .fims/LOG directory (only for maintenance)

     set s2t [ldelete_all [split $string_targ " "]  ""]
     set number [llength $s2t]
     for {set i 0} {$i < $number} { incr i 2 } {
         set s2t [ lreplace $s2t $i $i [lrange $s2t $i [expr $i+1] ] ]
         }
     for {set i [expr $number-1] } {$i > 0} { incr i -2 } {
         set s2t [ lreplace $s2t $i $i ]
         }
     set string2_targ [join $s2t "\n" ]

     set myfile $log_dir/$filename.targ
     #                             ----
     set fileId [open $myfile w ]
     puts $fileId $string2_targ
     close $fileId

     set s2f [ldelete_all [split $string_foc " "]  ""]
     set number [llength $s2f]
     for {set i 0} {$i < $number} { incr i 2 } {
         set s2f [ lreplace $s2f $i $i [lrange $s2f $i [expr $i+1] ] ]
         }
     for {set i [expr $number-1] } {$i > 0} { incr i -2 } {
         set s2f [ lreplace $s2f $i $i ]
         }
     set string2_foc [join $s2f "\n" ]

     set myfile $log_dir/$filename.focf
     #                             ----
     set fileId [open $myfile w ]
     puts $fileId $string2_foc
     close $fileId



     incr exposure_counter 1
    }


     #============================
     private method save_hits_TPX { filename } {
     #============================
     # hits and lss use this branch 
     #          ---
     # saving current parameters of one exposure
     # we save 5 !! files 
     # <mode><instrument number>.<expo_count>.<fits_name>.fims -> SET
     # <mode><instrument number>.<expo_count>.<fits_name>.targ -> LOG
     # <mode><instrument number>.<expo_count>.<fits_name>.focf -> LOG
     # <mode><instrument number>.<expo_count>.<fits_name>.p_targ -> SET
     # <mode><instrument number>.<expo_count>.<fits_name>.p_forc -> SET
     # e.g. hits2.15.ngc0815.fims is a local save by fims
     #      this file can be loaded again when [Load Mask] 
     # .focf contains the same as p_focf but with 
     #       carriage returns for better reading 
     # .targ contains the same as p_targ but with carrage 
     #       returns for better reading 
     # .p_focf contains only the value of the INS.FOC.SETUP 
     #         keyword to be inserted to an obx file by p2pp
     # .p_targ contains only the value of the INS.TARG.SETUP keyword ...
     # ipol.<ec>.<fits_name>.offs -> LOG 

     # save results as one long string
     set string_foc  ""
     set string_targ ""
     # empirical optical distortion of FORS1
     set corr_flag [$this cget -fors_frame]


     # modus either hits1 or hits2 or lss1 or lss2
     set mod "[$this cget -modus][string index [$this cget -instrument] 4]"
     if {$filename==""} {
         set myfile "$set_dir/$mod.$exposure_counter.fims"
         } else {
         set myfile "$set_dir/$filename.fims"
         } 
     set fileId [open $myfile w ]

     # write comment block 
     puts $lfId "FORS::save_hits_TPX:  $fileId $myfile"
     puts $fileId "# created by SKYCAT-TOOL V$s_v -- FIMS-plugin V$f_v"
     puts $fileId "# orig filename  : $myfile"
     puts stdout  "# orig filename  : $myfile"
     puts $fileId "# user-date-host : [exec whoami] [exec date -u +%Y-%m-%dT%H:%M:%S] $lochost"
     puts $fileId "# fits file dir  : [ [.skycat1 component image] cget -file]"
     puts $fileId "# fits file name : [$this get_fits_name]"
     puts $fileId "# fits instrument: [$m_image fits get INSTRUME]"
     puts $fileId "# fits origin    : [$m_image fits get ORIGIN]" 
     puts $fileId "# apply optidist : [$this cget -fors_frame]" 
     puts $fileId "# fits object    : [$m_image fits get OBJECT]" 
     puts $fileId "# catalog name   : $CatFileName"
     puts $fileId "# target setup for [$this cget -instrument]-[string toupper $mod]"
     puts $fileId "# --------------------------------------------"

     # TCS pointing 
         # for the .fims 
         $m_image convert coords $world_x $world_y "deg $EQX" \
           dummy_x dummy_y "wcs $EQX"
         lassign [split $dummy_x :] d1 d2 d3
         puts $fileId [format "TEL.TARG.ALPHA    %02s%02s%06.3f" $d1 $d2 $d3 ]
         lassign [split $dummy_y :] d1 d2 d3
         puts $fileId [format "TEL.TARG.DELTA    %02s%02s%06.3f" $d1 $d2 $d3 ]

             if {[string match lss [$this cget -modus]]} { 
                fix_mask_slit lss $corr_flag  
                } else {
                fix_mask_slit hits $corr_flag 
                }

   # particular ESO-Pipeline issue
   set last [string length $myfile] 
   set first [expr [string last "/" $myfile]  +1]
#  append string_foc [format "INS.FIMS.NAME %20s.p_focf    " \
#    [string range $myfile $first [expr $last - 6] ] ]
#  append string_targ [format "INS.FIMS.NAME %20s.p_targ    " \
#    [string range $myfile $first [expr $last - 6] ] ]
   # WH 9.5.2000
#   append string_foc [format "INS.FIMS.NAME  %1s_%-1s_%-1s.p_focf  " \
#   $s_v [lindex $f_v 0] [string range $myfile $first [expr $last - 6] ] ]
#   append string_targ [format "INS.FIMS.NAME  %1s_%-1s_%-1s.p_targ  " \
#   $s_v [lindex $f_v 0] [string range $myfile $first [expr $last - 6] ] ]

   append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s  " \
   [string range $myfile $first [expr $last - 6] ] $s_v [lindex $f_v 0] ]
   append string_targ [format "INS.FIMS.NAME %-1s.p_targ_%1s_%-1s  " \
   [string range $myfile $first [expr $last - 6] ] $s_v [lindex $f_v 0] ]

          if {$corr_flag} {
         lassign [split $targw(2,ca) :] d1 d2 d3
append string_targ  [format "  TEL.TARG.ALPHA    %02s%02s%06.3f" $d1 $d2 $d3 ]
append string_foc   [format " INS.FOCF.ALPHA    %02s%02s%06.3f" $d1 $d2 $d3 ]
         lassign [split $targw(2,cd) :] d1 d2 d3
append string_targ  [format "  TEL.TARG.DELTA    %02s%02s%06.3f" $d1 $d2 $d3 ]
append string_foc   [format " INS.FOCF.DELTA    %02s%02s%06.3f" $d1 $d2 $d3 ]
         } else {
         $m_image convert coords $world_x $world_y "deg $EQX" \
           dummy_x dummy_y "wcs $EQX"
         lassign [split $dummy_x :] d1 d2 d3
append string_targ  [format "  TEL.TARG.ALPHA    %02s%02s%06.3f" $d1 $d2 $d3 ]
append string_foc   [format " INS.FOCF.ALPHA    %02s%02s%06.3f" $d1 $d2 $d3 ]
         lassign [split $dummy_y :] d1 d2 d3
append string_targ  [format "  TEL.TARG.DELTA    %02s%02s%06.3f" $d1 $d2 $d3 ]
append string_foc   [format " INS.FOCF.DELTA    %02s%02s%06.3f" $d1 $d2 $d3 ]
         }


        # the rest is independent on correction 
       puts $fileId [format "TEL.ROT.OFFANGLE       %6.3f" [expr $PAM * -1]  ]
append string_targ  [format "  TEL.ROT.OFFANGLE     %6.3f" [expr $PAM * -1]  ]
append string_foc   [format " INS.FOCF.OFFANGLE     %6.3f" [expr $PAM * -1]  ]
#        puts $fileId [format "TEL.TARG.EPOCH          %6.1f" $EQX ]
# append string_targ  [format "  TEL.TARG.EPOCH     %6.1f" $EQX ]
       puts $fileId [format "TEL.TARG.EQUINOX        %6.1f" $EQX ]
append string_targ  [format "  TEL.TARG.EQUINOX    %6.1f" $EQX ]
       puts $fileId [format "TEL.TARG.TYPE    COORDINATE" ]
append string_targ  [format "  TEL.TARG.TYPE    COORDINATE" ]


     # slit or ht slit or pinhole 
     # ---------------------------
     # slit width string 
     set dummy [$this cget -long_width]

     # ht-slit  or lss slit 
     if {[string match lss [$this cget -modus]]} { 
        set ins_long "lSlit" 
        append ins_long \
        [string index $dummy 0] "_" [string index $dummy 2] "arcsec"
        } else {
        set ins_long "htSlit" 
        if {[string compare $dummy PH]==0} {
           # extra case : if htSlit0_3arcsec is choosen,
           #              take pinh0_95arcsec
           set ins_long [lindex $ph_range 0] 
           } else {
           append ins_long \
           [string index $dummy 0] "_" [string index $dummy 2] "arcsec"
           }
        }


    if {[string match *pinh* $ins_long]} {
     # extra case
     puts  $fileId [format "INS.PINH.NAID       %s" $ins_long ]
append string_foc [format "  INS.PINH.NAID       %s" $ins_long ]
       } else {
     # normal case
     puts  $fileId [format "INS.SLIT.NAID       %s" $ins_long ]
append string_foc [format "  INS.SLIT.NAID       %s" $ins_long ]
       }

     # SCIENCE TARGET
     # ----------------------
        if {[string length $targw(10,a)]==0} { 
           plot_ref_star_area 0
           
           set stop 1 
 
        info_dialog "Select the Science Target" .skycat1
        EnterObject .eo \
           -title {Science Target} \
           -image .skycat1.image \
           -labels {ra dec} \
           -command [code $this extra_lss 1 ] 
        tkwait window .eo
      # wcs EQX
      set targw(10,a) $lss_sli_ra 
      set targw(10,d) $lss_sli_de

      #delete cyan(-kali) area
      $m_canvas delete rtz
      set reftarg_zone 0
      set stop 0 
      }
      

      # for LSS without pics and HITS, science target default targ10
      lassign [split $targw(10,d) :] d1 d2 d3
      lassign [split $targw(10,a) :] a1 a2 a3
      # puts stdout  [format "INS.TARG1.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
        puts $fileId [format "INS.TARG1.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
   append string_foc [format " INS.TARG1.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
      # puts stdout  [format "INS.TARG1.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
        puts $fileId [format "INS.TARG1.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
   append string_foc [format " INS.TARG1.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
        puts $fileId [format "INS.TARG1.NAME   %20s" NN]
   append string_foc [format " INS.TARG1.NAME   %20s" NN]
        # plot a red circle around the science target
        $m_image convert coords $targw(10,a) $targw(10,d) "wcs $EQX" \
                 x y canvas 
        $m_canvas create oval [expr $x-$mos(0,r)] [expr $y-$mos(0,r)] \
                         [expr $x+$mos(0,r)] [expr $y+$mos(0,r)] \
                         -outline red -tag { A }


         if {$corr_flag} {
         $m_image convert coords $targw(10,a) $targw(10,d) "wcs $EQX" \
                  targw(10,x) targw(10,y) image 
         calc_zemax 10 $targw(10,x) $targw(10,y) 
         lassign [split $targw(10,ca) :] a1 a2 a3
         lassign [split $targw(10,cd) :] d1 d2 d3
         }
    append string_targ [format "  INS.TARG1.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
    append string_targ [format "  INS.TARG1.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
    append string_targ [format "  INS.TARG1.NAME   %20s" NN]




        # ----------------------------
        # reference objects are known 
        # ----------------------------
        # total numer of ref objects
        puts stdout "[C_ref_targ nort] Reference Objects selected"

     # Reference Objects already in correct coords
     # save reference objects to setup file
     for { set j 1} {$j <= [C_ref_targ nort] } { incr j 1 } {
         set z [expr $j + 30]; # the INS.TARG$z. number

         lassign [split [C_ref_targ get $j a] :] a1 a2 a3
      #  puts stdout  [format "INS.TARG$z.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]
         puts $fileId [format "INS.TARG$z.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3 ]

         lassign [split [C_ref_targ get $j d] :] d1 d2 d3
      #  puts stdout  [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
         puts $fileId [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

         $m_image convert coords \
           [C_ref_targ get $j a] [C_ref_targ get $j d] "wcs $EQX" x y image 
         # puts stdout "$j   $x $y" 


         if {$corr_flag} {
         set new_ref [calc_zemax_ref [C_ref_targ get $j a] \
                     [C_ref_targ get $j d] ]
         lassign [split $new_ref " " ] a0 d0
         puts stdout "new REF COORDS : >$a0<  >$d0<"

         # apply differential refraction 
         if {$dire_flag} {
            set new_ref [calc_diff_refrac 0 $a0 $d0]
            lassign [split $new_ref " " ] a0 d0
            puts stdout "new REF COORDS : >$a0<  >$d0<"
            }

         lassign [split $a0 :] a1 a2 a3
         lassign [split $d0 :] d1 d2 d3
         }
   append string_targ [format "  INS.TARG$z.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
   append string_targ [format "  INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
         }

     #--------------------
     # reference targets end
     #--------------------

     # if {[C_ref_targ nort]>0} {
     # if there is an alignment specify alignment slit
     #    append string_targ "  INS.MOS10.POS 0.000   INS.MOS10.WID  0.5"
     #     } 

     #  save also grism, OS filter and collimator 
     puts $fileId [format "SEQ.PREP.LMIN   %8.3f" $L_MIN ]
     puts $fileId [format "SEQ.PREP.LMAX   %8.3f" $L_MAX ]
     puts $fileId [format "SEQ.PREP.GRIS   %10s" [$this cget -grism] ]
     puts $fileId [format "SEQ.PREP.FILT   %10s" [$this cget -filter] ]
     puts $fileId [format "SEQ.PREP.COLL   [$this cget -collimat]" ]
     append string_targ "  INS.COLL.NAID   [$this cget -collimat]"

     close $fileId

     # save the same stuff again as one string to a
     # file to be called by p2pp
     set myfile $set_dir/$filename.p_targ
     set fileId [open $myfile w ]
     puts -nonewline $fileId $string_targ
     close $fileId

     set myfile $set_dir/$filename.p_focf
     set fileId [open $myfile w ]
     # special behaviour when saving an empty string to a file
     # the file  will have 1 char and P2PP will complain 
     # therefore dont copy empty string to file 
     if {[string length $string_foc]>0} {puts -nonewline $fileId $string_foc}
     close $fileId

     # save the same stuff again but in a more readable format
     # into the .fims/LOG directory (only for maintenance)

     set s2t [ldelete_all [split $string_targ " "]  ""]
     set number [llength $s2t]
     for {set i 0} {$i < $number} { incr i 2 } {
         set s2t [ lreplace $s2t $i $i [lrange $s2t $i [expr $i+1] ] ]
         }
     for {set i [expr $number-1] } {$i > 0} { incr i -2 } {
         set s2t [ lreplace $s2t $i $i ]
         }
     set string2_targ [join $s2t "\n" ]

     set myfile $log_dir/$filename.targ
     #                             ----
     set fileId [open $myfile w ]
     puts $fileId $string2_targ
     close $fileId

     set s2f [ldelete_all [split $string_foc " "]  ""]
     set number [llength $s2f]
     for {set i 0} {$i < $number} { incr i 2 } {
         set s2f [ lreplace $s2f $i $i [lrange $s2f $i [expr $i+1] ] ]
         }
     for {set i [expr $number-1] } {$i > 0} { incr i -2 } {
         set s2f [ lreplace $s2f $i $i ]
         }
     set string2_foc [join $s2f "\n" ]

     set myfile $log_dir/$filename.focf
     #                             ----
     set fileId [open $myfile w ]
     puts $fileId $string2_foc
     close $fileId



     incr exposure_counter 1
     }

     #===================================
     public  method init_mask_world2 {} {
     #===================================
     # The WCS command 'convert coords' does only work on 
     # frames but not on the canvas area beyond a FITS frame.
     # Therefore the instrumental mask uses a separate coord 
     # conversion (a linear approach, defined via bonsai_matrix 
     set debug 1
     # 
     # This member function draws the instrumental mask on the canvas
     # with the correct size (~6.4' x 6.4') and position angle 0.
     if {$debug} { puts stdout "world2:    RA            DEC\
            asec/pix (PXcen)  (PYcen) SizeX SizeY rot Epo equi proj" 
        }
     # all coordinates are stored in the large array mos(,) 
     # here we handle alp and del differently 
     # canvas coordinates in canvas pixels 
     # there is still a bug concerning south-pole 
     # first at all : central position in canvas
     # units of mos: canvas coords
     # units of targw: canvas
     if {$debug} { puts stdout "world2: [$m_image wcsset]" }

     set EQX [ lindex [$m_image wcsset] 8 ]

     # frameless mode
     if {$wx=="" || $wy==""} {
        set w1  [ lindex [$m_image wcsset] 0 ]
        set w2  [ lindex [$m_image wcsset] 1 ]
        $m_image convert coords $w1 $w2 "wcs $EQX" wx wy "deg $EQX"
        puts stdout "frameless mode reference pixel :$w1 $w2"
        } 
     
     $m_image convert coords $wx $wy "deg $EQX" ccx  ccy canvas 
     set mos(0,x0) $ccx; set mos(0,y0) $ccy
     set ccd_frame(0,x0) $ccx; set ccd_frame(0,y0) $ccy
     set ccd_frame(1,x0) $ccx; set ccd_frame(1,y0) $ccy
     
     if {$debug} {
         puts stdout "reference pixel : $wx $wy $ccx $ccy"
         }
            
     # retrieving the actual zooming parameter
     set canvscale [$m_image scale]
     set canvscale [lindex $canvscale 0]
     if {$canvscale<0} {
        set canvscale [expr -1.0/$canvscale]
        }



     # read the reference pixel from the fits header
     # ---------------------------------------------
     # this is independent on WCS notation
     if {$debug} {
     puts stdout ">[$m_image fits get CRPIX1]< \
	    >[$m_image fits get CRPIX2]< \
            >[$m_image fits get CRVAL1]< \
            >[$m_image fits get CRVAL2]<"
      }

      # read the equinox 
      if {$EQX==""} { set EQX [$m_image fits get EQUINOX]  }
    # if {$EQX==""} { set EQX [$m_image fits get EPOCH]    }
      if {$EQX==""} { set EQX [lindex [$m_image wcsset] 8 ] }
      if {$EQX==""} { puts stdout "Error: EQUINOX not defined !!!"  }

     # now the (signum of CROTA2 = CRO) , needed for init_mask 
     # first test if fites header contains CD,  
     # then test if fits header contains PC 
     # test test if fits header contains CROTA (WCS keywords)

     set no_wcs_flag 1; # WCS not yet defined by default 

     if {[$m_image fits get CD1_1] != ""} {
         set no_wcs_flag 0; # everything is still ok 
         if {$debug} { puts stdout "try CD notation ..." }
         set d11 [$m_image fits get CD1_1]
         set d12 [$m_image fits get CD1_2]
         set d21 [$m_image fits get CD2_1]
         set d22 [$m_image fits get CD2_2]
         if {$d12==""} {set no_wcs_flag 1; puts stdout "CD1_2 not found"}  
         if {$d21==""} {set no_wcs_flag 1; puts stdout "CD2_1 not found"}
         if {$d22==""} {set no_wcs_flag 1; puts stdout "CD2_2 not found"}

         if {$no_wcs_flag} {
            puts stdout "Error in FITS Header: \
            Incomplete CD-WCS keywords, try PC notation ...\n"
            } else {
            # everything correctly found up to now
            set d30 [expr $d11*$d22-$d12*$d21]
            set CSO [expr $d30/abs($d30)]; # = sign(CDELT1*CDELT2)
            if {$CSO==1} {
               # puts stdout "world2: WCS: right-hand coords -->East to right"
               # by convention if CSO<0 then CDELT1<0
               set fits_x_scale 1.0
               set fits_y_scale 1.0
               } else {
               # puts stdout "world2: WCS: left-hand coords East to left<--"
               # by convention if CSO<0 then CDELT1<0
               set fits_x_scale -1.0
               set fits_y_scale 1.0
               }
            set CRO [expr 57.29578*atan2($d30*$d12,$d22) ]
            if {$CRO == 0.0} {
               set CRO 1.0 
               } else {
               set CRO [expr $CRO / abs($CRO)] 
               }
            if {$debug} { 
               puts stdout "CD1,1,2,2, = \
               [$m_image fits get CD1_1] [$m_image fits get CD1_2] \
               [$m_image fits get CD2_1] [$m_image fits get CD2_2]"
               }
            }
           } 


      if {$no_wcs_flag} {
         # CD notation either incomplete or not available 
         if {[$m_image fits get PC001001] !=""} {
         set pc_flag 0 
         if {$debug} { puts stdout "try PC notation ..."}
         set d11 [$m_image fits get CDELT1]
         set d21 [$m_image fits get CDELT2]
         set d12 [$m_image fits get PC001002]
         set d22 [$m_image fits get PC002002]
         if {$d11==""} { set pc_flag 1; puts stdout "CDELT1 not found" }
         if {$d21==""} { set pc_flag 1; puts stdout "CDELT2 not found" }
         if {$d12==""} { set pc_flag 1; puts stdout "PC001002 not found" }
         if {$d22==""} { set pc_flag 1; puts stdout "PC002002 not found" }

         if {$pc_flag} { 
            puts stdout "Error in FITS Header: \
            Incomplete PC_coord_conv coeffWCS keywords, try CROTA notation ...\n"
            } else {
            set no_wcs_flag 0; # keyword set complete 
            set a [$m_image fits get CDELT1]
            set fits_x_scale [expr $a/abs($a) ]
            set b [$m_image fits get CDELT2]
            set fits_y_scale [expr $b/abs($b)]
            set CSO [expr $fits_x_scale * $fits_y_scale]
            if {$CSO==1} {
               # puts stdout "world2: WCS: right-hand coords -->East to right"
               } else {
               # puts stdout "world2: WCS: left-hand coords East to left<--"
               }
            set CRO [expr 57.29578*atan2($d12*abs($d11),$d22*abs($d21))]
            if {$CRO == 0.0} {
               set CRO 1.0 
               } else {
               set CRO [expr $CRO / abs($CRO)] 
               }
            if {$debug} {
               puts stdout "world2: PC11,12,21,22, CDELT1,2 = \
               [$m_image fits get PC001001] [$m_image fits get PC001002] \
               [$m_image fits get PC002001] [$m_image fits get PC002002] \
               [$m_image fits get CDELT1]   [$m_image fits get CDELT2]"
               }
            }
          }
         }

    if {$no_wcs_flag} {
         set cr_flag 0; # everything ok up to here 
         if {$debug} { puts stdout "try CDELT notation ..."}
         set a [$m_image fits get CDELT1]
         set b [$m_image fits get CDELT2]
         set cro1 [$m_image fits get CROTA1]
         set cro2 [$m_image fits get CROTA2]
         if {$a==""} { set cr_flag 1; puts stdout "CDELT1 not found" }
         if {$b==""} { set cr_flag 1; puts stdout "CDELT2 not found" }
         # at least one of CROTA should be defined
         if {$cro1=="" && $cro2==""} { 
             set cr_flag 1; puts stdout "CROTA1 && CROTA2 not found" } 


         if {$cr_flag} { 
            puts stdout "Error in FITS Header: \
            Incomplete CR-WCS keywords, try SkyCat ... \n"
            } else {
            set no_wcs_flag 0; 
            set fits_x_scale [expr $a/abs($a) ]
            set fits_y_scale [expr $b/abs($b)]
            set CSO [expr $fits_x_scale * $fits_y_scale]
            if {$debug} {
               if {$CSO==1} {
                  puts stdout "world2: WCS: right-hand coords -->East to right"
                  } else {
                  puts stdout "world2: WCS: left-hand coords East to left<--"
                  }
               }
            if {$cro1==""} {set CRO $cro2 } else { set CRO $cro1}
            if {$CRO == 0.0} {
               set CRO 1.0 
               } else {
               set CRO [expr $CRO / abs($CRO)] 
               }
            if {$debug} {
               puts stdout [format \
               "world2: CDELT1,2 CROTA1,2 = %10e %10e %6.2f" \
               [$m_image fits get CDELT1] \
	       [$m_image fits get CDELT2] \
               $CRO ]
               }
             }
           }

     if {$no_wcs_flag} {
      if {$debug} { puts stdout "try SkyCat ..."}
      # try skycat WCS as a last chance 
      set hello [$m_image wcsset]
      set CRO [lindex $hello 7]
         if {$CRO == 0.0} {
            set CRO 1.0 
            } else {
            set CRO [expr $CRO / abs($CRO)] 
            }
      set CSO -1
      set fits_x_scale -1.0
      set fits_y_scale 1.0
      puts stdout ".. got values from SkyCat"
      }


     if {$debug} {
     puts stdout "sign(CROTA2)         = CRO = $CRO"
     puts stdout "sign(CDELT1*CDELT2)  = CSO = $CSO"
     }


     # skycat image flip buttons 
     # --------------------------
     # state (if buttons are pressed)
     set flipr [$m_image rotate];   # [()]  button
     set flipx [$m_image flip x];   # [==]  button
     set flipy [$m_image flip y];   # [||]  button

     # puts stdout "flip rxy  $flipr $flipx $flipy"

     if {$flipr} { 
                 if {$CRO == -1} { set fits_x_scale [expr -1 * $fits_x_scale] }
                 set fits_y_scale [expr -1 * $fits_y_scale]
                 set CSO [expr -1 * $CSO] 
                 set CRO [expr -1 * $CRO] 
                 }
     if {$flipx} { 
                 set fits_x_scale [expr -1 * $fits_x_scale] 
                 set CSO [expr -1 * $CSO] 
                 set CRO [expr -1 * $CRO]
                 }
     if {$flipy} { 
                 set fits_y_scale [expr -1 * $fits_y_scale]
                 set CSO [expr -1 * $CSO] 
                 set CRO [expr -1 * $CRO]
                 }


     # CSO = 1 -> mouse button 2 (B2) mask rotates left -> decreasing 
     #            position angle (PA)
     # CSO =-1 -> mouse button 2 (B2) mask rotates left -> increasing 
     #            position angle (PA)
     
     # coord/mm * scal = coord/arcsec
     set scal [expr $atd * $focscale]
     # since mos(,) in world coordinates, canvscale not nessecarry
     # -----------------------------------------
     set trigo [expr 1/cos($wy*0.0174532)]
     # -----------------------------------------
     # trigo is for declination dependent hour-angle scale 
     puts $lfId "Init_Mask_World: canvscale=$canvscale \
             wx=$wx wy=$wy trigo=$trigo" 
     puts $lfId "Init_Mask_World: focscale=$focscale scal=$scal \
           arc_to_deg=$atd"
     # absolute alp/del coordinates
     # 19 pairs of slit blades

     # puts stdout ">[$this cget -modus]< >[$this cget -collimat]<"

     # initial mode-dependent bindings maybe be overridden by 
     # slitless spectro(polarime)try
     # hits, lss and ipol(hr) use only bind_frame
     
     switch -exact -- [$this cget -modus] {
         mos bind_mos
        pmos bind_mos
         img bind_img
        ipol { if {[string compare $mask "pol"]} { bind_ipol } 
               # bind_frame already in constr. 
               # bind_img called by bind_ipol
             }
         mxu { bind_mxu
               bind_align
             }
         lss bind_align 
        hits bind_align 
        echs { bind_align 
               bind_echs
             }
        default { # ipol(hr) 
                }
        }

     
     #  there is a slit-less MOS mode with $slit_width=open
     #  in this submode of MOS blades can be positioned as in IMG
     #  we handle this by a mode change. 
     #  this means mos->img; pmos->ipol
    # puts stdout "initial = [$this cget -modus] $mask"

     if {[string compare [$this cget -slit_width] open]==0} {

        if {[string compare [$this cget -modus] mos]==0} {
              # slitwidth=open in mos : handle as IMG 
              $this configure -modus img
              $this bind_img
              # puts stdout "SLS mos -> img | [$this cget -modus] $mask"

           } elseif {[string compare [$this cget -modus] img]==0} {
              $this configure -modus mos
              $this bind_mos
              # puts stdout "SLS img -> mos | [$this cget -modus] $mask"

           } elseif {[string compare [$this cget -modus] pmos]==0} {
              $this configure -modus ipol
              if {[string match *HR*  [$this cget -collimat] ]} {
                    set mask pol 
                    $this bind_ipol 
              # puts stdout "SLS pmos -> ipol | [$this cget -modus] $mask"
                    } else {
                    set mask mos
                    $this bind_img
              # puts stdout "SLS pmos -> ipol | [$this cget -modus] $mask"
                    }

           } elseif {[string compare [$this cget -modus] ipol]==0} {
              $this configure -modus pmos
              set mask mos
              $this bind_mos
              # puts stdout "SLS ipol -> pmos | [$this cget -modus] $mask"
           }
         }

     # the current instrumental modus
     set lmod [$this cget -modus]
     # puts stdout $lmod
    # puts stdout "after = [$this cget -modus] $lmod $mask"

     # collimator as a local variable 
     set co_flag [string match *SR* [$this cget -collimat] ]
     # --------------------------
     # set the initial slit-width
     # --------------------------
     #  [mos($i,b)] in arcsec 
     # park position beyond focal plane 
     set mos(0,b) [format "%3.1f" [expr 6.0 * $focscale] ] 
     switch -exact -- [$this cget -modus] {
     echs -
     mos {# for mos mode slit-width has a spectroscopic meaning (resolution)
          for {set i 1} {$i < 20} {incr i 1} {
              set mos($i,b) [$this cget -slit_width] }
         }
    pmos {
         # for pmos standard resolution collimator  mode 
         # the even slit blades are used for spectroscopy
         # odd slit blades are used as a dekker  
         for {set i 1} {$i < 20} {incr i 1} {
             set mos($i,b) [$this cget -slit_width]} 
         }
     img {
	 puts "MODE IMG ...."
         # for IMG, slit-width has the meaning of 'obscuration area'
         if {[string match *SR* [$this cget -collimat] ]} {
            # in img standard mode all slit blades are open
            #                                                        mm fp
            for {set i 1} {$i < 20} {incr i 1} { 
                 set mos($i,b) [expr 240 * $focscale ] }
            } else {
            # in img highres mode slit blades 1-5 and 15-20 are closed 
            # and are used as a dekker  to close the field beyond the 
            # projected CCD field.
            # slit blades 6-14 are open with respect to the highres scale
            #                                                          mm fp            
            if {[string compare [$this cget -instrument] FORS2]==0} { 
	       puts "IMG HR mode"
               for {set i  1} {$i <  5} {incr i 1} { set mos($i,b)   $mos(0,b) }
               for {set i  5} {$i < 17} {incr i 1} { 
                   set mos($i,b)   [expr 140 * $focscale] }
               for {set i 17} {$i < 20} {incr i 1} { set mos($i,b)   $mos(0,b) }
	    } else {
               for {set i  1} {$i <  6} {incr i 1} { set mos($i,b)   $mos(0,b) }
               for {set i  6} {$i < 15} {incr i 1} { 
                   set mos($i,b)   [expr 115 * $focscale] }
               for {set i 15} {$i < 20} {incr i 1} { set mos($i,b)   $mos(0,b) }
	     }
            }
         }
    ipol {
         # in ipol standard mode all even slit blades are open
         # all odd slit blades are closed, (10 is open)
         for {set i 2} {$i < 20} {incr i 2} { 
              set mos($i,b) [expr 240 * $focscale] } 
         for {set i 1} {$i < 20} {incr i 2} { set mos($i,b) $mos(0,b) } 
         # in ipol highres mode, all MOS-slit blades are open, and a
         # separate HIGHRES-IPOL-MASK is initialized
         # ......................
         }
     lss { # do nothing }
     hits { # do nothing }
     mxu {  
            # this might not be the right place
	    #
            set XSRMIN -90.0
            set XSRMAX 90.0
            # set YSRMIN -115.0
            # set YSRMAX 115.0
         }
    default {puts $lfId "Init_Mask_World: Warning modus"}
     }
 
     puts "flag 3"
     #set dummy [expr $mos($i,b)/$focscale]
     #puts $lfId "$i $mos($i,b) arcsec = $dummy mm"


     # ----------------------------------
     # set points on the North-South line 
     # -----------------------------------
     # chapter 1.0.0.0.0  set world coords
 switch -exact -- $mask {
 mos {
     # chapter 1.1.0.0.0  mask mos
     # all coords for the MOS mask 
     # first set only the coords of points with 
     # delta rectaszension=0. Those points are on the 
     # central north-south axis 
     # 19 slit blades, unit is world coords
     #  mos(1..19,bcfg) in WCS
     for { set i 1} {$i < 20} {incr i 1} {
      set mos($i,xb) $wx 
      set mos($i,yb) [expr $wy+$scal*$mosslitmin($i)]; # South side (i=1)
      set mos($i,xc) $wx  
      set mos($i,yc) $mos($i,yb)

      set mos($i,xf) $mos($i,xb)
      set mos($i,yf) [expr $wy+$scal*$mosslitmax($i)]
      set mos($i,xg) $mos($i,xf)  
      set mos($i,yg) $mos($i,yf)
      }
      # we draw slit blade 19 and slit blade 1 not larger than the CCD area
      # in order to avoid user-defined slit positions outside the CCD frame 
           # set mos(1,yb) [expr $wy+$scal*$YSRMIN]
           # set mos(1,yc) $mos(1,yb)
           # set mos(1,yf) [expr $wy+$scal*$YSRMIN]
           # set mos(1,yg) $mos(1,yf)

           # set mos(19,yb) [expr $wy+$scal*$YSRMAX]
           # set mos(19,yc) $mos(19,yb)
           # set mos(19,yf) [expr $wy+$scal*$YSRMAX]
           # set mos(19,yg) $mos(19,yf)
     }
 lss { 
     # chapter 1.2.0.0.0  mask lss
     # only 9 long-slits available
     for {set i 1} {$i < 10} {incr i 1} {
         # reference point on NS
         set mos($i,xb) $wx
         set mos($i,yb) [expr $wy+$scal*$longslitymin($i)]; # South side (i=1) 
         set mos($i,xf) $wx
         set mos($i,yf) [expr $wy+$scal*$longslitymax($i)]; # North side (i=1) 
         }
     }
hits { 
     # chapter 1.3.0.0.0  mask hits
     # only 8 hits-slits available + 2 pinholes
     # 9 = big pinhole, 10 = small pinhole
     for {set i 1} {$i < 10} {incr i 1} {
         # reference point on NS
         set mos($i,xb) $wx
         set mos($i,yb) [expr $wy+$scal*$htslitymin($i)]; # South side (i=1) 
         set mos($i,xf) $wx
         set mos($i,yf) [expr $wy+$scal*$htslitymax($i)]; # North side (i=1) 
         }
     # small pinhole in 10
     set mos(10,xb) $wx; set mos(10,xf) $wx
     set mos(10,yb) [expr $wy+$scal*$htslitymin(0)]; 
     set mos(10,yf) [expr $wy+$scal*$htslitymax(0)];
     }
pol  { 
     # chapter 1.4.0.0.0  mask pol
     # the high-res polarimetric imaging mask       
     # only 9  image stripes available
     for {set i 1} {$i < 10} {incr i 1} {
         # reference point on NS
         set mos($i,xb) $wx
         set mos($i,yb) [expr $wy+$scal*$polarimslitymin($i)]; # South side  
         set mos($i,xf) $wx
         set mos($i,yf) [expr $wy+$scal*$polarimslitymax($i)]; # North side  
         }
     }
mxu { # do nothing, there will be a particular create_mxu_slit }
 default {puts $lfId "Init_Mask_World: Warning mask"  }
     }


     # some presettings for different masks
     if {[string compare $mask "mos"]==0} {
        set local_max 20
        set local_set {b c f g}
        } else {
        set local_max 10
        set local_set {b f}
        }
     if {[string compare $mask "mxu"]==0} { set local_max 0 }

        # puts stdout "mask = >>$mask<< max=>>$local_max<< set=>>$local_set<<"

    
     #----------------------------------
     # transform points on North-South line 
     # from world to canvas coords 
     # -------------------------------------
     # chapter 2.0.0.0.0  world to canvas
     # we transform these points into canvas coords 
     # of middle-axis the 19 slit blades
     # for small frames, "deg $EQX -> canvas" does not work 
     # in this case we have to create our own scale for mask points 
     # outside the frame 
     set bonsai_flag 0
     # first test if everything would be ok 
     # : if the image is large enough
     for { set i 1} { $i < $local_max} { incr i 1} {
        foreach val $local_set {
            set a_w $mos($i,x$val); set b_w $mos($i,y$val)
            set a_c $a_w; set b_c $b_w
            catch { $m_image convert coords $a_w $b_w "deg $EQX" a_c b_c canvas }
            if ![expr $a_c-$a_w] {puts "flag 0" ;set bonsai_flag 1}
            }
     }
        # for IPOL high resolution mos($i,yi) is much smaller than the 
        # focal plane, therefore test additionally the focal plane 
           
            set a_w $wx; set b_w  [expr $wy+$scal*$YSRMAX]
           
            set a_c $a_w; set b_c $b_w
            catch {$m_image convert coords $a_w $b_w "deg $EQX" a_c b_c canvas }
            if ![expr $a_c-$a_w] {puts "flag 2" ; set bonsai_flag 1}

            set a_w $wx; set b_w  [expr $wy+$scal*$YSRMIN]
            set a_c $a_w; set b_c $b_w
            catch {$m_image convert coords $a_w $b_w "deg $EQX" a_c b_c canvas }
            if ![expr $a_c-$a_w] {set bonsai_flag 1}
            
           
     
     # here we need further requests for IPOL High-RES


     #set bonsai_flag 1
     
     #- Check bonsai effect for ccd dimmensions -#
     set m1x $wx; set m1y [expr $wy+$scal*$CCDMASTER_YSRMIN]
     set s1x $wx; set s1y [expr $wy+$scal*$CCDSLAVE_YSRMIN]    
      if {![catch {$m_image convert coords $m1x $m1y "deg $EQX" m1x m1y canvas}]} {set bonsai_flag 1}
      if {![catch {$m_image convert coords $m2x $m2y "deg $EQX" m2x m2y canvas}]} {set bonsai_flag 1}
      if {![catch {$m_image convert coords $s1x $s1y "deg $EQX" s1x s1y canvas}]} {set bonsai_flag 1}
      if {![catch {$m_image convert coords $s2x $s2y "deg $EQX" s2x s2y canvas}]} {set bonsai_flag 1}

     
     if $bonsai_flag {puts $lfId "world2: BONSAI ON " 
                     } else {
                      puts $lfId "world2: BONSAI OFF " }

     if $bonsai_flag {
	 puts  "world2: BONSAI ON "
        # puts stdout "world2: Warning: frame <= FORS Field  !!!"
        # puts stdout "world2: Warning: Use a larger frame, !!!"
        # puts stdout "world2: Warning: WCS extrapolation by hand ..."
         
        # calculate the matrix coefficients
        C_wc coeff_wc $wx $wy

        # now all coefficients are known 
        # convert 'by hand' 

        for { set i 1} { $i < $local_max} { incr i 1} {
           foreach val $local_set {
               set dum [C_wc w2c $mos($i,x$val) $mos($i,y$val)]
               set mos($i,x$val) [lindex $dum 0]
               set mos($i,y$val) [lindex $dum 1]
               }
           }
        } else {
      # the frame is large enough for the whole mask: 
      # if the test was ok, convert coords using the implemented WCS 
      foreach val $local_set {
        for { set i 1} { $i < $local_max} { incr i 1} {
            $m_image convert coords $mos($i,x$val) $mos($i,y$val) \
            "deg $EQX" mos($i,x$val) mos($i,y$val) canvas 
            }
        }
      }
 
    
     # ---------------------------------------
     # handle points beyond the central north-south line 
     # ---------------------------------------
     # chapter 3.0.0.0.0  beyond north-south line, border points 
     # MOS limit switches 
     # declination canvas coords
     # note : m can be zero 
     # take the FOCFIELD values only to  determine 
     # slope for all masks to 
     # lower south part
     set p1x $wx; set p1y [expr $wy+$scal*$YSRMIN]
     
     #- CCD Master -#
     set ccdm1x $wx; set ccdm1y [expr $wy+$scal*$CCDMASTER_YSRMIN]
     #- CCD Slave -#
     set ccds1x $wx; set ccds1y [expr $wy+$scal*$CCDSLAVE_YSRMIN]
     # upper northern point
     set p2x $wx; set p2y [expr $wy+$scal*$YSRMAX]
     #- CCD Master -#
     set ccdm2x $wx; set ccdm2y [expr $wy+$scal*$CCDMASTER_YSRMAX]
     #- CCD Slave -#
     set ccds2x $wx; set ccds2y [expr $wy+$scal*$CCDSLAVE_YSRMAX]
     
      
    
     
      
     # transform these points into canvas coords
     if {$bonsai_flag} {
	
        set dum [C_wc w2c $p1x $p1y]
        set p1x [lindex $dum 0] 
        set p1y [lindex $dum 1]
        set mdum [C_wc w2c $ccdm1x $ccdm1y]
	set ccdm1x [lindex $mdum 0] 
        set ccdm1y [lindex $mdum 1]
        set sdum [C_wc w2c $ccds1x $ccds1y]
	set ccds1x [lindex $sdum 0] 
        set ccds1y [lindex $sdum 1]

        set dum [C_wc w2c $p2x $p2y]
        set p2x [lindex $dum 0] 
        set p2y [lindex $dum 1]
	set mdum [C_wc w2c $ccdm2x $ccdm2y]
	set ccdm2x [lindex $mdum 0] 
        set ccdm2y [lindex $mdum 1]
        set sdum [C_wc w2c $ccds2x $ccds2y]
	set ccds2x [lindex $sdum 0] 
        set ccds2y [lindex $sdum 1]

      } else {
	
	puts "p1x: $p1x"
	puts "p2y: $p2y"
        $m_image convert coords $p1x $p1y "deg $EQX" p1x p1y canvas
        $m_image convert coords $p2x $p2y "deg $EQX" p2x p2y canvas
	
        puts "ccdm1x: $ccdm1x"
        puts "ccdm1y: $ccdm1y"
	puts "ccds1x: $ccdm1x"
        puts "ccds1y: $ccdm1y"
	$m_image convert coords $ccdm1x $ccdm1y "deg $EQX" ccdm1x ccdm1y canvas
        $m_image convert coords $ccdm2x $ccdm2y "deg $EQX" ccdm2x ccdm2y canvas
        $m_image convert coords $ccds1x $ccds1y "deg $EQX" ccds1x ccds1y canvas
        $m_image convert coords $ccds2x $ccds2y "deg $EQX" ccds2x ccds2y canvas
      }


     
     set dx [expr $p1x-$p2x]
     set dy [expr $p1y-$p2y] 

     set ccdmx [expr $ccdm1x - $ccdm2x]
     set ccdsx [expr $ccds1x - $ccds2x]
     set ccdmy [expr $ccdm1y - $ccdm2y]
     set ccdsy [expr $ccds1y - $ccds2y]
     set ctom [expr sqrt($dx*$dx+$dy*$dy)/($YSRMAX-$YSRMIN)]
     set ccdm_ctom [expr sqrt($ccdmx*$ccdmx+$ccdmy*$ccdmy)/($CCDMASTER_YSRMAX-$CCDMASTER_YSRMIN)]
     set ccds_ctom [expr sqrt($ccdsx*$ccdsx+$ccdsy*$ccdsy)/($CCDSLAVE_YSRMAX-$CCDSLAVE_YSRMIN)]
    
     # ne_flag=1 means, image is not rotated, 
     # the slope of the mask can be infinity in canvas coords
     # this case has to be handeled differently

     set ne_flag 0
     if {[expr abs($p1x-$p2x)]<1e-2} {set ne_flag 1} 
     # puts stdout [format "slope =  %e" [expr abs($p1x-$p2x)] ]

     if {$debug} {
      if {$ne_flag} {puts stdout "world2: frame || WCS -> not rotated" } else {
                     puts stdout "world2: frame \/ WCS -> rotated" }
        }


     if {$ne_flag} {
     # chapter 3.1.0.0.0  perpendicular 
     # the mask (the coords ) is perpendicular to the frame 
     # y-values are retained, x-values easy to handle
        for { set i 1} {$i < $local_max} {incr i 1} {
            set mos($i,ya) $mos($i,yb)
            set mos($i,yd) $mos($i,yb)
            set mos($i,ye) $mos($i,yf)
            set mos($i,yh) $mos($i,yf)
     switch -exact -- $mask {
            lss {
            # chapter 3.1.1.0.0  mask lss      
            # LSS mask
            set mos($i,xa) [expr $mos($i,xb)+\
                $fits_x_scale*$ctom*$longslitxmin($i)]
            set mos($i,xd) [expr $mos($i,xb)+\
                $fits_x_scale*$ctom*$longslitxmax($i)]
            set mos($i,xe) [expr $mos($i,xf)+\
                $fits_x_scale*$ctom*$longslitxmin($i)]
            set mos($i,xh) [expr $mos($i,xf)+\
                $fits_x_scale*$ctom*$longslitxmax($i)]
            }
           hits { 
            # chapter 3.1.2.0.0  mask hits     
            # small pinhole not supported
            # HITS-mask 
            set mos($i,xa) [expr $mos($i,xb)+\
                $fits_x_scale*$ctom*$htslitxmin($i)]
            set mos($i,xd) [expr $mos($i,xb)+\
                $fits_x_scale*$ctom*$htslitxmax($i)]
            set mos($i,xe) [expr $mos($i,xf)+\
                $fits_x_scale*$ctom*$htslitxmin($i)]
            set mos($i,xh) [expr $mos($i,xf)+\
                $fits_x_scale*$ctom*$htslitxmax($i)]
            }
            pol {
            # chapter 3.1.3.0.0  mask pol      
            # IPOL HIGHRES
            set mos($i,xa) [expr $mos($i,xb)+\
                $ctom*$fits_x_scale*$polarimslitxmin($i)]
            set mos($i,xd) [expr $mos($i,xb)+\
                $ctom*$fits_x_scale*$polarimslitxmax($i)]
            set mos($i,xe) [expr $mos($i,xf)+\
                $ctom*$fits_x_scale*$polarimslitxmin($i)]
            set mos($i,xh) [expr $mos($i,xf)+\
                $ctom*$fits_x_scale*$polarimslitxmax($i)]
            }
            mos {
            # chapter 3.1.4.0.0  mask mos      
            # MOS mask
            set mos($i,xa) [expr $mos($i,xb)+$fits_x_scale*$ctom*$XBLADEA]
            set mos($i,xd) [expr $mos($i,xb)+$fits_x_scale*$ctom*$XBLADEB]
            set mos($i,xe) [expr $mos($i,xf)+$fits_x_scale*$ctom*$XBLADEA]
            set mos($i,xh) [expr $mos($i,xf)+$fits_x_scale*$ctom*$XBLADEB]
            }
            mxu { # do nothing, there will be a particular create_mxu_slit }
            default { puts stdout "world2: Warning: mask" }
            }

            }
     } else {
     # chapter 3.2.0.0.0  slope       
     # the coord axis are not perpendicular to the frame 
     # first we need the slope of wcs on the canvas
     set loc_m [expr ($p1y-$p2y)/($p1x-$p2x)]
     set loc_m1 [expr 1.0/sqrt($loc_m*$loc_m+1.0)]
     set ccdm_loc_m [expr ($ccdm1y-$ccdm2y)/($ccdm1x-$ccdm2x)]
     set ccdm_loc_m1 [expr 1.0/sqrt($ccdm_loc_m*$ccdm_loc_m+1.0)]
     set ccds_loc_m [expr ($ccds1y-$ccds2y)/($ccds1x-$ccds2x)]
     set ccds_loc_m1 [expr 1.0/sqrt($ccds_loc_m*$ccds_loc_m+1.0)] 
     # puts stdout "m = $loc_m m1 = $loc_m1 ctom = $ctom"

# puts stdout "world2: CRO=$CRO CDELT1=$fits_x_scale 
#             prod=[expr $CRO*$fits_x_scale]"

#  switch -exact -- $mask {
#  mos { set loc_m [expr ($mos(19,yb)-$mos(1,yb))/($mos(19,xb)-$mos(1,xb))] }
#  lss { set loc_m [expr ($mos(1,yf)-$mos(1,yb))/($mos(1,xf)-$mos(1,xb))] }
#  pol { set loc_m [expr ($mos(9,yb)-$mos(1,yb))/($mos(9,xb)-$mos(1,xb))] }
#      }
#      set loc_m1 [expr 1.0/sqrt($loc_m*$loc_m+1.0)]
#      puts stdout "m = $loc_m m1 = $loc_m1 ctom = $ctom"


     #these are the border points of the 19 slit blades
     for { set i 1} {$i < $local_max} {incr i 1} {
       switch -exact -- $mask {
        lss {
            # chapter 3.2.1.0.0  mask lss
            # but here the long-slit mask 
         set mos($i,ya) [expr $mos($i,yb)-\
             $CRO*$fits_x_scale*$ctom*$longslitxmin($i)*$loc_m1]
         set mos($i,xa) [expr $mos($i,xb)-$loc_m*($mos($i,ya)-$mos($i,yb))]

         set mos($i,ye) [expr $mos($i,yf)-\
             $CRO*$fits_x_scale*$ctom*$longslitxmin($i)*$loc_m1]
         set mos($i,xe) [expr $mos($i,xf)-$loc_m*($mos($i,ye)-$mos($i,yf))]

         set mos($i,yd) [expr $mos($i,yb)-\
             $CRO*$fits_x_scale*$ctom*$longslitxmax($i)*$loc_m1]
         set mos($i,xd) [expr $mos($i,xb)-$loc_m*($mos($i,yd)-$mos($i,yb))]

         set mos($i,yh) [expr $mos($i,yf)-\
             $CRO*$fits_x_scale*$ctom*$longslitxmax($i)*$loc_m1]
         set mos($i,xh) [expr $mos($i,xf)-$loc_m*($mos($i,yh)-$mos($i,yf))]
            }
       hits {
            # chapter 3.2.2.0.0  mask hits 
            # but here the hits-slit mask 
         set mos($i,ya) [expr $mos($i,yb)-\
             $CRO*$fits_x_scale*$ctom*$htslitxmin($i)*$loc_m1]
         set mos($i,xa) [expr $mos($i,xb)-$loc_m*($mos($i,ya)-$mos($i,yb))]

         set mos($i,ye) [expr $mos($i,yf)-\
             $CRO*$fits_x_scale*$ctom*$htslitxmin($i)*$loc_m1]
         set mos($i,xe) [expr $mos($i,xf)-$loc_m*($mos($i,ye)-$mos($i,yf))]

         set mos($i,yd) [expr $mos($i,yb)-\
             $CRO*$fits_x_scale*$ctom*$htslitxmax($i)*$loc_m1]
         set mos($i,xd) [expr $mos($i,xb)-$loc_m*($mos($i,yd)-$mos($i,yb))]

         set mos($i,yh) [expr $mos($i,yf)-\
             $CRO*$fits_x_scale*$ctom*$htslitxmax($i)*$loc_m1]
         set mos($i,xh) [expr $mos($i,xf)-$loc_m*($mos($i,yh)-$mos($i,yf))]
            }
        pol {
            # chapter 3.2.3.0.0  mask pol 
            # IPOL HIGHRES
         set mos($i,ya) [expr $mos($i,yb)-\
             $CRO*$fits_x_scale*$ctom*$polarimslitxmin($i)*$loc_m1]
         set mos($i,xa) [expr $mos($i,xb)-$loc_m*($mos($i,ya)-$mos($i,yb))]

         set mos($i,ye) [expr $mos($i,yf)-\
             $CRO*$fits_x_scale*$ctom*$polarimslitxmin($i)*$loc_m1]
         set mos($i,xe) [expr $mos($i,xf)-$loc_m*($mos($i,ye)-$mos($i,yf))]

         set mos($i,yd) [expr $mos($i,yb)-\
             $CRO*$fits_x_scale*$ctom*$polarimslitxmax($i)*$loc_m1]
         set mos($i,xd) [expr $mos($i,xb)-$loc_m*($mos($i,yd)-$mos($i,yb))]

         set mos($i,yh) [expr $mos($i,yf)-\
             $CRO*$fits_x_scale*$ctom*$polarimslitxmax($i)*$loc_m1]
         set mos($i,xh) [expr $mos($i,xf)-$loc_m*($mos($i,yh)-$mos($i,yf))]
             }
        mos {
            # chapter 3.2.4.0.0  mask mos 
            # MOS mask
     set mos($i,ya) [expr $mos($i,yb)-$CRO*$fits_x_scale*$ctom*$XBLADEA*$loc_m1]
     set mos($i,xa) [expr $mos($i,xb)-$loc_m*($mos($i,ya)-$mos($i,yb))]

     set mos($i,ye) [expr $mos($i,yf)-$CRO*$fits_x_scale*$ctom*$XBLADEA*$loc_m1]
     set mos($i,xe) [expr $mos($i,xf)-$loc_m*($mos($i,ye)-$mos($i,yf))]

     set mos($i,yd) [expr $mos($i,yb)-$CRO*$fits_x_scale*$ctom*$XBLADEB*$loc_m1]
     set mos($i,xd) [expr $mos($i,xb)-$loc_m*($mos($i,yd)-$mos($i,yb))]

     set mos($i,yh) [expr $mos($i,yf)-$CRO*$fits_x_scale*$ctom*$XBLADEB*$loc_m1]
     set mos($i,xh) [expr $mos($i,xf)-$loc_m*($mos($i,yh)-$mos($i,yf))]
            }
     mxu { # do nothing }
         default { puts stdout "world2: Warning: mask beyond N-S" }
         }
         }
     # this works
     } 

   

#    for { set i 1} { $i <20} { incr i 1} {
#        puts stdout "$i  $mos($i,xa) $mos($i,xb) $mos($i,xc) $mos($i,xd) \
#           $mos($i,xe) $mos($i,xf) $mos($i,xg) $mos($i,xh)"  
#        puts stdout "$i  $mos($i,ya) $mos($i,yb) $mos($i,yc) $mos($i,yd) \
#           $mos($i,ye) $mos($i,yf) $mos($i,yg) $mos($i,yh)"  
#         }



     # ---------------------------------------------
     # now open the slit blades according to $mos($i,b)
     # --------------------------------------------
     # chapter 4.0.0.0.0  beyond north-south line, slit points 
     # mos(i,b) in arcsec, focscale = mm_to_as
     # mos(i,b)/mm_to_as/ctom = canvas 
     # for MOS, all slits are still in the park position, with common slit width

switch -exact -- [$this cget -modus] {
echs -
 mos {
     # chapter 4.1.0.0.0  modus mos                             
     if $ne_flag {
       # chapter 4.1.1.0.0  perpendicular                         
       #  note : initial slitwidth not 6 mmm but 'slit_width' arcsec
       for { set i 1} {$i < 20} {incr i 1} {
         # all y-values are retained
         #set mos($i,xb) [expr $mos($i,xb)-$mos($i,b)/$focscale*\
         #    $fits_x_scale*$ctom/2]
         #set mos($i,xc) [expr $mos($i,xc)+$mos($i,b)/$focscale*\
         #    $fits_x_scale*$ctom/2]
         #set mos($i,xf) [expr $mos($i,xf)-$mos($i,b)/$focscale*\
         #    $fits_x_scale*$ctom/2]
         #set mos($i,xg) [expr $mos($i,xg)+$mos($i,b)/$focscale*\
         #    $fits_x_scale*$ctom/2]
         # open beyond the focal plane 
         set mos($i,xb) $mos($i,xa)
         set mos($i,xc) \
             [expr $mos($i,xb)+$mos($i,b)/$focscale*$fits_x_scale*$ctom]
         set mos($i,xf) $mos($i,xe)
         set mos($i,xg) \
             [expr $mos($i,xf)+$mos($i,b)/$focscale*$fits_x_scale*$ctom]
         }
       } else {
       # chapter 4.1.2.0.0  slope                                 
       for { set i 1} {$i < 20} {incr i 1} {
         set dummy $mos($i,yb)
#         set mos($i,yb) [expr $dummy-\
#             $fits_y_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
#         set mos($i,xb) [expr $mos($i,xb)-$loc_m*($mos($i,yb)-$dummy)]
#
#         set dummy $mos($i,yf)
#         set mos($i,yf) [expr $dummy-\
#             $fits_y_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
#         set mos($i,xf) [expr $mos($i,xf)-$loc_m*($mos($i,yf)-$dummy)]
#
#         set dummy $mos($i,yc)
#         set mos($i,yc) [expr $dummy+\
#             $fits_y_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
#         set mos($i,xc) [expr $mos($i,xc)-$loc_m*($mos($i,yc)-$dummy)]
#
#         set dummy $mos($i,yg)
#         set mos($i,yg) [expr $dummy+\
#             $fits_y_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
#         set mos($i,xg) [expr $mos($i,xg)-$loc_m*($mos($i,yg)-$dummy)]
          
          # mos slitlets beyond the focal plane 
          # this position can be used to unselect mos slits 
          set mos($i,xb) $mos($i,xa); set mos($i,yb) $mos($i,ya)
          set mos($i,yc) [expr $mos($i,ya)+\
                         $fits_y_scale*$ctom*$mos($i,b)/$focscale*$loc_m1]
          set mos($i,xc) [expr  $mos($i,xa)-$loc_m*($mos($i,yc)-$dummy)]

          set mos($i,xf) $mos($i,xe); set mos($i,yf) $mos($i,ye);
          set mos($i,yg) [expr $mos($i,ye)+\
                         $fits_y_scale*$ctom*$mos($i,b)/$focscale*$loc_m1]
          set mos($i,xg) [expr $mos($i,xe)-$loc_m*($mos($i,yg)-$dummy)]
         }
       }
     }
pmos {
     # highres collimator caught in the constructor
     # chapter 4.2.0.0.0  modus pmos                            
     if $ne_flag {
       # chapter 4.2.1.0.0  perpendicular                         
       for { set i 2} {$i < 20} {incr i 2} {
         # all y-values are retained
         set mos($i,xb) [expr $mos($i,xb)-$mos($i,b)/2/$focscale*\
             $fits_x_scale*$ctom]
         set mos($i,xc) [expr $mos($i,xc)+$mos($i,b)/2/$focscale*\
             $fits_x_scale*$ctom]
         set mos($i,xf) [expr $mos($i,xf)-$mos($i,b)/2/$focscale*\
             $fits_x_scale*$ctom]
         set mos($i,xg) [expr $mos($i,xg)+$mos($i,b)/2/$focscale*\
             $fits_x_scale*$ctom]
       }
       } else {
       # chapter 4.2.2.0.0  slope                               
       for { set i 2} {$i < 20} {incr i 2} {
         set dummy $mos($i,yb)
         set mos($i,yb) [expr $dummy+\
             $fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xb) [expr $mos($i,xb)-$loc_m*($mos($i,yb)-$dummy)]
  
         set dummy $mos($i,yf)
         set mos($i,yf) [expr $dummy+\
             $fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xf) [expr $mos($i,xf)-$loc_m*($mos($i,yf)-$dummy)]

         set dummy $mos($i,yc)
         set mos($i,yc) [expr $dummy-\
             $fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xc) [expr $mos($i,xc)-$loc_m*($mos($i,yc)-$dummy)]

         set dummy $mos($i,yg)
         set mos($i,yg) [expr $dummy-\
             $fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xg) [expr $mos($i,xg)-$loc_m*($mos($i,yg)-$dummy)]
         }
       }

     # this is for both cases of ne_flag (close the odd slit blades )
     for { set i 1} {$i < 20} {incr i 2} {
         # closed
         set mos($i,yb) $mos($i,ya)
         set mos($i,xb) $mos($i,xa)
         set mos($i,yf) $mos($i,ye)
         set mos($i,xf) $mos($i,xe)
         set mos($i,yc) $mos($i,ya)
         set mos($i,xc) $mos($i,xa)
         set mos($i,yg) $mos($i,ye)
         set mos($i,xg) $mos($i,xe)
       }
     }
 img {
     # chapter 4.3.0.0.0  modus img                             
     # the slits of the central slit blades 6-14 will be at x=0 anyway
     if $ne_flag {
     # chapter 4.3.1.0.0  perpendicular                         
       # perpendicular case
       if $co_flag { #standard resolution collimator mode 
          # chapter 4.3.1.1.0  standard                              
       for { set i 1} {$i < 20} {incr i 1} {
         set mos($i,xb) [expr $mos($i,xb)-$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         set mos($i,xc) [expr $mos($i,xc)+$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         set mos($i,xf) [expr $mos($i,xf)-$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         set mos($i,xg) [expr $mos($i,xg)+$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         }
       } else {
       # chapter 4.3.1.2.0  highres                               
       # high-res modus 
       # close the outside slit blades explicitely 
      
       if {[string compare [$this cget -instrument] FORS2]==0} { 
	 puts "IMG HIGHRES MOS"
         foreach i { 1 2 3 4 17 18 19} {
         set mos($i,yb) $mos($i,ya)
         set mos($i,xb) $mos($i,xa)

         set mos($i,yf) $mos($i,ye)
         set mos($i,xf) $mos($i,xe)

         set mos($i,yc) $mos($i,ya)
         set mos($i,xc) $mos($i,xa)

         set mos($i,yg) $mos($i,ye)
         set mos($i,xg) $mos($i,xe)
         }
	    for { set i 5} {$i < 17} {incr i 1} {
         set mos($i,xb) [expr $mos($i,xb)-$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         set mos($i,xc) [expr $mos($i,xc)+$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         set mos($i,xf) [expr $mos($i,xf)-$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         set mos($i,xg) [expr $mos($i,xg)+$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         }
       } else { 
	   puts "FORS1 case"
         foreach i { 1 2 3 4 5 15 16 17 18 19} {	     
         set mos($i,yb) $mos($i,ya)
         set mos($i,xb) $mos($i,xa)

         set mos($i,yf) $mos($i,ye)
         set mos($i,xf) $mos($i,xe)

         set mos($i,yc) $mos($i,ya)
         set mos($i,xc) $mos($i,xa)

         set mos($i,yg) $mos($i,ye)
         set mos($i,xg) $mos($i,xe)
         }
       # open the inner slit blades accroding to mos(i,b)
       for { set i 6} {$i < 15} {incr i 1} {
	 puts "IMG HIGHRES FOR FORS1 IMAGES"
         set mos($i,xb) [expr $mos($i,xb)-$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         set mos($i,xc) [expr $mos($i,xc)+$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         set mos($i,xf) [expr $mos($i,xf)-$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         set mos($i,xg) [expr $mos($i,xg)+$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         }
     }
       }
     } else {
     # chapter 4.3.2.0.0  slope                                 
     # normal non-perpendicular case
     # if highres or standard: the inner slit blades are open according to 
     # mos(i,b)  
     if {[string compare [$this cget -instrument] FORS2] == 0} { 
	 puts "FORS2 IMG MOS SETTING ..."
         foreach i { 5 6 7 8 9 10 11 12 13 14 15 16} {
         set dummy $mos($i,yb)
         set mos($i,yb) [expr $dummy+\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xb) [expr $mos($i,xb)-$loc_m*($mos($i,yb)-$dummy)]

         set dummy $mos($i,yf)
         set mos($i,yf) [expr $dummy+\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xf) [expr $mos($i,xf)-$loc_m*($mos($i,yf)-$dummy)]

         set dummy $mos($i,yc)
         set mos($i,yc) [expr $dummy-\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xc) [expr $mos($i,xc)-$loc_m*($mos($i,yc)-$dummy)]

         set dummy $mos($i,yg)
         set mos($i,yg) [expr $dummy-\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xg) [expr $mos($i,xg)-$loc_m*($mos($i,yg)-$dummy)]
         }
	# if img and highres close the outside slit blades
     foreach i { 1 2 3 4 17 18 19} {
       if $co_flag { #standard
         # chapter 4.3.2.1.0  standard                              
         # open slit blades according to mos(i,b) 
         set dummy $mos($i,yb)
         set mos($i,yb) [expr $dummy+\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xb) [expr $mos($i,xb)-$loc_m*($mos($i,yb)-$dummy)]

         set dummy $mos($i,yf)
         set mos($i,yf) [expr $dummy+\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xf) [expr $mos($i,xf)-$loc_m*($mos($i,yf)-$dummy)]

         set dummy $mos($i,yc)
         set mos($i,yc) [expr $dummy-\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xc) [expr $mos($i,xc)-$loc_m*($mos($i,yc)-$dummy)]

         set dummy $mos($i,yg)
         set mos($i,yg) [expr $dummy-\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xg) [expr $mos($i,xg)-$loc_m*($mos($i,yg)-$dummy)]
       } else {
         # chapter 4.3.2.2.0  highres                               
         # highres modus, close them completely  
         set mos($i,yb) $mos($i,ya)
         set mos($i,xb) $mos($i,xa)

         set mos($i,yf) $mos($i,ye)
         set mos($i,xf) $mos($i,xe)

         set mos($i,yc) $mos($i,ya)
         set mos($i,xc) $mos($i,xa)

         set mos($i,yg) $mos($i,ye)
         set mos($i,xg) $mos($i,xe)
       }
   }
} else {
     foreach i { 6 7 8 9 10 11 12 13 14} {
	 puts "FORS1 IMG MOS SETTING ..."
         set dummy $mos($i,yb)
         set mos($i,yb) [expr $dummy+\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xb) [expr $mos($i,xb)-$loc_m*($mos($i,yb)-$dummy)]

         set dummy $mos($i,yf)
         set mos($i,yf) [expr $dummy+\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xf) [expr $mos($i,xf)-$loc_m*($mos($i,yf)-$dummy)]

         set dummy $mos($i,yc)
         set mos($i,yc) [expr $dummy-\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xc) [expr $mos($i,xc)-$loc_m*($mos($i,yc)-$dummy)]

         set dummy $mos($i,yg)
         set mos($i,yg) [expr $dummy-\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xg) [expr $mos($i,xg)-$loc_m*($mos($i,yg)-$dummy)]
         }
	  # if img and highres close the outside slit blades
     foreach i { 1 2 3 4 5 15 16 17 18 19} {
       if $co_flag { #standard
         # chapter 4.3.2.1.0  standard                              
         # open slit blades according to mos(i,b) 
         set dummy $mos($i,yb)
         set mos($i,yb) [expr $dummy+\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xb) [expr $mos($i,xb)-$loc_m*($mos($i,yb)-$dummy)]

         set dummy $mos($i,yf)
         set mos($i,yf) [expr $dummy+\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xf) [expr $mos($i,xf)-$loc_m*($mos($i,yf)-$dummy)]

         set dummy $mos($i,yc)
         set mos($i,yc) [expr $dummy-\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xc) [expr $mos($i,xc)-$loc_m*($mos($i,yc)-$dummy)]

         set dummy $mos($i,yg)
         set mos($i,yg) [expr $dummy-\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xg) [expr $mos($i,xg)-$loc_m*($mos($i,yg)-$dummy)]
     } else {
         # chapter 4.3.2.2.0  highres                               
         # highres modus, close them completely  
         set mos($i,yb) $mos($i,ya)
         set mos($i,xb) $mos($i,xa)

         set mos($i,yf) $mos($i,ye)
         set mos($i,xf) $mos($i,xe)

         set mos($i,yc) $mos($i,ya)
         set mos($i,xc) $mos($i,xa)

         set mos($i,yg) $mos($i,ye)
         set mos($i,xg) $mos($i,xe)
     }
 }
}
}
}
ipol { 
     # chapter 4.4.0.0.0  modus ipol                            
     if $co_flag {
     # chapter 4.4.1.0.0  standard                              
     # standard resolution collimator mode 
     if $ne_flag {
       # chapter 4.4.1.1.0  perpendicular                         
       for { set i 2} {$i < 20} {incr i 2} {
         # all y-values are retained
         set mos($i,xb) [expr $mos($i,xb)-$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         set mos($i,xc) [expr $mos($i,xc)+$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         set mos($i,xf) [expr $mos($i,xf)-$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         set mos($i,xg) [expr $mos($i,xg)+$mos($i,b)/$focscale*\
             $fits_x_scale*$ctom/2]
         # set mos($i,xb) [expr $mos($i,xb) - $mos($i,b)/2/$focscale*$ctom]
         # set mos($i,xc) [expr $mos($i,xc) + $mos($i,b)/2/$focscale*$ctom]
         # set mos($i,xf) [expr $mos($i,xf) - $mos($i,b)/2/$focscale*$ctom]
         # set mos($i,xg) [expr $mos($i,xg) + $mos($i,b)/2/$focscale*$ctom]
         }
       } else {
       # chapter 4.4.1.2.0  slope                                 
       # not perpendicular  
     for { set i 2} {$i < 20} {incr i 2} {
       # open
         set dummy $mos($i,yb)
         set mos($i,yb) [expr $dummy+\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xb) [expr $mos($i,xb)-$loc_m*($mos($i,yb)-$dummy)]

         set dummy $mos($i,yf)
         set mos($i,yf) [expr $dummy+\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xf) [expr $mos($i,xf)-$loc_m*($mos($i,yf)-$dummy)]

         set dummy $mos($i,yc)
         set mos($i,yc) [expr $dummy-\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xc) [expr $mos($i,xc)-$loc_m*($mos($i,yc)-$dummy)]

         set dummy $mos($i,yg)
         set mos($i,yg) [expr $dummy-\
             $CRO*$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
         set mos($i,xg) [expr $mos($i,xg)-$loc_m*($mos($i,yg)-$dummy)]

       # set dummy $mos($i,yb)
       # set mos($i,yb) [expr $dummy+\
       #     $fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
       # set mos($i,xb) [expr $mos($i,xb)-$loc_m*($mos($i,yb)-$dummy)]

       # set dummy $mos($i,yf)
       # set mos($i,yf) [expr $dummy+\
       #     $fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
       # set mos($i,xf) [expr $mos($i,xf)-$loc_m*($mos($i,yf)-$dummy)]

       # set dummy $mos($i,yc)
       # set mos($i,yc) [expr $dummy-\
       #     $fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
       # set mos($i,xc) [expr $mos($i,xc)-$loc_m*($mos($i,yc)-$dummy)]

       # set dummy $mos($i,yg)
       # set mos($i,yg) [expr $dummy-\
       #     $fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1]
       # set mos($i,xg) [expr $mos($i,xg)-$loc_m*($mos($i,yg)-$dummy)]
       }
       }
       # again this is for both cases of ne_flag
       # close odd slit blades 
     for { set i 1} {$i < 20} {incr i 2} {
       # closed
       set mos($i,yb) $mos($i,ya)
       set mos($i,xb) $mos($i,xa)

       set mos($i,yf) $mos($i,ye)
       set mos($i,xf) $mos($i,xe)

       set mos($i,yc) $mos($i,ya)
       set mos($i,xc) $mos($i,xa)

       set mos($i,yg) $mos($i,ye)
       set mos($i,xg) $mos($i,xe)
       }
     } else { 
     # chapter 4.4.2.0.0  highres                               
     # high resolution collimator mode 
     # end of the joy
     # do not modify 
     # for { set i 1} {$i < 19} {incr i 1} {
     # set mos($i,yb) [expr $mos(0,y0) - [random 100] ]
     # set mos($i,xb) [expr $mos(0,x0) + [random 100] ]

     # set mos($i,yf) [expr $mos(0,y0) - [random 100] ]
     # set mos($i,xf) [expr $mos(0,x0) + [random 100] ]

     # set mos($i,yc) [expr $mos(0,y0) + [random 100] ]
     # set mos($i,xc) [expr $mos(0,x0) - [random 100] ]

     # set mos($i,yg) [expr $mos(0,y0) + [random 100] ]
     # set mos($i,xg) [expr $mos(0,x0) - [random 100] ]
     # }
     }
   }
lss { # do nothing, since lss is a fixed mask, nothing to open }
hits { # do nothing, since lss is a fixed mask, nothing to open }
mxu  { # do nothing }
default {puts $lfId "Init_Mask_World: Warning mode"  }
     }

       # only for mask==mos 
       # by the way we initialize the world coords of the slit blade
       # positions, and the select_flag, as in mos mode  
     if {[string compare $mask "mos"]==0} {
       for { set i 1} {$i < 20} {incr i 1} {
           set targw($i,f) 0;  # default: MOS slit not set, not active, 
                               #          not positioned
           set targw($i,a) \
           [expr ($mos($i,xb)+$mos($i,xc)+$mos($i,xe)+$mos($i,xf))/4.0]
           set targw($i,d) \
           [expr ($mos($i,yb)+$mos($i,yc)+$mos($i,ye)+$mos($i,yf))/4.0]
           # puts stdout "$targw($i,a) $targw($i,d)"

           # $m_image convert coords $targw($i,a) $targw($i,d) canvas \
           #                  targw($i,a)  targw($i,d) "wcs $EQX"
           # puts stdout "targw : $targw($i,a) $targw($i,d) $targw($i,f)"

           # v0.9.6 close the unused mos slit blades again
           # only in mos/pmos; not in img/pimg
           # 14/04/99 also in echs 
           if {[string match "*mos" $lmod] || 
               [string match "*echs*" $lmod]} {
           set mos($i,xb) $mos($i,xa)
           set mos($i,yb) $mos($i,ya)
           set mos($i,xc) $mos($i,xa)
           set mos($i,yc) $mos($i,ya)
           set mos($i,xg) $mos($i,xe)
           set mos($i,yg) $mos($i,ye)
           set mos($i,xf) $mos($i,xe)
           set mos($i,yf) $mos($i,ye)
           }
           }
         }


     #++++++++++++++++++++++++++++++++++++++++++++++++++++++
     # after having initialized slit-less spectroscopy 
     # switch back to original mode 
     # puts stdout "inital end [$this cget -modus] $lmod $mask"

     if {[string compare [$this cget -slit_width] open]==0} {
        if {[string compare [$this cget -modus] img]==0} {
        $this configure -modus mos
        # puts stdout "SLS img -> mos | [$this cget -modus] $mask"
        }
        }
     if {[string compare [$this cget -slit_width] open]==0} {
        if {[string compare [$this cget -modus] ipol]==0} {
        $this configure -modus pmos
        # puts stdout "SLS ipol-> pmos | [$this cget -modus] $mask"
        }
        }
     # puts stdout " after end       [$this cget -modus] $lmod $mask"
     #++++++++++++++++++++++++++++++++++++++++++++++++++++++

  #--------------------
  # FRAME
  #--------------------
  # chapter 5.0.0.0.0  frame points                          
  # first two additional help points p1 and p2 
  # again on the central north-south line
  # p1 lies between mos(0,a) and mos(0,b) at x=0 but north
  # p2 lies between mos(0,c) and mos(0,d) at x=0 but south
  # lower south point
  set p1x $wx; set p1y [expr $wy+$scal*$YSRMIN]
  #- CCD Master -#
  set ccdm1x $wx; set ccdm1y [expr $wy+$scal*$CCDMASTER_YSRMIN]
  #- CCD Slave -#
  set ccds1x $wx; set ccds1y [expr $wy+$scal*$CCDSLAVE_YSRMIN]
  # upper northern point
  set p2x $wx; set p2y [expr $wy+$scal*$YSRMAX]
  #- CCD Master -#
  set ccdm2x $wx; set ccdm2y [expr $wy+$scal*$CCDMASTER_YSRMAX]
  #- CCD Slave -#
  set ccds2x $wx; set ccds2y [expr $wy+$scal*$CCDSLAVE_YSRMAX]
  # transform these points into canvas coords
  if $bonsai_flag {
     set dum [C_wc w2c $p1x $p1y]
     set p1x [lindex $dum 0] 
     set p1y [lindex $dum 1]
     set mdum [C_wc w2c $ccdm1x $ccdm1y]
     set ccdm1x [lindex $mdum 0] 
     set ccdm1y [lindex $mdum 1]
     set sdum [C_wc w2c $ccds1x $ccds1y]
     set ccds1x [lindex $sdum 0] 
     set ccds1y [lindex $sdum 1]
     set dum [C_wc w2c $p2x $p2y]
     set p2x [lindex $dum 0] 
     set p2y [lindex $dum 1]
     set mdum [C_wc w2c $ccdm2x $ccdm2y]
     set ccdm2x [lindex $mdum 0] 
     set ccdm2y [lindex $mdum 1]
     set sdum [C_wc w2c $ccds2x $ccds2y]
     set ccds2x [lindex $sdum 0] 
     set ccds2y [lindex $sdum 1]
   } else {
     $m_image convert coords $p1x $p1y "deg $EQX" p1x p1y canvas
     $m_image convert coords $p2x $p2y "deg $EQX" p2x p2y canvas
     $m_image convert coords $ccdm1x $ccdm1y "deg $EQX" ccdm1x ccdm1y canvas
     $m_image convert coords $ccdm2x $ccdm2y "deg $EQX" ccdm2x ccdm2y canvas
     $m_image convert coords $ccds1x $ccds1y "deg $EQX" ccds1x ccds1y canvas
     $m_image convert coords $ccds2x $ccds2y "deg $EQX" ccds2x ccds2y canvas
   }
  
     # chapter 5.1.0.0.0  standard                              
  if $ne_flag {
     # chapter 5.1.1.0.0  perpendicular                         
     # --------------------------
     # perpendicular to the frame  (fits header keyword CROTA2=0)
     # standard resolution 
     # lower=south part
     # chapter 5.1.1.1.0  south part
     set mos(0,ya) $p1y
     set mos(0,yb) $p1y
     set mos(0,yar) $p1y
     set mos(0,ybr) $p1y
     set mos(0,xa) [expr $p1x+$fits_x_scale*$ctom*$XSRMIN]
     set mos(0,xb) [expr $p1x+$fits_x_scale*$ctom*$XSRMAX]
     set ccd_frame(0,ya) $ccdm1y
     set ccd_frame(1,ya) $ccds1y
     set ccd_frame(0,yb) $ccdm1y
     set ccd_frame(1,yb) $ccds1y     
     set ccd_frame(0,yar) $p1y
     set ccd_frame(0,ybr) $p1y
     set ccd_frame(1,yar) $p1y
     set ccd_frame(1,ybr) $p1y
     set ccd_frame(0,xa) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*$CCDMASTER_XSRMIN]
     set ccd_frame(1,xa) [expr $ccds1x+$fits_x_scale*$ccds_ctom*$CCDSLAVE_XSRMIN]
     set ccd_frame(0,xb) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*$CCDMASTER_XSRMAX]
     set ccd_frame(1,xb) [expr $ccds1x+$fits_x_scale*$ccds_ctom*$CCDSLAVE_XSRMAX]
     
     # we assume red =  sky-west = x<0
     #          blue =  sky-east = x>0
     if {[string compare $mask "lss"]==0} {
     # --- LSS ---
     # in LSS mode measured from the ll position
     # Assume dispersion  direction BLUE=East=A=x>0, RED=W=Bx<0
     set mos(0,xar) [expr $p1x+$fits_x_scale*$ctom*\
         ($longslitxmax($long_number)+$specmax)]    
     set mos(0,xbr) [expr $p1x+$fits_x_scale*$ctom*\
         ($longslitxmin($long_number)-$specmin)]  
     set ccd_frame(0,xar) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmax($long_number)+$ccdm_specmax)]     
     set ccd_frame(1,xar) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmax($long_number)+$ccdm_specmax)]    
     set ccd_frame(0,xbr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(1,xbr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     } elseif {[string compare $mask "hits"]==0} {
     # --- HITS ---
     # in HITS mode measured from the ll position
     # Assume dispersion  direction BLUE=East=A=x>0, RED=W=Bx<0
     set mos(0,xar) [expr $p1x+$fits_x_scale*$ctom*\
         ($htslitxmax($long_number)+$specmax)]    
     set mos(0,xbr) [expr $p1x+$fits_x_scale*$ctom*\
         ($htslitxmin($long_number)-$specmin)]    

         } else {
     # --- MOS --- ( MXU , ipol also ? )
     # Assume dispersion  direction blue east A, red west B 
     
    
     set mos(0,xar) [expr $p1x+$fits_x_scale*$ctom*($XSRMIN+$specmin)]
     set ccd_frame(0,xar) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($CCDMASTER_XSRMIN+$ccdm_specmin)]     
     set ccd_frame(1,xar) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($CCDSLAVE_XSRMIN+$ccdm_specmin)] 
     set mos(0,xbr) [expr $p1x+$fits_x_scale*$ctom*($XSRMAX-$specmax)]
     set ccd_frame(0,xbr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($CCDMASTER_XSRMAX-$ccdm_specmax)]
     set ccd_frame(1,xbr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($CCDSLAVE_XSRMAX-$ccdm_specmax)]
    
 
         } 
     # now the same with the upper=northern part
     # chapter 5.1.1.2.0  north part
     set mos(0,yc) $p2y
     set mos(0,yd) $p2y
     set mos(0,ycr) $p2y
     set mos(0,ydr) $p2y
     set mos(0,xc) [expr $p2x+$fits_x_scale*$ctom*$XSRMAX]
     set mos(0,xd) [expr $p2x+$fits_x_scale*$ctom*$XSRMIN]
     set ccd_frame(0,yc) $ccdm2y
     set ccd_frame(1,yc) $ccds2y
     set ccd_frame(0,yd) $ccdm2y
     set ccd_frame(1,yd) $ccds2y
     set ccd_frame(0,ycr) $p2y
     set ccd_frame(0,ydr) $p2y
     set ccd_frame(1,ycr) $p2y
     set ccd_frame(1,ydr) $p2y
     set ccd_frame(0,xc) [expr $ccdm2x+$fits_x_scale*$ccdm_ctom*$CCDMASTER_XSRMAX]
     set ccd_frame(1,xc) [expr $ccds2x+$fits_x_scale*$ccds_ctom*$CCDSLAVE_XSRMAX]
     set ccd_frame(0,xd) [expr $ccdm2x+$fits_x_scale*$ccdm_ctom*$CCDMASTER_XSRMIN]
     set ccd_frame(1,xd) [expr $ccds2x+$fits_x_scale*$ccds_ctom*$CCDSLAVE_XSRMIN]
     if {[string compare $mask "lss"]==0} {
     # in LSS etwas anders, take the mos
     set mos(0,xdr) [expr $p1x+$fits_x_scale*$ctom*\
         ($longslitxmax($long_number)+$specmax)]
     set mos(0,xcr) [expr $p1x+$fits_x_scale*$ctom*\
         ($longslitxmin($long_number)-$specmin)]     
     set ccd_frame(0,xdr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmax($long_number)+$ccdm_specmax)]     
     set ccd_frame(1,xdr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmax($long_number)+$ccdm_specmax)]    
     set ccd_frame(0,xcr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(1,xcr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     } elseif {[string compare $mask "hits"]==0} {
     # in HITS etwas anders, take the mos
     set mos(0,xdr) [expr $p1x+$fits_x_scale*$ctom*\
         ($htslitxmax($long_number)+$specmax)]
     set mos(0,xcr) [expr $p1x+$fits_x_scale*$ctom*\
         ($htslitxmin($long_number)-$specmin)]

        } else {
        # --- MOS --- (mxu, ipol, img)
     set mos(0,xdr) [expr $p2x+$fits_x_scale*$ctom*($XSRMIN+$specmin)]
     set mos(0,xcr) [expr $p2x+$fits_x_scale*$ctom*($XSRMAX-$specmax)]
     set ccd_frame(0,xdr) [expr $ccdm2x+$fits_x_scale*$ccdm_ctom*($CCDMASTER_XSRMIN+$ccdm_specmin)]
     set ccd_frame(0,xcr) [expr $ccdm2x+$fits_x_scale*$ccdm_ctom*($CCDMASTER_XSRMAX-$ccdm_specmax)]
     set ccd_frame(1,xdr) [expr $ccds2x+$fits_x_scale*$ccds_ctom*($CCDSLAVE_XSRMIN+$ccdm_specmin)]
     set ccd_frame(1,xcr) [expr $ccds2x+$fits_x_scale*$ccds_ctom*($CCDSLAVE_XSRMAX-$ccdm_specmax)]
     } 
     } else {
     # ------------------------------------
     # chapter 5.1.2.0.0  slope
     # Not perpendicular to the image frame 
     # chapter 5.1.2.1.0  south part
     # Standard resolution lower=south part
     # west part full CCD
     # set mos(0,ya) [expr $p1y+$fits_y_scale*$ctom*$XSRMIN*$loc_m1]
     # set mos(0,xa) [expr $p1x-$loc_m*($mos(0,ya)-$p1y)]
     # east part full CCD
     # set mos(0,yb) [expr $p1y+$fits_y_scale*$ctom*$XSRMAX*$loc_m1]
     # set mos(0,xb) [expr $p1x-$loc_m*($mos(0,yb)-$p1y)]

     #  11/09/98 test 
     # west part full CCD
     set mos(0,ya) [expr $p1y-$CRO*$fits_x_scale*$ctom*$XSRMIN*$loc_m1]
     set mos(0,xa) [expr $p1x-$loc_m*($mos(0,ya)-$p1y)]

     set ccd_frame(0,ya) [expr $ccdm1y-$CRO*$fits_x_scale*$ccdm_ctom*$CCDMASTER_XSRMIN*$ccdm_loc_m1]
     set ccd_frame(1,ya) [expr $ccds1y-$CRO*$fits_x_scale*$ccds_ctom*$CCDSLAVE_XSRMIN*$ccds_loc_m1]
     set ccd_frame(0,xa) [expr $ccdm1x-$ccdm_loc_m*($ccd_frame(0,ya)-$ccdm1y)]
     set ccd_frame(1,xa) [expr $ccds1x-$ccds_loc_m*($ccd_frame(1,ya)-$ccds1y)]
 
     # east part full CCD
     set mos(0,yb) [expr $p1y-$CRO*$fits_x_scale*$ctom*$XSRMAX*$loc_m1]
     set mos(0,xb) [expr $p1x-$loc_m*($mos(0,yb)-$p1y)]

     set ccd_frame(0,yb) [expr $ccdm1y-$CRO*$fits_x_scale*$ccdm_ctom*$CCDMASTER_XSRMAX*$ccdm_loc_m1]
     set ccd_frame(1,yb) [expr $ccds1y-$CRO*$fits_x_scale*$ccds_ctom*$CCDSLAVE_XSRMAX*$ccds_loc_m1]
     set ccd_frame(0,xb) [expr $ccdm1x-$ccdm_loc_m*($ccd_frame(0,yb)-$ccdm1y)]
     set ccd_frame(1,xb) [expr $ccds1x-$ccds_loc_m*($ccd_frame(1,yb)-$ccds1y)]

     # reduced CCD

     # --- LSS ---
     if {[string compare $mask "lss"]==0} {
     # west part 
     set mos(0,yar) [expr $p1y-$CRO*$fits_x_scale*$ctom*\
         ($longslitxmax($long_number)+$specmax)*$loc_m1]
     set mos(0,xar) [expr $p1x-$loc_m*($mos(0,yar)-$p1y)]
     #east part
     set mos(0,ybr) [expr $p1y-$CRO*$fits_x_scale*$ctom*\
         ($longslitxmin($long_number)-$specmin)*$loc_m1]
     set mos(0,xbr) [expr $p1x-$loc_m*($mos(0,ybr)-$p1y)]
     set ccd_frame(0,yar) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmax($long_number)+$ccdm_specmax)]     
     set ccd_frame(1,yar) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmax($long_number)+$ccdm_specmax)]  
     set ccd_frame(0,xar) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmax($long_number)+$ccdm_specmax)]  
     set ccd_frame(1,xar) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmax($long_number)+$ccdm_specmax)] 
     set ccd_frame(0,ybr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(1,ybr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(0,xbr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(1,xbr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmin($long_number)-$ccdm_specmin)]

     # --- HITS --
     } elseif {[string compare $mask "hits"]==0} {
     # west part
     set mos(0,yar) [expr $p1y-$CRO*$fits_x_scale*$ctom*\
         ($htslitxmax($long_number)+$specmax)*$loc_m1]
     set mos(0,xar) [expr $p1x-$loc_m*($mos(0,yar)-$p1y)]
     #east part
     set mos(0,ybr) [expr $p1y-$CRO*$fits_x_scale*$ctom*\
         ($htslitxmin($long_number)-$specmin)*$loc_m1]
     set mos(0,xbr) [expr $p1x-$loc_m*($mos(0,ybr)-$p1y)]
        } else {
     # --- MOS --- (img, ipol, mxu )
     # west part

     
     set mos(0,yar) \
         [expr $p1y-$CRO*$fits_x_scale*$ctom*($XSRMIN+$specmin)*$loc_m1]
     
     #set ccd_frame(0,yar) \
     #    [expr $ccds1y-$CRO*$fits_x_scale*$ccdm_ctom*($CCDSLAVE_XSRMIN+$ccdm_specmin)*$ccdm_loc_m1]
     set ccd_frame(0,yar) [expr $ccdm1y-$CRO*$fits_x_scale*$ccdm_ctom*($CCDMASTER_XSRMIN+$ccdm_specmin)*$ccdm_loc_m1]
     set ccd_frame(1,yar) [expr $ccds1y-$CRO*$fits_x_scale*$ccds_ctom*($CCDSLAVE_XSRMIN+$ccdm_specmin)*$ccds_loc_m1]

     set mos(0,xar) [expr $p1x-$loc_m*($mos(0,yar)-$p1y)]
     set ccd_frame(0,xar) [expr $ccdm1x-$ccdm_loc_m*($ccd_frame(0,yar)-$ccdm1y)]
     set ccd_frame(1,xar) [expr $ccds1x-$ccdm_loc_m*($ccd_frame(1,yar)-$ccds1y)]
         
     # east part
     set mos(0,ybr) \
         [expr $p1y-$CRO*$fits_x_scale*$ctom*($XSRMAX-$specmax)*$loc_m1]
          
     set ccd_frame(0,ybr) \
         [expr $ccdm1y-$CRO*$fits_x_scale*$ccdm_ctom*($CCDMASTER_XSRMAX-$ccdm_specmax)*$ccdm_loc_m1]
     set ccd_frame(1,ybr) \
         [expr $ccds1y-$CRO*$fits_x_scale*$ccds_ctom*($CCDSLAVE_XSRMAX-$ccdm_specmax)*$ccds_loc_m1]

     set mos(0,xbr) [expr $p1x-$loc_m*($mos(0,ybr)-$p1y)]
     set ccd_frame(0,xbr) [expr $ccdm1x-$ccdm_loc_m*($ccd_frame(0,ybr)-$ccdm1y)]
     set ccd_frame(1,xbr) [expr $ccds1x-$ccds_loc_m*($ccd_frame(1,ybr)-$ccds1y)]

        }

     # Standard resolution upper north part
     # chapter 5.1.2.1.0  north part
     # full CCD 
     # set mos(0,yc) [expr $p2y+$fits_y_scale*$ctom*$XSRMAX*$loc_m1]
     # set mos(0,xc) [expr $p2x-$loc_m*($mos(0,yc)-$p2y)]

     # set mos(0,yd) [expr $p2y+$fits_y_scale*$ctom*$XSRMIN*$loc_m1]
     # set mos(0,xd) [expr $p2x-$loc_m*($mos(0,yd)-$p2y)]

     # 11/09/98 test 
     set mos(0,yc) [expr $p2y-$CRO*$fits_x_scale*$ctom*$XSRMAX*$loc_m1]
     set mos(0,xc) [expr $p2x-$loc_m*($mos(0,yc)-$p2y)]

     set ccd_frame(0,yc) [expr $ccdm2y-$CRO*$fits_x_scale*$ccdm_ctom*$CCDMASTER_XSRMAX*$ccdm_loc_m1]
     set ccd_frame(1,yc) [expr $ccds2y-$CRO*$fits_x_scale*$ccds_ctom*$CCDSLAVE_XSRMAX*$ccds_loc_m1]
     set ccd_frame(0,xc) [expr $ccdm2x-$ccdm_loc_m*($ccd_frame(0,yc)-$ccdm2y)]
     set ccd_frame(1,xc) [expr $ccds2x-$ccds_loc_m*($ccd_frame(1,yc)-$ccds2y)]


     set mos(0,yd) [expr $p2y-$CRO*$fits_x_scale*$ctom*$XSRMIN*$loc_m1]
     set mos(0,xd) [expr $p2x-$loc_m*($mos(0,yd)-$p2y)]

     set ccd_frame(0,yd) [expr $ccdm2y-$CRO*$fits_x_scale*$ccdm_ctom*$CCDMASTER_XSRMIN*$ccdm_loc_m1]
     set ccd_frame(1,yd) [expr $ccds2y-$CRO*$fits_x_scale*$ccds_ctom*$CCDSLAVE_XSRMIN*$ccds_loc_m1]
     set ccd_frame(0,xd) [expr $ccdm2x-$ccdm_loc_m*($ccd_frame(0,yd)-$ccdm2y)]
     set ccd_frame(1,xd) [expr $ccds2x-$ccds_loc_m*($ccd_frame(1,yd)-$ccds2y)]

     # reduced CCD
     if {[string compare $mask "lss"]==0} {
     set mos(0,ycr) [expr $p2y-$CRO*$fits_x_scale*$ctom*\
         ($longslitxmin($long_number)-$specmin)*$loc_m1]
     set mos(0,xcr) [expr $p2x-$loc_m*($mos(0,ycr)-$p2y)]
     set mos(0,ydr) [expr $p2y-$CRO*$fits_x_scale*$ctom*\
         ($longslitxmax($long_number)+$specmax)*$loc_m1]
     set mos(0,xdr) [expr $p2x-$loc_m*($mos(0,ydr)-$p2y)]
     set ccd_frame(0,ycr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmax($long_number)+$ccdm_specmax)]     
     set ccd_frame(0,xcr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmax($long_number)+$ccdm_specmax)]    
     set ccd_frame(0,ydr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(0,xdr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     
     set ccd_frame(1,ycr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmax($long_number)+$ccdm_specmax)]     
     set ccd_frame(1,xcr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmax($long_number)+$ccdm_specmax)]    
     set ccd_frame(1,ydr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(1,xdr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     } elseif {[string compare $mask "hits"]==0} {
     set mos(0,ycr) [expr $p2y-$CRO*$fits_x_scale*$ctom*\
         ($htslitxmin($long_number)-$specmin)*$loc_m1]
     set mos(0,xcr) [expr $p2x-$loc_m*($mos(0,ycr)-$p2y)]
     set mos(0,ydr) [expr $p2y-$CRO*$fits_x_scale*$ctom*\
         ($htslitxmax($long_number)+$specmax)*$loc_m1]
     set mos(0,xdr) [expr $p2x-$loc_m*($mos(0,ydr)-$p2y)]

      } else {
     # mos, ipol, mxu 
     set mos(0,ycr) \
         [expr $p2y-$CRO*$fits_x_scale*$ctom*($XSRMAX-$specmax)*$loc_m1]
     set mos(0,xcr) [expr $p2x-$loc_m*($mos(0,ycr)-$p2y)]

     set mos(0,ydr) \
         [expr $p2y-$CRO*$fits_x_scale*$ctom*($XSRMIN+$specmin)*$loc_m1]
     set mos(0,xdr) [expr $p2x-$loc_m*($mos(0,ydr)-$p2y)]

 
    
     
     #set ccd_frame(0,ycr) [expr $py-$CRO*$fits_x_scale*$ccdm_ctom*($CCDMASTER_XSRMAX-$ccdm_specmax)*$ccdm_loc_m1]
     set ccd_frame(0,ycr) [expr $ccdm2y-$CRO*$fits_x_scale*$ccdm_ctom*($CCDMASTER_XSRMAX-$ccdm_specmax)*$ccdm_loc_m1]
     set ccd_frame(0,xcr) [expr $ccdm2x-$ccdm_loc_m*($ccd_frame(0,ycr)-$ccdm2y)]
     set ccd_frame(1,ycr) [expr $ccds2y-$CRO*$fits_x_scale*$ccds_ctom*($CCDSLAVE_XSRMAX-$ccdm_specmax)*$ccds_loc_m1]
     set ccd_frame(1,xcr) [expr $ccds2x-$ccds_loc_m*($ccd_frame(1,ycr)-$ccds2y)]
     
     #set ccd_frame(0,ydr) [expr $ccdm2y-$CRO*$fits_x_scale*$ccdm_ctom*($CCDMASTER_XSRMIN+$ccdm_specmin)*$ccdm_loc_m1]
     set ccd_frame(0,ydr) [expr $ccdm2y-$CRO*$fits_x_scale*$ccdm_ctom*($CCDMASTER_XSRMIN+$ccdm_specmin)*$ccdm_loc_m1]
     set ccd_frame(0,xdr) [expr $ccdm2x-$ccdm_loc_m*($ccd_frame(0,ydr)-$ccdm2y)]
     set ccd_frame(1,ydr) [expr $ccds2y-$CRO*$fits_x_scale*$ccds_ctom*($CCDSLAVE_XSRMIN+$ccdm_specmin)*$ccds_loc_m1]
     set ccd_frame(1,xdr) [expr $ccds2x-$ccds_loc_m*($ccd_frame(1,ydr)-$ccds2y)]
       }
     }

     # chapter 5.2.0.0.0  highres                               
     # ---------------
     # High resolution 
     # ---------------

     set specmin [expr $specmin*$CCDscale_highres/$CCDscale_standard]
     set specmax [expr $specmax*$CCDscale_highres/$CCDscale_standard]

     set ccdm_specmin [expr $ccdm_specmin*$CCDscale_highres/$CCDscale_standard]
     set ccdm_specmax [expr $ccdm_specmax*$CCDscale_highres/$CCDscale_standard]

     # lower southern point
     set p1x $wx; set p1y [expr $wy+$scal*$YHRMIN]
     set ccdm1x $wx; set ccdm1y [expr $wy+$scal*$CCDMASTER_YHRMIN]
     set ccds1x $wx; set ccds1y [expr $wy+$scal*$CCDSLAVE_YHRMIN]
     # upper northern point
     set p2x $wx; set p2y [expr $wy+$scal*$YHRMAX]
     set ccdm2x $wx; set ccdm2y [expr $wy+$scal*$CCDMASTER_YHRMAX]
     set ccds2x $wx; set ccds2y [expr $wy+$scal*$CCDSLAVE_YHRMAX]
     if $bonsai_flag {
        set dum [C_wc w2c $p1x $p1y]
        set p1x [lindex $dum 0]
        set p1y [lindex $dum 1]

        set mdum [C_wc w2c $ccdm1x $ccdm1y]
        set ccdm1x [lindex $mdum 0] 
        set ccdm1y [lindex $mdum 1]

        set sdum [C_wc w2c $ccds1x $ccds1y]
        set ccds1x [lindex $sdum 0] 
        set ccds1y [lindex $sdum 1]
        
        set dum [C_wc w2c $p2x $p2y]
        set p2x [lindex $dum 0]
        set p2y [lindex $dum 1]
 
        set mdum [C_wc w2c $ccdm2x $ccdm2y]
        set ccdm2x [lindex $mdum 0] 
        set ccdm2y [lindex $mdum 1]

        set sdum [C_wc w2c $ccds2x $ccds2y]
        set ccds2x [lindex $sdum 0] 
        set ccds2y [lindex $sdum 1]

        } else {
        $m_image convert coords $p1x $p1y "deg $EQX" p1x p1y canvas
        $m_image convert coords $p2x $p2y "deg $EQX" p2x p2y canvas
        $m_image convert coords $ccdm1x $ccdm1y "deg $EQX" ccdm1x ccdm1y canvas
        $m_image convert coords $ccdm2x $ccdm2y "deg $EQX" ccdm2x ccdm2y canvas
        $m_image convert coords $ccds1x $ccds1y "deg $EQX" ccds1x ccds1y canvas
        $m_image convert coords $ccds2x $ccds2y "deg $EQX" ccds2x ccds2y canvas
        }

  if $ne_flag {
     # chapter 5.2.1.0.0  perpendicular                         
     # perpendicular to the frame 
     # high resolution 
     # lower=south part
     # chapter 5.2.1.1.0  south part                             
     set mos(0,ye) $p1y
     set mos(0,yf) $p1y
     set mos(0,yer) $p1y
     set mos(0,yfr) $p1y
     set mos(0,xe) [expr $p1x+$fits_x_scale*$ctom*$XHRMIN]
     set mos(0,xf) [expr $p1x+$fits_x_scale*$ctom*$XHRMAX]
     set ccd_frame(0,ye) $ccdm1y
     set ccd_frame(1,ye) $ccds1y
     set ccd_frame(0,yf) $ccdm1y
     set ccd_frame(1,yf) $ccds1y 
     set ccd_frame(0,yer) $ccdm1y
     set ccd_frame(1,yer) $ccds1y
     set ccd_frame(0,yfr) $ccdm1y
     set ccd_frame(1,yfr) $ccds1y
     set ccd_frame(0,xe) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*$CCDMASTER_XHRMIN]
     set ccd_frame(1,xe) [expr $ccds1x+$fits_x_scale*$ccds_ctom*$CCDSLAVE_XHRMIN]
     set ccd_frame(0,xf) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*$CCDMASTER_XHRMAX]
     set ccd_frame(1,xf) [expr $ccds1x+$fits_x_scale*$ccds_ctom*$CCDSLAVE_XHRMAX]
     if {[string compare $mask "lss"]==0} {
     # --- LSS ---
     set mos(0,xer) [expr $p1x+$fits_x_scale*$ctom*\
         ($longslitxmax($long_number)+$specmax)]
     set mos(0,xfr) [expr $p1x+$fits_x_scale*$ctom*\
         ($longslitxmin($long_number)-$specmin)]
     set ccd_frame(0,xfr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*\
         ($longslitxmin($long_number)-$specmin)]
     set ccd_frame(0,yer) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmax($long_number)+$ccdm_specmax)]     
     set ccd_frame(0,xer) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmax($long_number)+$ccdm_specmax)]    
     set ccd_frame(0,yfr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(0,xfr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(1,yer) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmax($long_number)+$ccdm_specmax)]     
     set ccd_frame(1,xer) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmax($long_number)+$ccdm_specmax)]    
     set ccd_frame(1,yfr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(1,xfr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     } elseif {[string compare $mask "hits"]==0} {

     # --- HITS ---
     set mos(0,xer) [expr $p1x+$fits_x_scale*$ctom*\
         ($htslitxmax($long_number)+$specmax)]
     set mos(0,xfr) [expr $p1x+$fits_x_scale*$ctom*\
         ($htslitxmin($long_number)-$specmin)]
     set ccdm_frame(0,xfr) [expr $p1x+$fits_x_scale*$ccdm_ctom*\
         ($htslitxmin($long_number)-$specmin)]
       } else {

     # --- MOS --- (mxu, ipol)
     set mos(0,xer) [expr $p1x+$fits_x_scale*$ctom*($XHRMIN+$specmin)]
     set ccd_frame(0,xer) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($CCDMASTER_XHRMIN+$ccdm_specmin)] 
     set ccd_frame(1,xer) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($CCDSLAVE_XHRMIN+$ccdm_specmin)]
     set mos(0,xfr) [expr $p1x+$fits_x_scale*$ctom*($XHRMAX-$specmax)]
     set ccd_frame(0,xfr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($CCDMASTER_XHRMAX-$ccdm_specmax)]
     set ccd_frame(1,xfr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($CCDSLAVE_XHRMAX-$ccdm_specmax)]
        }
     # now the same with the upper=northern part
     # chapter 5.2.1.2.0  north part                             
     set mos(0,yg) $p2y
     set mos(0,yh) $p2y
     set mos(0,ygr) $p2y
     set mos(0,yhr) $p2y
     set mos(0,xg) [expr $p2x+$fits_x_scale*$ctom*$XHRMAX]
     set mos(0,xh) [expr $p2x+$fits_x_scale*$ctom*$XHRMIN]
     
     set ccd_frame(0,yg) $ccdm2y
     set ccd_frame(1,yg) $ccds2y
     set ccd_frame(0,yh) $ccdm2y
     set ccd_frame(1,yh) $ccds2y
     set ccd_frame(0,ygr) $ccdm2y
     set ccd_frame(0,yhr) $ccdm2y
     set ccd_frame(1,ygr) $ccds2y
     set ccd_frame(1,yhr) $ccds2y
     set ccd_frame(0,xg) [expr $ccdm2x+$fits_x_scale*$ccdm_ctom*$CCDMASTER_XHRMAX]
     set ccd_frame(1,xg) [expr $ccds2x+$fits_x_scale*$ccds_ctom*$CCDSLAVE_XHRMAX]
     set ccd_frame(0,xh) [expr $ccdm2x+$fits_x_scale*$ccdm_ctom*$CCDMASTER_XHRMIN]
     set ccd_frame(1,xh) [expr $ccds2x+$fits_x_scale*$ccds_ctom*$CCDSLAVE_XHRMIN]

     
     if {[string compare $mask "lss"]==0} {
     # --- LSS ---
     set mos(0,xhr) [expr $p2x+$fits_x_scale*$ctom*\
         ($longslitxmax($long_number)+$specmax)]
     set mos(0,xgr) [expr $p2x+$fits_x_scale*$ctom*\
         ($longslitxmin($long_number)-$specmin)]
     set ccd_frame(0,xhr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmax($long_number)+$ccdm_specmax)]     
     set ccd_frame(1,xhr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmax($long_number)+$ccdm_specmax)]    
     set ccd_frame(0,xgr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(1,xgr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     } elseif {[string compare $mask "hits"]==0} {
     # --- HITS ---
     set mos(0,xhr) [expr $p2x+$fits_x_scale*$ctom*\
         ($htslitxmax($long_number)+$specmax)]
     set mos(0,xgr) [expr $p2x+$fits_x_scale*$ctom*\
         ($htslitxmin($long_number)-$specmin)]
          } else {
     # --- MOS ---
     set mos(0,xgr) [expr $p2x+$fits_x_scale*$ctom*($XHRMAX-$specmax)]
     set ccd_frame(0,xgr) [expr $ccdm2x+$fits_x_scale*$ccdm_ctom*($CCDMASTER_XHRMAX-$ccdm_specmax)]
     set ccd_frame(1,xgr) [expr $ccds2x+$fits_x_scale*$ccds_ctom*($CCDSLAVE_XHRMAX-$ccdm_specmax)]
     set mos(0,xhr) [expr $p2x+$fits_x_scale*$ctom*($XHRMIN+$specmin)]
     set ccd_frame(0,xhr) [expr $ccdm2x+$fits_x_scale*$ccdm_ctom*($CCDMASTER_XHRMIN+$ccdm_specmin)]
     set ccd_frame(1,xhr) [expr $ccds2x+$fits_x_scale*$ccds_ctom*($CCDSLAVE_XHRMIN+$ccdm_specmin)]
         }
     } else {
     # chapter 5.2.2.0.0  slope               
     # not perpendicular 
     set mos(0,ye) [expr $p1y-$CRO*$fits_x_scale*$ctom*$XHRMIN*$loc_m1]
     set mos(0,xe) [expr $p1x-$loc_m*($mos(0,ye)-$p1y)]

     set ccd_frame(0,ye) [expr $ccdm1y-$CRO*$fits_x_scale*$ccdm_ctom*$CCDMASTER_XHRMIN*$ccdm_loc_m1]
     set ccd_frame(1,ye) [expr $ccds1y-$CRO*$fits_x_scale*$ccds_ctom*$CCDSLAVE_XHRMIN*$ccds_loc_m1]
     set ccd_frame(0,xe) [expr $ccdm1x-$ccdm_loc_m*($ccd_frame(0,ye)-$ccdm1y)]
     set ccd_frame(1,xe) [expr $ccds1x-$ccds_loc_m*($ccd_frame(1,ye)-$ccds1y)]

     set mos(0,yf) [expr $p1y-$CRO*$fits_x_scale*$ctom*$XHRMAX*$loc_m1]
     set mos(0,xf) [expr $p1x-$loc_m*($mos(0,yf)-$p1y)]

     set ccd_frame(0,yf) [expr $ccdm1y-$CRO*$fits_x_scale*$ccdm_ctom*$CCDMASTER_XHRMAX*$ccdm_loc_m1]
     set ccd_frame(1,yf) [expr $ccds1y-$CRO*$fits_x_scale*$ccds_ctom*$CCDSLAVE_XHRMAX*$ccds_loc_m1]
     set ccd_frame(0,xf) [expr $ccdm1x-$ccdm_loc_m*($ccd_frame(0,yf)-$ccdm1y)]
     set ccd_frame(1,xf) [expr $ccds1x-$ccds_loc_m*($ccd_frame(1,yf)-$ccds1y)]

     # chapter 5.2.2.1.0  north part          

     # --- LSS ---
     if {[string compare $mask "lss"]==0} {
     set mos(0,yfr) [expr $p1y-$CRO*$fits_x_scale*$ctom*\
         ($longslitxmin($long_number)-$specmin)*$loc_m1]
     set ccd_frame(0,yfr) [expr $ccdm1y-$CRO*$fits_x_scale*$ccdm_ctom*\
         ($longslitxmin($long_number)-$specmin)*$loc_m1]
     set ccd_frame(1,yfr) [expr $ccdm1y-$CRO*$fits_x_scale*$ccdm_ctom*\
         ($longslitxmin($long_number)-$specmin)*$loc_m1]
     set mos(0,xfr) [expr $p1x-$loc_m*($mos(0,yfr)-$p1y)]
     set ccd_frame(0,xfr) [expr $ccdm1x-$loc_m*($ccd_frame(0,yfr)-$ccdm1y)]
     set ccd_frame(1,xfr) [expr $ccdm1x-$loc_m*($ccd_frame(0,yfr)-$ccdm1y)]
     set mos(0,yer) [expr $p1y-$CRO*$fits_x_scale*$ctom*\
         ($longslitxmax($long_number)+$specmax)*$loc_m1]
     set mos(0,xer) [expr $p1x-$loc_m*($mos(0,yer)-$p1y)]
     set ccd_frame(0,yer) [expr $ccdm1x-$loc_m*($mos(0,yfr)-$ccdm1y)]
     set ccd_frame(0,xer) [expr $ccdm1x-$loc_m*($mos(0,yer)-$ccdm1y)]
     set ccd_frame(1,yer) [expr $ccdm1x-$loc_m*($mos(0,yfr)-$ccdm1y)]
     set ccd_frame(1,xer) [expr $ccdm1x-$loc_m*($mos(0,yer)-$ccdm1y)]
     
     # --- HITS---
     } elseif {[string compare $mask "hits"]==0} {
     set mos(0,yfr) [expr $p1y-$CRO*$fits_y_scale*$ctom*\
         ($htslitxmin($long_number)-$specmin)*$loc_m1]
     set ccd_frame(0,yfr) [expr $ccdm1y-$CRO*$fits_y_scale*$ccdm_ctom*\
         ($htslitxmin($long_number)-$specmin)*$loc_m1]
     set mos(0,xfr) [expr $p1x-$loc_m*($mos(0,yfr)-$p1y)]
     set ccd_frame(0,xfr) [expr $ccdm1x-$loc_m*($ccd_frame(0,yfr)-$ccdm1y)]
     set mos(0,yer) [expr $p1y-$CRO*$fits_y_scale*$ctom*\
         ($htslitxmax($long_number)+$specmax)*$loc_m1]
     set mos(0,xer) [expr $p1x-$loc_m*($mos(0,yer)-$p1y)]
     
        } else {
     # --- MOS ---
     set mos(0,yer) [expr $p1y-$CRO*$fits_x_scale*$ctom*\
         ($XHRMIN+$specmin)*$loc_m1]
     set ccd_frame(0,yer) [expr $ccdm1y-$CRO*$fits_x_scale*$ccdm_ctom*\
         ($CCDMASTER_XHRMIN+$ccdm_specmin)*$ccdm_loc_m1]
     set ccd_frame(1,yer) [expr $ccds1y-$CRO*$fits_x_scale*$ccds_ctom*\
         ($CCDSLAVE_XHRMIN+$ccdm_specmin)*$ccds_loc_m1]

     set mos(0,xer) [expr $p1x-$loc_m*($mos(0,yer)-$p1y)]
     set ccd_frame(0,xer) [expr $ccdm1x-$ccdm_loc_m*($ccd_frame(0,yer)-$ccdm1y)]
     set ccd_frame(1,xer) [expr $ccds1x-$ccds_loc_m*($ccd_frame(1,yer)-$ccds1y)]

     set mos(0,yfr) [expr $p1y-$CRO*$fits_x_scale*$ctom*\
         ($XHRMAX-$specmax)*$loc_m1]

     set ccd_frame(0,yfr) [expr $ccdm1y-$CRO*$fits_x_scale*$ccdm_ctom*\
         ($CCDMASTER_XHRMAX-$ccdm_specmax)*$ccdm_loc_m1]
     set ccd_frame(1,yfr) [expr $ccds1y-$CRO*$fits_x_scale*$ccds_ctom*\
         ($CCDSLAVE_XHRMAX-$ccdm_specmax)*$ccds_loc_m1]  
   
     set mos(0,xfr) [expr $p1x-$loc_m*($mos(0,yfr)-$p1y)]
     set ccd_frame(0,xfr) [expr $ccdm1x-$ccdm_loc_m*($ccd_frame(0,yfr)-$ccdm1y)]
     set ccd_frame(1,xfr) [expr $ccds1x-$ccds_loc_m*($ccd_frame(1,yfr)-$ccds1y)]
       }
     # chapter 5.2.2.2.0  south part          
     # High resolution lower part
     set mos(0,yg) [expr $p2y-$CRO*$fits_x_scale*$ctom*$XHRMAX*$loc_m1]
     set mos(0,xg) [expr $p2x-$loc_m*($mos(0,yg)-$p2y)]

     set ccd_frame(0,yg) [expr $ccdm2y-$CRO*$fits_x_scale*$ccdm_ctom*$CCDMASTER_XHRMAX*$ccdm_loc_m1]
     set ccd_frame(1,yg) [expr $ccds2y-$CRO*$fits_x_scale*$ccds_ctom*$CCDSLAVE_XHRMAX*$ccds_loc_m1]
     set ccd_frame(0,xg) [expr $ccdm2x-$ccdm_loc_m*($ccd_frame(0,yg)-$ccdm2y)]
     set ccd_frame(1,xg) [expr $ccds2x-$ccds_loc_m*($ccd_frame(1,yg)-$ccds2y)]

     set mos(0,yh) [expr $p2y-$CRO*$fits_x_scale*$ctom*$XHRMIN*$loc_m1]
     set mos(0,xh) [expr $p2x-$loc_m*($mos(0,yh)-$p2y)]

     set ccd_frame(0,yh) [expr $ccdm2y-$CRO*$fits_x_scale*$ccdm_ctom*$CCDMASTER_XHRMIN*$ccdm_loc_m1]
     set ccd_frame(1,yh) [expr $ccds2y-$CRO*$fits_x_scale*$ccds_ctom*$CCDSLAVE_XHRMIN*$ccds_loc_m1]
     set ccd_frame(0,xh) [expr $ccdm2x-$ccdm_loc_m*($ccd_frame(0,yh)-$ccdm2y)]
     set ccd_frame(1,xh) [expr $ccds2x-$ccds_loc_m*($ccd_frame(1,yh)-$ccds2y)]

     if {[string compare $mask "lss"]==0} {
     # --- LSS ---
     set mos(0,yhr) [expr $p2y-$CRO*$fits_x_scale*$ctom*\
         ($longslitxmax($long_number)+$specmax)*$loc_m1]
     set mos(0,xhr) [expr $p2x-$loc_m*($mos(0,yhr)-$p2y)]
     set mos(0,ygr) [expr $p2y-$CRO*$fits_x_scale*$ctom*\
         ($longslitxmin($long_number)-$specmin)*$loc_m1]
     set mos(0,xgr) [expr $p2x-$loc_m*($mos(0,ygr)-$p2y)]
     set ccd_frame(0,yhr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmax($long_number)+$ccdm_specmax)]     
     set ccd_frame(0,xhr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmax($long_number)+$ccdm_specmax)]    
     set ccd_frame(0,ygr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(0,xgr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(1,yhr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmax($long_number)+$ccdm_specmax)]     
     set ccd_frame(1,xhr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmax($long_number)+$ccdm_specmax)]    
     set ccd_frame(1,ygr) [expr $ccdm1x+$fits_x_scale*$ccdm_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     set ccd_frame(1,xgr) [expr $ccds1x+$fits_x_scale*$ccds_ctom*($longslitxmin($long_number)-$ccdm_specmin)]
     } elseif {[string compare $mask "hits"]==0} {
     # --- HITS ---
     set mos(0,yhr) [expr $p2y-$CRO*$fits_y_scale*$ctom*\
         ($htslitxmax($long_number)+$specmax)*$loc_m1]
     set mos(0,xhr) [expr $p2x-$loc_m*($mos(0,yhr)-$p2y)]
     set mos(0,ygr) [expr $p2y-$CRO*$fits_y_scale*$ctom*\
         ($htslitxmin($long_number)-$specmin)*$loc_m1]
     set mos(0,xgr) [expr $p2x-$loc_m*($mos(0,ygr)-$p2y)]

        } else {
     # --- MOS ---- 
     set mos(0,ygr) [expr $p2y-$CRO*$fits_x_scale*$ctom*\
         ($XHRMAX-$specmax)*$loc_m1]
     set ccd_frame(0,ygr) [expr $ccdm2y-$CRO*$fits_x_scale*$ccdm_ctom*\
         ($CCDMASTER_XHRMAX-$ccdm_specmax)*$ccdm_loc_m1]
     set ccd_frame(1,ygr) [expr $ccds2y-$CRO*$fits_x_scale*$ccds_ctom*\
         ($CCDSLAVE_XHRMAX-$ccdm_specmax)*$ccds_loc_m1]
     set mos(0,xgr) [expr $p2x-$loc_m*($mos(0,ygr)-$p2y)]
     set ccd_frame(0,xgr) [expr $ccdm2x-$ccdm_loc_m*($ccd_frame(0,ygr)-$ccdm2y)]
     set ccd_frame(1,xgr) [expr $ccds2x-$ccds_loc_m*($ccd_frame(1,ygr)-$ccds2y)]

     set mos(0,yhr) [expr $p2y-$CRO*$fits_x_scale*$ctom*\
         ($XHRMIN+$specmin)*$loc_m1]
     set ccd_frame(0,yhr) [expr $ccdm2y-$CRO*$fits_x_scale*$ccdm_ctom*\
         ($CCDMASTER_XHRMIN+$specmin)*$ccdm_loc_m1]
     set ccd_frame(1,yhr) [expr $ccds2y-$CRO*$fits_x_scale*$ccds_ctom*\
         ($CCDSLAVE_XHRMIN+$ccdm_specmin)*$ccds_loc_m1]
     set mos(0,xhr) [expr $p2x-$loc_m*($mos(0,yhr)-$p2y)]
     set ccd_frame(0,xhr) [expr $ccdm2x-$ccdm_loc_m*($ccd_frame(0,yhr)-$ccdm2y)]
     set ccd_frame(1,xhr) [expr $ccds2x-$ccds_loc_m*($ccd_frame(1,yhr)-$ccds2y)]
      }
     }

     # and back again to default values 
     set specmin [expr $specmin/$CCDscale_highres*$CCDscale_standard]
     set specmax [expr $specmax/$CCDscale_highres*$CCDscale_standard]
     set ccdm_specmin [expr $ccdm_specmin/$CCDscale_highres*$CCDscale_standard]
     set ccdm_specmax [expr $ccdm_specmax/$CCDscale_highres*$CCDscale_standard]

    # puts stdout "SR $mos(0,xa) $mos(0,ya) $mos(0,xb) $mos(0,yb) \
    #                 $mos(0,xc) $mos(0,yc) $mos(0,xd) $mos(0,yd)"

     # finally the central green circle
     # mos(0,x/y) in canvas  coords, 
     # ccx ccy actual canvas coords
     # $m_image convert coords $wx $wy "deg $EQX" \
     #    ccx  ccy canvas 
     # set mos(0,x0) $ccx; set mos(0,y0) $ccy
     set oldpos(x) $ccx; set oldpos(y) $ccy
     puts $lfId "Init_Mask_World: new center ($ccx,$ccy) = ($wx,$wy)"
     # the radius of the green circle in world coords 
     set mos(0,r) [expr $scal*$XSRMAX/40]
     # puts stdout "mos(0,r)=$mos(0,r)"
     if $bonsai_flag {
       # radius = 0.4 of a MOS stripe 
       # 1 MOS stripe ~= 1/(18*sqrt(2)) of diagonal
       set dumx [expr ($mos(0,xa)-$mos(0,xc))/50.0]
       set dumy [expr ($mos(0,ya)-$mos(0,yc))/50.0]
       set mos(0,r) [expr sqrt($dumx*$dumx+$dumy*$dumy)*0.4]
        } else {
        $m_image convert dist  $mos(0,r) 0 "deg $EQX" \
               mos(0,r) dummy canvas
        if {$mos(0,r) < $dummy} { set mos(0,r) $dummy}
        }
     # puts stdout "radius = $mos(0,r)"


     #some debug tests
     $m_image convert coords $wx $wy "deg $EQX" p1 p2 canvas
     puts -nonewline $lfId "Init_Mask_World: ($wx,$wy) = canvas($p1,$p2)"
     $m_image convert coords $wx $wy "deg $EQX" p1 p2 image
     puts -nonewline $lfId "= image($p1,$p2)"
     $m_image convert coords $wx $wy "deg $EQX" p1 p2 screen
     puts -nonewline $lfId " = screen($p1,$p2)"
     $m_image convert coords $wx $wy "deg $EQX" p1 p2 "wcs $EQX"
     puts $lfId " = wcs($p1,$p2)"

     # the angle between mask and pixel/canvas/screen system
     set abs_ang_rad 0.0
     # the angle between mask and WCS, depends on the orientation
     # of the WCS,  see CSO
     set PAM 0.0

     # 23/SEP/98 : a very special event
     # FORS1 is mounted on UT1/VLT in the correct way 
     # but the sky field orientation in the FORS1 focal field is 
     # different to the Basic Telescope Definiton 
     # in contradiction to the FORS-Reference-System manual
     # solution: the zero point of the rotator corresponds to 
     # 180 degrees position angle 

     if {$mountangle == 180} {
     set dummy [$this cget -rotangle]
     $this configure -rotangle $mountangle
     $this rotate $m_canvas $mos(0,x0) $mos(0,y0) 1
     $this configure -rotangle $dummy
     set abs_ang_rad 0.0
     set PAM 0.0
     set tcs_pam 0.0
     # puts stdout "instrument mounting like FORS1 COMM123/SV"
     } else {
     # puts stdout ">>>>>> WARNING : instrument mounting not like FORS1"
     }

     # -----------------------------------------------
     # the mc coefficients have to be calculated 
     # -----------------------------------------------
     # independent on the bonsai flag 
     if {[string compare $lmod mxu]==0} {
     fors::MXU_SLIT::coeff $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                           $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                           $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc) 

         # configure grism dispersion for interference filters
         # puts stdout "MXU with interference filters"
         # puts stdout "-----------------------------"
         # puts stdout "lamcen(grism)=$A_disp($good,cen) nm "

         # grism dispersion from [nm/mm(CCD)] to [mm(ff)/nm]  
         # last check 15/09/99 hummel
         set ldisp [expr $CCDscale_standard/$focscale/$A_disp($good,disp)/$PIX]
         # puts stdout "disp(grism)=$A_disp($good,disp) = $ldisp nm/mm"

         set pos [lsearch -exact $filt_range [$this cget -filter] ]
         # puts stdout ">[$this cget -filter]< appears at position \
         # >$pos< in \
         # >>$filt_range<<"
         # puts stdout "fwhm($pos) = [lindex $fwhm_range $pos] nm "
         # puts stdout "lamc($pos) = [lindex $lamcent_range $pos] nm "

         # when order separation filters are used instead of interference
         # filters, mxu_class will indicate the dispersion stripe until
         # the rim of the CCD otherwise the FWHM of the interference filter 
         # is used to indicate the predicted length of the spectrum. 
         # but there is one exception: the low dispersion grisms 
         # GRIS_150I+27 and GRIS_200I+28. 
         # therefore for order seperation filters we will calculate the 
         # broad band FWHM and the wavelength of the OS filter center 
         # from the disperion table 
         # Hummel: 08.08.2000 No, broad band filter extra branch 
         # no longer supported
         # puts stdout "--------------------------$ldisp"
       # if {$ldisp<0.3} {
       #    # puts stdout "low dispersion grism >>>"
       #    # this must one of the low disperion grisms
       #    # 150I or 200I 

       #    # puts stdout "> [$this cget -filter]< >$A_disp($good,filter)<"
       #    if {[string compare [$this cget -filter] $A_disp($good,filter)]==0} {
       #        # puts stdout "filter = order separation"
       #        # its one of the order separation filters
       #        set l_lam0 [expr ( $A_disp($good,min) + $A_disp($good,max) ) * 0.5 ]
       #        set l_fwhm [expr ( $A_disp($good,max) - $A_disp($good,min) ) * 1.0 ]
       #        } else {
       #        puts stdout "filter = interference or broad band"
       #        # its a interference filter or a broad band filter
       #        set l_lam0 [lindex $lamcent_range $pos]
       #        set l_fwhm [lindex $fwhm_range $pos]
       #        }
       #    } else {
            # puts stdout "med dispersion grism >>>"
            # this must be a medium dispersion grism 
            set l_lam0 [lindex $lamcent_range $pos]
            set l_fwhm [lindex $fwhm_range $pos]
       #    }


         # grism-disp[mm/nm]  grism-lam0[nm]  lam0-of-filt[nm]  fwhm-of-filt[nm]
         set 4val [list $ldisp [expr $A_disp($good,cen)*1.0] $l_lam0 $l_fwhm ]

         # puts stdout "mxu-disp: $4val"
         fors::MXU_SLIT::disp $4val

         }
        if {[string match "*echs*" $lmod]} {
                 fors::CONV_MC::coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                                $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                                $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
           }

     # for targ2 
     if {[string match "*hits*" $lmod]} { 
        set corr_flag [$this cget -fors_frame]
        fix_mask_slit hits $corr_flag 
        }

     # for targ2 
     if {[string match "*lss*" $lmod]} { 
        set corr_flag [$this cget -fors_frame]
        fix_mask_slit  lss $corr_flag 
        }

#     if {[string match "*mxu*" $lmod]} { 
#	 puts "Introduce HERE the loading of FAD file"
#        load_mxu_fad $set_dir/mxu2.009.standard.fad
#        }
     set mask_initialized 1

     
     
     }


#======================================
private method echs_pos { } {
#======================================
    # Position 1 or 2 ECHS slits after mask rotation 
    # and after reset_mask
    # we handle both ECHS slits, built by MOS as 
    # a fixed mask 
       set y1   0.0;  # hardcoded center of MOS 10
       set y2 -10.0;  # hardcoded center of  MOS 9
     # set y2  10;    # hardcoded center of MOS 11, no longer used
      
       # we have three hardcoded x-positions, 
       # dependent on blaze angle positions of XGRISM
       # default value is -80
       # puts stdout ">$echelle_range<"
       set x -80.0
       foreach val $echelle_range {
             # puts stdout ">[$this cget -grism]<"
             # puts stdout ">$val<"
           if {[string match [$this cget -grism] $val]} {
              # a constant mos position for all EGRIS-XGRIS-OSfilter combinations 
              set x 0.0
              } 
           }
        if {$x==-80.0} { 
           puts "Warning >>> XGRIS not correctly defined"
           foreach val $echelle_range {
                   # puts -nonewline stdout ">$val<" 
                   }
                   # puts stdout ""
                   for {set i 1} {$i<=4} {incr i 1} {
                       # puts -nonewline stdout ">[$this cget -grism]<"
                       }
                   # puts stdout ""
           } else {

           # XGRIS is correctly defined 

       # 2 transformations mm->canvas->screen
       # MOS blade pair 10 
       set a [fors::CONV_MC::m2c $x $y1]
       $m_image convert coords [lindex $a 0] [lindex $a 1] canvas \
                               sx sy screen
       # some diagnostics
       $m_image convert coords [lindex $a 0] [lindex $a 1] canvas \
                               loca locd "wcs $EQX"
       puts stdout "echs_pos10: desired position = $loca $locd"

       set tx [lindex $a 0]; set ty [lindex $a 1]

             set a [$m_canvas find overlapping \
                   [expr $tx-1] [expr $ty-1] \
                   [expr $tx+1] [expr $ty+1] ]
          #  foreach c $a {
          #        set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
          #        puts stdout $local_tags
          #        }

       $m_canvas delete ali 
       if [catch { $this move_slit $m_canvas $sx $sy 1 0 } result] {
        puts stdout "echs_pos10: result = $result, grism,x,y= [$this cget -grism] $x $y1"
        puts stdout [format ">>targw(10,x,y,cx,cy)= %8.4f %8.4f %8.4f %8.4f" \
                     $targw(10,x) $targw(10,y) $targw(10,cx) $targw(10,cy) ]
        puts stdout [format ">>targw(10,a,d,ca,cd)= %8.4f %8.4f %8.4f %8.4f" \
                    $targw(10,a) $targw(10,d) $targw(10,ca) $targw(10,cd)]
        } else {
        puts stdout "MOS \#10 positioned"
        }

       if {$echs_flag} { 

       # MOS blade pair 9 is also active  
       set a [fors::CONV_MC::m2c $x $y2]

       $m_image convert coords [lindex $a 0] [lindex $a 1] canvas \
                               sx sy screen

       # some diagnostics
       $m_image convert coords [lindex $a 0] [lindex $a 1] canvas \
                               loca locd "wcs $EQX"
       puts stdout "echs_pos 9: desired position = $loca $locd"

       set tx [lindex $a 0]; set ty [lindex $a 1]

             set a [$m_canvas find overlapping \
                   [expr $tx-1] [expr $ty-1] \
                   [expr $tx+1] [expr $ty+1] ]
           # foreach c $a {
           #       set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
           #       puts stdout $local_tags
           #       }

       $m_canvas delete ali 
       if [ catch { $this move_slit $m_canvas $sx $sy 1 0 } result] {
        puts stdout "echs_pos 9: result = $result, grism,x,y= [$this cget -grism] $x $y2"
        puts stdout [format ">>targw(9,x,y,cx,cy)= %8.4f %8.4f %8.4f %8.4f" \
                     $targw(9,x) $targw(9,y) $targw(9,cx) $targw(9,cy) ]
        puts stdout [format ">>targw(9,a,d,ca,cd)= %8.4f %8.4f %8.4f %8.4f" \
                     $targw(9,a) $targw(9,d) $targw(9,ca) $targw(9,cd) ]
                } else {
        puts stdout "MOS \#9 positioned"
        }
        } else  {
        # puts stdout " MOS 9 not active "
        }

     }
     puts stdout "echs_pos end: [$this cget -grism]"


     # return = grism error 
     if {$x==-80.0} { return 1 } else {return 0}

     }

#======================================
private method echs_targ {} {
#======================================
# update one or two echs targw values, 
#                        -----
# after the mask has been shifted
    if {$echs_flag} {
       set start  9; set end 10 } else {
       set start 10; set end 10 }
        

for {set my $start} {$my <=$end} { incr my 1} {
    # canvas slit center 
    set xc [expr ($mos($my,xb)+$mos($my,xc)+$mos($my,xf)+$mos($my,xg) )/4]
    set yc [expr ($mos($my,yb)+$mos($my,yc)+$mos($my,yf)+$mos($my,yg) )/4]

    # canvas -> wcs of MOS 10 or (MOS 10 and MOS 9)
    if [catch {
    $m_image convert coords $xc $yc canvas \
            targw($my,a) targw($my,d) "wcs $EQX"} result ] {
    puts stdout "echs_targ $my: $result" }

            
    # canvas -> pixel of MOS 10 or (MOS 10 and MOS 9)
    $m_image convert coords $xc $yc canvas \
            targw($my,x) targw($my,y) image 
    }

    # now we apply the correction for optical distortion
    # of FORS images and differential refraction
    set corr_flag [$this cget -fors_frame]
    fix_mask_slit echs $corr_flag

set last_action "Move Mask"
catch {SP update}
}

#======================================
private method reset_slits {} {
#======================================
# called by button1-release, 
# after mask shift, reset slit positions 

  # update the 19 slit-positions of the mos
  if {[string compare $mask "mos"]==0} {

     # this means mos,pmos,img,pimg-SR,echs

     # v0.9.6 close the unused mos slit blades again
     # only in mos/pmos, not in img/pimg
     # exception (p)mos slit-les spectroscopy

     if {[string match "*mos" [$this cget -modus] ]} {
     if {[string compare open [$this cget -slit_width] ]} {
        # its mos or pmos, but not slitless spectroscopy (IMS), (PIMS)
          for { set i 1} { $i < 20 } { incr i 1} {
              # close all 19 mos slit blades 
              set mos($i,xb) $mos($i,xa)
              set mos($i,yb) $mos($i,ya)
              set mos($i,xc) $mos($i,xa); # + some value to open to 6 mm
              set mos($i,yc) $mos($i,ya)
              set mos($i,xf) $mos($i,xe)
              set mos($i,yf) $mos($i,ye)
              set mos($i,xg) $mos($i,xe); # + some value to open to 6 mm
              set mos($i,yg) $mos($i,ye)
              set targw($i,f) 0;          # deselect mos slit 
              }
            }
          }

           # take the mean 

          for { set i 1} { $i < 20 } { incr i 1} {
              # update the slit postion values 
              # obsolete in mos, pmos, since f=0
              set targw($i,a) [expr ($mos($i,xc)+$mos($i,xb)+\
                              $mos($i,xf)+$mos($i,xg))/4.0] 
              set targw($i,d) [expr ($mos($i,yc)+$mos($i,yb)+\
                              $mos($i,yf)+$mos($i,yg))/4.0] 
           }
     }

      # MXU specials delete all instances of MXU 
       if {[string compare [$this cget -modus] mxu ]==0} {
           if {[fors::MXU_SLIT::get_mmff]} {
              # do nothing, retain the standard mask as a fixed mask
              } else {
              set noms [fors::MXU_SLIT::nos]
              if {$noms>0} {		 
                 for {set i 1} {$i <= $noms} { incr i 1} { 
		     if {$i > $nf_refslit} {
			 delete object C_mxus$i 
		     }
                     }
                  }
              }
           }

     # now we apply the correction for optical distortion
     # of FORS images and differential refraction
     if {[string compare [$this cget -modus] hits]==0} {
        set corr_flag [$this cget -fors_frame]
        fix_mask_slit hits $corr_flag
        }



 update_and_replot

  set last_action {Move Mask}
  catch {SP update}
  }
 


     #==============================
     private method mark { w xs ys } {
     #==============================
     # mark mask pointing 
     $m_image convert coords $xs $ys screen oldpos(x) oldpos(y) canvas
     }

     #=======================
     public  method update_mask {} {
     #=======================
     # What happens if you zoom the image or 
     # shift screen versus canvas ?
     # 1. rescale the mask according to the new image zoom
     #    update the matrix mos(..,..)
     # 2. world coordinates retain

     # check on new scale, only used for trace 
     set newscale [lindex [$m_image scale] 0]
     if {$newscale<0} {
        set newscale [expr -1.0/$newscale]
        }
     #puts $lfId "NewScale=$newscale OldScale=$canvscale"
     set stretch [expr (1.0*$newscale)/(1.0*$canvscale)]
     # puts stdout "update_mask: newscale=$newscale stretch=$stretch"

    
     #check on shift between screen and canvas
     #puts $lfId "$ccx $ccy"
     # $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
     #       nwx nwy "deg $EQX"
     # puts $lfId "OLD: $canvscale $mos(0,x0) $mos(0,y0) $nwx $nwy \
     #                   $world_x $world_y"
     # do not update mos(0,x0), mos(0,y0) alone !!!
     # $m_image convert coords $world_x $world_y "deg $EQX" \
     #          ccx ccy canvas
     set canvscale $newscale


     # 1. determine new canvas postion using retained world coords
     # do this in all cases 
        set dum [C_wc w2c $world_x $world_y]
        set x [lindex $dum 0]
        set y [lindex $dum 1]
     # puts stdout " bonsai x y $x $y"
     # puts stdout [$m_image convert coords $world_x $world_y "deg $EQX" \
     #              x y canvas ]
     # puts stdout " convertx y $x $y"
     # the distance between old and new position in canvas coordinates
     # this is not the mark position but the mask center
     # puts stdout [format "->   %6.1f == %6.1f %6.1f == %6.1f" \
     #             $x $ccx $y $ccy ] 
     set shift_x [expr $x - $ccx]
     set shift_y [expr $y - $ccy]


     # set new center coordinates
     set ccx $x; set ccy $y
     # puts stdout [format "->   shift = %6.1f %6.1f" $shift_x $shift_y ] 
     # puts stdout [format "->   %6.1f == %6.1f %6.1f == %6.1f" \
     #             $x $ccx $y $ccy ] 

     puts $lfId "update_mask: scale=$newscale stretch = $stretch \
           world=$world_x,$world_y"

     # shift the mos to the new canvas but old world position
     # 19 slit blades 
     switch -exact -- $mask  {
     mos {
         for { set i 1 } { $i < 20 } { incr i 1} {
            foreach val { a b c d e f g h } {
              set mos($i,x$val) [expr $mos($i,x$val)+$shift_x]
              set mos($i,y$val) [expr $mos($i,y$val)+$shift_y]
             }
	  }          
          }
     hits -
     lss {
         for { set i 1 } { $i < 10 } { incr i 1} {
            foreach val { a b d e f h } {
              set mos($i,x$val) [expr $mos($i,x$val)+$shift_x]
              set mos($i,y$val) [expr $mos($i,y$val)+$shift_y]
             }
          }
          }
      pol {
         for { set i 1 } { $i < 10 } { incr i 1} {
            foreach val { a b d e f h } {
              set mos($i,x$val) [expr $mos($i,x$val)+$shift_x]
              set mos($i,y$val) [expr $mos($i,y$val)+$shift_y]
             }
          }
          }
      mxu {
          }
  default { puts stdout "Warning : mask update_mask" }
          } 

     # the frame and the central position
     # ----------------------------------
     foreach val { 0 ar br cr dr er fr gr hr a b c d e f g h} {
       set mos(0,x$val) [expr $mos(0,x$val)+$shift_x]
       set mos(0,y$val) [expr $mos(0,y$val)+$shift_y]
       }


     # CCD frame and the central position
     # ----------------------------------
     foreach val { 0 ar br cr dr er fr gr hr a b c d e f g h} {
       set ccd_frame(0,x$val) [expr $ccd_frame(0,x$val)+$shift_x]
       set ccd_frame(0,y$val) [expr $ccd_frame(0,y$val)+$shift_y]       
       set ccd_frame(1,x$val) [expr $ccd_frame(1,x$val)+$shift_x]
       set ccd_frame(1,y$val) [expr $ccd_frame(1,y$val)+$shift_y]
       }

    

     if {$stretch != 1} {
       # rescale the size of the mos according to the new zoom
       # 19 slit blades 


       # the frame 
       # ---------
       foreach val { 0 a b c d e f g h ar br cr dr er fr gr hr } {
        set mos(0,x$val) [expr $mos(0,x0)+($mos(0,x$val)-$mos(0,x0))*$stretch]
        set mos(0,y$val) [expr $mos(0,y0)+($mos(0,y$val)-$mos(0,y0))*$stretch]
        }
       
       #
       # CCD frame
       foreach val {0 ar br cr dr er fr gr hr a b c d e f g h} {
          set ccd_frame(0,x$val) [expr $ccd_frame(0,x0)+($ccd_frame(0,x$val)-$ccd_frame(0,x0))*$stretch]          
	  set ccd_frame(0,y$val) [expr $ccd_frame(0,y0)+($ccd_frame(0,y$val)-$ccd_frame(0,y0))*$stretch]
          set ccd_frame(1,x$val) [expr $ccd_frame(1,x0)+($ccd_frame(1,x$val)-$ccd_frame(1,x0))*$stretch]	  
          set ccd_frame(1,y$val) [expr $ccd_frame(1,y0)+($ccd_frame(1,y$val)-$ccd_frame(1,y0))*$stretch]
          
       }

      
       # the green circle (radius is a distance)
       set mos(0,r) [expr $mos(0,r)*$stretch]
       # aleady done 
       }

 switch -exact -- $mask  {
  mos {
   for { set i 1 } { $i < 20 } { incr i 1} {
    foreach val { a b c d e f g h } {
      set mos($i,x$val) [expr $mos(0,x0)+($mos($i,x$val)-$mos(0,x0))*$stretch]
      set mos($i,y$val) [expr $mos(0,y0)+($mos($i,y$val)-$mos(0,y0))*$stretch]
      }
     }
    }
  pol {
   for { set i 1 } { $i < 10 } { incr i 1} {
    foreach val { a b d e f h } {
      set mos($i,x$val) [expr $mos(0,x0)+($mos($i,x$val)-$mos(0,x0))*$stretch]
      set mos($i,y$val) [expr $mos(0,y0)+($mos($i,y$val)-$mos(0,y0))*$stretch]
      }
     }
    }
 hits -
  lss {
   for { set i 1 } { $i < 10 } { incr i 1} {
    foreach val { a b d e f h } {
      set mos($i,x$val) [expr $mos(0,x0)+($mos($i,x$val)-$mos(0,x0))*$stretch]
      set mos($i,y$val) [expr $mos(0,y0)+($mos($i,y$val)-$mos(0,y0))*$stretch]
      }
     }
    }
      mxu {
           fors::MXU_SLIT::coeff    $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                           $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                           $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc) 
          }
  default { puts stdout "Warning: mask update_mask" }
    }



        # the reference stars
        # -------------------
       for {set i 1} { $i <=10} { incr i 1 } {
           catch {
           set ref($i,x) [expr $mos(0,x0)+($ref($i,x)-$mos(0,x0))*$stretch]
           set ref($i,y) [expr $mos(0,y0)+($ref($i,y)-$mos(0,y0))*$stretch]
           }
           }

      
      # set the index according to long_width 
       switch -exact -- [$this cget -long_width] {
          all {set long_number 0 }
          0.3 {set long_number 1 }
          0.4 {set long_number 2 }
          0.5 {set long_number 3 }
          0.7 {set long_number 4 }
          1.0 {set long_number 5 }
          1.3 {set long_number 6 }
          1.6 {set long_number 7 }
          2.0 {set long_number 8 }
          2.5 {set long_number 9 }
        default {puts stdout "Warning: LSS slit" }
           }


}

       #=============================
       public method update_color {} {
       # ============================
       switch -exact -- [$this cget -colorset] {
           Red/Yellow    {set color1 red;     set color2 yellow}
           Black/White   {set color1 black;   set color2 white}
           Violet/Orange {set color1 violet;  set color2 orange}
           Green/Red     {set color1 green;   set color2 red}
           Gray          {set color1 #9f9f9f; set color2 #6f6f6f}
           Dark          {set color1 #000000; set color2 #2f2f2f}
           Bright        {set color1 white;   set color2 yellow}
           default       {puts $lfId "UpdateColor: Warning ColorSet"}
        }
        set color3 $color1; set color4 $color2
          # tranparency 
          switch -glob -- [$this cget -graytype] {
             0    { set color1 ""; set color2 ""; # look through 
                    set trapa gray25;             # for green circle
                  }
            ??    { set trapa gray[$this cget -graytype]}
           100    { set trapa "";   # background obscured  }
	     1    { set trapa gray25; }
        default   {puts $lfId "UpdateColor: Warning GrayType"}
           }
     } 


     #=======================
     public  method plot_mask {} {
     #=======================
     # draws the whole mask once more with the new settings
     # draw the odd yellow slit blades 
     # tags are mm, m1, m3, .. m19
     # the mos(,) array contains coordinates in canvas coords, 
     # ready for direct printing
     # no coords transformations have to be done 
     #
     # tag A  = all elements
     # tag C  = collimator/CCD-window
     # tag P2 = white circle, rot+shift, dither 1/2 in IPOL
     # tag P3 = grey  circle, rot+shift, dither 1/3 in IPOL
     # tag m  = green circle, rotation and shift point
     # tag sw = slit width label
     # tag out = slit number rim label 
     # tag D  = display, dont move 

     # for imaging, each slilet needs a mosa and a mosb tag
     # tag mm   = all mos slit blades (ma$i, mb$i, ms$i, m$i)
     # tag m$i  = one of the 19 slit blade pairs (ma$i and mb$i sw out)
     # tag ma$i = one of the 19 sky-west mosA slit blade only 
     # tag mb$i = one of the 19 sky-east mosB slit blade only 
     # tag ms$i = one of the 19 mos slits (invisible, transparent)

    
     set debug 1

     if {$debug} { puts -nonewline stdout "p" }
     set lmod [$this cget -modus]   
     set lcol [$this cget -collimat]
     # puts stdout "PLOT_MASK: mod=$lmod mask=$mask sw=[$this cget -slit_width]"

     # used in ipol/img/mos(open)
     # in mos all slits are closed by default (slitrim=9)
     # in img all mos blades are out of the focal field by default (slitrim=1)
     # in ims all mos blades are out of the focal field by default (slitrim=1)
     # 
     set slitrim 0 
     if {[string compare $lmod ipol]==0 || [string compare $lmod img]==0 || 
         [string compare [$this cget -slit_width] open]==0 } { 
         set slitrim 1
         }
  
    
    # -----------------------------------
    # plot the 19 MOS slit blades or
    # plot one of the 9 lss slits or 
    # plot one of the 8 hits windows and 2 pinholes 
    # plot the 9 ipol stripes
    # -----------------------------------

    # general tasks
    # plot the focal field rim (blue frame and tag play grounds)
    # plot ccd field (green frame)
    # plot the slits, the contents of the focal field
    # plot the labels (slit labels and focf labels)


     
     if {$debug} { puts -nonewline stdout "l" }

     # --------------------------------
     # 
     # plot the focal field (rim)
     #
     # -------------------------------


     # both SR and HR blue frames 
     # both full and reduced frame (user-defined wavelength region)

     # SR always plotted, as a thin line  
     $m_canvas create line $mos(0,xa) $mos(0,ya) $mos(0,xb) $mos(0,yb) \
        $mos(0,xc) $mos(0,yc) $mos(0,xd) $mos(0,yd) $mos(0,xa) $mos(0,ya) \
        -fill blue -tag  { A C }

     if {$debug} {
       puts $lfId "FOCAL FIELD coordinates:"
       puts $lfId "create line line $mos(0,xa) $mos(0,ya) $mos(0,xb) $mos(0,yb) \
        $mos(0,xc) $mos(0,yc) $mos(0,xd) $mos(0,yd) $mos(0,xa) $mos(0,ya) -fill blue"

     }

     
     

     # draw the focfield frame in blue
     # reduced CCD frame in blue dashed 
     # tag = mm
  if {[string match *HR* $lcol ]} {
     # -------------
     # -------------
     # HR Collimator
     # -------------
     # -------------
    

     if {[string compare [$this cget -instrument] FORS2]==0} {     
     # CCD frame
     #- Master Part -#
     $m_canvas create line $ccd_frame(0,xe) $ccd_frame(0,ye) $ccd_frame(0,xf) $ccd_frame(0,yf) \
      $ccd_frame(0,xg) $ccd_frame(0,yg) $ccd_frame(0,xh) $ccd_frame(0,yh) $ccd_frame(0,xe) $ccd_frame(0,ye) \
      -fill green -width 1 -tag { A C}       
     #- Slave Part -#
      $m_canvas create line $ccd_frame(1,xe) $ccd_frame(1,ye) $ccd_frame(1,xf) $ccd_frame(1,yf) \
      $ccd_frame(1,xg) $ccd_frame(1,yg) $ccd_frame(1,xh) $ccd_frame(1,yh) $ccd_frame(1,xe) $ccd_frame(1,ye) \
      -fill green -width 1 -tag { A C}       
      
     $m_canvas create text [expr ($ccd_frame(1,xe)+$ccd_frame(1,xf))/2] \
                           [expr ($ccd_frame(1,yh)+5)] \
                           -anchor n  -fill green      \
                           -text "     CCD slave     " \
                           -tag { A C m }
     $m_canvas create text [expr ($ccd_frame(0,xg)+$ccd_frame(0,xh))/2] \
                           [expr ($ccd_frame(0,ye)-5)] \
                           -anchor s  -fill green      \
                           -text  "    CCD master    " \
                           -tag { A C m }
	       
      } else {
         # full frame 
         $m_canvas create line $mos(0,xe) $mos(0,ye) $mos(0,xf) $mos(0,yf) \
         $mos(0,xg) $mos(0,yg) $mos(0,xh) $mos(0,yh) $mos(0,xe) $mos(0,ye) \
         -fill blue -width 3 -tag { A C }
      }
     
    

    

     # for mxu we neeed a tag play ground 
     #     ---
     if {[string compare $lmod mxu]==0} {
	 
      $m_canvas create poly $mos(0,xe) $mos(0,ye) $mos(0,xf) $mos(0,yf) \
      $mos(0,xg) $mos(0,yg) $mos(0,xh) $mos(0,yh)  \
      -fill "" -smooth false -stipple "" -tag { A C mxu }
              # -fill "white" -stipple "gray12"
     }

     # for echs we neeed a tag play ground 
     #     ----
     # for slit positioning with center/gauss
     if {[string compare $lmod echs]==0} {
       $m_canvas create poly $mos(0,xe) $mos(0,ye) $mos(0,xf) $mos(0,yf) \
       $mos(0,xg) $mos(0,yg) $mos(0,xh) $mos(0,yh)  \
        -fill "" -smooth false -stipple "" -tag { A C echs_ref_targ }

        # if the center/gauss option is active 
       if {$pics} { $m_canvas addtag ali withtag echs_ref_targ 

       # $m_canvas lower echs_ref_targ m9

       # -----------------
       # cyan center cross
       # -----------------
       $m_canvas create line [expr ($mos(10,xb)+$mos(10,xf))*0.5]  \
                             [expr ($mos(10,yb)+$mos(10,yf))*0.5]  \
                             [expr ($mos(10,xc)+$mos(10,xg))*0.5]  \
                             [expr ($mos(10,yc)+$mos(10,yg))*0.5]  \
        -fill "cyan" -smooth false -tag { A C }
       $m_canvas create line [expr ($mos(10,xb)+$mos(10,xc))*0.5]  \
                             [expr ($mos(10,yb)+$mos(10,yc))*0.5]  \
                             [expr ($mos(10,xf)+$mos(10,xg))*0.5]  \
                             [expr ($mos(10,yf)+$mos(10,yg))*0.5]  \
        -fill "cyan" -smooth false -tag { A C }
       }
       }

     # for mxu standard mask mode we neeed a (invisible) tag play ground 
     # for standard mask aligment  with center/gauss
     # this is done via a special trick 
     # we add the ali tag to the mxu tagged play gorund
       if {$pics} { 
          # if the center/gauss option is active 
          if {[string compare $lmod mxu]==0} {
             # if we are in standard mask mode 
             if {[fors::MXU_SLIT::get_mmff]} {
                $m_canvas addtag ali withtag mxu 
                $m_canvas addtag PG  withtag mxu 
                $m_canvas   dtag mxu  
                } else {
                $m_canvas addtag mxu withtag ali 
                $m_canvas   dtag ali  
                $m_canvas   dtag PG   
                }
             }
        }
      # puts stdout "tag of mxu PG=[$m_canvas gettags PG]"

     # for hits we neeed a (invisible) tag play ground 
     # for pinhole alignment with center/gauss
     # HITS-highres : only big pinhole 
     # hits_ref_targ is for reference target verify
     # tag ali is for center/slit alignment 

     if {[string compare $lmod hits]==0} {
     $m_canvas create poly $mos(0,xe) $mos(0,ye) $mos(0,xf) $mos(0,yf) \
      $mos(0,xg) $mos(0,yg) $mos(0,xh) $mos(0,yh)  \
      -fill "" -stipple ""  -smooth false  -tag { A C hits_ref_targ }

       # if the center/gauss option is active 
       if {$pics} { $m_canvas addtag ali withtag hits_ref_targ }
     }

     # for lss we neeed a (invisible) tag play ground 
     # for lss alignment with center/gauss
     # if the center/gauss option is active 
     # lss_ref_trag is used by refernce target verify
     # ali us ised by center/slit alignment

     if {[string compare $lmod lss]==0} {
     $m_canvas create poly $mos(0,xe) $mos(0,ye) $mos(0,xf) $mos(0,yf) \
      $mos(0,xg) $mos(0,yg) $mos(0,xh) $mos(0,yh)  \
      -fill "" -stipple ""  -smooth false  -tag { A C lss_ref_targ }

       # if center/slit option is ON, add the 'ali' tag
       if {$pics} { $m_canvas addtag ali withtag lss_ref_targ 
       
       # ----------------------
       # cyan slit center cross
       # ----------------------
       $m_canvas create line \
          [expr ($mos($long_number,xa)+$mos($long_number,xd))*0.5]  \
          [expr ($mos($long_number,ya)+$mos($long_number,yd))*0.5]  \
          [expr ($mos($long_number,xe)+$mos($long_number,xh))*0.5]  \
          [expr ($mos($long_number,ye)+$mos($long_number,yh))*0.5]  \
        -fill "cyan" -smooth false -tag { A C }
       $m_canvas create line \
          [expr ($mos($long_number,xa)+$mos($long_number,xe))*0.5]  \
          [expr ($mos($long_number,ya)+$mos($long_number,ye))*0.5]  \
          [expr ($mos($long_number,xd)+$mos($long_number,xh))*0.5]  \
          [expr ($mos($long_number,yd)+$mos($long_number,yh))*0.5]  \
        -fill "cyan" -smooth false -tag { A C }
     }
    } 

     # reduced frame (dispersion limits) only in spectroscopic 
     # modes mos, pmos, lss, hits and mxu and ims and pims 
     switch -exact -- $lmod  {
     mos -
    pmos -
     mxu -
    hits -
     lss {
       if {[string compare [$this cget -instrument] FORS2]==0} {
       $m_canvas create line $ccd_frame(0,xer) $ccd_frame(0,yer) $ccd_frame(0,xhr) $ccd_frame(0,yhr) \
         -fill blue -width 1 -stipple "" -tag  { A C }
       #$m_canvas create line $ccd_frame(1,xer) $ccd_frame(1,yer) $ccd_frame(1,xhr) $ccd_frame(1,yhr) \
       #  -fill blue -width 1 -stipple "" -tag  { A C }
       $m_canvas create line $ccd_frame(0,xfr) $ccd_frame(0,yfr) $ccd_frame(0,xgr) $ccd_frame(0,ygr) \
         -fill blue -width 1 -stipple "" -tag { A C }
       #$m_canvas create line $ccd_frame(1,xfr) $ccd_frame(1,yfr) $ccd_frame(1,xgr) $ccd_frame(1,ygr) \
       #  -fill blue -width 1 -stipple "" -tag { A C } 
       } else {
	$m_canvas create line $mos(0,xer) $mos(0,yer) $mos(0,xhr) $mos(0,yhr) \
         -fill blue -width 1 -stipple "" -tag  { A C }
        $m_canvas create line $mos(0,xfr) $mos(0,yfr) $mos(0,xgr) $mos(0,ygr) \
         -fill blue -width 1 -stipple "" -tag { A C }   
       }
      }
    }

     } else {
     #--------------
     #--------------
     # SR Collimator  
     #--------------
     # -------------
     # full frame 
     $m_canvas create line $mos(0,xa) $mos(0,ya) $mos(0,xb) $mos(0,yb) \
      $mos(0,xc) $mos(0,yc) $mos(0,xd) $mos(0,yd) $mos(0,xa) $mos(0,ya) \
      -fill blue -width 3 -tag { A C } 
     
      if {[string compare [$this cget -instrument] FORS2]==0} {     
     # CCD frame
     #- Master Part -#
     $m_canvas create line $ccd_frame(0,xa) $ccd_frame(0,ya) $ccd_frame(0,xb) $ccd_frame(0,yb) \
      $ccd_frame(0,xc) $ccd_frame(0,yc) $ccd_frame(0,xd) $ccd_frame(0,yd) $ccd_frame(0,xa) $ccd_frame(0,ya) \
      -fill green -width 1 -tag { A C}       
     #- Slave Part -#
      $m_canvas create line $ccd_frame(1,xa) $ccd_frame(1,ya) $ccd_frame(1,xb) $ccd_frame(1,yb) \
      $ccd_frame(1,xc) $ccd_frame(1,yc) $ccd_frame(1,xd) $ccd_frame(1,yd) $ccd_frame(1,xa) $ccd_frame(1,ya) \
      -fill green -width 1 -tag { A C}       
     
      $m_canvas create text [expr ($ccd_frame(0,xa)+$ccd_frame(0,xb))/2] \
                           [expr ($ccd_frame(0,ya)-15)] \
                           -anchor n  -fill green      \
                           -text "     CCD master     " \
                           -tag { A C m }
     $m_canvas create text [expr ($ccd_frame(1,xc)+$ccd_frame(1,xd))/2] \
                           [expr ($ccd_frame(1,yc)+15)] \
                           -anchor s  -fill green      \
                           -text  "    CCD slave      " \
                           -tag { A C m }
      }
     

    
     # for mxu we neeed a taged play ground, therefore test here as polygon
     #     ---
     if {[string compare $lmod mxu]==0} {
	
	     $m_canvas create poly $mos(0,xa) $mos(0,ya) $mos(0,xb) $mos(0,yb) \
		     $mos(0,xc) $mos(0,yc) $mos(0,xd) $mos(0,yd)  \
		     -fill "" -smooth false -stipple "" -tag { A C mxu }
	
     }

     # for hits we neeed a (invisible) tag play ground 
     # for pinhole alignment with center/gauss
     # HITS-highres : only big pinhole 
     # hits_ref_targ is used by reference target verify
     # ali us ised by center/slit alignment

     if {[string compare $lmod hits]==0} {
     $m_canvas create poly $mos(0,xa) $mos(0,ya) $mos(0,xb) $mos(0,yb) \
      $mos(0,xc) $mos(0,yc) $mos(0,xd) $mos(0,yd)  \
      -fill "" -stipple ""  -smooth false  -tag { A C hits_ref_targ }
     
      # if the center/slit option is ON, add the ali tag
     if {$pics} { $m_canvas addtag ali withtag hits_ref_targ }
     # no cyan center cross in slit
     }

     # for echs we neeed a tag play ground 
     # for slit positioning with center/gauss
     # echs_ref_trag is used by reference target verify
     # ali is used by center/slit alignment

     if {[string compare $lmod echs]==0} {
     $m_canvas create poly $mos(0,xa) $mos(0,ya) $mos(0,xb) $mos(0,yb) \
      $mos(0,xc) $mos(0,yc) $mos(0,xd) $mos(0,yd)  \
      -fill "" -stipple ""  -smooth false  -tag { A C echs_ref_targ }

       # if center/slit option is ON, add the 'ali' tag
       if {$pics} { $m_canvas addtag ali withtag echs_ref_targ 

       # $m_canvas lower echs_ref_targ m9

       # -----------------
       # cyan center cross
       # -----------------
       $m_canvas create line [expr ($mos(10,xb)+$mos(10,xf))*0.5]  \
                             [expr ($mos(10,yb)+$mos(10,yf))*0.5]  \
                             [expr ($mos(10,xc)+$mos(10,xg))*0.5]  \
                             [expr ($mos(10,yc)+$mos(10,yg))*0.5]  \
        -fill "cyan" -smooth false -tag { A C }
       $m_canvas create line [expr ($mos(10,xb)+$mos(10,xc))*0.5]  \
                             [expr ($mos(10,yb)+$mos(10,yc))*0.5]  \
                             [expr ($mos(10,xf)+$mos(10,xg))*0.5]  \
                             [expr ($mos(10,yf)+$mos(10,yg))*0.5]  \
        -fill "cyan" -smooth false -tag { A C }
       }
       }

     # for mxu standard mask mode e neeed a (invisible) tag play ground 
     # for standard mask aligment  with center/gauss
     # this is done via a special trick 
     # we add the ali tag to the mxu tagged play gorund
       if {$pics} { 
          # if the center/gauss option is active 
          if {[string compare $lmod mxu]==0} {
             # if we are in standard mask mode 
             if {[fors::MXU_SLIT::get_mmff]} {
                $m_canvas addtag ali withtag mxu 
                $m_canvas addtag PG  withtag mxu 
                $m_canvas   dtag mxu  
                } else {
                $m_canvas addtag mxu withtag ali 
                $m_canvas   dtag ali  
                $m_canvas   dtag PG   
                }
             }
        }
      # puts stdout "tag of mxu PG=[$m_canvas gettags PG]"

     # for lss we neeed a (invisible) tag play ground 
     # for lss alignment with center/gauss
     # lss_ref_trag is used by refernce target verify
     # ali is used by center/slit alignment

     if {[string compare $lmod lss]==0} {
     $m_canvas create poly $mos(0,xa) $mos(0,ya) $mos(0,xb) $mos(0,yb) \
      $mos(0,xc) $mos(0,yc) $mos(0,xd) $mos(0,yd)  \
      -fill "" -stipple ""  -smooth false  -tag { A C lss_ref_targ }

     # if the center/slit option is ON add the 'ali' tag
     if {$pics} { $m_canvas addtag ali withtag lss_ref_targ 
    

       # ----------------------
       # cyan slit center cross
       # ----------------------
       $m_canvas create line \
          [expr ($mos($long_number,xa)+$mos($long_number,xd))*0.5]  \
          [expr ($mos($long_number,ya)+$mos($long_number,yd))*0.5]  \
          [expr ($mos($long_number,xe)+$mos($long_number,xh))*0.5]  \
          [expr ($mos($long_number,ye)+$mos($long_number,yh))*0.5]  \
        -fill "cyan" -smooth false -tag { A C }
       $m_canvas create line \
          [expr ($mos($long_number,xa)+$mos($long_number,xe))*0.5]  \
          [expr ($mos($long_number,ya)+$mos($long_number,ye))*0.5]  \
          [expr ($mos($long_number,xd)+$mos($long_number,xh))*0.5]  \
          [expr ($mos($long_number,yd)+$mos($long_number,yh))*0.5]  \
        -fill "cyan" -smooth false -tag { A C }
     }
     }

     # reduced frame only in spectroscopic modes 
     # mos, pmos, lss, hits and mxu 
     # but not for echs, img, ipol
     switch -exact -- $lmod  {
     mos -
    pmos -
     mxu -
    hits -
     lss {
       
       if {[string compare [$this cget -instrument] FORS2]==0} {
       #$m_canvas create line $mos(0,xar) $mos(0,yar) $ccd_frame(0,xdr) $ccd_frame(0,ydr) \
       #   -fill blue -width 1 -stipple "" -tag  { A C }
       #$m_canvas create line $ccd_frame(0,xbr) $ccd_frame(0,ybr) $ccd_frame(0,xcr) $ccd_frame(0,ycr) \
       #   -fill blue -width 1 -stipple "" -tag  { A C }

       $m_canvas create line $ccd_frame(0,xar) $ccd_frame(0,yar) $ccd_frame(0,xdr) $ccd_frame(0,ydr) \
         -fill blue -width 1 -stipple "" -tag  { A C }
       #$m_canvas create line $ccd_frame(0,xar) $ccd_frame(1,yar) $ccd_frame(0,xdr) $ccd_frame(0,ydr) \
       #  -fill blue -width 1 -stipple "" -tag  { A C }
       #$m_canvas create line $ccd_frame(0,xbr) $ccd_frame(0,ybr) $ccd_frame(0,xgr) $ccd_frame(0,ygr) \
       #  -fill blue -width 1 -stipple "" -tag { A C }
       $m_canvas create line $ccd_frame(0,xbr) $ccd_frame(0,ybr) $ccd_frame(0,xcr) $ccd_frame(0,ycr) \
         -fill blue -width 1 -stipple "" -tag { A C } 
       } else {
	 $m_canvas create line $mos(0,xar) $mos(0,yar) $mos(0,xdr) $mos(0,ydr) \
          -fill blue -width 1 -stipple "" -tag  { A C }
       $m_canvas create line $mos(0,xbr) $mos(0,ybr) $mos(0,xcr) $mos(0,ycr) \
          -fill blue -width 1 -stipple "" -tag  { A C }
        }  
       }
     }

   }


     # -----------------------------
     #
     # plot the SLITS
     #
     # -----------------------------

 switch -exact -- $mask  {
     mos {
         # mos and pmos, img, ipol 

         # default slit blade number mos  
         set start 1; set end 19
         set bb mm
        if {[string compare $lmod "echs"]==0} { 
           set bb ee; # to avoid MOS/IMG bindings in ECHS
           if {$echs_flag} { set start 9; set end 9
              } else { set start 0; set end -1}
           }

         # ODD 
     for {set i $start} {$i <= $end} {incr i 2} {
         # odd blades 

	# plot modified  slit blades shaded
        # in IPOL plot odd slit blades shaded
	if {[$this cget -graytype]} {
        if {[string compare $targw($i,f) "0"]==0} { 
           set color1 "";  set color2 ""
           } else {
           set trapa gray25; set color1 $color3; set color2 $color4
           }

         if { [ string match ipol $lmod ] || [ string match pmos $lmod ] } {
           set trapa gray25; set color1 $color3; set color2 $color4
           }

         }

        # puts stdout "plot_mask: $i, $targw($i,f) $trapa >$color1 >$color2 >$color3 >$color4"

     $m_canvas create polygon $mos($i,xa) $mos($i,ya) $mos($i,xb) $mos($i,yb) \
        $mos($i,xf) $mos($i,yf) $mos($i,xe) $mos($i,ye) -outline $color3 \
        -fill $color1 -stipple $trapa -tag " A $bb m$i ma$i "   
	
     $m_canvas create polygon $mos($i,xd) $mos($i,yd) $mos($i,xc) $mos($i,yc) \
        $mos($i,xg) $mos($i,yg) $mos($i,xh) $mos($i,yh) -outline $color3 \
        -fill $color1 -stipple $trapa -tag " A $bb m$i mb$i "

     #if {$debug} {
     #	 puts $lfId "create polygon $mos($i,xd) $mos($i,yd) $mos($i,xc) $mos($i,yc) \
     #   $mos($i,xg) $mos($i,yg) $mos($i,xh) $mos($i,yh) -outline $color3 \
     #   -fill $color1 -stipple"
     #}
     # create invisible objects, the slits. they can be called by their
     # tag to open and close slit blades in imaging mode

     if {$slitrim} {
     $m_canvas create polygon $mos($i,xb) $mos($i,yb) $mos($i,xc) $mos($i,yc) \
        $mos($i,xg) $mos($i,yg) $mos($i,xf) $mos($i,yf) -outline "" \
        -fill "" -stipple "" -tag " A $bb m$i ms$i "
        }
     # puts stdout "plot_mask:mos:odd loop: ok"
     }

     # draw the even $color2 slit blades 
     # tags are mm, m2, .. m18

     # default slit blade numbers for mos 
     set start 2; set end 18
     if {[string compare $lmod "echs"]==0} { set start 10; set end 10 }

     # EVEN
     for {set i $start} {$i <= $end} {incr i 2} {

        if {[$this cget -graytype]} {
        if {[string compare $targw($i,f) "0"]==0} { 
           set color1 "";  set color2 ""
           } else {
           set trapa gray25; set color1 $color3; set color2 $color4
           }
         }

        # puts stdout "plot_mask: $i, $targw($i,f) $trapa >$color1 >$color2 >$color3 >$color4"

     $m_canvas create polygon $mos($i,xa) $mos($i,ya) $mos($i,xb) $mos($i,yb) \
        $mos($i,xf) $mos($i,yf) $mos($i,xe) $mos($i,ye) -outline $color4 \
        -fill $color2 -stipple $trapa -tag " A $bb m$i ma$i "

     $m_canvas create polygon $mos($i,xd) $mos($i,yd) $mos($i,xc) $mos($i,yc) \
        $mos($i,xg) $mos($i,yg) $mos($i,xh) $mos($i,yh) -outline $color4 \
        -fill $color2 -stipple $trapa -tag " A $bb m$i mb$i "
     
     if {$slitrim} { 
     $m_canvas create polygon $mos($i,xb) $mos($i,yb) $mos($i,xc) $mos($i,yc) \
        $mos($i,xg) $mos($i,yg) $mos($i,xf) $mos($i,yf) -outline "" \
        -fill "" -stipple "" -tag " A $bb m$i ms$i "
     }
     # puts stdout "plot_mask:mos:eve loop: ok"
     }
     }
  pol {
      # polarimetric imaging mask for high res 
      # first fill the mask, not the stripes 
     $m_canvas create polygon  \
      $mos(0,xa) $mos(0,ya) $mos(0,xb) $mos(0,yb) \
      $mos(0,xc) $mos(0,yc) $mos(0,xd) $mos(0,yd) \
      $mos(9,xe) $mos(9,ye) $mos(9,xh) $mos(9,yh) \
      $mos(1,xd) $mos(1,yd) $mos(1,xa) $mos(1,ya) \
      $mos(9,xe) $mos(9,ye) $mos(0,xd) $mos(0,yd) \
      $mos(0,xa) $mos(0,ya) \
      -outline "" -fill $color1 -stipple $trapa -width 1 -tag A 

       for {set i 1} {$i < 9 } {incr i 1} {
          set j [expr $i + 1]
     $m_canvas create polygon $mos($i,xe) $mos($i,ye) $mos($i,xh) $mos($i,yh) \
        $mos($j,xd) $mos($j,yd) $mos($j,xa) $mos($j,ya) \
        $mos($i,xe) $mos($i,ye) \
      -outline "" -fill $color1 -stipple $trapa -width 1  -tag A
         }

       # and the open stripes
       for {set i 1} {$i < 10} {incr i 1} {
     $m_canvas create polygon $mos($i,xa) $mos($i,ya) $mos($i,xd) $mos($i,yd) \
        $mos($i,xh) $mos($i,yh) $mos($i,xe) $mos($i,ye) -outline $color4 \
        -fill ""  -stipple ""  -tag A
         }
       }
  hits {
     # the current ht-slit 
     # the 0.3'' and 0.4'' slit is not supported
     switch -exact -- $long_number {
        3 -
        4 -
        5 -
        6 -
        7 -
        8 {
     $m_canvas create polygon $mos($long_number,xa) $mos($long_number,ya) \
        $mos($long_number,xd) $mos($long_number,yd) \
        $mos($long_number,xh) $mos($long_number,yh) \
        $mos($long_number,xe) $mos($long_number,ye) -outline $color4 \
        -fill "" -stipple "" -tag A
          }

      1 {
     # draw the pinhole 
     $m_canvas create polygon $mos(9,xa) $mos(9,ya) \
     $mos(9,xd) $mos(9,yd) $mos(9,xh) $mos(9,yh) \
     $mos(9,xe) $mos(9,ye) -outline $color4 -fill "" -stipple "" -tag A
     # tcltk too stupid, cannot draw oval with angle
     set x [expr $mos(9,xa)-$mos(9,xh)]; 
     set y [expr $mos(9,ya)-$mos(9,yh)]
     set r [expr sqrt($x*$x+$y*$y)/2.0/sqrt(2.0)]
     set x [expr ($mos(9,xa)+$mos(9,xh))/2.0] 
     set y [expr ($mos(9,ya)+$mos(9,yh))/2.0]
     $m_canvas create oval [expr $x-$r] [expr $y-$r] [expr $x+$r] [expr $y+$r] \
        -fill "" -stipple "" -outline $color4 -tag A 
          }
       0 {
     # long_number ==0  
     # 'show-all 7 slits'  modus  
     for {set i 3 } {$i<10} { incr i 1} {
     $m_canvas create polygon $mos($i,xa) $mos($i,ya) \
        $mos($i,xd) $mos($i,yd) $mos($i,xh) $mos($i,yh) \
        $mos($i,xe) $mos($i,ye) -outline $color4 \
        -fill "" -stipple "" -tag A
        }
     # draw the oval pinhole 
     # tcltk too stupid, cannot draw oval with angle
     set x [expr $mos(9,xa)-$mos(9,xh) ]; set y [expr $mos(9,ya)-$mos(9,yh) ]
     set r [expr sqrt($x*$x+$y*$y)/2.0/sqrt(2.0)]
     set x [expr ($mos(9,xa)+$mos(9,xh))/2.0] 
     set y [expr ($mos(9,ya)+$mos(9,yh))/2.0]
     $m_canvas create oval [expr $x-$r] [expr $y-$r] [expr $x+$r] [expr $y+$r] \
        -fill "" -stipple "" -outline $color4 -tag A 
        }
       }
       }
   lss {
     # the current long-slit 
     if {$long_number>0} {
     $m_canvas create polygon $mos($long_number,xa) $mos($long_number,ya) \
        $mos($long_number,xd) $mos($long_number,yd) \
        $mos($long_number,xh) $mos($long_number,yh) \
        $mos($long_number,xe) $mos($long_number,ye) -outline $color4 \
        -fill "" -stipple "" -tag A
        } else {
     for {set i 1 } {$i<10} { incr i 1 } {
     $m_canvas create polygon $mos($i,xa) $mos($i,ya) \
        $mos($i,xd) $mos($i,yd) \
        $mos($i,xh) $mos($i,yh) \
        $mos($i,xe) $mos($i,ye) -outline $color4 \
        -fill "" -stipple "" -tag A
        }
        }
       }
   mxu {  
          # update the color for all slits 
          fors::MXU_SLIT::put_col [list $color1 $color2 $color3 $color4 $trapa]
          # number of mxu slits 
          set noms [fors::MXU_SLIT::nos]
          if {$noms>0} {
          for {set i 1} {$i <= $noms} {incr i 1} {
              if {[C_mxus$i cget -ON]} {
                 switch -exact -- [C_mxus$i cget -shape] {
                 STRAIGHT { C_mxus$i update_slit   }
                 CIRCLE   { C_mxus$i update_circle }
                 CURVED   { C_mxus$i update_curve  }
                  }

                 C_mxus$i plot_m 
                 }
              }
             }
          }
   default { puts stdout "Warning: mask plot_mask" }
      }



     if {$debug} { puts -nonewline stdout "o" }

     # -----------------------------------------
     # draw the green circle 
     # -----------------------------------------
     # showing the ccx ccy position = rotation axis
     # tag = mm , m
     # puts stdout "plot green point: \
     #     [expr $mos(0,x0)-$mos(0,r)]"
     $m_canvas create oval \
        [expr $mos(0,x0)-$mos(0,r)] [expr $mos(0,y0)-$mos(0,r)] \
        [expr $mos(0,x0)+$mos(0,r)] [expr $mos(0,y0)+$mos(0,r)] \
        -fill green -stipple $trapa -tag { A m m10 } 
     $m_canvas create oval \
        [expr $mos(0,x0)-$mos(0,r)/20] [expr $mos(0,y0)-$mos(0,r)/20] \
        [expr $mos(0,x0)+$mos(0,r)/20] [expr $mos(0,y0)+$mos(0,r)/20] \
        -fill "" -stipple "" -tag { A m m10 } 

     # defines orientation: at x>0,y>0 North East
     # here only for some tests




     if {$debug} { puts -nonewline stdout "t" }

     # ----------------------------------------------
     #
     # plot the LABELS 
     #
     #-----------------------------------------------

     if {[$this cget -label]} {


     # label orientation 

       if {$fits_x_scale ==  1.0} {set oriA e; set oriB w }
       if {$fits_x_scale == -1.0} {set oriA w; set oriB e }
       if {$mountangle == 180} {
		  # puts stdout "mountangle = $mountangle ori"
		  set dummy $oriA; set oriA $oriB; set oriB $dummy
			       }

     # puts stdout "\nplot_mask: >$world_x< >$world_y< $EQX\n"

     $m_image convert coords $world_x $world_y "deg $EQX" lwx lwy "wcs $EQX"

     if {$debug} { puts -nonewline stdout "-" }

     # -------------------------------
     # 1.0 focal field labels (blue frame)
     # -------------------------------
     if {$mountangle == 180} { 
	 # puts stdout "mountangle = $mountangle EW"
     if {[string compare $mask "mos"]==0} {
     # draw labels to the blue focal plane 
     $m_canvas create text [expr ($mos(10,xa)+$mos(10,xe))/2] \
                           [expr ($mos(10,ya)+$mos(10,ye))/2] \
                           -anchor $oriA -fill $color4      \
                           -text "     E A blue      " \
                           -tag { A C m }
     $m_canvas create text [expr ($mos(10,xd)+$mos(10,xh))/2] \
                           [expr ($mos(10,yd)+$mos(10,yh))/2] \
                           -anchor $oriB -fill $color4      \
                           -text "     W B red      " \
                           -tag { A C m }
       } else {
     # draw labels at the switch-off limits of MOS 
     $m_canvas create text [expr ($mos(0,xa)+$mos(0,xd))/2] \
                           [expr ($mos(0,ya)+$mos(0,yd))/2] \
                           -anchor $oriA -fill $color4      \
                           -text "     E A blue      " \
                           -tag { A C m }
     $m_canvas create text [expr ($mos(0,xb)+$mos(0,xc))/2] \
                           [expr ($mos(0,yb)+$mos(0,yc))/2] \
                           -anchor $oriB -fill $color4      \
                           -text "     W B red      " \
                           -tag { A C m }
     }
     $m_canvas create text [expr ($mos(0,xc)+$mos(0,xd))/2] \
                           [expr ($mos(0,yc)+$mos(0,yd))/2] \
                           -anchor n  -fill $color4      \
                           -text "     BOTTOM 19 $lmod     " \
                           -tag { A C m }
     $m_canvas create text [expr ($mos(0,xa)+$mos(0,xb))/2] \
                           [expr ($mos(0,ya))] \
                           -anchor s  -fill $color4      \
                           -text "     TOP 1  [$this cget -instrument]     " \
                           -tag { A C m }     
      } else {
     $m_canvas create text [expr ($mos(9,xa)+$mos(9,xd))/2] \
                           [expr ($mos(9,ya)+$mos(9,yd))/2] \
                           -anchor $oriA -fill $color4      \
                           -text "     W A red      "  \
                           -tag { A C m }
     $m_canvas create text [expr ($mos(9,xc)+$mos(9,xb))/2] \
                           [expr ($mos(9,yc)+$mos(9,yb))/2] \
                           -anchor $oriB -fill $color4      \
                           -text "     E B blue      " \
                           -tag { A C m }
     $m_canvas create text [expr ($mos(0,xc)+$mos(0,xd))/2] \
                           [expr ($mos(0,yc)+$mos(0,yd))/2] \
                           -anchor s  -fill $color4      \
                           -text "     TOP 19 [$this cget -instrument]     " \
                           -tag { A C m }
     $m_canvas create text [expr ($mos(0,xa)+$mos(0,xb))/2] \
                           [expr ($mos(0,ya)+$mos(0,yb))/2] \
                           -anchor n  -fill $color4      \
                           -text "     S 1  $lmod     " \
                           -tag { A C m }
      }


     if {$debug} { puts -nonewline stdout "m" }

     # -------------------------------
     # 2.0 TCS labels (upper left corner)
     # ------------------------------1-
     catch {$m_canvas delete D}
     plot_tcs_label 

     if {$debug} { puts -nonewline stdout "a" }

     # -------------------------------
     # 3.0 modes
     # -------------------------------
     # some text to help orientation
     #-----------------------
     switch -exact -- $mask {
     mos {
         # -------------------------------
         # 3.1 MOS-mask  (mos,pmos,img,ipol)
         # -------------------------------
         # 3.1.1 cyan circle
         # -------------------------------
       # these are only for the progammer  for diagnostic purposes
       # mos blade 10 A (center west)
       # set dum_x [expr ($mos(10,xa)+$mos(10,xb)+$mos(10,xe)+$mos(10,xf))/4.0]
       # set dum_y [expr ($mos(10,ya)+$mos(10,yb)+$mos(10,ye)+$mos(10,yf))/4.0]
       # $m_canvas create oval \
       #  [expr $dum_x - $mos(0,r)] [expr $dum_y - $mos(0,r)] \
       #  [expr $dum_x + $mos(0,r)] [expr $dum_y + $mos(0,r)] \
       # -fill cyan -tag A

       # $m_canvas create oval \
       #  [expr $mos(1,xa)-2.05*$mos(0,r)] [expr $mos(1,ya)-2.05*$mos(0,r)] \
       #  [expr $mos(1,xa)+2.05*$mos(0,r)] [expr $mos(1,ya)+2.05*$mos(0,r)] \
       # -outline red -tag A
       # $m_canvas create oval \
       #  [expr $mos(1,xb)-0.05*$mos(0,r)] [expr $mos(1,yb)-0.05*$mos(0,r)] \
       #  [expr $mos(1,xb)+0.05*$mos(0,r)] [expr $mos(1,yb)+0.05*$mos(0,r)] \
       # -outline  yellow -tag A



         # -------------------------------
         # 3.1.2 blade labels
         # -------------------------------
         # ODD 
         set start 1; set end 19
         if {[string compare $lmod "echs"]==0} {
            if {$echs_flag} { set start 9; set end 9 
               } else {
               set start 0; set end -1
               }
            }

       for {set i $start} { $i <= $end} {incr i 2} {
           # slit labels 
           # at the target positions
           # puts stdout "$i $targw($i,f)"
           # puts stdout "> [format " %2.1f'' " $mos($i,b) ] <"
           if {$targw($i,f) != 0 } {
              # slit number label
              $m_canvas create text [expr ($mos($i,xb)+$mos($i,xf))/2] \
                        [expr ($mos($i,yb)+$mos($i,yf))/2] \
                        -anchor $oriA -fill $color4 -text " #$i " -tag "A lb m$i"
              # slit width label
              $m_canvas create text [expr ($mos($i,xc)+$mos($i,xg))/2] \
                        [expr ($mos($i,yc)+$mos($i,yg))/2] \
                        -anchor $oriB -fill $color4 \
                        -text [format " %2.1f'' " $mos($i,b) ] -tag "A sw m$i"
              }
           # at both rims 
           # we give these labels the individual slit tag, since 
           # it could be that the labels are in the slit itself
           # east drive A
           $m_canvas create text [expr ($mos($i,xa)+$mos($i,xe))/2] \
                     [expr ($mos($i,ya)+$mos($i,ye))/2] \
                     -anchor $oriA -fill $color4 -text "#$i" \
                     -tag "A out m$i ma$i"
           # west drive B
           $m_canvas create text [expr ($mos($i,xd)+$mos($i,xh))/2] \
                     [expr ($mos($i,yd)+$mos($i,yh))/2] \
                     -anchor $oriB  -fill $color4 -text "#$i" \
                     -tag "A out m$i mb$i"
           # make the mos slits visible 
           if {[string match *mos $lmod] || 
               [string match  echs $lmod]} {
              $m_canvas create polygon $mos($i,xb) $mos($i,yb) \
                        $mos($i,xc) $mos($i,yc) $mos($i,xg) $mos($i,yg) \
                        $mos($i,xf) $mos($i,yf) -outline $color4 \
                        -fill ""  -tag "A $bb m$i ms$i "
              }
           }

         # EVEN 
         set start 2; set end 18
         if {[string compare $lmod "echs"]==0} { set start 10; set end 10 }


    for {set i $start} { $i <= $end} {incr i 2} {
        # slit labels 
        # at the target positions (only if have been set) 
        # puts stdout "$i $targw($i,f)"
        if {$targw($i,f) != 0} {
           # slit number label
           $m_canvas create text [expr ($mos($i,xb)+$mos($i,xf))/2] \
                     [expr ($mos($i,yb)+$mos($i,yf))/2] \
                     -anchor $oriA -fill $color3 -text " #$i " \
                     -tag "A lb m$i"
           # slit width label
           $m_canvas create text [expr ($mos($i,xc)+$mos($i,xg))/2] \
                     [expr ($mos($i,yc)+$mos($i,yg))/2] \
                     -anchor $oriB -fill $color3 \
                     -text [format " %2.1f'' " $mos($i,b)] \
                     -tag "A sw m$i"
           }
        # at the both frame rims 
        # east drive A
        $m_canvas create text [expr ($mos($i,xa)+$mos($i,xe))/2] \
                  [expr ($mos($i,ya)+$mos($i,ye))/2] \
                  -anchor $oriA -fill $color3 -text "#$i" \
                  -tag " A out m$i ma$i"
        # west drive B
        $m_canvas create text [expr ($mos($i,xd)+$mos($i,xh))/2] \
                  [expr ($mos($i,yd)+$mos($i,yh))/2] \
                 -anchor $oriB -fill $color3 -text "#$i" \
                 -tag " A out m$i mb$i"

        # make the mos slits visible 
        if {[string match *mos $lmod] ||
            [string match echs $lmod]} {
           $m_canvas create polygon $mos($i,xb) $mos($i,yb) \
                     $mos($i,xc) $mos($i,yc) $mos($i,xg) $mos($i,yg) \
                     $mos($i,xf) $mos($i,yf) -outline $color3 \
                     -fill ""  -tag " A $bb m$i ms$i "
            }
         if {[string match echs $lmod]} {
            $m_canvas create oval \
            [expr $mos(10,xg)-$mos(0,r)/2] [expr $mos(10,yg)-$mos(0,r)/2] \
            [expr $mos(10,xg)+$mos(0,r)/2] [expr $mos(10,yg)+$mos(0,r)/2] \
            -fill cyan -stipple $trapa -tag { A m PE } 
           }
        }

         # -------------------------------
         # 3.1.3 wavelength range labels
         # -------------------------------
      # indicate wavelength range only for mos, pmos
      # not for IMG and IPOL-SR (they also have mask=mos)
      # not for ECHS (they also have mask=mos)
      if {[string match "*mos" $lmod] } {
      plot_mos_dispersion_label $oriA $oriB
      }
     }
    mxu  {
         # -------------------------------
         # 3.2 MXU-mask  
         # -------------------------------
         plot_mos_dispersion_label $oriA $oriB

         # workaround for the preliminary commissioning 
         # of the mxu, WHummel 13.Sept 2000
         # plot the 1000 Pixel area in SR = 200 arcsea
         # we simply use the HR collimator box
         
	 #AAG: workarround removed - MIT CCDs
	 #set loc_rad [expr $mos(0,r)/2.0]

         #set p1x [expr (3* $mos(0,xa) + $mos(0,xb) ) * 0.25 ]
         #set p1y [expr (3* $mos(0,ya) + $mos(0,yb) ) * 0.25 ]
         #set p2x [expr ($mos(0,xa) + 3* $mos(0,xb) ) * 0.25 ]
         #set p2y [expr ($mos(0,ya) + 3* $mos(0,yb) ) * 0.25 ]

         #set p3x [expr (3* $mos(0,xd) + $mos(0,xc) ) * 0.25 ]
         #set p3y [expr (3* $mos(0,yd) + $mos(0,yc) ) * 0.25 ]
         #set p4x [expr ($mos(0,xd) + 3* $mos(0,xc) ) * 0.25 ]
         #set p4y [expr ($mos(0,yd) + 3* $mos(0,yc) ) * 0.25 ]


         #$m_canvas create line $p1x $p1y $p3x $p3y \
	 #                     -fill cyan -stipple "" -arrow none \
         #                     -width 1 -tags " A"

         #$m_canvas create line $p2x $p2y $p4x $p4y \
         #                      -fill cyan -stipple "" -arrow none \
         #                      -width 1 -tags " A"


          # number of mxu slits 
          if {$noms>0} {
          for {set i 1} {$i <= $noms} {incr i 1} {
              if {[C_mxus$i cget -ON]} {
              C_mxus$i plot_label $oriA $oriB
              }
              }

             if {[fors::MXU_SLIT::get_mmff]} {
                # if technical mask mode
                # draw a cyan move circle at the first of many
                # technical slits

                # get x y width length in mm focal plane
                set first_slit [C_mxus1 get_slit]
                # puts stdout "slit center x/y(mm) >[lindex $first_slit 0]< >[lindex $first_slit 1]<";

                # first_slit 0,1 = x,y center of slit
                #  draw cyan circle 3 times a slit width away from the slit center 
                # take the smallest (either width or length)
                # note: the radius of the green move-circle is about 8'' or ~4mm 
                # the cyan cricle should be at least 16'' or 8 mm away from the 
                # green circle 
                if { [lindex $first_slit 2] < [lindex $first_slit 3] } {
                   set shift [lindex $first_slit 2]
                   } else {
                   set shift [lindex $first_slit 3]
                   }  
                 if { $shift < 4 } { set shift 4 } 

                set px [expr [lindex $first_slit 0]+$shift*3.0 ]
                set py [expr [lindex $first_slit 1]-$shift*3.0 ]

                # slit center in canvas to plot
                set a [fors::CONV_MC::m2c $px $py]
                # puts stdout "cyan(mm)=($px,$py) (canvas)=$a shift(constant)=$shift"
                set loc_rad [expr $mos(0,r)/2.0]

                $m_canvas create oval \
                   [expr [lindex $a 0] - $loc_rad ] \
                   [expr [lindex $a 1] - $loc_rad ] \
                   [expr [lindex $a 0] + $loc_rad ] \
                   [expr [lindex $a 1] + $loc_rad ] \
                   -fill cyan -tag {A m}
                }
             }
         # puts stdout "plot mxu label ok"
         }
    hits {
         # -------------------------------
         # 3.2 HITS-mask  
         # -------------------------------
         # 3.2.1 cyan move button
         # -------------------------------
         switch -exact -- $long_number {
         3 -
         4 -
         5 -
         6 -
         7 -
         8 {
     # extra move button 
     $m_canvas create oval \
        [expr $mos($long_number,xe)-$mos(0,r)/2.0 ] \
        [expr $mos($long_number,ye)-$mos(0,r)/2.0 ] \
        [expr $mos($long_number,xe)+$mos(0,r)/2.0 ] \
        [expr $mos($long_number,ye)+$mos(0,r)/2.0 ] \
        -fill cyan -tag {A m} 
         # -------------------------------
         # 3.2.2 slit label 
         # -------------------------------
      $m_canvas create text $mos($long_number,xh) $mos($long_number,yh) \
         -anchor $oriB -fill $color4 -text "  [$this cget -long_width]''  "  \
         -tag " A "
           }
         1 { 
           $m_canvas create text $mos(9,xa) $mos(9,ya) \
               -anchor $oriA  -fill $color4 -text "----(big pinhole)----"  \
               -tag " A "
            }
           }

         # -------------------------------
         # 3.2.3 wavelength range labels
         # -------------------------------
      # indicate wavelength range 
      if {[string match *HR* $lcol ]} {
      if {[string compare [$this cget -instrument] FORS2]==0} {
      $m_canvas create text $ccd_frame(0,xhr) $ccd_frame(0,yhr) \
               -anchor $oriB -fill $color4 \
               -text [format "%4.0fA" [expr $L_MAX*10.0] ] -tag " A"
      $m_canvas create text $ccd_frame(0,xgr) $ccd_frame(0,ygr) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $L_MIN*10.0] ] -tag " A"
      } else {
	$m_canvas create text $mos(0,xhr) $mos(0,yhr) \
               -anchor $oriB -fill $color4 \
               -text [format "%4.0fA" [expr $L_MAX*10.0] ] -tag " A"
      $m_canvas create text $mos(0,xgr) $mos(0,ygr) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $L_MIN*10.0] ] -tag " A"  
      }
         } else {
      $m_canvas create text $mos(0,xdr) $mos(0,ydr) \
               -anchor $oriB -fill $color4 \
               -text [format "%4.0fA" [expr $L_MAX*10.0] ] -tag " A"
      $m_canvas create text $mos(0,xcr) $mos(0,ycr) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $L_MIN*10.0] ] -tag " A"
         }
        }
     lss { 
         # -------------------------------
         # 3.3 LSS-mask  
         # -------------------------------
         # 3.3.1 slit label
         # -------------------------------
      # indicate slit
      $m_canvas create text $mos($long_number,xh) $mos($long_number,yh) \
         -anchor $oriB -fill $color4 -text "  [$this cget -long_width]''  "  \
         -tag " A "

     # extra move-button
     # $m_canvas create oval \
     #    [expr $mos($long_number,xe)-$mos(0,r)/2.0 ] \
     #    [expr $mos($long_number,ye)-$mos(0,r)/2.0 ] \
     #    [expr $mos($long_number,xe)+$mos(0,r)/2.0 ] \
     #    [expr $mos($long_number,ye)+$mos(0,r)/2.0 ] \
     #    -fill cyan -tag {A m} 
       # $m_canvas create oval \
       #  [expr $mos(1,xa)-0.1*$mos(0,r)] [expr $mos(1,ya)-0.1*$mos(0,r)] \
       #  [expr $mos(1,xa)+0.1*$mos(0,r)] [expr $mos(1,ya)+0.1*$mos(0,r)] \
       # -outline red -tag A
       # $m_canvas create oval \
       #  [expr $mos(1,xd)-0.1*$mos(0,r)] [expr $mos(1,yd)-0.1*$mos(0,r)] \
       #  [expr $mos(1,xd)+0.1*$mos(0,r)] [expr $mos(1,yd)+0.1*$mos(0,r)] \
       # -outline yellow -tag A

         # -------------------------------
         # 3.3.2 wavelength range labels
         # -------------------------------
      # indicate wavelength range 
      if {[string match *HR* $lcol ]} {
      if {[string compare [$this cget -instrument] FORS2]==0} {
      
      $m_canvas create text $ccd_frame(0,xgr) $ccd_frame(0,ygr) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $L_MIN*10.0] ] -tag " A"
      $m_canvas create text $ccd_frame(0,xhr) $ccd_frame(0,yhr) \
               -anchor $oriB -fill $color4 \
               -text [format "%4.0fA" [expr $L_MAX*10.0] ] -tag " A"
         } else {
      $m_canvas create text $mos(0,xcr) $mos(0,ycr) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $L_MIN*10.0] ] -tag " A"
      $m_canvas create text $mos(0,xdr) $mos(0,ydr) \
               -anchor $oriB -fill $color4 \
               -text [format "%4.0fA" [expr $L_MAX*10.0] ] -tag " A"
         }
       }
    }
     pol { 
         # -------------------------------
         # 3.4 POL-mask
         # -------------------------------
         # $m_canvas create oval \
         #   [expr $mos(1,xa)-$mos(0,r)] [expr $mos(1,ya)-$mos(0,r)] \
         #   [expr $mos(1,xa)+$mos(0,r)] [expr $mos(1,ya)+$mos(0,r)] \
         #   -outline red  -tag A 
         # $m_canvas create oval \
         #   [expr $mos(1,xd)-$mos(0,r)] [expr $mos(1,yd)-$mos(0,r)] \
         #   [expr $mos(1,xd)+$mos(0,r)] [expr $mos(1,yd)+$mos(0,r)] \
         #   -outline yellow -tag A 
         }
     }


         # -------------------------------
         # 3.0 all masks 
         # -------------------------------
          
     # the white circle for (x>0,y>0 = NE) in focal field coord system  
     if {$mountangle == 180} {

     if {[string match *HR* $lcol ] } {
       if {[string compare [$this cget -instrument] FORS2]==0} {
        # hr
     $m_canvas create oval \
        [expr $ccd_frame(0,xe)-$mos(0,r)] [expr $ccd_frame(0,yf)-$mos(0,r)] \
        [expr $ccd_frame(0,xe)+$mos(0,r)] [expr $ccd_frame(0,yf)+$mos(0,r)] \
        -fill white  -tag { A P2 m }
        # gray circle for CCD reference pixel = (1,1 = SE)
      $m_canvas create oval \
        [expr $ccd_frame(1,xh)-$mos(0,r)] [expr $ccd_frame(1,yh)-$mos(0,r)] \
        [expr $ccd_frame(1,xh)+$mos(0,r)] [expr $ccd_frame(1,yh)+$mos(0,r)] \
         -fill gray   -tag { A P3 m }
      } else {
        $m_canvas create oval \
        [expr $mos(0,xe)-$mos(0,r)] [expr $mos(0,ye)-$mos(0,r)] \
        [expr $mos(0,xe)+$mos(0,r)] [expr $mos(0,ye)+$mos(0,r)] \
        -fill white  -tag { A P2 m }
        # gray circle for CCD reference pixel = (1,1 = SE)
      $m_canvas create oval \
        [expr $mos(0,xh)-$mos(0,r)] [expr $mos(0,yh)-$mos(0,r)] \
        [expr $mos(0,xh)+$mos(0,r)] [expr $mos(0,yh)+$mos(0,r)] \
         -fill gray   -tag { A P3 m }
      }
        } else {
        # sr 
     $m_canvas create oval \
        [expr $mos(0,xa)-$mos(0,r)] [expr $mos(0,ya)-$mos(0,r)] \
        [expr $mos(0,xa)+$mos(0,r)] [expr $mos(0,ya)+$mos(0,r)] \
        -fill white  -tag { A P2 m }
     # gray circle for CCD reference pixel = (1,1 = SE)
      $m_canvas create oval \
        [expr $mos(0,xd)-$mos(0,r)] [expr $mos(0,yd)-$mos(0,r)] \
        [expr $mos(0,xd)+$mos(0,r)] [expr $mos(0,yd)+$mos(0,r)] \
         -fill gray   -tag { A P3 m }
        }

	if {[string compare ipol [$this cget -modus]] } {
           # its not IPOL
	   $m_canvas dtag P2
	   $m_canvas dtag P3
	   }

        } else {

     if {[string match *HR* $lcol ] } {
        # hr
     if {[string compare [$this cget -instrument] FORS2]==0} {
        # hr
     $m_canvas create oval \
        [expr $ccd_frame(0,xa)-$mos(0,r)] [expr $ccd_frame(1,yf)-$mos(0,r)] \
        [expr $ccd_frame(0,xa)+$mos(0,r)] [expr $ccd_frame(1,yf)+$mos(0,r)] \
        -fill white  -tag { A P2 m }
        # gray circle for CCD reference pixel = (1,1 = SE)
      $m_canvas create oval \
        [expr $ccd_frame(1,xe)-$mos(0,r)] [expr $ccd_frame(0,yh)-$mos(0,r)] \
        [expr $ccd_frame(1,xe)+$mos(0,r)] [expr $ccd_frame(0,yh)+$mos(0,r)] \
         -fill gray   -tag { A P3 m }
      } else {
     $m_canvas create oval \
        [expr $mos(0,xg)-$mos(0,r)] [expr $mos(0,yg)-$mos(0,r)] \
        [expr $mos(0,xg)+$mos(0,r)] [expr $mos(0,yg)+$mos(0,r)] \
        -fill white  -tag { A P2 m }
     # gray circle for CCD reference pixel = (1,1 = SE)
      $m_canvas create oval \
        [expr $mos(0,xf)-$mos(0,r)] [expr $mos(0,yf)-$mos(0,r)] \
        [expr $mos(0,xf)+$mos(0,r)] [expr $mos(0,yf)+$mos(0,r)] \
         -fill gray   -tag { A P3 m }
       }
        } else {
        # sr 
     $m_canvas create oval \
        [expr $mos(0,xc)-$mos(0,r)] [expr $mos(0,yc)-$mos(0,r)] \
        [expr $mos(0,xc)+$mos(0,r)] [expr $mos(0,yc)+$mos(0,r)] \
        -fill white  -tag { A P2 m }
     # gray circle for CCD reference pixel = (1,1 = SE)
      $m_canvas create oval \
        [expr $mos(0,xb)-$mos(0,r)] [expr $mos(0,yb)-$mos(0,r)] \
        [expr $mos(0,xb)+$mos(0,r)] [expr $mos(0,yb)+$mos(0,r)] \
         -fill gray   -tag { A P3 m }
        }

      }

     } 





     if {$debug} { puts -nonewline stdout "s" }

     if {$reftarg_zone} {
     if {[C_ref_targ cget -mos10ali]==0} {
        plot_ref_star_area 0 
        
        } else { 
        plot_ref_star_area 1 
        }
     
     }

     # AAG - PPRS 10127
     ::fors::FORS::my_mask  edit_ref_targ plot

     if {$debug} { puts stdout "k" }
     return 0
     }


     #======================
     private method plot_tcs_label {} {
     #====================== 
     # extra coords and PA in the upper left corner 
     $m_image convert coords $world_x $world_y "deg $EQX" lwx lwy "wcs $EQX"
     $m_image convert coords 10 20 screen px20 py20 canvas
     $m_canvas create text $px20 $py20 -fill $color4 \
         -text "ROT = $tcs_pam" -anchor w -tag {D}
     $m_canvas create text $px20 [expr $py20 +20] -fill $color4 \
         -text "  RA = $lwx" -anchor w -tag {D}
     $m_canvas create text $px20 [expr $py20 +40] -fill $color4 \
         -text "DEC = $lwy" -anchor w -tag {D}

      if {$pics} {
      $m_canvas create text $px20 [expr $py20 +60] -fill $color4 \
          -text ">CENTER<" -anchor w -tag {D}
       }
     # $m_canvas create text $px20 [expr $py20 +80] -fill $color4 \
     #     -text [$this cget -filter] -anchor w -tag {D}
     
     } 

     #============================================
     private method plot_mos_dispersion_label { oriA oriB } {
     #============================================
      if {[string match *HR* [$this cget -collimat] ]} {
       if {[string compare [$this cget -instrument] FORS2]==0} {  
      $m_canvas create text $ccd_frame(0,xh) $ccd_frame(1,yh) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $L_MIN*10.0] ] -tag " A"
      $m_canvas create text $ccd_frame(0,xg) $ccd_frame(1,yg) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $L_MAX*10.0] ] -tag " A"
      $m_canvas create text $ccd_frame(1,xer) $ccd_frame(0,yer) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $A_disp($good,cen)*10.0] ] -tag " A"
      $m_canvas create text $ccd_frame(1,xfr) $ccd_frame(0,yfr) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $A_disp($good,cen)*10.0] ] -tag " A"

      $m_canvas create line $ccd_frame(0,xfr) $ccd_frame(0,yfr) mos(0,xg) $mos(0,yg) \
               -fill blue -width 1  -stipple "" -tag {A C}
      $m_canvas create line $ccd_frame(0,xgr) $ccd_frame(0,ygr) $mos(0,xf) $mos(0,yf) \
               -fill blue -width 1  -stipple "" -tag {A C}

      $m_canvas create line $ccd_frame(0,xer) $ccd_frame(0,yer) $mos(0,xh) $mos(,yh) \
               -fill blue -width 1  -stipple "" -tag {A C}
      $m_canvas create line $ccd_frame(0,xhr) $ccd_frame(0,yhr) $mos(0,xe) $mos(0,ye) \
               -fill blue -width 1  -stipple "" -tag {A C}
   } else {
        $m_canvas create text $mos(0,xh) $mos(0,yh) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $L_MIN*10.0] ] -tag " A"
      $m_canvas create text $mos(0,xg) $mos(0,yg) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $L_MAX*10.0] ] -tag " A"
      $m_canvas create text $mos(0,xer) $mos(0,yer) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $A_disp($good,cen)*10.0] ] -tag " A"
      $m_canvas create text $mos(0,xfr) $mos(0,yfr) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $A_disp($good,cen)*10.0] ] -tag " A"

      $m_canvas create line $mos(0,xfr) $mos(0,yfr) $mos(0,xg) $mos(0,yg) \
               -fill blue -width 1  -stipple "" -tag {A C}
      $m_canvas create line $mos(0,xgr) $mos(0,ygr) $mos(0,xf) $mos(0,yf) \
               -fill blue -width 1  -stipple "" -tag {A C}
      $m_canvas create line $mos(0,xer) $mos(0,yer) $mos(0,xh) $mos(0,yh) \
               -fill blue -width 1  -stipple "" -tag {A C}
      $m_canvas create line $mos(0,xhr) $mos(0,yhr) $mos(0,xe) $mos(0,ye) \
	      -fill blue -width 1  -stipple "" -tag {A C}
  }
         } else {
	if {[string compare [$this cget -instrument] FORS2]==0} {     
      $m_canvas create text $mos(0,xd) $mos(0,yd) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA"  [expr $L_MIN*10.0] ] -tag " A"
      $m_canvas create text $mos(0,xc) $mos(0,yc) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA"  [expr $L_MAX*10.0] ] -tag " A"
      $m_canvas create text $ccd_frame(0,xar) $ccd_frame(0,yar) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $A_disp($good,cen)*10.0] ] -tag " A"
      $m_canvas create text $ccd_frame(0,xbr) $ccd_frame(0,ybr) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $A_disp($good,cen)*10.0] ] -tag " A"

      $m_canvas create line $ccd_frame(0,xbr) $ccd_frame(0,ybr) $mos(0,xc) $mos(0,yc) \
               -fill blue -width 1  -stipple "" -tag {A C}
      $m_canvas create line $ccd_frame(0,xcr) $ccd_frame(0,ycr) $mos(0,xb) $mos(0,yb) \
               -fill blue -width 1  -stipple "" -tag {A C}
      $m_canvas create line $ccd_frame(0,xar) $ccd_frame(0,yar) $mos(0,xd) $mos(0,yd) \
               -fill blue -width 1  -stipple "" -tag {A C}
      $m_canvas create line $ccd_frame(0,xdr) $ccd_frame(0,ydr) $mos(0,xa) $mos(0,ya) \
               -fill blue -width 1  -stipple "" -tag {A C}      
      } else {     
      $m_canvas create text $mos(0,xd) $mos(0,yd) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA"  [expr $L_MIN*10.0] ] -tag " A"
      $m_canvas create text $mos(0,xc) $mos(0,yc) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA"  [expr $L_MAX*10.0] ] -tag " A"
      $m_canvas create text $mos(0,xar) $mos(0,yar) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $A_disp($good,cen)*10.0] ] -tag " A"
      $m_canvas create text $mos(0,xbr) $mos(0,ybr) \
               -anchor $oriA -fill $color4 \
               -text [format "%4.0fA" [expr $A_disp($good,cen)*10.0] ] -tag " A"

      $m_canvas create line $mos(0,xbr) $mos(0,ybr) $mos(0,xc) $mos(0,yc) \
               -fill blue -width 1  -stipple "" -tag {A C}
      $m_canvas create line $mos(0,xcr) $mos(0,ycr) $mos(0,xb) $mos(0,yb) \
               -fill blue -width 1  -stipple "" -tag {A C}
      $m_canvas create line $mos(0,xar) $mos(0,yar) $mos(0,xd) $mos(0,yd) \
               -fill blue -width 1  -stipple "" -tag {A C}
      $m_canvas create line $mos(0,xdr) $mos(0,ydr) $mos(0,xa) $mos(0,ya) \
               -fill blue -width 1  -stipple "" -tag {A C}
   }

}
}

     private method plot_ref_star_area_ccd2 {} {         
         if {[string match *SR* [$this cget -collimat] ]} {
	 #- Slave part plus gap -#
         $m_canvas create polygon  $ccd_frame(0,xc) $ccd_frame(0,yc) $ccd_frame(0,xd) $ccd_frame(0,yd) \
                   $ccd_frame(1,xd) $ccd_frame(1,yd) $ccd_frame(1,xc) $ccd_frame(1,yc) -outline cyan\
                   -fill cyan  -stipple "" -tag " A mm rtz"

         #- It should be forbidden between field stop and master CCD -#
	 #- Upper area -#
	 set xua  $ccd_frame(0,xa)
	 set yua  $ccd_frame(0,ya)
         set xub  $ccd_frame(0,xb)
         set yub  $ccd_frame(0,yb)
         set xuc  $mos(0,xb)
         set yuc  $mos(0,yb) 
         set xud  $mos(0,xa)
         set yud  $mos(0,ya)  
         $m_canvas create polygon $xua $yua $xub $yub $xuc $yuc $xud $yud -outline cyan\
                   -fill cyan  -stipple "" -tag " A mm rtz"
         #- Left area  -#
         set xla  $ccd_frame(0,xa)
	 set yla  $ccd_frame(0,ya)
         set xlb  $mos(0,xa)
         set ylb  $mos(0,ya)
         set xlc  $mos(0,xd)
         set ylc  $mos(0,yd)
         set xld  $ccd_frame(1,xd)
         set yld  $ccd_frame(1,yd)
         $m_canvas create polygon $xla $yla $xlb $ylb $xlc $ylc $xld $yld -outline cyan\
                   -fill cyan  -stipple "" -tag " A mm rtz"
         #- Right area -#
         set xra  $mos(0,xb)
	 set yra  $mos(0,yb)
         set xrb  $ccd_frame(0,xb)
         set yrb  $ccd_frame(0,yb)
         set xrc  $ccd_frame(1,xc)
         set yrc  $ccd_frame(1,yc)
         set xrd  $mos(0,xc)
         set yrd  $mos(0,yc)
         $m_canvas create polygon $xra $yra $xrb $yrb $xrc $yrc $xrd $yrd -outline cyan\
                   -fill cyan  -stipple "" -tag " A mm rtz"
	 } else {           
	 #- Slave part plus gap -#
         $m_canvas create polygon  $ccd_frame(0,xg) $ccd_frame(0,yg) $ccd_frame(0,xh) $ccd_frame(0,yh) \
                   $ccd_frame(1,xh) $ccd_frame(1,yh) $ccd_frame(1,xg) $ccd_frame(1,yg) -outline cyan \
                   -fill cyan  -stipple "" -tag " A mm rtz"
	 }
	 
     }
     #======================
     private method plot_ref_star_area { mos10flag } {
     #====================== 
     # plots some cyan areas which are forbidden for 
     # the refernce stars 
     # all objects have tag : rfz
     if {[string compare [$this cget -instrument] FORS2]==0} {
	 plot_ref_star_area_ccd2
     }

     if {$mos10flag} {
                     # draw full slit blade 10 
                     $m_canvas create polygon $mos(9,xe) $mos(9,ye) \
                       $mos(9,xh) $mos(9,yh) $mos(11,xd) $mos(11,yd) \
                       $mos(11,xa) $mos(11,ya) -outline cyan\
                       -fill cyan  -stipple "" -tag " A mm rtz"
                     } 

                     # draw the cyan vorbidden rim region 
                     # note : 10.0/396.0 means 10 arcsec of 396 arcsec


                     if {[string match *mos* $mask]} {

                        # mos, echs, pmos, img, ipol/sr 

                     if {[string match *HR* [$this cget -collimat] ]} {

	              if {[string compare [$this cget -instrument] FORS2]==0} {
	             set fa [expr 10.0/198.0]
                     # AB-bar
                     set ax1 [expr $ccd_frame(0,xe) + ($ccd_frame(0,xh)-$ccd_frame(0,xe))*$fa]
                     set ay1 [expr $ccd_frame(0,ye) + ($ccd_frame(0,yh)-$ccd_frame(0,ye))*$fa]
                     set bx1 [expr $ccd_frame(0,xf) + ($ccd_frame(0,xg)-$ccd_frame(0,xf))*$fa]
                     set by1 [expr $ccd_frame(0,yf) + ($ccd_frame(0,yg)-$ccd_frame(0,yf))*$fa]
                     $m_canvas create polygon $mos(1,xa) $mos(1,ya) \
                         $mos(1,xd) $mos(1,yd) $bx1 $by1 $ax1 $ay1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # DC-bar
                     set dx1 [expr $ccd_frame(1,xh) + ($ccd_frame(1,xe)-$ccd_frame(1,xh))*$fa]
                     set dy1 [expr $ccd_frame(1,yh) + ($ccd_frame(0,ye)-$ccd_frame(1,yh))*$fa]
                     set cx1 [expr $ccd_frame(1,xg) + ($ccd_frame(1,xf)-$ccd_frame(1,xg))*$fa]
                     set cy1 [expr $ccd_frame(1,yg) + ($ccd_frame(0,yf)-$ccd_frame(1,yg))*$fa]
                     $m_canvas create polygon $mos(19,xe) $mos(19,ye) \
                         $mos(19,xh) $mos(19,yh) $cx1 $cy1 $dx1 $dy1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # AD-bar
                     set ax1 [expr $ccd_frame(0,xe) + ($ccd_frame(0,xf)-$ccd_frame(0,xe))*$fa]
                     set ay1 [expr $ccd_frame(0,ye) + ($ccd_frame(0,yf)-$ccd_frame(0,ye))*$fa]
                     set dx1 [expr $ccd_frame(1,xh) + ($ccd_frame(1,xg)-$ccd_frame(1,xh))*$fa]
                     set dy1 [expr $ccd_frame(1,yh) + ($ccd_frame(1,yg)-$ccd_frame(1,yh))*$fa]
                     $m_canvas create polygon $mos(1,xa) $mos(1,ya) \
                         $mos(19,xe) $mos(19,ye) $dx1 $dy1 $ax1 $ay1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # BC-bar
                     set bx1 [expr $ccd_frame(0,xf) + ($ccd_frame(0,xe)-$ccd_frame(0,xf))*$fa]
                     set by1 [expr $ccd_frame(0,yf) + ($ccd_frame(0,ye)-$ccd_frame(0,yf))*$fa]
                     set cx1 [expr $ccd_frame(1,xg) + ($ccd_frame(1,xh)-$ccd_frame(1,xg))*$fa]
                     set cy1 [expr $ccd_frame(1,yg) + ($ccd_frame(1,yh)-$ccd_frame(1,yg))*$fa]
                     $m_canvas create polygon $mos(1,xd) $mos(1,yd) \
                         $mos(19,xh) $mos(19,yh) $cx1 $cy1 $bx1 $by1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
		      } else {
                     set fa [expr 10.0/198.0]
                     # AB-bar
                     set ax1 [expr $mos(0,xe) + ($mos(0,xh)-$mos(0,xe))*$fa]
                     set ay1 [expr $mos(0,ye) + ($mos(0,yh)-$mos(0,ye))*$fa]
                     set bx1 [expr $mos(0,xf) + ($mos(0,xg)-$mos(0,xf))*$fa]
                     set by1 [expr $mos(0,yf) + ($mos(0,yg)-$mos(0,yf))*$fa]
                     $m_canvas create polygon $mos(1,xa) $mos(1,ya) \
                         $mos(1,xd) $mos(1,yd) $bx1 $by1 $ax1 $ay1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # DC-bar
                     set dx1 [expr $mos(0,xh) + ($mos(0,xe)-$mos(0,xh))*$fa]
                     set dy1 [expr $mos(0,yh) + ($mos(0,ye)-$mos(0,yh))*$fa]
                     set cx1 [expr $mos(0,xg) + ($mos(0,xf)-$mos(0,xg))*$fa]
                     set cy1 [expr $mos(0,yg) + ($mos(0,yf)-$mos(0,yg))*$fa]
                     $m_canvas create polygon $mos(19,xe) $mos(19,ye) \
                         $mos(19,xh) $mos(19,yh) $cx1 $cy1 $dx1 $dy1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # AD-bar
                     set ax1 [expr $mos(0,xe) + ($mos(0,xf)-$mos(0,xe))*$fa]
                     set ay1 [expr $mos(0,ye) + ($mos(0,yf)-$mos(0,ye))*$fa]
                     set dx1 [expr $mos(0,xh) + ($mos(0,xg)-$mos(0,xh))*$fa]
                     set dy1 [expr $mos(0,yh) + ($mos(0,yg)-$mos(0,yh))*$fa]
                     $m_canvas create polygon $mos(1,xa) $mos(1,ya) \
                         $mos(19,xe) $mos(19,ye) $dx1 $dy1 $ax1 $ay1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # BC-bar
                     set bx1 [expr $mos(0,xf) + ($mos(0,xe)-$mos(0,xf))*$fa]
                     set by1 [expr $mos(0,yf) + ($mos(0,ye)-$mos(0,yf))*$fa]
                     set cx1 [expr $mos(0,xg) + ($mos(0,xh)-$mos(0,xg))*$fa]
                     set cy1 [expr $mos(0,yg) + ($mos(0,yh)-$mos(0,yg))*$fa]
                     $m_canvas create polygon $mos(1,xd) $mos(1,yd) \
                         $mos(19,xh) $mos(19,yh) $cx1 $cy1 $bx1 $by1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
		      }
                     } else {

                     set fa [expr 10.0/396.0]
                     # AB-bar
                     set ax1 [expr $mos(0,xa) + ($mos(0,xd)-$mos(0,xa))*$fa]
                     set ay1 [expr $mos(0,ya) + ($mos(0,yd)-$mos(0,ya))*$fa]
                     set bx1 [expr $mos(0,xb) + ($mos(0,xc)-$mos(0,xb))*$fa]
                     set by1 [expr $mos(0,yb) + ($mos(0,yc)-$mos(0,yb))*$fa]
                     $m_canvas create polygon $mos(1,xa) $mos(1,ya) \
                         $mos(1,xd) $mos(1,yd) $bx1 $by1 $ax1 $ay1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # DC-bar
                     set dx1 [expr $mos(0,xd) + ($mos(0,xa)-$mos(0,xd))*$fa]
                     set dy1 [expr $mos(0,yd) + ($mos(0,ya)-$mos(0,yd))*$fa]
                     set cx1 [expr $mos(0,xc) + ($mos(0,xb)-$mos(0,xc))*$fa]
                     set cy1 [expr $mos(0,yc) + ($mos(0,yb)-$mos(0,yc))*$fa]
                     $m_canvas create polygon $mos(19,xe) $mos(19,ye) \
                         $mos(19,xh) $mos(19,yh) $cx1 $cy1 $dx1 $dy1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # AD-bar
                     set ax1 [expr $mos(0,xa) + ($mos(0,xb)-$mos(0,xa))*$fa]
                     set ay1 [expr $mos(0,ya) + ($mos(0,yb)-$mos(0,ya))*$fa]
                     set dx1 [expr $mos(0,xd) + ($mos(0,xc)-$mos(0,xd))*$fa]
                     set dy1 [expr $mos(0,yd) + ($mos(0,yc)-$mos(0,yd))*$fa]
                     $m_canvas create polygon $mos(1,xa) $mos(1,ya) \
                         $mos(19,xe) $mos(19,ye) $dx1 $dy1 $ax1 $ay1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # BC-bar
                     set bx1 [expr $mos(0,xb) + ($mos(0,xa)-$mos(0,xb))*$fa]
                     set by1 [expr $mos(0,yb) + ($mos(0,ya)-$mos(0,yb))*$fa]
                     set cx1 [expr $mos(0,xc) + ($mos(0,xd)-$mos(0,xc))*$fa]
                     set cy1 [expr $mos(0,yc) + ($mos(0,yd)-$mos(0,yc))*$fa]
                     $m_canvas create polygon $mos(1,xd) $mos(1,yd) \
                         $mos(19,xh) $mos(19,yh) $cx1 $cy1 $bx1 $by1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
			
                     }

                     } else {

                        # lss, hits, mxu, ipol/hr 

                     if {[string match *HR* [$this cget -collimat] ]} {

                     set fa [expr 10.0/198.0]
                     # AB-bar
                     set ax1 [expr $mos(0,xe) + ($mos(0,xh)-$mos(0,xe))*$fa]
                     set ay1 [expr $mos(0,ye) + ($mos(0,yh)-$mos(0,ye))*$fa]
                     set bx1 [expr $mos(0,xf) + ($mos(0,xg)-$mos(0,xf))*$fa]
                     set by1 [expr $mos(0,yf) + ($mos(0,yg)-$mos(0,yf))*$fa]
                     $m_canvas create polygon $mos(0,xa) $mos(0,ya) \
                         $mos(0,xb) $mos(0,yb) $bx1 $by1 $ax1 $ay1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # DC-bar
                     set dx1 [expr $mos(0,xh) + ($mos(0,xe)-$mos(0,xh))*$fa]
                     set dy1 [expr $mos(0,yh) + ($mos(0,ye)-$mos(0,yh))*$fa]
                     set cx1 [expr $mos(0,xg) + ($mos(0,xf)-$mos(0,xg))*$fa]
                     set cy1 [expr $mos(0,yg) + ($mos(0,yf)-$mos(0,yg))*$fa]
                     $m_canvas create polygon $mos(0,xd) $mos(0,yd) \
                         $mos(0,xc) $mos(0,yc) $cx1 $cy1 $dx1 $dy1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # AD-bar
                     set ax1 [expr $mos(0,xe) + ($mos(0,xf)-$mos(0,xe))*$fa]
                     set ay1 [expr $mos(0,ye) + ($mos(0,yf)-$mos(0,ye))*$fa]
                     set dx1 [expr $mos(0,xh) + ($mos(0,xg)-$mos(0,xh))*$fa]
                     set dy1 [expr $mos(0,yh) + ($mos(0,yg)-$mos(0,yh))*$fa]
                     $m_canvas create polygon $mos(0,xa) $mos(0,ya) \
                         $mos(0,xd) $mos(0,yd) $dx1 $dy1 $ax1 $ay1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # BC-bar
                     set bx1 [expr $mos(0,xf) + ($mos(0,xe)-$mos(0,xf))*$fa]
                     set by1 [expr $mos(0,yf) + ($mos(0,ye)-$mos(0,yf))*$fa]
                     set cx1 [expr $mos(0,xg) + ($mos(0,xh)-$mos(0,xg))*$fa]
                     set cy1 [expr $mos(0,yg) + ($mos(0,yh)-$mos(0,yg))*$fa]
                     $m_canvas create polygon $mos(0,xb) $mos(0,yb) \
                         $mos(0,xc) $mos(0,yc) $cx1 $cy1 $bx1 $by1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"

                        } else {

                     set fa [expr 10.0/396.0]
                     # AB-bar
                     set ax1 [expr $mos(0,xa) + ($mos(0,xd)-$mos(0,xa))*$fa]
                     set ay1 [expr $mos(0,ya) + ($mos(0,yd)-$mos(0,ya))*$fa]
                     set bx1 [expr $mos(0,xb) + ($mos(0,xc)-$mos(0,xb))*$fa]
                     set by1 [expr $mos(0,yb) + ($mos(0,yc)-$mos(0,yb))*$fa]
                     $m_canvas create polygon $mos(0,xa) $mos(0,ya) \
                         $mos(0,xb) $mos(0,yb) $bx1 $by1 $ax1 $ay1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # DC-bar
                     set dx1 [expr $mos(0,xd) + ($mos(0,xa)-$mos(0,xd))*$fa]
                     set dy1 [expr $mos(0,yd) + ($mos(0,ya)-$mos(0,yd))*$fa]
                     set cx1 [expr $mos(0,xc) + ($mos(0,xb)-$mos(0,xc))*$fa]
                     set cy1 [expr $mos(0,yc) + ($mos(0,yb)-$mos(0,yc))*$fa]
                     $m_canvas create polygon $mos(0,xd) $mos(0,yd) \
                         $mos(0,xc) $mos(0,yc) $cx1 $cy1 $dx1 $dy1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # AD-bar
                     set ax1 [expr $mos(0,xa) + ($mos(0,xb)-$mos(0,xa))*$fa]
                     set ay1 [expr $mos(0,ya) + ($mos(0,yb)-$mos(0,ya))*$fa]
                     set dx1 [expr $mos(0,xd) + ($mos(0,xc)-$mos(0,xd))*$fa]
                     set dy1 [expr $mos(0,yd) + ($mos(0,yc)-$mos(0,yd))*$fa]
                     $m_canvas create polygon $mos(0,xa) $mos(0,ya) \
                         $mos(0,xd) $mos(0,yd) $dx1 $dy1 $ax1 $ay1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                     # BC-bar
                     set bx1 [expr $mos(0,xb) + ($mos(0,xa)-$mos(0,xb))*$fa]
                     set by1 [expr $mos(0,yb) + ($mos(0,ya)-$mos(0,yb))*$fa]
                     set cx1 [expr $mos(0,xc) + ($mos(0,xd)-$mos(0,xc))*$fa]
                     set cy1 [expr $mos(0,yc) + ($mos(0,yd)-$mos(0,yc))*$fa]
                     $m_canvas create polygon $mos(0,xb) $mos(0,yb) \
                         $mos(0,xc) $mos(0,yc) $cx1 $cy1 $bx1 $by1  -outline cyan  \
                         -fill cyan  -stipple "" -tag " A mm rtz"
                        }

                     }

     # the fortbidden reftarg zone is plotted 
     set reftarg_zone 1
     }

     #======================
     public  method help {} {
     #====================== 
     # prints minihelp on the screen 
     # no longer needed 2.12.97
     puts $lfId "FORS Simulator V0.9"
     puts $lfId "------------------ >> Mini Help <<"
     puts $lfId "(B1=Mouse Button-1 = left Mous button)"
     puts $lfId "B1 on green circle = Shift Mask"
     puts $lfId "B2 on green circle = Rotate Mask anti-clock"
     puts $lfId "B3 on green circle = Rotate Mask clock"

     puts $lfId "B1 on slit blade      = Move Slitlet"
     puts $lfId "B2 on slit blade      = this info"
     puts $lfId "B3 on CCD          = update"
     puts $lfId "B2 on CCD          = reset"
     puts $lfId "--------------------------------------------"
     puts $lfId $this
     }



     #=============================
     private method shift { w xs ys } {
     #=============================
     if {$stop} { return }
     # A) shifts the mask via tcl move command
     # B) updates the mos-array
     # xs,ys are the screen coords where to be shifted 
     #
     # Comments:
     # declare bord as soon as possible as global
     # not yet moved outside frame 
     set over 0

     # convert last drag coord to pixels 
     $m_image convert coords $xs $ys screen xi yi image 

     #puts stdout "pixel ($xi,$yi) screen ($xs,$ys) ($width,$height)"
     # border in image coords 
     if {$bonsai_flag} {set over 1}
     if {$xi < 1 || $xi >= $width} { set over 1}
     if {$yi < 1 || $yi >= $height} { set over 1}

     # screen to canvas
     $m_image convert coords $xs $ys screen x y canvas

     # mark and shift position != mask center
     # the distance between old and new position in canvas coordinates
     set shift_x [expr $x - $oldpos(x)]
     set shift_y [expr $y - $oldpos(y)]


     # moves the mask on the screen without recreating it via plot_mask
     $w move A  $shift_x $shift_y

     #update the old-position for permanent shifts 
     set oldpos(x) $x; set oldpos(y) $y

     # apply shift mask center = telescope pointing (canvas)
     set ccx [expr $ccx+$shift_x]
     set ccy [expr $ccy+$shift_y]

     # update world coords of mask center
     # if the frame is smaller than the mask or the mask is
     # shifted outside the frame use bonsai-wcs
     if {$over} {
                 set dum [C_wc c2w $ccx $ccy]
                 set world_x [lindex $dum 0]
                 set world_y [lindex $dum 1]
                 } else {
     if [catch {
     $m_image convert coords $ccx $ccy canvas world_x world_y "deg $EQX"
               } result ] {
     puts stdout "shift: Big Error!!!!!!!!!!!!! $result" }

                 }

      # Since the fors instrumental mask can be rotated by 
      # an minimum amount of 1 deg, we have to keep an internal 
      # array containing all mask coords 

     # update the mos 
     # 19 slit blades 
        set local_max 10
        set local_set {a b d e f h}
     if {[string compare $mask "mos"]==0} {
        set local_max 20
        set local_set {a b c d e f g h}
        }
     if {[string compare $mask "mxu"]==0} { set local_max 0 
        # later we will ask for the mxu local slit counter
        }
 

        # update the MOS slit blades 
        for { set i 1 } { $i < $local_max } { incr i 1} {
         foreach val $local_set {
           set mos($i,x$val) [expr $mos($i,x$val)+$shift_x]
           set mos($i,y$val) [expr $mos($i,y$val)+$shift_y]
           }
         }

   
         # update the ccd frame including collimator and wavelength borders
         foreach val {0 a b c d e f g h ar br cr dr er fr gr hr} {
           set mos(0,x$val) [expr $mos(0,x$val)+$shift_x]
           set mos(0,y$val) [expr $mos(0,y$val)+$shift_y]
           }

          foreach val {0 a b c d e f g h ar br cr dr er fr gr hr} {
           set ccd_frame(0,x$val) [expr $ccd_frame(0,x$val)+$shift_x]
           set ccd_frame(0,y$val) [expr $ccd_frame(0,y$val)+$shift_y]           
           set ccd_frame(1,x$val) [expr $ccd_frame(1,x$val)+$shift_x]
           set ccd_frame(1,y$val) [expr $ccd_frame(1,y$val)+$shift_y] 
           }
         

     # this must be correct, since we can put up the mask 
     # not only in the center of the green point, but also 
     # at some biased position
     # Cursor position, note canvas-wcs canvas-deg does not work
     #                       outside frame  
     # $m_image convert coords $xs $ys screen lwx lwy "wcs $EQX"

     # cursor position, not TCS pointing 
     $m_image convert coords $xs $ys screen xc yc canvas
     set dum [C_wc c2w $xc $yc]
     set xw [lindex $dum 0]
     set yw [lindex $dum 1]

     if [catch {
     $m_image convert coords $xw $yw "deg $EQX" lcx lcy "wcs $EQX"
               } result ] {
     puts stdout "shift: $result" }

     # mask-tcs position (= green point center)
     # $m_image convert coords $world_x $world_y "deg $EQX" lwx lwy "wcs $EQX"
     #puts stdout [format "Shift Mask to c(%10.3f,%10.3f) (%10.3f,%10.3f)" \
     #         $mos(0,x0) $mos(0,y0) $world_x $world_y ]
     # puts stdout [format "T%10s %10s C%10s %10s c%6.1f %6.1f s%6.1f %6.1f" \
     #                    $lwx $lwy $lcx $lcy $x $y $xs $ys] 

     # renew only tcs labels 
     if {[$this cget -label]} {
        catch {$w delete D}
        plot_tcs_label
        }

    # and the badpixelmap 
    if {$badpixelmap} {
    fors::CONV_MC::coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                      $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                      $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc) 
     update_BPM
     }

     }

     #==================================
     private method calc_zemax { my x y } {
     #==================================
     # optical correction based on zemax data
     # we correct the underlying FORS image 
     # puts stdout "zemax input $my $x $y"
     # x and y are CCD pixel coords, my = MOS slit nr 
     #
     # if fr_x="" then frameless mode
     # there is no underlying FORS image to be corrected
    
     set go 0

     if {[string match *FORS* [$m_image fits get INSTRUME] ]} {
        # if the underlying image is from FORS1 or from FORS2 
        set go 1 }

     # W.H 13.Sept 2000 , workaround 
     # FORS1 images are aliens for MXU 
     #if {[string compare [$this cget -modus] mxu]==0} {
     #if {[string compare [$m_image fits get INSTRUME] "FORS1"]==0} {
     #   set go 0 } }


     if {$go} {
     set x_r  [expr $x - $fr_x]
     set y_r  [expr $y - $fr_y]
     puts stdout "zemax dist $x_r $y_r"
     set r [expr sqrt($x_r * $x_r + $y_r * $y_r) ]
     # read scale from fits header, 0.1=HR 0.2=SR
     set fisc [lindex [$m_image wcsset] 2]
     if {$fisc>0.15} {
        set ab [ablage_SR  $r]
        puts stdout [format "zemax_SR %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f" \
               $r $ab [expr ($r-$ab)/$r]  [expr ($r+$ab)/$r] $x_r $y_r ] 
        } else {
        set ab [ablage_HR  $r]
        puts stdout [format "zemax_HR %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f" \
           $r $ab [expr ($r-$ab)/$r]  [expr ($r+$ab)/$r] $x_r $y_r ] 
        }
     # |radius| must be made smaller 
     if {$r< 1e-10} { set x_new $fr_x; set y_new $fr_y; } else {
     set x_new [expr ($r-$ab)/$r*$x_r + $fr_x ] 
     set y_new [expr ($r-$ab)/$r*$y_r + $fr_y ] 
     }
      puts stdout [format \
      "ZEMAX = X: %5.3f %+5.3f = %5.3f Y: %5.3f %+5.3f = %5.3f" \
                   $x [expr $x_new-$x] $x_new $y [expr $y_new-$y] $y_new ] 
     set targw($my,cx) $x_new
     set targw($my,cy) $y_new
     # puts stdout "zemax $targw($my,ca) $targw($my,cd)"

     } else {
     
     # in frameless mode or image not from FORS1/2
     # do not apply optical correction 
     set targw($my,cx) $targw($my,x)
     set targw($my,cy) $targw($my,y)
     }

     # in both cases again 
     if [catch {
     $m_image convert coords $targw($my,cx) $targw($my,cy) image \
                              targw($my,ca)  targw($my,cd) "wcs $EQX" 
              } result ] {
     set targw($my,ca) $targw($my,a)
     set targw($my,cd) $targw($my,d)
     puts stdout "calc_zemax: $result" 
     return 1 } else { return 0 }


     }

     #==================================
     private method calc_zemax_ref { a d } {
     #==================================
     # optical correction based on zemax data
     # we correct the underlying FORS image
     # puts stdout "zemax input $x $y"
     # x and y are CCD pixel coords, my = MOS slit nr
     # input a d in wcs 
     #
     # if fr_x="" then frameless mode
     # there is no underlying FORS image to be corrected

     set go 0

     if {[string match *FORS* [$m_image fits get INSTRUME] ]} {
        # if the underlying image is from FORS1 or from FORS2 
        set go 1 }

     # W.H 14.Sept 2000 , workaround 
     # FORS1 images are aliens for MXU 
     #if {[string compare [$this cget -modus] mxu]==0} {
     #if {[string compare [$m_image fits get INSTRUME] "FORS1"]==0} {
     #   set go 0 } }

     if {$go} {

     $m_image convert coords $a $d "wcs $EQX" x y image
     set x_r  [expr $x - $fr_x]
     set y_r  [expr $y - $fr_y]
     # puts stdout "using $a $d $x $y $fr_x $fr_y"
     # puts stdout "zemax dist $x_r $y_r"
     set r [expr sqrt($x_r * $x_r + $y_r * $y_r) ]
     # read scale from fits header, 0.1=HR 0.2=SR
     set fisc [lindex [$m_image wcsset] 2]
     if {$fisc>0.15} {
        set ab [ablage_SR  $r]
     # puts stdout [format "zemax_SR %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f" \
     #  $r $ab [expr ($r-$ab)/$r]  [expr ($r+$ab)/$r] $x_r $y_r ]
        } else {
        set ab [ablage_HR  $r]
     # puts stdout [format "zemax_HR %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f" \
     #  $r $ab [expr ($r-$ab)/$r]  [expr ($r+$ab)/$r] $x_r $y_r ]
        }
     # |radius| must be made smaller
     set x_new [expr ($r-$ab)/$r*$x_r + $fr_x ]
     set y_new [expr ($r-$ab)/$r*$y_r + $fr_y ]

     #puts stdout [format \
     #"ZEMAX_REF_TARG = X: %5.3f %+5.3f = %5.3f Y: %5.3f %+5.3f = %5.3f" \
     #               $x [expr $x_new-$x] $x_new \
     #               $y [expr $y_new-$y] $y_new ]
     $m_image convert coords $x_new $y_new image \
      a d "wcs $EQX"
     # puts stdout "zemax_ref_targ $a $d"
     return "$a $d"
     } else {

       # frameless mode, or underlying image not from FORS1 or FORS2 
       # do not apply image distrotion correction 
       return "$a $d"
       }

     }

     #==================================
     private method calc_oc { x_mm x_pix my a2 d2 a3 d3 } {
     #==================================
     # x_mm   = slit position in mm focal plane 
     # x_pix  = slit position in pixel 
     # my     = slit number [1..19]
     # a2 d2  = slit position RA DEC  / deg  
     # a3 d3  = slit position RA DEC  / wcs   
     # -------------------
     # optical correction
     # -------------------
     # the distortion is corrected via two terms 
     # 1. the absolute distortion at the 10th slit in pixel
     set pix [expr $x_mm -7.7225]; set pix [expr $pix * $pix];
     set pix [expr $pix * $pix];
     set pix [expr 9.6542e-9 * $pix]
     if {$x_mm>0} { set pix [expr $pix * -1 ] }
     # 2. relative distortion of slit y with respect to slit 10 
     set rel_x_pix [expr abs($x_pix - 1020.0) ]; # distance from x=0
     set corr_x [expr $rel_x_pix*([od_F $x_mm $my]-1.0)]; # relative correction
     if {$x_mm>0} { set corr_x [expr $corr_x * -1 ] }

     puts stdout "abs = $pix   ||  rel = $corr_x"
     set pix [expr $pix + $corr_x ];     # total in pixel
     set as  [expr $pix*0.2       ];      # in arscsec
     set del_x [expr $pix*0.2/$focscale]; # in mm
     set new_x [expr $x_mm + $del_x ]
     set da [expr $as*$CSO*cos($abs_ang_rad)]
     set dd [expr $as*$CSO*sin($abs_ang_rad)]
     # these are the relative distortions with respect to  slit nr 10

     puts stdout [format \
          "Move #%2d from x=%5.3f by %5.3f to %5.3f =%5.3f \
               pixel = %5.3f'' Dalp=%6.4f'' Ddel=%6.4f'' rel=%6.4f" \
        $my      $x_mm      $del_x    $new_x $pix $as $da $dd  $corr_x ]

     # now we have to copy these valaus to the targw array
     set deg_da [expr $da*15.0/3600.0*$trigo]
     set deg_dd [expr $dd/3600.0 ]
     set deg_da [expr $deg_da + $a2 ]
     set deg_dd [expr $deg_dd + $d2 ]
     $m_image convert coords $deg_da $deg_dd "deg $EQX" \
               targw($my,ca) targw($my,cd) "wcs $EQX"
     set targw($my,cx) [expr $targw($my,x)+$pix]
     puts stdout "old   $a3 $d3"
     puts stdout "new   $targw($my,ca) $targw($my,cd)"
     }


     #======================================================
     private method calc_diff_refrac { my aw dw } {
     #======================================================
     #  calculates differential correction 
     #  aw dw = RA DEC in wcs of target 
     #  my is MOS slit number, but 
     # my=0 means reference target 

     # puts stdout "reference pixel = $fr_x $fr_y"
     # puts stdout "reference wcs   = $wx $wy"
     set h [$m_image fits get "HIERARCH ESO TEL ALT" ]
     # puts stdout "reference height= $h"
     set p [$m_image fits get "HIERARCH ESO TEL PARANG START" ]
     # puts stdout "HIERARCH ESO TEL PARANG START >$p<  deg" 

     # images without ALT or PARANG keywords 
     # 
     if {$h == "" || $p==""}  {
        if {$my !=0} { return 
                    } else {
                      # reference target 
                      return [list $aw $dw]
                    }
       }

     set p [expr $p*0.0174532]

     $m_image convert coords $aw $dw "wcs $EQX" a d "deg $EQX"
     # puts stdout "ref  $wx $wy target  $a $d"
     
     # distance between target and ref pixel in deg 
     set del_alp [expr $a - $wx ]
     set del_del [expr $d - $wy ]
     # puts stdout "delta RA = $del_alp deg  delta DEC = $del_del deg "

     # and in arcsec 
     # set xx [expr $del_alp * 3600.0 * cos($d) ]; 
     # set yy [expr $del_del * 3600.0]
     # set rr [expr sqrt($xx*$xx+$yy*$yy)]
     # puts stdout "delta RA = $xx ''  delta DEC = $yy '' r=$rr"

     
     # transform RA DEC into Azi Alti using parallactic angle p
     set del_azi [expr -cos($p)*$del_alp*cos($d) -sin($p)*$del_del  ]
     set del_hei [expr -sin($p)*$del_alp*cos($d) +cos($p)*$del_del  ]
     # puts stdout "delta A = $del_azi deg delta H = $del_hei deg"

     # show results in arcsec 
     # set xx [expr $del_azi * 3600.0 ]
     # set yy [expr $del_hei * 3600.0]
     # set rr [expr sqrt($xx*$xx+$yy*$yy)] 
     # puts stdout "a_targ - a_ref = $xx '' h_targ - h_ref = $yy '' r=$rr"

     # set z1 [expr (90.0 - $h)]
     # puts stdout "h = $h deg    z = $z1 deg "
     set z [expr (90.0 - $h)*0.0174532]
     # formula for differential refraction 
     set refrac [expr (57.717-3*0.066*tan($z)*tan($z)) / (cos($z)*cos($z))]
     set refrac [expr $refrac * $del_hei*0.0174532] 
     # puts stdout "refrac at z=($z1^0+$del_hei deg) = $refrac ''"
     set refrac [expr $refrac / 3600.0]
     # puts stdout "refrac at z=($z1^0+$del_hei deg) = $refrac deg"
      
     set aa [expr $refrac / cos($d) * sin($p)]; # RA component
     set a1 [expr $a - $aa ];                   # correction
     # puts stdout "$a + $aa = $a1"
     
     set dd [expr $refrac * cos($p)];           # DEC component 
     set d1 [expr $dd + $d];                    # correction 
     # puts stdout "$d + $dd = $d1"
     
     $m_image convert coords $a1 $d1 "deg $EQX" a d "wcs $EQX"
     # puts stdout "RA  $targw($my,ca) ---> $a"
     # puts stdout "DEC $targw($my,cd) ---> $d"

     # update corrected coordinates 
     if {$my !=0} {
        # MOS targets 
        set targw($my,ca) $a
        set targw($my,cd) $d
        } else {
        # reference targets
        return "$a $d"
        }
     }

     #======================================================
     private method move_slit { w sx sy {on 1} {click 1} } {
     #======================================================
     # moves the MOS slit (MOS, PMOS)
     # w = canvas widget
     # sx, sy is cursor pos in screen coords 
     # on==1  : select, on==0 : un-select, give slit free 
     # click==1 : called by binding (click), moved manually 
     # click==0 : called by private member functions (e.g. load_mask), 
     #            do not use center gauss
     #            do not use optical correction
     # stop==1: do nothing (e.g. when selecting a reference object)
     # stop==0: do the normal procedure 
     if {$stop} { return }

     $m_image convert coords $sx $sy screen x y canvas 
     set pmos_flag [expr [string compare [$this cget -modus] "pmos"]==0]

     # find the tag of the slit blade belonging to the target object
     #puts $lfId $curtag
     #puts $lfId " coords of $curtag : [$w coords $curtag]"
     #puts $lfId "$mos(12,xa) $mos(12,ya) $mos(12,xb)"
     set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
     set my [string range $curtag  1 end]
     #  puts stdout ">[$w find closest $x $y]< \
     #            >[$w gettags [$w find closest $x $y]]< \
     #           curtag= >$curtag<  my= >$my<"

     if {$on==0} {
       if {$my!=""} {
         if {$click} {
         set wid .skycat1.save
         catch {destroy $wid}
         set w [DialogWidget $wid \
            -text "You De-Select MOS slit #$my" \
            -bitmap warning \
            -default -1 \
            -buttons {OK Cancel} ]
         if { [$d activate] } return 
            }

        #  we have to deselect 
        set targw($my,f) 0;       # deselect slit 
        set mos($my,b) $mos(0,b); # set slit width to park position slit width 
        if { [expr abs($mos(0,xa)-$mos(0,xb))] < 0.01 } {
            # puts stdout "deselect perpendicular" 
            set mos($my,xb) $mos($my,xa)
            set mos($my,xc) \
                [expr $mos($my,xb)+$mos($my,b)/$focscale*$fits_x_scale*$ctom]
            set mos($my,xf) $mos($my,xe)
            set mos($my,xg) \
                [expr $mos($my,xf)+$mos($my,b)/$focscale*$fits_x_scale*$ctom]
            } else {
            # slope 
            # puts stdout "deselect slope" 
            set mos($my,xb) $mos($my,xa) 
            set mos($my,yb) $mos($my,ya)
            set mos($my,xc) \
                [expr $mos($my,xa)+($mos($my,xd)-$mos($my,xa))/80.0 ]
            set mos($my,yc) \
                [expr $mos($my,ya)+($mos($my,yd)-$mos($my,ya))/80.0 ]
  
            set mos($my,xf) $mos($my,xe) 
            set mos($my,yf) $mos($my,ye)
            set mos($my,xg) [expr $mos($my,xe)+($mos($my,xh)-$mos($my,xe))/80.0]
            set mos($my,yg) [expr $mos($my,ye)+($mos($my,yh)-$mos($my,ye))/80.0]
            }
          set mos($my,b) [$this cget -slit_width]; # set slit width  back again
          }
          # scratch the whole old mask and plot_mask it
          catch {$w delete A} 
          update_and_replot; #  <------------------------
         } else {
         # no deselect 

     if { [string length $my] == 0 } { 
         # if the slit was deselected before 
         set mos($my,b) [$this cget -slit_width ] 
     $m_image convert coords $sx $sy screen nx ny "deg $EQX" 
     $m_image convert coords $sx $sy screen tx ty "wcs $EQX" 
     puts stdout "Warning cannot move #$my to ($nx,$ny)=($tx,$ty)" 
     puts $lfId "MoveSlit: Warning cannot move #$my to ($nx,$ny)=($tx,$ty)" 
     } elseif { [expr $my-($my/2)*2] && $pmos_flag} {
     $m_image convert coords $sx $sy screen nx ny "deg $EQX" 
     $m_image convert coords $sx $sy screen tx ty "wcs $EQX" 
     puts $lfId "MoveSlit:  Warning cannot move #$my to ($nx,$ny)=($tx,$ty)" 
    puts stdout "Warning cannot move #$my to ($nx,$ny)=($tx,$ty)" 
        } else {
     # from here on : everything is ok : carry out the move 

     #------------------
     # center-slit option 
     #------------------
     if {$click} { 
     if {[$this cget -pics]} {
        # sx,sy are approximate coords, 
        # use RtdImagePick to center slit on image target
        $m_image convert coords $sx $sy screen a0 d0 image 
        # puts stdout "Button-Press canvas=$x $y screen=$sx $sy pixel=$a0 $d0"
        center_slit 
        if {$lss_sli_ra != ""} {
        $m_image convert coords $lss_sli_ra $lss_sli_de image sx sy screen
        $m_image convert coords $lss_sli_ra $lss_sli_de image  x  y canvas 

        # puts stdout "[$w gettags current]"
        set my [string range [lindex [$w gettags current] 2] 1 end]
        if {$my==""} { 
            puts stdout "Cursor not on any MOS slit"
            return
            }

        # set CCD pixel of target 
        set targw($my,x) $lss_sli_ra
        set targw($my,y) $lss_sli_de
        puts stdout [format "targw(%2d,x) = %8.3f, targw(%2d,y) = %8.3f" \
           $my $targw($my,x) $my $targw($my,y) ]
        # puts stdout "center_slit from ($a0,$d0) on center_gauss result=\
        #        ($lss_sli_ra,$lss_sli_de) pixel"
           } else {
            # there are two posibilities
            # a) : return, do not specify any position 
            # b) : use the Button-Release coords 
            return 
           }   
        }
     }
     #-----------------------
     # center-slit option end
     #-----------------------

     #puts $lfId "number = >[string range $curtag  1 end]<"
     $m_image convert coords $sx $sy screen a1 d1 image 
     $m_image convert coords $sx $sy screen a2 d2 "deg $EQX"
     $m_image convert coords $sx $sy screen a3 d3 "wcs $EQX"
     # CCD pixel of target 
     set targw($my,x) $a1
     set targw($my,y) $d1
     # puts stdout "targw($my,x) = $targw($my,x), targw($my,y) = $targw($my,y)" 
     # puts stdout [ format \
     # "MS: in=(%8.3f,%8.3f)screen (%8.3f,%8.3f)ima (%s,%s)wcs"  \
     #  $sx $sy $a1 $d1 $a3 $d3 ]
     # targw in  wcs
     # -------------
     $m_image convert coords $x $y canvas targw($my,a) targw($my,d) "wcs $EQX"
     set targw($my,f) 1
     # $m_image convert dist   $mos($my,b) dummy canvas \
     #                "deg $EQX" wid dummy
     set l_focscale 1.89393939; # nominal focscale 
     set scal [expr $atd * $l_focscale]

     # a2-world_x is difference delta alpha
     # since x-axis points to the west and y-axis points to the south
     # there is (delta alpha > 0) -> (delta x < 0)
     # and      (delta decli > 0) -> (delta y < 0)
     # for wrong instrument mounting 
     # if {$mountangle == 180} {
     # set xd_mm [expr ($world_x-$a2)/$trigo/$scal]
     # set yd_mm [expr ($world_y-$d2)/$scal]
     # } else {
     # set xd_mm [expr ($a2-$world_x)/$trigo/$scal]
     # set yd_mm [expr ($d2-$world_y)/$scal]
     #  }
     #  puts stdout "ang=$abs_ang_rad $xd_mm $yd_mm"
     #  set x_mm [expr $xd_mm*cos($abs_ang_rad)-$yd_mm*sin($abs_ang_rad)]
     #  set y_mm [expr $xd_mm*sin($abs_ang_rad)+$yd_mm*cos($abs_ang_rad)]

     # we calculate x in mm focplane of the new slit position
     # first the slit-center at x==0 in canvas
     set mid_x [expr ($mos($my,xa)+$mos($my,xd)+$mos($my,xe)+$mos($my,xh))/4.0]
     set mid_y [expr ($mos($my,ya)+$mos($my,yd)+$mos($my,ye)+$mos($my,yh))/4.0]
     # second the slit position in canvas (=x,y) already known
     # third the distance between both points as an approx.
      set mid_x [expr $mid_x - $x]; set mid_y [expr $mid_y - $y]
      set mid_x [expr $mid_x * $mid_x]; set mid_y [expr $mid_y * $mid_y]
      set x_mm [expr sqrt($mid_x+$mid_y) / $ctom  ]
     # check the absolute value (+ or - x)
      set mid_x [expr $mos($my,xa)-$x] 
      set mid_y [expr $mos($my,ya)-$y]
      set mad_x [expr $mos($my,xd)-$x] 
      set mad_y [expr $mos($my,yd)-$y]
      set mid_x [expr sqrt($mid_x*$mid_x+$mid_y*$mid_y)]
      set mad_x [expr sqrt($mad_x*$mad_x+$mad_y*$mad_y)]
      if {$mid_x<$mad_x} { set x_mm [expr $x_mm * -1.0] }
 
      puts $lfId  [format "Slit #%2d to (%s,%s)=(%6.3f,%6.3f) mm(%6.3f)" \
 		 $my $a3 $d3 $a2 $d2 $x_mm ]
      puts stdout [format "Move #%2d to (%s,%s)=(%6.3f,%6.3f) \
                          mm(%6.3f) %6.3f %6.3f" \
		 $my $a3 $d3 $a2 $d2 $x_mm $x $y]

    
      calc_zemax $my $a1 $d1;   # pixel coords  
      # apply optical correction in any case
      #                   deg     wcs
      # calc_oc $x_mm $a1 $my $a2 $d2 $a3 $d3 

      # apply correction of differential refraction (10.5.99)
      # input slit position, corrected for optical distortion
      if {$dire_flag} { calc_diff_refrac $my $targw($my,ca) $targw($my,cd) }
 
     # we calculate x in mm focplane of the slit position
     # here once more with optical correction 
     # first the slit-center at x==0 in canvas
     set mid_x [expr ($mos($my,xa)+$mos($my,xd)+$mos($my,xe)+$mos($my,xh))/4.0]
     set mid_y [expr ($mos($my,ya)+$mos($my,yd)+$mos($my,ye)+$mos($my,yh))/4.0]
     # second the slit position in canvas (=x,y) already known
     # but not with the optical correction value
     $m_image convert coords $targw($my,ca) $targw($my,cd) "wcs $EQX" \
                             x5 y5 canvas
     # third the distance between both points as an approx.
     set mid_x [expr $mid_x - $x5]; set mid_y [expr $mid_y - $y5]
     set mid_x [expr $mid_x * $mid_x]; set mid_y [expr $mid_y * $mid_y]
     set x_mm [expr sqrt($mid_x+$mid_y) / $ctom  ]
     # check the absolute value (+ or - x)
     set mid_x [expr $mos($my,xa)-$x5]
     set mid_y [expr $mos($my,ya)-$y5]
     set mad_x [expr $mos($my,xd)-$x5]
     set mad_y [expr $mos($my,yd)-$y5]
     set mid_x [expr sqrt($mid_x*$mid_x+$mid_y*$mid_y)]
     set mad_x [expr sqrt($mad_x*$mad_x+$mad_y*$mad_y)]
     if {$mid_x<$mad_x} { set x_mm [expr $x_mm * -1.0] }

     # show optical correction 
     # puts $lfId  [format "Slit #%2d to (%s,%s)=(%6.3f,%6.3f) mm(%6.3f)" \
     #             $my $targw($my,ca) $targw($my,cd) 0.0  0.0 $x_mm ]
     # puts stdout [format "Move #%2d to (%s,%s)=(%6.3f,%6.3f) \
     #                      mm(%6.3f) %6.3f %6.3f" \
     #             $my $targw($my,ca) $targw($my,cd) 0.0 0.0  $x_mm $x5 $y5]



     # set ctom [eval calc_ctom] 
     # puts stdout $ctom
     # now the graphical part: perpendicular 
     if { [expr abs($mos(0,xa)-$mos(0,xb))] < 0.01 } {
        # slit blade move direction is up-down
	# in order to avoid deviding by zero, special treatment
	puts $lfId "MoveSlit:  >> special up-down <<"
	if {$mountangle == 180} {
	set mos($my,yc) [expr $y - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	set mos($my,yg) [expr $y - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	set mos($my,yb) [expr $y + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	set mos($my,yf) [expr $y + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	} else {
	set mos($my,yc) [expr $y + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	set mos($my,yg) [expr $y + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	set mos($my,yb) [expr $y - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	set mos($my,yf) [expr $y - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	}

	# the rest coordinates remain unchanged
        # for slit-less spectroscopy, the slit may be moved out of the 
        # focal field. In this case draw the slit smaller; 
        # next move_slit inside focal field will draw the full slit again
        # in original size mos(b)
        set lam [expr ($mos($my,yf)-$mos($my,ye))/($mos($my,yh)-$mos($my,ye))]
        # puts stdout "0 < $lam < 1"
        if {$lam<0} {
           set mos($my,xf) $mos($my,xe); set mos($my,yf) $mos($my,ye)
           set mos($my,xb) $mos($my,xa); set mos($my,yb) $mos($my,ya)
           }
        set lam [expr ($mos($my,yg)-$mos($my,ye))/($mos($my,yh)-$mos($my,ye))]
        if {$lam>1} {
           set mos($my,xg) $mos($my,xh); set mos($my,yg) $mos($my,yh)
           set mos($my,xc) $mos($my,xd); set mos($my,yc) $mos($my,yd)
           }
	} elseif { [expr abs($mos(0,ya)-$mos(0,yb))] < 1e-2 } {
	# in order to avoid deviding by zero, special treatment
	# slit blade move direction is left-right 
        # puts stdout "move slit in NE direction"
	puts $lfId "MoveSlit: >> special left-right <<"
	if {$mountangle == 180} {
	set mos($my,xb) [expr $x + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	set mos($my,xf) [expr $x + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	set mos($my,xc) [expr $x - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	set mos($my,xg) [expr $x - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	} else {
	set mos($my,xb) [expr $x - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	set mos($my,xf) [expr $x - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	set mos($my,xc) [expr $x + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	set mos($my,xg) [expr $x + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
	}
        # for slitless spectroscopy, the slit may be moved out of the 
        # focal field. In this case draw the slit smaller; 
        # next move_slit inside focal field will draw the full slit again
        # in original size mos(b)
        set lam [expr ($mos($my,xf)-$mos($my,xe))/($mos($my,xh)-$mos($my,xe))]
        # puts stdout "0 < $lam < 1"
        if {$lam<0} {
           set mos($my,xf) $mos($my,xe); set mos($my,yf) $mos($my,ye)
           set mos($my,xb) $mos($my,xa); set mos($my,yb) $mos($my,ya)
           }
        set lam [expr ($mos($my,xg)-$mos($my,xe))/($mos($my,xh)-$mos($my,xe))]
        if {$lam>1} {
           set mos($my,xg) $mos($my,xh); set mos($my,yg) $mos($my,yh)
           set mos($my,xc) $mos($my,xd); set mos($my,yc) $mos($my,yd)
           }
	} else {
        # slope 
	# now the standard case 
	# deviding by zero does no more occur
	# analytical expressions to adjust the slit blade in such a 
	# way that the slit center matches the position of the target 
	# star, even if the target star is not on the 
        # slilet central line  
     puts $lfId "MoveSlit: >> usual shear <<"
     set m1 [expr ($mos($my,ya)-$mos($my,yd))/($mos($my,xa)-$mos($my,xd))]
     set m2 [expr ($mos($my,ye)-$mos($my,yh))/($mos($my,xe)-$mos($my,xh))]
     set b1 [expr ($mos($my,ya)*$mos($my,xd)-$mos($my,xa)*$mos($my,yd))]
     set b1 [expr $b1/($mos($my,xd)-$mos($my,xa))]
     set b2 [expr ($mos($my,ye)*$mos($my,xh)-$mos($my,xe)*$mos($my,yh))]
     set b2 [expr $b2/($mos($my,xh)-$mos($my,xe))]
     #puts $lfId "m1 = $m1  m2 = $m2 b1=$b1 b2=$b2"
     set h2 [expr $m1+1/$m1]
     if {$mountangle == 180} {
     set h1 [expr  $h2*$fits_x_scale*$ctom*$mos($my,b)\
            /$focscale/2/sqrt($m1*$m1+1)] 
     } else {
     set h1 [expr -$h2*$fits_x_scale*$ctom*$mos($my,b)\
            /$focscale/2/sqrt($m1*$m1+1)] 
     }
     set bl [expr $y + $x/$m1 + $h1]
     set br [expr $y + $x/$m1 - $h1]
     #puts $lfId "h1 = $h1  h2 = $h2  br = $br  bl = $bl"
     set x1 [expr ($bl-$b1)/$h2]
     set x2 [expr ($br-$b1)/$h2]
     set x3 [expr ($bl-$b2)/$h2]
     set x4 [expr ($br-$b2)/$h2]
     set y1 [expr $m1*$x1+$b1]
     set y2 [expr $m1*$x2+$b1]
     set y3 [expr $m1*$x3+$b2]
     set y4 [expr $m1*$x4+$b2]
     #puts $lfId "$x1 $y1, $x2 $y2, $x3 $y3, $x4 $y4"
     # index1=c 2=b 3=g 4=f
     #set mos($my,xc) $x1; set mos($my,yc) $y1
     #set mos($my,xb) $x2; set mos($my,yb) $y2
     #set mos($my,xg) $x3; set mos($my,yg) $y3
     #set mos($my,xf) $x4; set mos($my,yf) $y4

     set mos($my,xb) $x1; set mos($my,yb) $y1
     set mos($my,xc) $x2; set mos($my,yc) $y2
     set mos($my,xf) $x3; set mos($my,yf) $y3
     set mos($my,xg) $x4; set mos($my,yg) $y4

     # for slit-les spectroscopy, the slit may be moved out of the 
     # focal field. In this case draw the slit smaller; 
     # next move_slit inside focal field will draw the full slit again
     # in original size mos(b)
     set lam [expr ($mos($my,xf)-$mos($my,xe))/($mos($my,xh)-$mos($my,xe)) ]
     # puts stdout "0 < $lam < 1"
     if {$lam<0} {
        set mos($my,xf) $mos($my,xe); set mos($my,yf) $mos($my,ye)
        set mos($my,xb) $mos($my,xa); set mos($my,yb) $mos($my,ya)
        }
     set lam [expr ($mos($my,xg)-$mos($my,xe))/($mos($my,xh)-$mos($my,xe)) ]
     if {$lam>1} {
        set mos($my,xg) $mos($my,xh); set mos($my,yg) $mos($my,yh)
        set mos($my,xc) $mos($my,xd); set mos($my,yc) $mos($my,yd)
        }
     }
      }
     # scratch the whole old mask and plot_mask it
     catch {$w delete A} 
     plot_mask; #  <------------------------
     if {$badpixelmap} { plot_BPM }
     }

     set last_action "Move MOS \#$my"
     catch {SP update}

     return $my
     }


     #======================================================
     private method align_mask { w sx sy } {
     #======================================================
     # for echs, lss, mxu (standard maks mode) or hits :
     # move the mask to the center/gauss result
     # w = canvas widget
     # sx, sy is cursor pos in screen coords 
     # on==1  : select, on==0 : un-select, give slit free 
     # stop==1: do nothing (when selecting a reference object)
     # stop==0: do the normal procedure 
     if {$stop} { return }
     # puts stdout "in align_mask"
     set my 1 
     set l_mod [$this cget -modus] 

     $m_image convert coords $sx $sy screen x y canvas 

     # old (current mask center)
     if {$bonsai_flag} {
        set dum [C_wc w2c $world_x $world_y]
     $m_image convert coords [lindex $dum 0] [lindex $dum 1] canvas ocx ocy screen 
     $m_image convert coords $world_x $world_y "deg $EQX" a b "wcs $EQX"
     } else {
     $m_image convert coords $world_x $world_y "deg $EQX" ocx ocy screen 
     $m_image convert coords $world_x $world_y "deg $EQX" a b "wcs $EQX" 
     }

     puts $lfId "AliMask: jump slit to $a $b"
     # calculate new mask position in screen coords 
     # new_mask = old_mask + (center/gauss - current slit )
     # sx sy screen are cursor or center/gauss 
     # current slit is either MOS 10 (echs) or HITS ? (hits) or pinhole
     
     #------------------
     # center-slit option 
     #------------------
     if {[$this cget -pics]} {
        # sx,sy are approximate coords, 
        # use RtdImagePick to center slit on image target
        $m_image convert coords $sx $sy screen a0 d0 image 
        # puts stdout "Cursor canvas=$x $y screen=$sx $sy pixel=$a0 $d0"
        center_slit 
        if {$lss_sli_ra != ""} {
        $m_image convert coords $lss_sli_ra $lss_sli_de image sx sy screen
        $m_image convert coords $lss_sli_ra $lss_sli_de image  x  y canvas 
        # set CCD pixel of target 
        # set targw($my,x) $lss_sli_ra
        # set targw($my,y) $lss_sli_de
        # puts stdout [format "targw(%2d,x) = %8.3f, targw(%2d,y) = %8.3f" \
        #    $my $targw($my,x) $my  $targw($my,y) ]
        # puts stdout "center_slit from ($a0,$d0) on center_gauss result=\
        #        ($lss_sli_ra,$lss_sli_de) pixel"
           } else {
        # puts stdout "center_slit failed, use cursor at ($a0,$d0) pixel"
           }   
        } else { return }

     #-----------------------
     # center-slit option end
     #-----------------------

     
     # MXU  --------------------
     # we use the first created mxu slit 
     if {[string match $l_mod mxu]} {
        fors::MXU_SLIT::coeff $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                              $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                              $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc) 
        C_mxus1 M2W; # x,y -> xc,yc -> ra,dec in mxu  
        set targw(1,a) [C_mxus1 get_RA]
        set targw(1,d) [C_mxus1 get_DEC]
        $m_image convert coords $targw(1,a) $targw(1,d) "wcs $EQX" \
                                       ssx ssy screen 
        # puts stdout "current MXU position is $ssx $ssy"
        }

     # ECHS --------------------
     # we use the center of MOS 10 
     if {[string match $l_mod echs]} {
        set X [expr ($mos(10,xb)+$mos(10,xc)+$mos(10,xf)+$mos(10,xg) )*0.25] 
        set Y [expr ($mos(10,yb)+$mos(10,yc)+$mos(10,yf)+$mos(10,yg) )*0.25] 
        $m_image convert coords $X $Y canvas ssx ssy screen 

        # puts stdout "current ECHS position is $ssx $ssy"
        }

     # LSS- --------------------
     # we use the center of the long slit in the mask
     # which is not necessarily at Y=0 in the focal field
     if {[string match $l_mod lss]} {
        set X [expr ($mos($long_number,xa)+$mos($long_number,xd)+ \
                     $mos($long_number,xe)+$mos($long_number,xh) )*0.25] 
        set Y [expr ($mos($long_number,ya)+$mos($long_number,yd)+ \
                     $mos($long_number,ye)+$mos($long_number,yh) )*0.25] 
        $m_image convert coords $X $Y canvas ssx ssy screen 

        # puts stdout "current LSS position is $ssx $ssy"
        }

     # HITS --------------------
     if { [string match $l_mod hits] } {
        if {$long_number ==1 } {
        # --------------- PINHOLE ALIGNMENT 
        # highres collimator with hits -> pinhole 
        # where is it ? in mos(9,adeh) (long_number==1)
        # square points, we assume long_number==1 
        set X [expr ($mos(9,xa)+$mos(9,xd)+$mos(9,xe)+$mos(9,xh))/4]
        set Y [expr ($mos(9,ya)+$mos(9,yd)+$mos(9,ye)+$mos(9,yh))/4]
        $m_image convert coords $X $Y canvas ssx ssy screen 
        puts stdout "current BPH position is $ssx $ssy"
        } else {
        # --------------- HT-SLIT ALIGNMENT 
        # some parts of the ht slit is outside the CCD. 
        # consider only that part of the htslit inside 
        # the focalfield (=CCD). We dont put the 
        # center/gauss-target on the center of the 
        # focal field part of the htslit but very close 
        # to the rim of the slit. Make the ht-slit length 
        # smaller until there is a square (slit-width==slit_length). 
        # put target on center of this square. 

        # any of the available slits, not the pinhole !!!
        # slit width in canvas
        set dx [expr $mos($long_number,xa)-$mos($long_number,xd)]
        set dy [expr $mos($long_number,ya)-$mos($long_number,yd)]
        set A [expr sqrt($dx*$dx+$dy*$dy) ]

        # full slit length in canvas 
        set dx [expr $mos($long_number,xd)-$mos($long_number,xh)]
        set dy [expr $mos($long_number,yd)-$mos($long_number,yh)]
        set B [expr sqrt($dx*$dx+$dy*$dy) ]

        # square points 
        set xhn [expr $mos($long_number,xd)+ \
                ($mos($long_number,xh)-$mos($long_number,xd))*$A/$B]
        set yhn [expr $mos($long_number,yd)+ \
                ($mos($long_number,yh)-$mos($long_number,yd))*$A/$B]
        set xen [expr $mos($long_number,xa)+ \
                ($mos($long_number,xe)-$mos($long_number,xa))*$A/$B]
        set yen [expr $mos($long_number,ya)+ \
                ($mos($long_number,ye)-$mos($long_number,ya))*$A/$B]

        set X [expr ($mos($long_number,xa)+$mos($long_number,xd)+$xhn+$xen)/4]
        set Y [expr ($mos($long_number,ya)+$mos($long_number,yd)+$yhn+$yen)/4]
        $m_image convert coords $X $Y canvas ssx ssy screen 
        # puts stdout "current HITS position is $ssx $ssy"
        }
        }
        # HITS 


     set newx [expr $ocx + $sx - $ssx ]
     set newy [expr $ocy + $sy - $ssy ]

     # before we shift, update the oldpos in mark 
     set oldpos(x) $mos(0,x0); set oldpos(y) $mos(0,y0)
     # shift mask to new screen coords 
     shift $w $newx $newy 

     if {[string match $l_mod echs]} { echs_targ }   

     # now we apply the correction for optical distortion 
     # of FORS images and differential refraction 
     set corr_flag [$this cget -fors_frame]

     # <><><><><><><><><><><><><><><><><><><><><>
     # workaround 13. Sept 2000 WH
     # FORS1 images are aliens fro MXU 
     #if {[string match $l_mod mxu]} {
     #if {[string compare [$m_image fits get INSTRUME] "FORS1"]==0} {
     #   set corr_flag 0 } }
     # <><><><><><><><><><><><><><><><><><><><><>

     fix_mask_slit $l_mod $corr_flag 

     }

     #==================================
     private method fix_mask_slit  { mod cf } {
     #==================================
     # calculate zemax correction for 
     # slit (either echs, lss, hits, pinh or
     #  mxu-standard-mask-mode ) but apply 
     # correction to pointing (mask center) 
     # mod must be hits, lss  or echs 
     # puts stdout "in fix_mask_slit"

     # mask center in pixel 
     $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x y image 

      # MXU 
     if {[string compare $mod mxu]==0} {
        # puts "correct a fixed mask = MXU"
        # TCS 
        # world_new = world_old - (targ_corrected - targ_initial)
        $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
        targw(2,a) targw(2,d) "wcs $EQX"  
        $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
        targw(2,x) targw(2,y) image 
        if {$cf} {
           calc_zemax 2 $targw(2,x) $targw(2,y) 
           if {$dire_flag} {
              calc_diff_refrac 2 $targw(2,ca) $targw(2,cd)
              puts stdout "old pointing $targw(2,a) $targw(2,d)"
              puts stdout "new pointing $targw(2,ca) $targw(2,cd)"
              }
           }

        # slit
        # slit center pf C_mxus1 
        # this is probably not necessary since it will be repeated in 
        # part prepare of save_mxu_TPX ? 
        C_mxus1 M2W; # x,y -> xc,yc -> ra,dec in mxu  
        set targw(10,a) [C_mxus1 get_RA]
        set targw(10,d) [C_mxus1 get_DEC]
        $m_image convert coords $targw(10,a) $targw(10,d) "wcs $EQX" \
                                 targw(10,x)  targw(10,y) image 
        if {$cf} { calc_zemax 10 $targw(10,x) $targw(10,y) 
        if {$dire_flag} {
           calc_diff_refrac 10 $targw(10,ca) $targw(10,cd)
           # puts stdout "old mos10    $targw(10,a) $targw(10,d)"
           # puts stdout "new mos10    $targw(10,ca) $targw(10,cd)"
           }
           }
        # puts stdout "$targw(2,x) $targw(2,y) $targw(2,cx) $targw(2,cy)"
        # puts stdout "$targw(2,a) $targw(2,d) $targw(2,ca) $targw(2,cd)"
        # puts stdout "$targw(10,x) $targw(10,y) $targw(10,cx) $targw(10,cy)"
        # puts stdout "$targw(10,a) $targw(10,d) $targw(10,ca) $targw(10,cd)"
        }

      # ECHS
     if {[string compare $mod echs]==0} {
        # puts "correct a fixed mask = ECHS"
        # TCS 
        # world_new = world_old - (targ_corrected - targ_initial)
        if [catch {
        $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
        targw(2,a) targw(2,d) "wcs $EQX"  
                  } result ] { puts stdout "fix_mask ECHS: $result" }

        if [catch {
        $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
        targw(2,x) targw(2,y) image 
                  } result ] { puts stdout "fix_mask ECHS: $result" }

        if {$cf} { calc_zemax 2 $targw(2,x) $targw(2,y) 
        if {$dire_flag} {
           calc_diff_refrac 2 $targw(2,ca) $targw(2,cd)
           puts stdout "old pointing $targw(2,a) $targw(2,d)"
           puts stdout "new pointing $targw(2,ca) $targw(2,cd)"
           }
          

        # slit
        # mask center = mos 10 in pixel 
        calc_zemax 10 $targw(10,x) $targw(10,y) 
        if {$dire_flag} {
           calc_diff_refrac 10 $targw(10,ca) $targw(10,cd)
           # puts stdout "old mos10    $targw(10,a) $targw(10,d)"
           # puts stdout "new mos10    $targw(10,ca) $targw(10,cd)"
           }
           }
        # puts stdout "$targw(2,x) $targw(2,y) $targw(2,cx) $targw(2,cy)"
        # puts stdout "$targw(2,a) $targw(2,d) $targw(2,ca) $targw(2,cd)"
        # puts stdout "$targw(10,x) $targw(10,y) $targw(10,cx) $targw(10,cy)"
        # puts stdout "$targw(10,a) $targw(10,d) $targw(10,ca) $targw(10,cd)"
        }

      # LSS 
     if {[string compare $mod lss]==0} {
        # puts "correct a fixed mask = LSS"
        # TCS
        # world_new = world_old - (targ_corrected - targ_initial)
        
        if [catch {
        $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
                 targw(2,a) targw(2,d) "wcs $EQX"
                 } result ] { puts stdout "fix_mask:LSS: $result" }

        $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
                 targw(2,x) targw(2,y) image

        
        if {$cf} { calc_zemax 2 $targw(2,x) $targw(2,y)
        if {$dire_flag} {
           calc_diff_refrac 2 $targw(2,ca) $targw(2,cd)
           puts stdout "old pointing $targw(2,a) $targw(2,d)"
           puts stdout "new pointing $targw(2,ca) $targw(2,cd)"
           }
           }

        # SLIT (=science target)
        set dx [expr $mos($long_number,xa)-$mos($long_number,xd)]
        set dy [expr $mos($long_number,ya)-$mos($long_number,yd)]
        set A [expr sqrt($dx*$dx+$dy*$dy) ]
        #where is lss slit center ? in mos($long_number,adeh)
        set X [expr ($mos($long_number,xa)+$mos($long_number,xd)+ \
                     $mos($long_number,xe)+$mos($long_number,xh))/4]
        set Y [expr ($mos($long_number,ya)+$mos($long_number,yd)+ \
                     $mos($long_number,ye)+$mos($long_number,yh))/4]
        $m_image convert coords $X $Y canvas targw(10,x) targw(10,y) image
        if [catch {
        $m_image convert coords $X $Y canvas targw(10,a) targw(10,d) "wcs $EQX"
                 } result ] { puts stdout "fix_mask:LSS: $result" }

        # puts stdout "current LSS position is $targw(10,a) $targw(10,d)"

        if {$cf} { calc_zemax 10 $targw(10,x) $targw(10,y) }

       }
       
      # HITS
     if {[string compare $mod hits]==0} {
        # puts "correct a fixed mask = HITS"
        # TCS
        # world_new = world_old - (targ_corrected - targ_initial)

        if $bonsai_flag {
           set dum [C_wc c2w $mos(0,x0) $mos(0,y0)]
           $m_image convert coords [lindex $dum 0] [lindex $dum 1] "deg $EQX" \
                    targw(2,a) targw(2,d) "wcs $EQX"
           } else {
           if [catch { 
              $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
                    targw(2,a) targw(2,d) "wcs $EQX"  
                  } result ] {
               puts stdout "fix_max:HITS:TCS  $result" }
           }


        if [catch { 
        $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
        targw(2,x) targw(2,y) image 
                  } result ] {
        puts stdout "fix_mask:HITS:TCS $result" } else {

        if {$cf} { 
           if {[calc_zemax 2 $targw(2,x) $targw(2,y) ]} {
              puts stdout "No Distortion Correction Applied to TEL !!"
              }
        if {$dire_flag} {
           calc_diff_refrac 2 $targw(2,ca) $targw(2,cd)
           puts stdout "old pointing $targw(2,a) $targw(2,d)"
           puts stdout "new pointing $targw(2,ca) $targw(2,cd)"
           }
           }

        }
        # slit
        # similar to center-slit in hits 
        # not necessary, since slit and TCS must be inside 
        # frame 
        # set dum [C_wc w2c $p1x $p1y]
        # set p1x [lindex $dum 0]
        # set p1y [lindex $dum 1]

        # --- SR ---
        if {$long_number != 1} {
        # any of the available slits, not the pinhole !!!
        # slit width in canvas
        set dx [expr $mos($long_number,xa)-$mos($long_number,xd)]
        set dy [expr $mos($long_number,ya)-$mos($long_number,yd)]
        set A [expr sqrt($dx*$dx+$dy*$dy) ]

        # full slit length in canvas
        set dx [expr $mos($long_number,xd)-$mos($long_number,xh)]
        set dy [expr $mos($long_number,yd)-$mos($long_number,yh)]
        set B [expr sqrt($dx*$dx+$dy*$dy) ]

        # square points
        set xhn [expr $mos($long_number,xd)+ \
                ($mos($long_number,xh)-$mos($long_number,xd))*$A/$B]
        set yhn [expr $mos($long_number,yd)+ \
                ($mos($long_number,yh)-$mos($long_number,yd))*$A/$B]
        set xen [expr $mos($long_number,xa)+ \
                ($mos($long_number,xe)-$mos($long_number,xa))*$A/$B]
        set yen [expr $mos($long_number,ya)+ \
                ($mos($long_number,ye)-$mos($long_number,ya))*$A/$B]

        set X [expr ($mos($long_number,xa)+$mos($long_number,xd)+$xhn+$xen)/4]
        set Y [expr ($mos($long_number,ya)+$mos($long_number,yd)+$yhn+$yen)/4]



        if {$bonsai_flag} {
           # if underlying frame is small 
           set dum [C_wc c2w $X $Y]
           $m_image convert coords [lindex $dum 0] [lindex $dum 1] "deg $EQX" \
                    targw(10,a) targw(10,d) "wcs $EQX"
           } else {
           # underlying frame is large enough 
        if [catch {$m_image convert coords $X $Y canvas \
                 targw(10,a) targw(10,d) "wcs $EQX"  } result] {
             puts stdout "fix_mask:HIUTS:SLIT: $result" }
             }

        if [catch {$m_image convert coords $X $Y canvas \
                 targw(10,x) targw(10,y) image } result] {
           puts stdout "fix_mask:HITS:SLIT: $result"
           }

        # puts stdout "current HITS Slit at $targw(10,a) $targw(10,d)"

        } else {

        # --- PINHOLE ---
        #highres collimator with hits -> pinhole
        #where is it ? in mos(9,adeh) (long_number==1)
        #square points, we assume long_number==1
        set X [expr ($mos(9,xa)+$mos(9,xd)+$mos(9,xe)+$mos(9,xh))/4]
        set Y [expr ($mos(9,ya)+$mos(9,yd)+$mos(9,ye)+$mos(9,yh))/4]
        $m_image convert coords $X $Y canvas targw(10,x) targw(10,y) image
        $m_image convert coords $X $Y canvas \
        targw(10,a) targw(10,d) "wcs $EQX"
        puts stdout "current BPH position is $targw(10,a) $targw(10,d)"
        }

        if {$cf} { 
           if {[calc_zemax 10 $targw(10,x) $targw(10,y) ]} {
              puts stdout "fix_mask:HITS:SLIT: Distortion not corrected for TARG !!"
              }
        if {$dire_flag} {
           calc_diff_refrac 10 $targw(10,ca) $targw(10,cd)
           puts stdout "old slit     $targw(10,a) $targw(10,d)"
           puts stdout "new slit     $targw(10,ca) $targw(10,cd)"
           }
        }
        }

     # in any case the new correct pointing will be stored 
     # to MOS 2
     # corrected telescope pointing
     if {$cf} {
     $m_image convert coords $targw(2,ca) $targw(2,cd) "wcs $EQX" \
              ta td "deg $EQX"
     # for slit difference 
     $m_image convert coords $targw(10,a) $targw(10,d) "wcs $EQX" \
               a d "deg $EQX"
     $m_image convert coords $targw(10,ca) $targw(10,cd) "wcs $EQX" \
               ca cd "deg $EQX"

     set ta [expr $ta - $ca + $a]
     set td [expr $td - $cd + $d]
     $m_image convert coords $ta $td "deg $EQX" \
                             targw(2,ca) targw(2,cd) "wcs $EQX"
     }
     puts $lfId "FixMaskSlit to $targw(10,a) $targw(10,d)"
     }


     #==================================
     private method move_slit_blade { w sx sy but } {
     #==================================
     # moves only one slit blade of the pair
     # for IMS, PIMS, IMG, IPOL-SR
     # sx,sy screen coords, but = mouse button
     # but = 0 1 out in 
     #       out : of the focal field, delete blade setting 
     #         1 : move blade A (if clicked inside the slit)
     #         0 : move blade B (if clicked inside the slit)
     # stop==1: do nothing (when selecting a reference object)
     # stop==0: do the normal procedure 
     if {$stop} { return }

     #puts stdout "Move Slitlet : $w sx=$sx sy=$sy but=$but"
     #puts stdout " coords of m1 : [$w coords m1]"
     #puts stdout [$w gettags m1]
     #puts stdout [$w find closest $sx $sy]
     #puts stdout [$w gettags [$w find closest $sx $sy]]
     # shows the current tag clicked on 
     #puts stdout [lindex [$w gettags [$w find closest $sx $sy]] 1]
     $m_image convert coords $sx $sy screen x y canvas 

     # find the tag of the slit blade belonging to the cursor position 
     # puts stdout "[$w gettags [$w find closest $x $y]]"
     set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
     set sidtag [lindex [$w gettags [$w find closest $x $y]] 3]
     # puts stdout " coords of $curtag : [$w coords $curtag]"
     # puts $lfId "$mos(12,xa) $mos(12,ya) $mos(12,xb)"
     set c1 [expr [string compare [$this cget -modus] "ipol"]==0]
     set c2 [expr [string compare [$this cget -modus] "pmos"]==0 && \
                  [string compare [$this cget -slit_width] open]==0 ]
     set ipol_flag [expr $c1 || $c2]
     # puts stdout "ipol_flag = $ipol_flag"
     # set ipol_flag [expr [string compare [$this cget -modus] "ipol"]==0]

     set my [string range $curtag  1 end]
     set sid [string range $sidtag  1 1]

     # puts stdout ">$curtag< >$sidtag< >$my< >$sid<"

     if {[string compare $but out]==0} {
        if { ![expr $my-($my/2)*2] && $ipol_flag || !$ipol_flag } {
           set mos($my,b) $mos(0,b)
           set mos($my,xb) $mos($my,xa); set mos($my,yb) $mos($my,ya)
           set mos($my,xc) $mos($my,xd); set mos($my,yc) $mos($my,yd)
           set mos($my,xf) $mos($my,xe); set mos($my,yf) $mos($my,ye)
           set mos($my,xg) $mos($my,xh); set mos($my,yg) $mos($my,yh)
           set targw($my,f) 0
           }
     catch {$w delete A} 
     plot_mask; #  <------------------------
     return;
       }

     if { [string length $my] == 0 } { 
     # puts stdout "Warning cannot move #$my to ($x,$y)" 
     } elseif { [expr $my-($my/2)*2] && $ipol_flag} {
     # puts stdout "Warning cannot move #$my to ($x,$y)" } else {
     # puts stdout "number = >[string range $curtag  1 end]<"
     # puts stdout "side   = >[string range $sidtag  1 1  ]<"
           # [expr $sid-($sid/2)*2]
           # this is a odd number 
     $m_image convert coords $sx $sy screen a1 d1 image 

     if [ catch {
     $m_image convert coords $sx $sy screen a2 d2 "deg $EQX" } result ] {
           puts stdout "move_slit_blade: $result" }

     if [ catch {
     $m_image convert coords $sx $sy screen a3 d3 "wcs $EQX" } result ] {
           puts stdout "move_slit_blade: $result" }

     set targw($my,f) 1
     # $m_image convert dist   $mos($my,b) dummy canvas \
     #                "deg $EQX" wid dummy
     set scal [expr $atd * $focscale]
     # a2-world_x is difference delta alpha
     # since x-axis points to east and y-axis points to north
     # there is (delta alpha > 0) -> (delta x > 0)
     # and      (delta decli > 0) -> (delta y > 0)
     set x_mm [expr ($a2-$world_x)/$trigo/$scal]
     set y_mm [expr ($d2-$world_y)/$scal]
puts $lfId  [format "blade #%2d%s to (%s,%s)=(%6.3f,%6.3f) mm(%6.3f,%6.3f)" \
		 $my $sid $a3 $d3 $a2 $d2 $x_mm $y_mm ]
puts stdout [format "blade #%2d%1s to (%s,%s)=(%6.3f,%6.3f) mm(%6.3f,%6.3f)" \
		 $my $sid $a3 $d3 $a2 $d2 $x_mm $y_mm ]
     # b=[expr $wid*$atod] arcsec" \
     #   c($x,$y) s($sx,$sy) i($a1,$d1)"


     if { [expr abs($mos(0,xa)-$mos(0,xb))] < 0.01 } {
        # slit blade move direction is up-down
	# in order to avoid deviding by zero, special treatment
	puts $lfId "MoveBlade: >> special up-down <<"
        # ERROR
        switch -exact -- $sid {
           a { set mos($my,yb) $y
               set mos($my,yf) $y
             }
           b { set mos($my,yc) $y
               set mos($my,yg) $y
             }
           s {# which point is closer ? 
              switch -exact -- $but {
              1 { # take MOS A
                 set mos($my,yb) $y
                 set mos($my,yf) $y
                 } 
              0 { # take MOS B
                 set mos($my,yc) $y
                 set mos($my,yg) $y
                 }
              }
             }
          default {puts $lfId "MoveBlade: Warning "}
           }
	# the rest coordinated remain unchanged
	} elseif { [expr abs($mos(0,ya)-$mos(0,yb))] < 1e-2 } {
	# in order to avoid deviding by zero, special treatment
	# slit blade move direction is left-right 
	# puts $lfId ">> special left-right <<"
        # ERROR
        switch -exact -- $sid {
           a {set mos($my,xb) $x
              set mos($my,xf) $x
             }
           b {set mos($my,xc) $x
              set mos($my,xg) $x
             }
           s {# which point is closer ? 
              switch -exact -- $but {
               1 {
                 # take MOS A
                 set mos($my,xb) $x
                 set mos($my,xf) $x
                 } 
               0 {
                 # take MOS B
                 set mos($my,xc) $x
                 set mos($my,xg) $x
                 }
              }
             }
          default {puts $lfId "MoveBlade: Warning shift single slit blade"}
           }
	} else {
	# now the standard case 
	# deviding by zero does no more occur
	# analytical expressions to adjust the slit blade in such a 
	# way that the slit center matches the position of the target 
	# star, even if the target star is not on the 
        # slilet central line  
     # puts $lfId ">> usual shear <<"
     # index 1 line ABCD
     # index 2 line EFGH
     set m1 [expr ($mos($my,ya)-$mos($my,yd))/($mos($my,xa)-$mos($my,xd))]
     set m2 [expr ($mos($my,ye)-$mos($my,yh))/($mos($my,xe)-$mos($my,xh))]
     set b1 [expr ($mos($my,ya)*$mos($my,xd)-$mos($my,xa)*$mos($my,yd))]
     set b1 [expr $b1/($mos($my,xd)-$mos($my,xa))]
     set b2 [expr ($mos($my,ye)*$mos($my,xh)-$mos($my,xe)*$mos($my,yh))]
     set b2 [expr $b2/($mos($my,xh)-$mos($my,xe))]
     #puts $lfId "m1 = $m1  m2 = $m2 b1=$b1 b2=$b2"
     set h2 [expr $m1+1/$m1]
     set b3 [expr $y + $x/$m1]
     #puts $lfId "h1 = $h1  h2 = $h2  br = $br  bl = $bl"
     set x1 [expr ($b3-$b1)/$h2]
     set x2 [expr ($b3-$b2)/$h2]
     set y1 [expr $m1*$x1+$b1]
     set y2 [expr $m1*$x2+$b2]

     switch -exact -- $sid {
        a {set mos($my,xb) $x1; set mos($my,yb) $y1
           set mos($my,xf) $x2; set mos($my,yf) $y2
          }
        b {set mos($my,xc) $x1; set mos($my,yc) $y1
           set mos($my,xg) $x2; set mos($my,yg) $y2
          }
        s {# which point is closer ? 
           # set da [expr $mos($my,xb)-$x1]; set da [expr $da*$da]
           # set d1 [expr $mos($my,yb)-$y1]; set d1 [expr $d1*$d1]
           # set da [expr $da+$d1]
           # set db [expr $mos($my,xc)-$x1]; set db [expr $db*$db]
           # set d1 [expr $mos($my,yc)-$y1]; set d1 [expr $d1*$d1]
           # set db [expr $db+$d1]
           # if {$da<$db} 
           switch -exact -- $but {
           1 {
              # take MOS A
              set mos($my,xb) $x1; set mos($my,yb) $y1
              set mos($my,xf) $x2; set mos($my,yf) $y2
	      set sid a
              } 
           0 {
             # take MOS B
              set mos($my,xc) $x1; set mos($my,yc) $y1
              set mos($my,xg) $x2; set mos($my,yg) $y2
	      set sid b
              }
            }
          }
       default {puts $lfId "MoveBlade: Warning" }
        }

     # set mos($my,xb) $x1; set mos($my,yb) $y1
     # set mos($my,xc) $x2; set mos($my,yc) $y2
     # set mos($my,xf) $x3; set mos($my,yf) $y3
     # set mos($my,xg) $x4; set mos($my,yg) $y4
     }
     # the new slitwidth 
     set xdum [expr $mos($my,xb) - $mos($my,xc)]
     set ydum [expr $mos($my,yb) - $mos($my,yc)]
     $m_image convert dist $xdum $ydum canvas x y "deg $EQX"
     set mos($my,b) [format "%3.1f" [expr sqrt($x*$x*1.296e7+$y*$y*1.296e7)] ]
     # puts stdout "Width #$my = $mos($my,b) arcsec   \
     #         = [expr $mos($my,b) * $focscale] mm"
     set targw($my,a) \
         [expr ($mos($my,xc)+$mos($my,xb)+$mos($my,xf)+$mos($my,xg))/4.0] 
     set targw($my,d) \
         [expr ($mos($my,yc)+$mos($my,yb)+$mos($my,yf)+$mos($my,yg))/4.0] 
     # targw in wcs 
     # ------------
     $m_image convert coords $targw($my,a) $targw($my,d) canvas \
                              targw($my,a) targw($my,d) "wcs $EQX"
     # scratch the whole old mask and plot_mask it
     # tcs-label needs not to be updated
     catch {$w delete A} 
     plot_mask; #  <------------------------
     if {$badpixelmap} { plot_BPM }

     set last_action "Move MOS \#$my[string toupper  $sid]"
     catch {SP update}
     }
     }


     #==================================
     private method mxu_create_slit { w sx sy but {on 1} {click 1} } {
     #==================================
     # creates a new slit for the FORS2-MXU
     # sx,sy screen coords, but = mouse button
     # stop==1: do nothing (when selecting a reference object)
     # stop==0: do the normal procedure

     #puts stdout "mxu_create_slit : $w sx=$sx sy=$sy but=$but"
     if {$stop} { return }

     $m_image convert coords $sx $sy screen x y canvas 
     $m_image convert coords $sx $sy screen px py image
     # puts stdout "[$w gettags [$w find closest $x $y]]"
     # puts stdout  "$x $y ->  [C_wc c2m $x $y]"

     # adapt mos slit length 
     set mos_sl  [$this cget -slit_length]
     if {[string compare $mos_sl real]==0} {
        set mxu_sl 22.0 } else { set mxu_sl $mos_sl }
     if {[string match *CIRCLE* [$this cget -mxu_shape] ]} {
         set mxu_sl [$this cget -slit_width]
         }

     # adapt collimator
     if {[string compare [$this cget -instrument] FORS2]==0} { 
	 set ff [list $CCDMASTER_XSRMIN $CCDMASTER_XSRMAX]
     } else {
	 set ff [list $XSRMIN $XSRMAX]
     }
     if {[string match *HR* [$this cget -collimat] ]} {
	 if {[string compare [$this cget -instrument] FORS2]==0} { 
	     set ff [list $CCDMASTER_XSRMIN $CCDMASTER_XSRMAX]
	 } else {
	     set ff [list $XHRMIN $XHRMAX] }
	 }

     #------------------
     # center-slit option
     #------------------
     if {$click} {
     if {[$this cget -pics]} {
        # sx,sy are approximate coords,
        # use RtdImagePick to center slit on image target
        $m_image convert coords $sx $sy screen px py image
        # puts stdout "Cursor canvas=$x $y screen=$sx $sy pixel=$px $py"
        center_slit
        if {$lss_sli_ra != ""} {
        $m_image convert coords $lss_sli_ra $lss_sli_de image x y canvas
        # puts stdout "center_slit from ($px,$py) on center_gauss result=\
        #        ($lss_sli_ra,$lss_sli_de) pixel"
        set px $lss_sli_ra; set py $lss_sli_de
           } else {
        # puts stdout "center_slit failed, use cursor at ($px,$py) pixel"
           }
        }
     }
     #-----------------------
     # center-slit option end
     #-----------------------
     
     # this is necessary for calc_zemax
     set targw(1,f) 1
     set targw(1,x) $px
     set targw(1,y) $py
     $m_image convert coords $px $py image targw(1,a) targw(1,d) "wcs $EQX"
     # this is also called in load_mxu_TPX. 
     # this is convenient, for science masks, when slits are on
     # their position. For standard masks for which slits may change their
     # position we need another procedure (see load_fad)
     # when we allow to shift MXU slits, ... more complicated 
    

     # create one instance of MXU SLIT
     # the number of mxu slits
     set noms [fors::MXU_SLIT::nos]
     if {$noms==$mnomxus} {
        info_dialog "Error: Cannot handle more than\n $mnomxus MXU slits" .skycat1
        return
        }

     # we dont use the auto, since auto continues with 7 when 
     # reset_mask seleted the old 6 
     # instances are C_mxus1, C_mxus2 ...
    
     set a [fors::MXU_SLIT C_mxus[incr noms 1] \
           $m_canvas \
           $m_image  \
           $x $y \
           -eqx $EQX \
           -slit_width [$this cget -slit_width] \
           -slit_length $mxu_sl \
           -pos_ang 0.0  \
           -shape [$this cget -mxu_shape] \
           -ff $ff \
           -ON 1 \
           -child $this ]

      switch -exact -- [$this cget -mxu_shape]  { 
      STRAIGHT -
      CIRCLE {
           # optical correction of just created slit or circle, 
           # we use targw 1
           set targw(1,x) $px; set targw(1,y) $py
           set targw(1,a) [C_mxus$noms get_RA]
           set targw(1,d) [C_mxus$noms get_DEC]
           calc_zemax 1 $targw(1,x) $targw(1,y) 
           # targw(1,cx) and targw(1,cy) is not used 
           C_mxus$noms put_OC  [list $targw(1,ca) $targw(1,cd)]


           # apply correction of differential refraction (10.5.99)
           # input slit position, corrected for optical distortion
             if {$dire_flag} {calc_diff_refrac 1 $targw(1,ca) $targw(1,cd)}
             C_mxus$noms put_OC  [list $targw(1,ca) $targw(1,cd)]
             }
      CURVED {
           # special note for CURVED
           # the click-point/center-gauss-point is not on the curved 
           # slit. 
           # we determine the curved slitcenter, apply correction of 
           # of optical distortion and save correct values to mxu slit
           # this point has only meaning for DRS 
           set dummy [C_mxus$noms update_curved_center]
           $m_image convert coords [lindex $dummy 0] [lindex $dummy 1] canvas \
                    targw(1,x) targw(1,y) image 
           set targw(1,a) [C_mxus$noms get_RA]
           set targw(1,d) [C_mxus$noms get_DEC]
           calc_zemax 1 $targw(1,x) $targw(1,y) 
           # targw(1,cx) and targw(1,cy) is not used 
           C_mxus$noms put_OC  [list $targw(1,ca) $targw(1,cd)]
             }
             }

             # C_mxus$noms status

      # for status panel
      set last_action "Create MXU \#$noms"

      catch {SP update}
      } 

     #==================================
     public method arsc {} {
     #==================================
     # returns the number of all reference mxu slits  
     set noms [fors::MXU_SLIT::nos]
     set s_c 0; 

     for {set i 1} {$i <= $noms} { incr i 1} {
         if {[C_mxus$i cget -ON]} { 
         if {[C_mxus$i cget -ref_slit]} { incr s_c 1 
            }
            }
            }
     return $s_c 
     }

     #==================================
     public method anoms {} {
     #==================================
     # returns the number of all non-deleted mxu slits 
     set noms [fors::MXU_SLIT::nos]
     set s_c 0; 

     for {set i 1} {$i <= $noms} { incr i 1} {
         if {[C_mxus$i cget -ON]} { incr s_c 1 
            }
            }
     return $s_c 
     }

     #==================================
     private method rotate { w x y dir} {
     #==================================
     if {$stop} { return }

     # rotate the whole mask with all slits
     # del_ang is incremental rotation del_angle of the object 
     # default + or - 5 degr 

     # rad-deg conversion 
     # rotangle in deg, del_ang in rad 
     set del_ang [expr $dir*$rotangle*0.0174532]
     # puts $lfId "RotAng >>$rotangle<< DelAng >>$del_ang<<"
     # abs_ang_rad is the current rot-angle with respect to the 
     # initial object, = mask-wcs angle from north to east
     set abs_ang_rad [expr $abs_ang_rad + $del_ang]
     set PAM [expr $PAM + $dir*$rotangle*$CSO] 
     set PAM [expr fmod($PAM,360) ]
     set tcs_pam [format "%6.2f" [expr $PAM * -1] ]
     # set myob($w,obj) [$w find closest $x $y]
     # new coordinates
     # we rotate the all coordinates around the axis 
     # located in the center of the figure = ccx, ccy
     # the angle step is del_angle
     # the new total angle = abs_ang_rad

     set local_max 10
     set local_set {a b d e f h}
     if {[string compare $mask "mos"]==0} {
         set local_max 20 
         set local_set {a b c d e f g h }
          }
     if {[string compare $mask "mxu"]==0} {
         set local_max 0 
         # later we will use the mxu internal counter
         } 

     # ---------------------
     # rotate the mos-slit blades
     # ---------------------
     set cda [expr cos($del_ang)]; set sda [expr sin($del_ang)]
     #  rotation via matrix multiplication
     puts $lfId "rotate at $world_x,$world_y to PA=$tcs_pam"
     for { set i 1} { $i < $local_max } { incr i 1} {
       foreach val $local_set {
   set d1 [expr $ccx + ($mos($i,x$val)-$ccx)*$cda - ($mos($i,y$val)-$ccy)*$sda]
   set d2 [expr $ccy + ($mos($i,x$val)-$ccx)*$sda + ($mos($i,y$val)-$ccy)*$cda]
   set mos($i,x$val) $d1; set mos($i,y$val) $d2
        }
     }

  # update the 19 slit-positions of the mos
  if {[string compare $mask "mos"]==0} {
  for { set i 1} { $i < $local_max } { incr i 1} {

      # close the unused mos slit blades again in MOS/PMOS
      # keep them open in IMG IPOL ECHS (mask=mos)
      # exception : MOS slit-les spectroscopy 
      if {[string match "*mos" [$this cget -modus] ]} {
      if {[string compare "open" [$this cget -slit_width] ]} {
         # deselect all mos slits 
         set targw($i,f) 0
         set mos($i,xb) $mos($i,xa)
         set mos($i,yb) $mos($i,ya)
         set mos($i,xc) $mos($i,xa)
         set mos($i,yc) $mos($i,ya)
         set mos($i,xg) $mos($i,xe)
         set mos($i,yg) $mos($i,ye)
         set mos($i,xf) $mos($i,xe)
         set mos($i,yf) $mos($i,ye)
         }
       } else {
      # img, ipol, echs 
      # mean value 
      set A [expr ($mos($i,xc)+$mos($i,xb)+$mos($i,xf)+$mos($i,xg))/4.0] 
      set B [expr ($mos($i,yc)+$mos($i,yb)+$mos($i,yf)+$mos($i,yg))/4.0] 
      if {$targw($i,f) != 0 } {
      $m_image convert coords $A $B canvas targw($i,a) targw($i,d) "wcs $EQX"
      }
      }
    }
  }


#    ToDo test and replace 
#    foreach k {a b c d e f g h ar br cr dr er fr gr hr} {
#    set d1 [expr $ccx + ($mos(0,x$k)-$ccx)*$cda - ($mos(0,y$k)-$ccy)*$sda]
#    set d2 [expr $ccy + ($mos(0,x$k)-$ccx)*$sda + ($mos(0,y$k)-$ccy)*$cda]
#    set mos(0,x$k) $d1; set mos(0,y$k) $d2
#    }
     
     # rotate the mos-frame 
     set d1 [expr $ccx + ($mos(0,xa)-$ccx)*$cda - ($mos(0,ya)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xa)-$ccx)*$sda + ($mos(0,ya)-$ccy)*$cda]
     set mos(0,xa) $d1; set mos(0,ya) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xa)-$ccx)*$cda - ($ccd_frame(0,ya)-$ccy)*$sda]
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xa)-$ccx)*$sda + ($ccd_frame(0,ya)-$ccy)*$cda]
     set ccd_frame(0,xa) $ccdmd1 ; set ccd_frame(0,ya) $ccdmd2
     
     set ccdsd1 [expr $ccx + ($ccd_frame(1,xa)-$ccx)*$cda - ($ccd_frame(1,ya)-$ccy)*$sda]
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xa)-$ccx)*$sda + ($ccd_frame(1,ya)-$ccy)*$cda]
     set ccd_frame(1,xa) $ccdsd1 ; set ccd_frame(1,ya) $ccdsd2

     set d1 [expr $ccx + ($mos(0,xb)-$ccx)*$cda - ($mos(0,yb)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xb)-$ccx)*$sda + ($mos(0,yb)-$ccy)*$cda]
     set mos(0,xb) $d1; set mos(0,yb) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xb)-$ccx)*$cda - ($ccd_frame(0,yb)-$ccy)*$sda]
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xb)-$ccx)*$sda + ($ccd_frame(0,yb)-$ccy)*$cda]
     set ccd_frame(0,xb) $ccdmd1 ; set ccd_frame(0,yb) $ccdmd2
     
     set ccdsd1 [expr $ccx + ($ccd_frame(1,xb)-$ccx)*$cda - ($ccd_frame(1,yb)-$ccy)*$sda]
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xb)-$ccx)*$sda + ($ccd_frame(1,yb)-$ccy)*$cda]
     set ccd_frame(1,xb) $ccdsd1 ; set ccd_frame(1,yb) $ccdsd2

     set d1 [expr $ccx + ($mos(0,xc)-$ccx)*$cda - ($mos(0,yc)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xc)-$ccx)*$sda + ($mos(0,yc)-$ccy)*$cda]
     set mos(0,xc) $d1; set mos(0,yc) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xc)-$ccx)*$cda - ($ccd_frame(0,yc)-$ccy)*$sda]
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xc)-$ccx)*$sda + ($ccd_frame(0,yc)-$ccy)*$cda]
     set ccd_frame(0,xc) $ccdmd1 ; set ccd_frame(0,yc) $ccdmd2
     
     set ccdsd1 [expr $ccx + ($ccd_frame(1,xc)-$ccx)*$cda - ($ccd_frame(1,yc)-$ccy)*$sda]
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xc)-$ccx)*$sda + ($ccd_frame(1,yc)-$ccy)*$cda]
     set ccd_frame(1,xc) $ccdsd1 ; set ccd_frame(1,yc) $ccdsd2

     set d1 [expr $ccx + ($mos(0,xd)-$ccx)*$cda - ($mos(0,yd)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xd)-$ccx)*$sda + ($mos(0,yd)-$ccy)*$cda]
     set mos(0,xd) $d1; set mos(0,yd) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xd)-$ccx)*$cda - ($ccd_frame(0,yd)-$ccy)*$sda]
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xd)-$ccx)*$sda + ($ccd_frame(0,yd)-$ccy)*$cda]
     set ccd_frame(0,xd) $ccdmd1 ; set ccd_frame(0,yd) $ccdmd2
     
     set ccdsd1 [expr $ccx + ($ccd_frame(1,xd)-$ccx)*$cda - ($ccd_frame(1,yd)-$ccy)*$sda]
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xd)-$ccx)*$sda + ($ccd_frame(1,yd)-$ccy)*$cda]
     set ccd_frame(1,xd) $ccdsd1 ; set ccd_frame(1,yd) $ccdsd2

     # rotate the CCD 
     set d1 [expr $ccx + ($mos(0,xe)-$ccx)*$cda - ($mos(0,ye)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xe)-$ccx)*$sda + ($mos(0,ye)-$ccy)*$cda]
     set mos(0,xe) $d1; set mos(0,ye) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xe)-$ccx)*$cda - ($ccd_frame(0,ye)-$ccy)*$sda]
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xe)-$ccx)*$sda + ($ccd_frame(0,ye)-$ccy)*$cda]
     set ccd_frame(0,xe) $ccdmd1 ; set ccd_frame(0,ye) $ccdmd2
     
     set ccdsd1 [expr $ccx + ($ccd_frame(1,xe)-$ccx)*$cda - ($ccd_frame(1,ye)-$ccy)*$sda]
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xe)-$ccx)*$sda + ($ccd_frame(1,ye)-$ccy)*$cda]
     set ccd_frame(1,xe) $ccdsd1 ; set ccd_frame(1,ye) $ccdsd2

     set d1 [expr $ccx + ($mos(0,xf)-$ccx)*$cda - ($mos(0,yf)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xf)-$ccx)*$sda + ($mos(0,yf)-$ccy)*$cda]
     set mos(0,xf) $d1; set mos(0,yf) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xf)-$ccx)*$cda - ($ccd_frame(0,yf)-$ccy)*$sda]
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xf)-$ccx)*$sda + ($ccd_frame(0,yf)-$ccy)*$cda]
     set ccd_frame(0,xf) $ccdmd1 ; set ccd_frame(0,yf) $ccdmd2
     
     set ccdsd1 [expr $ccx + ($ccd_frame(1,xf)-$ccx)*$cda - ($ccd_frame(1,yf)-$ccy)*$sda]
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xf)-$ccx)*$sda + ($ccd_frame(1,yf)-$ccy)*$cda]
     set ccd_frame(1,xf) $ccdsd1 ; set ccd_frame(1,yf) $ccdsd2

     set d1 [expr $ccx + ($mos(0,xg)-$ccx)*$cda - ($mos(0,yg)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xg)-$ccx)*$sda + ($mos(0,yg)-$ccy)*$cda]
     set mos(0,xg) $d1; set mos(0,yg) $d2

      set ccdmd1 [expr $ccx + ($ccd_frame(0,xg)-$ccx)*$cda - ($ccd_frame(0,yg)-$ccy)*$sda]
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xg)-$ccx)*$sda + ($ccd_frame(0,yg)-$ccy)*$cda]
     set ccd_frame(0,xg) $ccdmd1 ; set ccd_frame(0,yg) $ccdmd2
     
     set ccdsd1 [expr $ccx + ($ccd_frame(1,xg)-$ccx)*$cda - ($ccd_frame(1,yg)-$ccy)*$sda]
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xg)-$ccx)*$sda + ($ccd_frame(1,yg)-$ccy)*$cda]
     set ccd_frame(1,xg) $ccdsd1 ; set ccd_frame(1,yg) $ccdsd2

     set d1 [expr $ccx + ($mos(0,xh)-$ccx)*$cda - ($mos(0,yh)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xh)-$ccx)*$sda + ($mos(0,yh)-$ccy)*$cda]
     set mos(0,xh) $d1; set mos(0,yh) $d2

      set ccdmd1 [expr $ccx + ($ccd_frame(0,xh)-$ccx)*$cda - ($ccd_frame(0,yh)-$ccy)*$sda]
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xh)-$ccx)*$sda + ($ccd_frame(0,yh)-$ccy)*$cda]
     set ccd_frame(0,xh) $ccdmd1 ; set ccd_frame(0,yh) $ccdmd2
     
     set ccdsd1 [expr $ccx + ($ccd_frame(1,xh)-$ccx)*$cda - ($ccd_frame(1,yh)-$ccy)*$sda]
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xh)-$ccx)*$sda + ($ccd_frame(1,yh)-$ccy)*$cda]
     set ccd_frame(1,xh) $ccdsd1 ; set ccd_frame(1,yh) $ccdsd2

     # rotate the reduced mos-frame 
     set d1 [expr $ccx + ($mos(0,xar)-$ccx)*$cda - ($mos(0,yar)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xar)-$ccx)*$sda + ($mos(0,yar)-$ccy)*$cda]
     set mos(0,xar) $d1; set mos(0,yar) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xar)-$ccx)*$cda - ($ccd_frame(0,yar)-$ccy)*$sda]        
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xar)-$ccx)*$sda + ($ccd_frame(0,yar)-$ccy)*$cda]
     set ccd_frame(0,xar) $ccdmd1; set ccd_frame(0,yar) $ccdmd2

     set ccdsd1 [expr $ccx + ($ccd_frame(1,xar)-$ccx)*$cda - ($ccd_frame(1,yar)-$ccy)*$sda]        
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xar)-$ccx)*$sda + ($ccd_frame(1,yar)-$ccy)*$cda]
     set ccd_frame(1,xar) $ccdsd1; set ccd_frame(1,yar) $ccdsd2

     set d1 [expr $ccx + ($mos(0,xbr)-$ccx)*$cda - ($mos(0,ybr)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xbr)-$ccx)*$sda + ($mos(0,ybr)-$ccy)*$cda]
     set mos(0,xbr) $d1; set mos(0,ybr) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xbr)-$ccx)*$cda - ($ccd_frame(0,ybr)-$ccy)*$sda]     
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xbr)-$ccx)*$sda + ($ccd_frame(0,ybr)-$ccy)*$cda]
     set ccd_frame(0,xbr) $ccdmd1 ; set ccd_frame(0,ybr) $ccdmd2

     set ccdsd1 [expr $ccx + ($ccd_frame(1,xbr)-$ccx)*$cda - ($ccd_frame(1,ybr)-$ccy)*$sda]     
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xbr)-$ccx)*$sda + ($ccd_frame(1,ybr)-$ccy)*$cda]
     set ccd_frame(1,xbr) $ccdsd1 ; set ccd_frame(1,ybr) $ccdsd2

     set d1 [expr $ccx + ($mos(0,xcr)-$ccx)*$cda - ($mos(0,ycr)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xcr)-$ccx)*$sda + ($mos(0,ycr)-$ccy)*$cda]
     set mos(0,xcr) $d1; set mos(0,ycr) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xcr)-$ccx)*$cda - ($ccd_frame(0,ycr)-$ccy)*$sda]  
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xcr)-$ccx)*$sda + ($ccd_frame(0,ycr)-$ccy)*$cda] 
     set ccd_frame(0,xcr) $ccdmd1 ; set ccd_frame(0,ycr) $ccdmd2

     set ccdsd1 [expr $ccx + ($ccd_frame(1,xcr)-$ccx)*$cda - ($ccd_frame(1,ycr)-$ccy)*$sda]  
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xcr)-$ccx)*$sda + ($ccd_frame(1,ycr)-$ccy)*$cda] 
     set ccd_frame(1,xcr) $ccdsd1 ; set ccd_frame(1,ycr) $ccdsd2

     set d1 [expr $ccx + ($mos(0,xdr)-$ccx)*$cda - ($mos(0,ydr)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xdr)-$ccx)*$sda + ($mos(0,ydr)-$ccy)*$cda]
     set mos(0,xdr) $d1; set mos(0,ydr) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xdr)-$ccx)*$cda - ($ccd_frame(0,ydr)-$ccy)*$sda]    
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xdr)-$ccx)*$sda + ($ccd_frame(0,ydr)-$ccy)*$cda] 
     set ccd_frame(0,xdr) $ccdmd1 ; set ccd_frame(0,ydr) $ccdmd2

     set ccdsd1 [expr $ccx + ($ccd_frame(1,xdr)-$ccx)*$cda - ($ccd_frame(1,ydr)-$ccy)*$sda]    
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xdr)-$ccx)*$sda + ($ccd_frame(1,ydr)-$ccy)*$cda] 
     set ccd_frame(1,xdr) $ccdsd1 ; set ccd_frame(1,ydr) $ccdsd2

     # rotate the reduced CCD 
     set d1 [expr $ccx + ($mos(0,xer)-$ccx)*$cda - ($mos(0,yer)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xer)-$ccx)*$sda + ($mos(0,yer)-$ccy)*$cda]
     set mos(0,xer) $d1; set mos(0,yer) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xer)-$ccx)*$cda - ($ccd_frame(0,yer)-$ccy)*$sda]     
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xer)-$ccx)*$sda + ($ccd_frame(0,yer)-$ccy)*$cda] 
     set ccd_frame(0,xer) $ccdmd1 ; set ccd_frame(0,yer) $ccdmd2

     set ccdsd1 [expr $ccx + ($ccd_frame(1,xer)-$ccx)*$cda - ($ccd_frame(1,yer)-$ccy)*$sda]     
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xer)-$ccx)*$sda + ($ccd_frame(1,yer)-$ccy)*$cda] 
     set ccd_frame(1,xer) $ccdsd1 ; set ccd_frame(1,yer) $ccdsd2

     set d1 [expr $ccx + ($mos(0,xfr)-$ccx)*$cda - ($mos(0,yfr)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xfr)-$ccx)*$sda + ($mos(0,yfr)-$ccy)*$cda]
     set mos(0,xfr) $d1; set mos(0,yfr) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xfr)-$ccx)*$cda - ($ccd_frame(0,yfr)-$ccy)*$sda]     
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xfr)-$ccx)*$sda + ($ccd_frame(0,yfr)-$ccy)*$cda] 
     set ccd_frame(0,xfr) $ccdmd1; set ccd_frame(0,yfr) $ccdmd2

     set ccdsd1 [expr $ccx + ($ccd_frame(1,xfr)-$ccx)*$cda - ($ccd_frame(1,yfr)-$ccy)*$sda]     
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xfr)-$ccx)*$sda + ($ccd_frame(1,yfr)-$ccy)*$cda] 
     set ccd_frame(1,xfr) $ccdsd1; set ccd_frame(1,yfr) $ccdsd2

     set d1 [expr $ccx + ($mos(0,xgr)-$ccx)*$cda - ($mos(0,ygr)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xgr)-$ccx)*$sda + ($mos(0,ygr)-$ccy)*$cda]
     set mos(0,xgr) $d1; set mos(0,ygr) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xgr)-$ccx)*$cda - ($ccd_frame(0,ygr)-$ccy)*$sda]     
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xgr)-$ccx)*$sda + ($ccd_frame(0,ygr)-$ccy)*$cda] 
     set ccd_frame(0,xgr) $ccdmd1; set ccd_frame(0,ygr) $ccdmd2

     set ccdsd1 [expr $ccx + ($ccd_frame(1,xgr)-$ccx)*$cda - ($ccd_frame(1,ygr)-$ccy)*$sda]     
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xgr)-$ccx)*$sda + ($ccd_frame(1,ygr)-$ccy)*$cda] 
     set ccd_frame(1,xgr) $ccdsd1; set ccd_frame(1,ygr) $ccdsd2

     set d1 [expr $ccx + ($mos(0,xhr)-$ccx)*$cda - ($mos(0,yhr)-$ccy)*$sda]
     set d2 [expr $ccy + ($mos(0,xhr)-$ccx)*$sda + ($mos(0,yhr)-$ccy)*$cda]
     set mos(0,xhr) $d1; set mos(0,yhr) $d2

     set ccdmd1 [expr $ccx + ($ccd_frame(0,xhr)-$ccx)*$cda - ($ccd_frame(0,yhr)-$ccy)*$sda]     
     set ccdmd2 [expr $ccy + ($ccd_frame(0,xhr)-$ccx)*$sda + ($ccd_frame(0,yhr)-$ccy)*$cda] 
     set ccd_frame(0,xhr) $ccdmd1; set ccd_frame(0,yhr) $ccdmd2

     set ccdsd1 [expr $ccx + ($ccd_frame(1,xhr)-$ccx)*$cda - ($ccd_frame(1,yhr)-$ccy)*$sda]     
     set ccdsd2 [expr $ccy + ($ccd_frame(1,xhr)-$ccx)*$sda + ($ccd_frame(1,yhr)-$ccy)*$cda] 
     set ccd_frame(1,xhr) $ccdsd1; set ccd_frame(1,yhr) $ccdsd2

     # update echs slits  targw
     # if {[string compare echs [$this cget -modus]]==0} { 
     # $m_image convert coords $A $B canvas targw(10,a) targw(10,d) "wcs $EQX" 
     #   }

     if {[string compare $mask "mxu"]==0} {
          # update mxu focal field <-> canvas transformation matrix
       fors::MXU_SLIT::coeff $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                             $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                             $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc) 
         set noms [fors::MXU_SLIT::nos]
         if {[fors::MXU_SLIT::get_mmff]} {
            # do nothing for standard masks
            } else {
         # delete all mxu slits if there are any
         if {$noms>0} {
            for {set i 1} {$i <= $noms} { incr i 1} { 
		if {$i > $nf_refslit} {
		    delete object C_mxus$i 
		}
		}
            }
            }
          }

     # delete the object and plot_mask it 
     catch {$w delete A}
     catch {$w delete D}
     plot_mask; #  <------------------------ 

    if {$badpixelmap} {
    fors::CONV_MC::coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                      $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                      $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc) 
     update_BPM
     }

     # puts stdout [format "Rotate Mask around %5.1f  %5.1f from \
     # %3.1f by %3.1f to %3.1f  (degr)"  $ccx $ccy \
     # [expr $abs_ang_rad/0.0174532-$dir*$rotangle] \
     # [expr $dir*$rotangle] \
     # [expr $abs_ang_rad/0.0174532] ] 

     #  puts stdout [format "Rotate Mask around %5.1f  %5.1f from \
     #  %5.1f by %5.1f to %5.1f  (PA )"  $ccx $ccy \
     #  [expr $PAM-$dir*$rotangle*$CSO] \
     #  [expr $dir*$rotangle*$CSO] \
     #  [expr $PAM] ] 

     set last_action "Rotate Mask"
     catch {SP update}
     }

     #===========================
     private method calc_dither { c frac } {
     #===========================
     # for polarimetric imaging IPOL 
     # this method moves the mask one stripe further
     # highres  = 11.36''
     # standard = 22.73''
     # c =+-1 = up down
     # frac = 1        : 2 exposures, shift = '1' *  22.73 ''
     # frac = 0.666666 : 3 exposures, shift = '2/3' *22.73'

     # take the half for highres collimator 
     if {[string match *HR* [$this cget -collimat] ] } {
       set dith [expr 22.73/3600.0/2.0 * $frac ] 
         } else { set dith [expr 22.73/3600.0 * $frac ] }
     # puts stdout "Dither : shift by $dith deg, PAM = $PAM  deg"

     # deg to rad for sin/cos 
     set pa [expr $PAM*0.0174532] 
     if { $c < 0 } { set pa [expr ( $PAM + 180.0 ) * 0.0174532 ] }

     # puts stdout [ format \
     # "w-%8.3f %8.3f c-%8.3f %8.3f m-%8.3f %8.3f o-%8.3f %8.3f" \
     #  $world_x $world_y $ccx $ccy $mos(0,x0) $mos(0,y0) $oldpos(x) $oldpos(y)]

     $m_image convert coords $mos(0,x0) $mos(0,y0) canvas a0 d0 "wcs $EQX"
     $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x1 y1 "deg $EQX"
     # $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x2  y2 canvas
     # $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x3  y3 screen 
     # $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x4  y4 image 

     puts stdout [ format "Di old: %10s  %10s %8.5f %8.5f" $a0 $d0 $x1 $y1 ]

     # calculation, linear scale 
     $m_image convert coords $mos(0,x0) $mos(0,y0) canvas a0 d0 "deg $EQX"
     set a1 [expr $a0 + $dith * sin($pa)/cos($d0*0.0174532) ]
     set d1 [expr $d0 + $dith * cos($pa)]
     #
     set dither_offset_a [ expr $dith * sin($pa)/cos($d0*0.0174532) ]
     set dither_offset_d [ expr $dith * cos($pa)]
     # puts stdout "offset RA  = $dither_offset_a "
     # puts stdout "offset DEC = $dither_offset_d"
     # offset from deg to arcsec
     set dither_offset_a [expr $dither_offset_a*3600.0/15.0/cos($d0*0.0174532)]
     set dither_offset_d [expr $dither_offset_d*3600.0 ]

     $m_image convert coords $a1 $d1 "deg $EQX" a0 d0 "wcs $EQX"
     puts stdout [ format "Di new: %10s  %10s %8.5f %8.5f" $a0 $d0 $a1 $d1 ]
     # $m_image convert coords $a1 $d1 "deg $EQX" x2 y2 canvas
     # $m_image convert coords $a1 $d1 "deg $EQX" x3 y3 screen 
     # $m_image convert coords $a1 $d1 "deg $EQX" x4 y4 image

     puts stdout [ format "SEQ.OFF.ALPHA  = %8.5f''"  $dither_offset_a ]
     puts stdout [ format "SEQ.OFF.DELTA  = %8.5f''"  $dither_offset_d ]
     if {$frac<0.9} { set number 2 } else { set number 1 }
     puts stdout [ format "SEQ.OFF.NUMBER =  %1d"  $number ]

     # puts stdout [ format "new TCS : %10s  %10s %8.5f %8.5f" $a0 $d0 $a1 $d1 ]
     return "$a1 $d1 $dither_offset_a $dither_offset_d"
     }

     #===========================
     private method dither_pol { w c frac } {
     #===========================
     # do not dither when specifying reference targets 
     if {$stop} { return }

     # for polarimetric imaging IPOL 
     # this moves the mask one stripe further
     # highres  = 11.36''
     # standard = 22.73''
     # frac = 1        : 2 exposures, shift = '1' * 22.73''
     # frac = 0.666666 : 3 exposures, shift = '2/3' *22.73''

     # ---------------------------------
     set result [calc_dither $c $frac]
     # ---------------------------------
     if {$frac<0.9} { set dith_num 2 } else { set dith_num 1 } 
     # puts stdout ">$result<"
     set result [split $result " " ]
     set a0 [lindex $result 0];      # wcs 
     set d0 [lindex $result 1];      # wcs
     # set off_a [lindex $result 2];   # arcsec
     # set off_d [lindex $result 3];   # arcsec 
     # puts stdout "result = >$a0< >$d0< >$off_a< >$off_d<"

     # we shift the mask without mark
     set oldpos(x) $mos(0,x0); set oldpos(y) $mos(0,y0)

     # $m_image convert coords $a0 $d0 "deg $EQX" a0 d0 "wcs $EQX"
     # $m_image convert coords $a0 $d0 "deg $EQX" x2 y2 canvas
       $m_image convert coords $a0 $d0 "deg $EQX" x3 y3 screen 
     # $m_image convert coords $a0 $d0 "deg $EQX" x4 y4 image

     # puts stdout [ format \
     # "w-%8.3f %8.3f c-%8.3f %8.3f m-%8.3f %8.3f o-%8.3f %8.3f" \
     # $world_x $world_y $ccx $ccy $mos(0,x0) $mos(0,y0) $oldpos(x) $oldpos(y)]
     # puts stdout "   --- shift ---"
     # ------------------------------
     [code $this shift $w $x3 $y3 ]
     # ------------------------------

     # puts stdout [ format \
     # "w-%8.3f %8.3f c-%8.3f %8.3f m-%8.3f %8.3f o-%8.3f %8.3f" \
     # $world_x $world_y $ccx $ccy $mos(0,x0) $mos(0,y0) $oldpos(x) $oldpos(y)]

     # ------------------------------
     update_and_replot
     # ------------------------------

     # puts stdout [ format "%8.3f  %8.3f -- %8.3f  %8.3f -- %8.3f %8.3f" \
     #    $world_x $world_y $ccx $ccy $mos(0,x0) $mos(0,y0)]
     set last_action [format "Dither Y %5.3f" [expr $c * $frac] ]
     catch {SP update}
     }


     #===========================
     private method bind_frame { } {
     #===========================
     # tag m = green circle, tag C = blue CCD window 
     # if mouse in m (green central circle) then mark

     $m_canvas bind m <Button-1>        [code $this mark %W %x %y]
     $m_canvas bind m <B1-Motion>       [code $this shift %W %x %y]

     if {[string compare [$this cget -modus] echs]==0} {
     $m_canvas bind m <ButtonRelease-1> [code $this echs_targ]
     } else {
     # only MOS ? 
     $m_canvas bind m <ButtonRelease-1> [code $this reset_slits]
     }

     # if mouse is in mc and clicked, rotate clockwise
     $m_canvas bind m <Button-2> [code $this rotate %W %x %y -1]
     # if mouse is in mc and shift-b2 clicked, rotate anti-clock-wise
     $m_canvas bind m <Shift-Button-2> [code $this rotate %W %x %y 1]

     # ccd window frame 
     # $m_canvas bind C <Button-1>        [code $this test_2]
     puts $lfId "$this bind frame"
     }

     #===========================
     private method bind_mos { } {
     #===========================
     # if mouse is on any object
     # set 
     $m_canvas bind mm <Button-1> [code $this move_slit %W %x %y 1 1 ]

     # scratch 
     $m_canvas bind lb  <Button-2> [code $this move_slit %W %x %y 0 1 ]
     $m_canvas bind out <Button-1> [code $this move_slit %W %x %y 0 1 ]

     # change individual MOS slit width 
     $m_canvas bind sw <Button-1> [code $this indi_slit_width %W %x %y ]
     }


     #===========================
     private method bind_img { } {
     #===========================
     # if mouse is on any object
     # the last arg for mosa or mosb tag

       # B1 inside slit, move blade A, otherwise default blade 
       $m_canvas bind mm  <Button-1> [code $this move_slit_blade %W %x %y 1]

       # B2 inside slit, move blade B, otherwise default blade 
       $m_canvas bind mm  <Button-2> [code $this move_slit_blade %W %x %y 0]

       # B1 on slit rim label or B2 on slit label : out of focal field
       $m_canvas bind out <Button-1> [code $this move_slit_blade %W %x %y out]
       $m_canvas bind lb  <Button-2> [code $this move_slit_blade %W %x %y out]

       # individual slit width 
       $m_canvas bind sw  <Button-1> [code $this indi_slit_width %W %x %y ]

     puts $lfId "$this bind img"
     }


     #===========================
     private method bind_ipol { } {
     #===========================
     bind_img 
     # white  circle : dither 1 MOS slit width 
     $m_canvas bind P2 <Button-1>       [code $this dither_pol %W  1 1.0 ]
     $m_canvas bind P2 <Shift-Button-1> [code $this dither_pol %W -1 1.0 ]

     # grey  circle : dither 2/3 MOS slit width
     $m_canvas bind P3 <Button-1>       \
                             [code $this dither_pol %W  1 [expr 2.0/3.0] ]
     $m_canvas bind P3 <Shift-Button-1> \
                             [code $this dither_pol %W -1 [expr 2.0/3.0] ]
     puts $lfId "$this bind ipol"
     }

     #===========================
     private method bind_echs { } {
     #===========================
     # the cyan circle is used to (de)-activate the MOS 9 slit 
     $m_canvas bind PE <Shift-Button-1> [code $this flip_echs ]
     $m_canvas bind PE <Shift-Button-1> [code $this flip_echs ]
     }

     #===========================
     private method bind_mxu { } {

     #===========================
     $m_canvas bind mxu <Button-1> [code $this mxu_create_slit %W %x %y 0 ]

     # click on the slit width label (.e.g  '1.3"') to change individual 
     # MXU slit width 
     $m_canvas bind sw <Button-1> [code $this indi_slit_mxu %W %x %y ]

     # click on slit label (e.g. '#15') at the rim of the focal field to
     # delete mxu slit
     $m_canvas bind kill <Button-1> [code $this delete_slit_mxu %W %x %y ]

     # click on the slit label (e.g. '#15') at the slit to assign slit as 
     # reference slit for 'Fast Alignment' 
     #$m_canvas bind rs <Button-1> [code $this ref_slit_mxu %W %x %y 1 ]
     #$m_canvas bind rs <Button-2> [code $this ref_slit_mxu %W %x %y 0 ]

     # scratch 
     # AAG, De-Select mxu slit
     $m_canvas bind rs <Button-2> [code $this delete_slit_mxu %W %x %y ]

     # for the bezier points 
     $m_canvas bind door <Button-1>        [code $this mark_cms %W %x %y]
     $m_canvas bind door <B1-Motion>       [code $this shift_cms %W %x %y]
     $m_canvas bind door <ButtonRelease-1> [code $this end_cms %W %x %y]
     }

     #===========================
     private method bind_align { } {
     #===========================
     # for hits, lss and echs we want to shift the mask 
     # that the slit is on the center/gauss - result 
     # in case the center/slit option is on 
     $m_canvas bind ali <Button-1> [code $this align_mask %W %x %y ]
     }

     #===========================
     private method flip_echs { } { 
     #===========================
     # ECHS uses either MOS slit 10 alone (echs_flag==0)  
     # or MOS 10 and 9 together (echs_flag==1)  
     # puts stdout "flip_echs: [$this cget -grism]" 
 
     if {$echs_flag} {set echs_flag 0 
                      set targw(9,f) 0
                     } else { 
                      set echs_flag 1
                      set targw(9,f) 1
                     }

     # for status panel 
     catch {SP update}

     update_and_replot 
     # plot first echs slit 9, then position 
     if {$echs_flag} { echs_pos }
     }

     #===========================
     private method center_slit { } { 
     #===========================
     set lss_sli_ra ""; set lss_sli_de  ""
     # set w [utilNamespaceTail $this]
     set w .skycat1
     set w [$w component image]
     $w pick_dialog [code $this extra_pick ] 
     }

     #===========================
     public  method AutoPos { kind } {
     #===========================
     # 1) create file AutoPos.asc
     # 2) call FIMSROOT/bin/AP
 
     set debug 0 

     puts stdout "Hello in AutoPos -Method = $kind"
     # kind may be static, shift, rotate, all

     # select one of the local catalogs 
     set a_c [get_cat] 

     if {$a_c==""} { info_dialog "No catalog specified" .skycat1
                     return
                   }
    
     # test flag  
     set p 0

     set loc_list [lindex [$a_c info variable info_] 4 ]
     # do not process empty list
     if {$loc_list == ""} { 
        set p 1 
        if {$debug} { puts stdout "AutoPos : Error catalog empty "
                    }
        } else {
        if {$debug} { puts stdout "AutoPos : Hurray catalog not empty"
                    }
        }


     #--SET Priority-Mag-Flag

     # process headings; whats available
     set loc_head [$a_c info variable headings_]
     set loc_head [lindex $loc_head 4]
     if {$debug} { puts stdout "AutoPos : catalog_head = >$loc_head<" }

     # search in table header
     # mandatory keywords 

     # several tries 
     set i_ra  [lsearch -glob  $loc_head *ra*]
     if {$i_ra == -1} { set i_ra  [lsearch -glob  $loc_head *RA*] }
     if {$i_ra == -1} { set i_ra  [lsearch -glob  $loc_head *ra*] }
     if {$i_ra == -1} { puts stdout "AutoPos : Error catalog header RA" 
                        set p 1 }

     set i_dec [lsearch -glob  $loc_head *dec*]
     if {$i_dec == -1} { set i_dec  [lsearch -glob  $loc_head *DEC*] }
     if {$i_dec == -1} { set i_dec  [lsearch -glob  $loc_head *Dec*] }
     if {$i_dec == -1} { puts stdout "AutoPos : Error catalog header DEC" 
                         set p 1 }

     set i_id  [lsearch -glob  $loc_head *id*]
     if {$i_id == -1} { set i_id  [lsearch -glob  $loc_head *ID*] }
     if {$i_id == -1} { set i_id  [lsearch -glob  $loc_head *Id*] }
     if {$i_id == -1} { set i_id  [lsearch -glob  $loc_head *name*] }
     if {$i_id == -1} { set i_id  [lsearch -glob  $loc_head *target*] }
     if {$i_id == -1} { set i_id  [lsearch -glob  $loc_head *star*] }
     if {$i_id == -1} { set i_id  [lsearch -glob  $loc_head *object*] }
     if {$i_id == -1} { puts stdout "AutoPos : Error catalog header ID" 
                        set i_id 0; # assuming the first entry is the ID
                        set p 1 }


     if {$debug} {
     if {!$p} { puts stdout "AutoPos : Hurray catalog header "
              }
              }
           
     # optional, keywords  
     set p 0
     set i_mag  [lsearch -glob  $loc_head *mag*]
     if {$i_mag == -1} { set i_mag [lsearch -glob  $loc_head *MAG*] }
     if {$i_mag == -1} { set i_mag [lsearch -glob  $loc_head *Mag*] }
     if {$i_mag == -1} { puts stdout "AutoPos : Warning catalog header MAG" 
                         set p 1 }

     set i_prio [lsearch -glob  $loc_head *pri*]
     if {$i_prio == -1} { set i_prio [lsearch -glob  $loc_head *PRI*] }
     if {$i_prio == -1} { set i_prio [lsearch -glob  $loc_head *Pri*] }
     if {$i_prio == -1} { puts stdout "AutoPos : Warning catalog header PRIO" 
                          set p 1 }

     if {$debug} {
        if {!$p} { puts stdout "AutoPos : MAG and PRIO found in catalog header"
                 } 
                 }



     if {$i_prio >=0 } { set mp_header "1" } else { set mp_header "0" }
     if {$i_mag >=0 } { set mp_header 1$mp_header } else {
               set mp_header 0$mp_header }

     # puts stdout "$i_mag  $i_prio $mp_header" 

     # -- Frame Rotation -- Scale
     # WP = angle between Frame (pixel-coords) and WCS = CROTA2 
     # WP > 0 means north counterclock, independent on E-axis 
     # WP * CSO = mathematical rotation angle  
     #
     # Note: Position angle (PA) is a direction in the WCS
     #       it is measured from N to E
     #
     # angle between mask and wcs = abs_ang_rad =  PAM
     # = position angle of the mask with respect to the WCS
     # Since the WCS can be right or left oriented we need 
     # CSO in order to determine PAM  
     # PAM (position angle of the mask)
     # 
     # fits_scale= ++ || --           -------------
     # angle between mask and pixel = MP = WP + PAM
     #                                -------------
     # fits_scale= +- || -+           -------------
     # angle between mask and pixel = MP = WP - PAM
     #                                -------------
     # WP and PAM are given to Autopos
     # Autopos returns the position angle = PAM  in 
     # the target acquisition file opt1.targ

     set fitsheader [$m_image wcsset]
     if {$debug} { 
     puts stdout "AutoPos : fits header = $fitsheader"
     puts stdout "AutoPos : scale = [lindex $fitsheader 2] arcsec/pixel"
     puts stdout "AutoPos : rot   = [lindex $fitsheader 7] deg"
     }
     set WP [lindex $fitsheader 7]; # CROTA2 = World-Pixel 
     if {$debug} { 
     puts stdout "AutoPos: >[lindex $fitsheader 10]<" 
     }   

     # MP = mask-pixel angle
     # PAM = mask-wcs angle  (=position angle of instrument)
     # WP = wcs-pixel angle (=fits keyword CROTA2)
     # give AutoPos neg MW, since AutoPos uses mathemat. angles
     # set MP [expr -$WP*$CSO - $PAM ]
     set MP [expr $WP - $PAM*$CSO ]; # mathemat mask angle 

     if {$debug} { puts stdout "AutoPos : WP= $WP fixed PAM= $PAM MP= $MP" }

     set myfile $log_dir/AutoPos.asc
     puts stdout "AutoPos : >$myfile<" 
     set fileId [open $myfile w ]
     puts stdout "AutoPos : open $myfile" 

    # we need to know the collimator, here as 
    # high-res collimator flag 
    if {[string match *HR* [$this cget -collimat] ] } {
       set sc 1 } else { set sc 0 }
     
    # CCD -- Window 

   # furthermore the CCD-window del_ra, del_dec 
     if {[string match *HR* [$this cget -collimat] ] } {
     set del_dec [expr ($YHRMAX-$YHRMIN)*$focscale]
     } else {
     set del_dec [expr ($YSRMAX-$YSRMIN)*$focscale]
     }

     #
     if {[string match *HR* [$this cget -collimat] ] } {
     # Assume dispersion direction BLUE=E=B=x>0, RED=W=A=x<0
     set ra_min [expr ($XHRMIN+$specmax/2)*$focscale]
     set ra_max [expr ($XHRMAX-$specmin/2)*$focscale]
     set hrcolli 1
     # Assume dispersion direction RED=E=B=x>0, BLUE=W=A=x<0
     # set ra_min [expr ($XHRMIN+$specmin/2)*$focscale]
     # set ra_max [expr ($XHRMAX-$specmax/2)*$focscale]
     } else {
     # Assume dispersion direction BLUE=E=B=x>0, RED=W=A=x<0
     set ra_min [expr ($XSRMIN+$specmax)*$focscale]
     set ra_max [expr ($XSRMAX-$specmin)*$focscale]
     set hrcolli 0
     # Assume dispersion direction RED=E=B=x>0, BLUE=W=A=x<0
     # set ra_min [expr ($XSRMIN+$specmin)*$focscale]
     # set ra_max [expr ($XSRMAX-$specmax)*$focscale]
     }


     # convert the current mask position in wcs and pixel
     $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
         lwx lwy "deg $EQX"
     $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
         px py image 
 
     # Control parameter

	# AAG - PPRS 10127
	# Added 'simple' option

      switch -exact -- $kind {
      static {set co 0}
      shift  {set co 1}
      rotat  {set co 2}
      all    {set co 3}
      simple {set co 4}
         }

     switch -exact -- [$this cget -modus] {
     mos { set mosp 0 }
    pmos { set mosp 1 }
     mxu { set mosp 2 }
    default { puts stdout "AutoPos : invalid mode" }
       }

    # puts stdout "modus = [$this cget -modus], mosp = $mosp"
    # puts stdout "FORS  = [string index [$this cget -instrument] 4 ]"

     # mask_frame angle

      set big_string [format "%1d %1d %2d %1d %8.6f %1d %1d %6.3f %6.3f " \
          $exposure_counter \
          [string index [$this cget -instrument] 4] \
          $mp_header $co $focscale \
           $mosp $hrcolli [lindex $fitsheader 2] $WP ]

      if {$debug} { puts stdout "big_string= $big_string" }

      set app_string [format "%6.3f %10.8f %10.8f %6.3f %6.3f %6.3f " \
          $PAM $lwx $lwy $px $py $del_dec ]

      if {$debug} { puts stdout "app_string= $app_string" }

       # in mos default slit length is 'real', although slit_length is not 
       # used for mos, only in mxu, where real is omitted
       if {[string match *real* [$this cget -slit_length] ]} {
           set sl 22.0 } else { set sl [$this cget -slit_length] }

       # puts stdout "sw, sl = [$this cget -slit_width], $sl"


      set ap2_string [format "%6.3f %6.3f %4.1f %4.1f %6.3f %6.3f" \
       $ra_min $ra_max $fits_x_scale $fits_y_scale \
       [$this cget -slit_width] $sl ]

      if {$debug} { puts stdout "ap2_string= $ap2_string" }

      if {$debug} { 
      puts stdout "ecmp K B P C   scal   CROT    PA     wx    wy       px    py    dec      mi/ma/ra ori"
      puts stdout  "$big_string $app_string $ap2_string"
      }
      puts $fileId "$big_string $app_string $ap2_string"



      set counter 0
     if {$debug} { puts stdout ">>$loc_list<<" }
     foreach star $loc_list {
       incr counter 1
       # puts stdout "$star >[lindex $star $i_ra]< >[lindex $star $i_dec]<"
       set px 0; set py 0; set alp 0; set dec 0;
       catch {$m_image convert coords \
              [lindex $star $i_ra] [lindex $star $i_dec] "wcs $EQX" \
              alp dec "deg $EQX" }
       catch {$m_image convert coords \
              [lindex $star $i_ra] [lindex $star $i_dec] "wcs $EQX" \
              px py image }
       # puts stdout "a=$alp d=$dec px=$px py=$py"
       set id [lindex $star $i_id]
       # target name strings must not contain blanks, hence fill them with "_"
       set id [string_exchange $id " " "_" ]



     set plotstring [format "%s %3.6f %3.6f %6.2f %6.2f" $id $alp $dec $px $py]
       if {$i_mag >=0} { set plotstring "$plotstring [lindex $star $i_mag]" }
       if {$i_prio >=0} { set plotstring "$plotstring [lindex $star $i_prio]" }

       # now all together
       if {$debug} { puts stdout "AutoPos: $counter $plotstring" }
       # consider only targets with correctly converted coords
       if {$px>0} {puts $fileId $plotstring }
       # puts stdout $plotstring 
       }
     close $fileId

     puts stdout "AutoPos : close $myfile"

     set exfile $int_dir 
     append exfile "/bin/AP"

     set source_file "AutoPos.asc"

     puts stdout "AutoPos : start external $exfile $source_file ..."
     #
     set result [exec $exfile $source_file]
     puts stdout "AutoPos : receive result : $result"
     puts stdout "AutoPos : load setupfile ... end"

     switch -exact -- [$this cget -modus] {
    pmos { load_mos_TPX  "" }
     mos { load_mos_TPX  "" }
     mxu { load_mxu_TPX  "" }
     }

     # autpos has created a mos/mxu1/2.$expo_counter.auto_pos.fims file 
     # which should not be overwritten by the next autopos try

     incr exposure_counter 1
     }

     #===========================
     private method catalog { a_c } {
     #===========================
     set debug 0
     # 1) get the target list from the Astrocat widget
     # 2) compare entries with the list of slit positions 
     #    just selected
     # 3) delete the selected entries in the target list
     # 4) put reduced target list back to AstroCat widget
     # use two times the MOS slit width 
     # puts stdout "in catalog with $a_c" 
     if {[string compare [$this cget -slit_width] "open"]==0} { 
        set error_box 22.7
        } else {
        set error_box [expr [$this cget -slit_width] * 2.0 ]  
        }
     set error_box [expr $error_box * 2.7777778e-4]
     puts $lfId "CatalogInterface: ErrorBox =  \
           [expr $error_box / 2.7777778e-4] ''"
     # puts stdout "----------------------------"
     # if you use more than one catalog 
     
     
     # puts stdout "###->$a_c info class    = [$a_c info class]"
     # puts stdout "###->$a_c info inherit  = [$a_c info inherit]"
     # puts stdout "###->$a_c info heritage = [$a_c info heritage]"
     #----list the functions 
     # set f [$a_c info function]
     #    set f [split $f " "]
     #    set l [llength $f]
     #    for {set i 0} {$i< $l} { incr i 1} {
     #        if {[string match [lindex $f $i] *rem* ]==1} {
     #        puts stdout [lindex $f $i] 
     #        }
     #        }
     #---list the variables 
     # set f [$a_c info variable]
    #      set f [split $f " "]
    #      set l [llength $f]
    #      for {set i 0} {$i< $l} { incr i 1} {
    #          if {[string length [$a_c info variable [lindex $f $i] ] ]<100} {
    #           puts stdout [$a_c info variable [lindex $f $i] ] 
    #            }
    #          }
    #  puts stdout "###->$a_c configure = [$a_c configure]"
    #  puts stdout "###->$a_c itcl::find objects -class AstroCat = \
    #              [$a_c itcl::find objects -class AstroCat]"
    #  puts stdout "###->$a_c info variable n_instances_ = \
    #              [$a_c info variable n_instances_ ]"
    #  puts stdout "###->$a_c info variable instances_ = \
    #              [$a_c info variable instances_ ]"
    #  puts stdout "###->$a_c info variable astrocat_ = \
    #              [$a_c info variable astrocat_ ]"
    #  puts stdout "###->$a_c info variable instance_idx_ = \
    #              [$a_c info variable instance_idx_ ]"
    #  puts stdout "###->$a_c info variable catalog_menu_info_ = \
    #              [$a_c info variable count_ ]"
    #  puts stdout "###->$a_c info variable current_instance_ = \
    #              [$a_c info variable current_instance_]"
    # 
#
#      puts stdout "###->$a_c cget              = [$a_c cget ]"
#      puts stdout "###->$a_c cget -image       = [$a_c cget -image]"
#      puts stdout "###->$a_c cget -debug       = [$a_c cget -debug]"
#      puts stdout "###->$a_c cget -catalog     = [$a_c cget -catalog]"
#      puts stdout "###->$a_c cget -catalogtype = [$a_c cget -catalogtype]"
#      puts stdout "###->$a_c cget -tcs         = [$a_c cget -tcs]"
#      puts stdout "###->$a_c cget -transient   = [$a_c cget -transient]"
#      puts stdout "###->$a_c cget -center      = [$a_c cget -center]"
#      puts stdout "###->$a_c info function remove_FORS = \
#                  [$a_c info function remove_FORS]"
#      puts stdout "###->$a_c info function save_as = \
#                  [$a_c info function save_as]"
#     puts stdout "###->$a_c info variable info_ = \
#                 [$a_c info variable info_]"
#      puts stdout "###->$a_c  = [$a_c get_equinox]"
#      puts stdout "###->$a_c  = [$a_c select_columns]"
#      puts stdout "###->$a_c  = [$a_c select_result_row]"

     # there is no way to get the 'highlight objects from a catalog
     # maybe we have to plugin something to overwrite a SkyCat Method


     
     # 1) get the target list from the AstroCat widget
     # default, the first created AstroCat instance
     # puts stdout [set ac [itcl::find objects -class ::skycat::SkyCat] ]
     # search the current AstroCat instance

     set cur [$a_c info variable current_instance_]
     set hea [$a_c info variable headings_]
     set hea [lindex $hea 4]
     # puts stdout "cur = >>$cur<<"
     # puts stdout "hea = >>$hea<<"
     # set cur [lindex $cur 4]
     # puts stdout "cur = >>$cur<<"
     # set current AstroCat instance to the last 
     # cerated AstroCat instance 
     # this could be chnaged by a user-defined instance
     # selected by a menu button ??
     # set a_c cat::AstroCat::$cur
     set filename [lindex [$a_c info variable instance_idx_] 4]
     set filename [split $filename ","]
     set filename [lindex $filename 0]

     # puts stdout $filename 
     #puts stdout "###->$a_c cget -catalog     = [$a_c cget -catalog]"
     #puts stdout "###->CatalogInterface: catname=$a_c"
     #puts stdout "###->CatalogInterface: filename= \
     # [lindex [$a_c info variable instance_idx_] 4]"

     puts $lfId "CatalogInterface: catname  = $a_c"
     puts $lfId "CatalogInterface: filename = $filename"

     # puts stdout "CatalogInterface: catname  = $a_c"
     # puts stdout "CatalogInterface: filename = $filename" 
     set CatFileName $filename 

     
     set loc_list [$a_c info variable info_] 
     if {$debug} { puts stdout ">>$loc_list<<" }
     set loc_list [lindex $loc_list 4]
     # puts stdout ">>$loc_list<<" 

     # scratch those targets in the catalog
     # which are in mos

     # 2) compare the target list with the just selected
     # -------------------------------------------------
     #    slit positions 
     # -----------------
     if {$debug} { puts stdout "CatalogInterface: search in list ... "}
     set found_list ""
     set rest_list $loc_list
     set work_list $loc_list
     set found_counter 0
     # puts stdout "LOC_LIST  = >>$loc_list<<"
     # puts stdout "FOUND_LIST= >>$found_list<<"
     # puts stdout "REST_LIST = >>$rest_list<<"

     if {[string compare [$this cget -modus] mxu]==0} {
        set max [fors::MXU_SLIT::nos]
        } else {
        # this should be MOS 
        set max 19
        }


     for {set i 1} {$i <= $max} {incr i 1} {
         # all 19 slit blades, or all mxu-slits 


     if {[string compare [$this cget -modus] mxu]==0} {
        set on [C_mxus$i cget -ON]
        } else {
        # this should be MOS 
        set on  $targw($i,f)
        }

         if {$on} {
             if {$debug} { puts stdout "search for \#$i with $on"}
         # if the slit is explicitely selected
         # random coincidence with unused slits is not accepted 

         if {[string compare [$this cget -modus] mxu]==0} {
            set loc_RA  [C_mxus$i get_RA]
            set loc_DEC [C_mxus$i get_DEC]
            } else {
            # this should be MOS
            set loc_RA  $targw($i,a)
            set loc_DEC $targw($i,d)
            }

         $m_image convert coords $loc_RA  $loc_DEC "wcs $EQX" aa dd "deg $EQX"
         # aa dd are RA DEC of selected slit in deg 
         if {$debug} {puts stdout \
                     "\#$i $loc_RA $loc_DEC -> ($aa,$dd)"
                     }
         # now search in the target list for coord coincidence
         set list_counter 0
         set free 1
    foreach star $work_list {
            if {$free} {
                       if {$debug} { 
                       puts stdout $star
                       puts stdout "[lindex $star 1] [lindex $star 2]"
                       }
            $m_image convert coords [lindex $star 1] [lindex $star 2] \
            "wcs $EQX" alp dec "deg $EQX"
                if {$debug} { puts stdout "a=$alp d=$dec"}
            # alp dec are RA DEC of current test target
            # calculate distance between target and cat entry
            # assume simple linear scale, and constant cos(dec)
            set dec [expr $dec - $dd]
            set alp [expr ($alp - $aa)*cos($dd)]
                if {$debug} { puts stdout \
                              "Da= [expr $alp*3600]''  Dd=[expr $dec*3600]''"
                            }
            set dec [expr $dec*$dec]
            set alp [expr $alp*$alp]
            set dec [expr sqrt($dec+$alp)]
            if {$dec<$error_box } {
               # coord coincidence !!!!!!!!!!!!
               puts $lfId "CI: \#$i ($loc_RA,$loc_DEC) ($aa,$dd) with \
               [lindex $star 0] ([lindex $star 1],[lindex $star 2]) \
               within [expr $dec*3600] ''"
               set rest_list [lreplace $work_list $list_counter $list_counter]
                       if {$debug} { puts stdout \
                       "[llength $work_list] - 1 = [llength $rest_list]"
                       }
               #  element $list_c replaced by nothing  -> reduced list
               # transfer the target name
               if {[string compare [$this cget -modus] mxu]==0} {
                  C_mxus$i configure -target_name [lindex $star 0]
                  } else {
                  set targw($i,f) [lindex $star 0]
                  set targw($i,f) [string_exchange $targw($i,f) " " "_" ]
                  }
               # puts stdout $targw($i,f)
               # and add to found list
               lappend found_list [lindex $work_list $list_counter]
               incr found_counter 1
               set free 0 
               # dont assign further names to slit  
               }  ; # end coincidence 
            incr list_counter 1
            } ; # free 
            } ; # list loop (foreach star)
            # foreach star 

        # update the working list, reduce it by 1  
        set work_list $rest_list
        # keep this for later purposes
        # set index [lsearch $star high]
        # if {$index==-1} { set index [lsearch $star low] }
        # if {$index>1} {set prio [lindex $star $index]}
        }
        if {[string compare [$this cget -modus] mxu]==0} {
        puts stdout [ format "CatalogInterface: Slit \#%2d %s" $i \
                  [C_mxus$i cget -target_name] ]
                  } else {
        puts stdout [ format "CatalogInterface: Slit \#%2d %s" $i $targw($i,f)]
                  }
        }
        #puts stdout [$a_c component results remove_selected ]
        #puts stdout [$a_c search ]
        # puts stdout "LOC_LIST  = >>$loc_list<<"
        # puts stdout "FOUND_LIST= >>$found_list<<"
        # puts stdout "REST_LIST = >>$rest_list<<"
     puts stdout "CatalogInterface: initial : [llength $loc_list], \
          found-and-purged : [llength $found_list], \
                 remaining : [llength $rest_list]"
     puts $lfId "CatalogInterface: $found_counter objetcs identified"
     # puts stdout "[llength $loc_list] - [llength $found_list] + \
     #        [llength $rest_list]"

    #  5 delete from the AstroCat current local catalog 
    #  puts stdout "a_c = $a_c"
    #  puts stdout "###->$a_c info class    = [$a_c info class]"
    #  puts stdout "###->$a_c info inherit  = [$a_c info inherit]"
    #  puts stdout "###->$a_c info heritage = [$a_c info heritage]"
    #  puts stdout "###->$a_c info function = [$a_c info function]"
    #  puts stdout "###->$a_c info variable = [$a_c info variable]"
    #  puts stdout "###->$a_c itcl::find objects -class AstroCat = \
    #              [$a_c itcl::find objects -class AstroCat]"



     # 4) delete, the found_list from the target_list
     set a [$a_c query_done "" $hea $rest_list 0] 

     # puts stdout [set ll [ llength $rest_list ]  ]
     # for {set i 0 } { $i < $ll } { incr i 1} {
     #     puts stdout [lindex $rest_list $i]
     #     }
      
     puts stdout "CatalogInterface: end"
     }


#-----------------------------
public  method update_scale {} {
#-----------------------------
       # WCS - canvas coords
       # only for mxu 

       C_wc coeff_wc $wx $wy

       switch -exact -- [$this cget -modus] {
       mxu {
           }
      echs {
           fors::CONV_MC::coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                          $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                          $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
           }
        }

       set ctom [eval calc_ctom]
       set ccdm_ctom [eval calc_ccdm_ctom]
       set ccds_ctom [eval calc_ccds_ctom]
       }

#-----------------------------
private method calc_ctom { } {
#------------------------------
     set scal [expr $atd * $focscale]
     set p1x $wx; set p1y [expr $wy+$scal*$YSRMIN]
     # upper northern point
     set p2x $wx; set p2y [expr $wy+$scal*$YSRMAX]

     # transform these points into canvas coords
     if $bonsai_flag {
        set dum [C_wc w2c $p1x $p1y]
        set p1x [lindex $dum 0]
        set p1y [lindex $dum 1]
        set dum [C_wc w2c $p2x $p2y]
        set p2x [lindex $dum 0]
        set p2y [lindex $dum 1]
      } else {
        $m_image convert coords $p1x $p1y "deg $EQX" p1x p1y canvas
        $m_image convert coords $p2x $p2y "deg $EQX" p2x p2y canvas
      }


     set dx [expr $p1x-$p2x]
     set dy [expr $p1y-$p2y]
     set ctom [expr sqrt($dx*$dx+$dy*$dy)/($YSRMAX-$YSRMIN)]
     # puts stdout [format "calc_ctom: new scale: %8.3f" $ctom]
     return $ctom
}

#-----------------------------
private method calc_ccdm_ctom { } {
#------------------------------
     set scal [expr $atd * $focscale]
    
     set ccdm1x $wx; set ccdm1y [expr $wy+$scal*$CCDMASTER_YSRMIN]
     
     set ccdm2x $wx; set ccdm2y [expr $wy+$scal*$CCDMASTER_YSRMAX]
     

     # transform these points into canvas coords
     if $bonsai_flag {
        
	 
        set mdum [C_wc w2c $ccdm1x $ccdm1y]
	set ccdm1x [lindex $mdum 0] 
        set ccdm1y [lindex $mdum 1]
         

        set mdum [C_wc w2c $ccdm2x $ccdm2y]
	set ccdm2x [lindex $mdum 0] 
        set ccdm2y [lindex $mdum 1]
       

      } else {        
	$m_image convert coords $ccdm1x $ccdm1y "deg $EQX" ccdm1x ccdm1y canvas
        $m_image convert coords $ccdm2x $ccdm2y "deg $EQX" ccdm2x ccdm2y canvas       
      }
     
     set ccdmx [expr $ccdm1x - $ccdm2x]     
     set ccdmy [expr $ccdm1y - $ccdm2y]     
     
     set ccdm_ctom [expr sqrt($ccdmx*$ccdmx+$ccdmy*$ccdmy)/($CCDMASTER_YSRMAX-$CCDMASTER_YSRMIN)]    
     return $ccdm_ctom
}

#-----------------------------
private method calc_ccds_ctom { } {
#------------------------------

     set scal [expr $atd * $focscale]         
     set ccds1x $wx; set ccds1y [expr $wy+$scal*$CCDSLAVE_YSRMIN]     
     set ccds2x $wx; set ccds2y [expr $wy+$scal*$CCDSLAVE_YSRMAX]

     # transform these points into canvas coords
     if $bonsai_flag {
       
        set sdum [C_wc w2c $ccds1x $ccds1y]
	set ccds1x [lindex $sdum 0] 
        set ccds1y [lindex $sdum 1]      
       
        set sdum [C_wc w2c $ccds2x $ccds2y]
	set ccds2x [lindex $sdum 0] 
        set ccds2y [lindex $sdum 1]

      } else {        	
        $m_image convert coords $ccds1x $ccds1y "deg $EQX" ccds1x ccds1y canvas
        $m_image convert coords $ccds2x $ccds2y "deg $EQX" ccds2x ccds2y canvas
      }
          
     set ccdsx [expr $ccds1x - $ccds2x]     
     set ccdsy [expr $ccds1y - $ccds2y]
     
     set ccds_ctom [expr sqrt($ccdsx*$ccdsx+$ccdsy*$ccdsy)/($CCDSLAVE_YSRMAX-$CCDSLAVE_YSRMIN)]    
     return $ccds_ctom
   
}



#-----------------------------
public method wave_config_mask { three } {
#------------------------------
# this is callled by the wavelength EntryForm 
# this is also called by public variable grism

# input is lmin lmax who 

# who == grism (called by public variable grism)
#        update when a new grism has been selected 
#        even the L_MIN and L_MAX look like not 
#        having been updated 
# who = wave (called by the wavelength entry widget)
#       update only when the new user-defined wavelength 
#       border differs from L_MIN, L_MAX

set three [split $three " " ]
set flag 0

puts "WAVE CONFIG MASK"
# puts stdout "update WaveLeBord with >[lindex $three 0]< >[lindex $three 1]< >[lindex $three 2]<"

# check if there are really changes with respect to the current 
# wavelength range setting
if {[string compare [lindex $three 2] "grism"]==0} {
   if {$mask_initialized} { set flag 1 }
   # puts stdout "called by Grism, flag = $flag"
} else {
   if {$L_MIN != [lindex $three 0]} { set flag 1; set L_MIN [lindex $three 0]}
   if {$L_MAX != [lindex $three 1]} { set flag 1; set L_MAX [lindex $three 1]}
   # puts stdout "called by Wave, flag = $flag"
}



# if so
if {$flag} { set L_MIN [expr $L_MIN * 1.0]; # int to float
             set L_MAX [expr $L_MAX * 1.0]; # int to float 
             wave_limit; 
             # if you want to keep the slits after selecting a new 
             # wavelength range you must code here something awful

    set ins_lamc $A_disp($good,cen)
    set ins_disp $A_disp($good,disp)


     if {$mask_initialized} {
     fors::CONV_MC::coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
          $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
          $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
        }

    switch -exact -- [$this cget -modus] {
    
        lss -
        hits {
        set specmin [expr ($ins_lamc-$L_MIN)/$ins_disp/$focscale]
        set specmin [expr $specmin/$PIX*$CCDscale_standard]
        set specmin [expr $specmin - $longslitxmin($long_number) ]

        set specmax [expr ($L_MAX-$ins_lamc)/$ins_disp/$focscale]
        set specmax [expr $specmax/$PIX*$CCDscale_standard]
        set specmax [expr $specmax + $longslitxmax($long_number) ]

     set a [fors::CONV_MC::m2c -$specmin $YSRMIN]; set mos(0,xbr) [lindex $a 0]; set mos(0,ybr) [lindex $a 1]
     set a [fors::CONV_MC::m2c  $specmax $YSRMIN]; set mos(0,xar) [lindex $a 0]; set mos(0,yar) [lindex $a 1]
     set a [fors::CONV_MC::m2c -$specmin $YSRMAX]; set mos(0,xcr) [lindex $a 0]; set mos(0,ycr) [lindex $a 1]
     set a [fors::CONV_MC::m2c  $specmax $YSRMAX]; set mos(0,xdr) [lindex $a 0]; set mos(0,ydr) [lindex $a 1]

    
   
     # HR 
     set specmin [expr $specmin*$CCDscale_highres/$CCDscale_standard]
     set specmax [expr $specmax*$CCDscale_highres/$CCDscale_standard]


     set a [fors::CONV_MC::m2c -$specmin $YHRMIN]; set mos(0,xfr) [lindex $a 0]; set mos(0,yer) [lindex $a 1]
     set a [fors::CONV_MC::m2c  $specmax $YHRMIN]; set mos(0,xer) [lindex $a 0]; set mos(0,yfr) [lindex $a 1]
     set a [fors::CONV_MC::m2c -$specmin $YHRMAX]; set mos(0,xgr) [lindex $a 0]; set mos(0,ygr) [lindex $a 1]
     set a [fors::CONV_MC::m2c  $specmax $YHRMAX]; set mos(0,xhr) [lindex $a 0]; set mos(0,yhr) [lindex $a 1]

         }
       mos -
       mxu -
      pmos {
	
        set specmin [expr ($ins_lamc-$L_MIN)/$ins_disp/$focscale]
        set ccdm_specmin [expr ($ins_lamc-$L_MIN)/$ins_disp/$focscale]
        set specmin [expr $specmin/$PIX*$CCDscale_standard]
        set ccdm_specmin [expr $ccdm_specmin/$PIX*$CCDscale_standard]
        set specmin [expr -$specmin + $XSRMAX ]
        set ccdm_specmin [expr -$ccdm_specmin + $CCDMASTER_XSRMAX ]
       

        set specmax [expr ($L_MAX-$ins_lamc)/$ins_disp/$focscale]
        set specmax [expr $specmax/$PIX*$CCDscale_standard]
        set specmax [expr -$specmax - $XSRMIN ]

        set ccdm_specmax [expr ($L_MAX-$ins_lamc)/$ins_disp/$focscale]
        set ccdm_specmax [expr $ccdm_specmax/$PIX*$CCDscale_standard]        
        set ccdm_specmax [expr -$ccdm_specmax - $CCDMASTER_XSRMIN ]

     set a [fors::CONV_MC::m2c -$specmin $YSRMIN]; set mos(0,xar) [lindex $a 0]; set mos(0,yar) [lindex $a 1]
     set a [fors::CONV_MC::m2c  $specmax $YSRMIN]; set mos(0,xbr) [lindex $a 0]; set mos(0,ybr) [lindex $a 1]
     set a [fors::CONV_MC::m2c -$specmin $YSRMAX]; set mos(0,xdr) [lindex $a 0]; set mos(0,ydr) [lindex $a 1]
     set a [fors::CONV_MC::m2c  $specmax $YSRMAX]; set mos(0,xcr) [lindex $a 0]; set mos(0,ycr) [lindex $a 1]


     #if {$mask_initialized} {
     #fors::CONV_MC::coeff_mc $CCDMASTER_XSRMIN $YSRMIN $ccd_frame(0,xa) $mos(0,ya) \
     #     $CCDMASTER_XSRMAX $YSRMIN $ccd_frame(0,xb) $mos(0,yb) \
     #     $CCDSLAVE_XSRMAX $YSRMAX $ccd_frame(1,xc) $mos(1,yc)
     #   }

     
     set m [fors::CONV_MC::m2c -$ccdm_specmin $YSRMIN]; set ccd_frame(0,xar) [lindex $m 0]; 
     set ccd_frame(0,yar) [lindex $m 1]
     set m [fors::CONV_MC::m2c  $ccdm_specmax $YSRMIN]; set ccd_frame(0,xbr) [lindex $m 0]; 
     set ccd_frame(0,ybr) [lindex $m 1]
     set m [fors::CONV_MC::m2c -$ccdm_specmin $YSRMAX]; set ccd_frame(0,xdr) [lindex $m 0]; 
     set ccd_frame(0,ydr) [lindex $m 1]
     set m [fors::CONV_MC::m2c  $ccdm_specmax $YSRMAX]; set ccd_frame(0,xcr) [lindex $m 0]; 
     set ccd_frame(0,ycr) [lindex $m 1] 

    # set m [fors::CONV_MC::m2c -$ccdm_specmin $YSRMIN]; set ccd_frame(0,xar) [lindex $m 0]; 
    # set ccd_frame(0,yar) [lindex $m 1]
    # set m [fors::CONV_MC::m2c  $ccdm_specmax $YSRMIN]; set ccd_frame(0,xbr) [lindex $m 0]; 
    # set ccd_frame(0,ybr) [lindex $m 1]
    # set m [fors::CONV_MC::m2c -$ccdm_specmin $YSRMAX]; set ccd_frame(0,xdr) [lindex $m 0]; 
    # set ccd_frame(0,ydr) [lindex $m 1]
    # set m [fors::CONV_MC::m2c  $ccdm_specmax $YSRMAX]; set ccd_frame(0,xcr) [lindex $m 0]; 
    # set ccd_frame(0,ycr) [lindex $m 1] 

	  

     # HR 
     set specmin [expr $specmin*$CCDscale_highres/$CCDscale_standard]
     set specmax [expr $specmax*$CCDscale_highres/$CCDscale_standard]
     set ccdm_specmin [expr $ccdm_specmin*$CCDscale_highres/$CCDscale_standard]
     set ccdm_specmax [expr $ccdm_specmax*$CCDscale_highres/$CCDscale_standard]

     set a [fors::CONV_MC::m2c -$specmin $YHRMIN]; set mos(0,xer) [lindex $a 0]; set mos(0,yer) [lindex $a 1]
     set a [fors::CONV_MC::m2c  $specmax $YHRMIN]; set mos(0,xfr) [lindex $a 0]; set mos(0,yfr) [lindex $a 1]
     set a [fors::CONV_MC::m2c -$specmin $YHRMAX]; set mos(0,xhr) [lindex $a 0]; set mos(0,yhr) [lindex $a 1]
     set a [fors::CONV_MC::m2c  $specmax $YHRMAX]; set mos(0,xgr) [lindex $a 0]; set mos(0,ygr) [lindex $a 1]

    
     set a [fors::CONV_MC::m2c -$ccdm_specmin $CCDMASTER_YHRMIN]; set ccd_frame(0,xer) [lindex $a 0]; set ccd_frame(0,yer) [lindex $a 1]
     set a [fors::CONV_MC::m2c  $ccdm_specmax $CCDMASTER_YHRMIN]; set ccd_frame(0,xfr) [lindex $a 0]; set ccd_frame(0,yfr) [lindex $a 1]
     set a [fors::CONV_MC::m2c -$ccdm_specmin $CCDMASTER_YHRMAX]; set ccd_frame(0,xhr) [lindex $a 0]; set ccd_frame(0,yhr) [lindex $a 1]
     set a [fors::CONV_MC::m2c  $ccdm_specmax $CCDMASTER_YHRMAX]; set ccd_frame(0,xgr) [lindex $a 0]; set ccd_frame(0,ygr) [lindex $a 1]

     set a [fors::CONV_MC::m2c -$ccdm_specmin $CCDSLAVE_YHRMIN]; set ccd_frame(1,xer) [lindex $a 0]; set ccd_frame(1,yer) [lindex $a 1]
     set a [fors::CONV_MC::m2c  $ccdm_specmax $CCDSLAVE_YHRMIN]; set ccd_frame(1,xfr) [lindex $a 0]; set ccd_frame(1,yfr) [lindex $a 1]
     set a [fors::CONV_MC::m2c -$ccdm_specmin $CCDSLAVE_YHRMAX]; set ccd_frame(1,xhr) [lindex $a 0]; set ccd_frame(1,yhr) [lindex $a 1]
     set a [fors::CONV_MC::m2c  $ccdm_specmax $CCDSLAVE_YHRMAX]; set ccd_frame(1,xgr) [lindex $a 0]; set ccd_frame(1,ygr) [lindex $a 1]
            
           }
         }

    if {$mask_initialized} { update_and_replot  } 
    }
}

#-----------------------------
public method set_wcs { in } {
#------------------------------
puts stdout "set_wcs  $in" 
set in [split $in " " ]
puts stdout "[lindex $in 0] [lindex $in 1] \
             [lindex $in 2] [lindex $in 3] \
             [lindex $in 4] [lindex $in 5] \
             [lindex $in 6] [lindex $in 7] \
             2000 1998.5 -TAN"
$m_image wcsset [lindex $in 0] [lindex $in 1] \
                [lindex $in 2] [lindex $in 3] \
                [lindex $in 4] [lindex $in 5] \
                [lindex $in 6] [lindex $in 7] \
                2000 1998.5 -TAN
puts stdout [$m_image wcsset]
}

#-----------------------------
public method new_wcs { } {
#------------------------------
# external configuration of the wavelength range
# Cancel button -> destroy
global .ew
puts stdout "new_wcs"

if [catch {wm deiconify .ew}] {
   EntryForm .ew \
   -title "Define WCS (no frame)" \
   -labels {ra dec secpix xrefpix yrefpix nxpix nypix rotate equinox} \
   -command "fors::FORS::my_mask set_wcs"  \
   -values  { {01:01:00} {-72:01:00} {2.0} {100} {100} {400} {400} {0.0} {2000.0}} 
   } else {
   .ew configure -values { 
              {01:00:00} {-72:00:00} {2.0} {100} {100} {400} {400} {0.0} {2000.0}
                         }
   }
   tkwait window .ew
}


#------------------------------------------
public method indi_slit_width { w sx sy } {
#------------------------------------------
# external configuration of an individual MOS slit width  
     # puts stdout "We change individual MOS slit width"
     $m_image convert coords $sx $sy screen x y canvas 
     set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
     set my [string range $curtag  1 end]
     # puts stdout ">[$w find closest $x $y]< \
     #           >[$w gettags [$w find closest $x $y]]< \
     #          curtag= >$curtag<  my= >$my<"


global .msw 

if [catch {wm deiconify .msw}] {
   EntryForm .msw \
   -title "Slit Width of MOS #$my in arcsec" \
   -labels {Width} \
   -command "fors::FORS::my_mask config_slit $my" \
   -values  "$mos($my,b)" 
   } else {
   .msw configure -values "$mos($my,b)"
   } 
set last_action "Set Slit Width \#$my" 
catch {SP update} 
}


# --------------------------------
private method warn {} {
# --------------------------------
set number [random 7]


switch -exact -- $number {
0 - 
1 {
     set result 1
     while {$result==0} {

     set wid .skycat1.save
     catch {destroy $wid}
     set w [DialogWidget $wid \
         -text "Are you Really Shure ?" \
         -bitmap warning \
         -default 0  \
         -buttons {YES NO} ]

     set result [$w activate] 
    }
  }
2 {  info_dialog "You have pressed \[Ok\]\n\nPlease verify" .skycat1
     info_dialog "You have pressed \[Ok\]\n\nPlease verify" .skycat1
     info_dialog "\[Ok\] Accepted" .skycat1
     info_dialog "\[Ok\] Not Accepted" .skycat1
     info_dialog "You have pressed \[Ok\]\n\nPlease verify" .skycat1}

3 { expire_date }
4 { info_dialog "Not Accepted\n\n\Press again \[YES\]" .skycat1            
    info_dialog "Not Accepted\n\n\Press again \[NO\]" .skycat1
    info_dialog "Not Accepted\n\n\Press again \[OK\]" .skycat1  }

5 { info_dialog "No Save Permission for\n\n\User : [exec whoami]\n\n\Press \[NO\]" .skycat1
    info_dialog "No Save Permission for\n\n\User : [exec whoami]\n\n\Press \[NO\]" .skycat1}
6 { 
    for {set j 1 } { $j<5} { incr j 1 } {

     set text "Agreement:\n\n\
               This setup file\n\
               remains a property of ESO\n\
               even if never scheduled."  

     set result 1
     set haha [random 3]

     while {$result != $haha} {

     set wid .skycat1.save
     catch {destroy $wid}
     set w [DialogWidget $wid \
         -text $text  \
         -bitmap warning \
         -default 0 \
         -buttons {YES OK Accept Thanks} ]

     set result [$w activate] 
      }
     }
    }
  }
}

#-----------------------------
private method shift_cms { w sx sy } {
#------------------------------
# external configuration of an individual MXU slit : CURVED 

     $m_image convert coords $sx $sy screen x y canvas 
     set particle [$w find closest $x $y]
     set curtag [lindex [$w gettags $particle] 2]
     set beztag [lindex [$w gettags $particle] 3]
     set my [string range $curtag  3 end]

if {$my==""} { info_dialog  "Not so Fast !!!\nTry 20'' SlitWidth !!!" .skycat1
             } else {

#     puts stdout ">[$w find closest $x $y]< \
#              >[$w gettags [$w find closest $x $y]]< \
#              curtag= >$curtag<  beztag= >$beztag< my= >$my<"

# the shift distance in canvas units 
set shift_x [expr $x-$op(x)]
set shift_y [expr $y-$op(y)]

# puts stdout "$my $beztag $x +- $shift_x $y +- $shift_y"
# modify position of one bezier point
C_mxus$my mod_bez $beztag $shift_x $shift_y  

# plot the slit contour of curve $my 
C_mxus$my plot_m 
# delete and replot the two non-moved bez-points
C_mxus$my plot_bez $beztag

# shift the selected bezier point on the canvas
$w move $beztag $shift_x $shift_y 

# update cursor oldpos for permanent shifts
set op(x) $x
set op(y) $y

}
}

#------------------------------------
private method mark_cms { w xs ys } {
#------------------------------------
     fors::CONV_MC::coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
          $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
          $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)

$m_image convert coords $xs $ys screen op(x) op(y) canvas
}

#-----------------------------
private method end_cms { w sx sy } {
#------------------------------
# update the curved mxu slit
     $m_image convert coords $sx $sy screen x y canvas 
     set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
     set my [string range $curtag  3 end]

# delete the mxc tags 
$m_canvas delete mxc$my

if {$label} { C_mxus$my plot_label e w }

# update the optical correction for all three bezier points
# get out the values of class mxu, apply correction and copy them 
# in to class mxu again 
update_bez_oc $my

# set a [C_mxus$my get_curve_ca]
# set targw(1,a) [C_mxus$my get_RA]
# set targw(1,d) [C_mxus$my get_DEC]
# 
# $m_image convert coords [lindex $a 0] [lindex $a 1] canvas \
#          targw(1,x) targw(1,y) image 
# calc_zemax 1 $targw(1,x) $targw(1,y)
# C_mxus$my   put_b_oc 1 [list $targw(1,cx) $targw(1,cy)]
# 
# $m_image convert coords [lindex $a 2] [lindex $a 3] canvas \
#          targw(1,x) targw(1,y) image 
# calc_zemax 1 $targw(1,x) $targw(1,y)
# C_mxus$my   put_b_oc 2 [list $targw(1,cx) $targw(1,cy)]
# 
# $m_image convert coords [lindex $a 4] [lindex $a 5] canvas \
#          targw(1,x) targw(1,y) image 
# calc_zemax 1 $targw(1,x) $targw(1,y)
# C_mxus$my   put_b_oc 3 [list $targw(1,cx) $targw(1,cy)]
# 
# $m_image convert coords [lindex $a 6] [lindex $a 7] canvas \
#          targw(1,x) targw(1,y) image 
# calc_zemax 1 $targw(1,x) $targw(1,y)
# C_mxus$my   put_b_oc 4 [list $targw(1,cx) $targw(1,cy)]
# 
# $m_image convert coords [lindex $a 8] [lindex $a 9] canvas \
#          targw(1,x) targw(1,y) image 
# calc_zemax 1 $targw(1,x) $targw(1,y)
# C_mxus$my   put_b_oc 5 [list $targw(1,cx) $targw(1,cy)]
# 
# $m_image convert coords [lindex $a 10] [lindex $a 11] canvas \
#          targw(1,x) targw(1,y) image 
# calc_zemax 1 $targw(1,x) $targw(1,y)
# C_mxus$my   put_b_oc 6 [list $targw(1,cx) $targw(1,cy)]
# 
# C_mxus$my status

  # apply correction of differential refraction (10.5.99)
  # input slit position, corrected for optical distortion
  #  if {$dire_flag} {calc_diff_refrac 1 $targw(1,ca) $targw(1,cd)}
  #  C_mxus$noms put_OC  [list $targw(1,ca) $targw(1,cd)]

  set a [size_CS $my]
}

# ===============================
private method size_CS { my } {
# ===============================
 # WH 14.Sept 2000
 # finally we test if the slit is small enough
  set c [C_mxus$my get_b_oc_mmu]
             # index of the six bezier points
             #
             #    0,1 +----+ 6,7
             #         \    \
             #          \    \
             #           \    \
             #           /    /  2,3 +     + 8,9
             #      4,5 +----+ 10,11
             #
             # flash the first point
  set p1x [expr ([lindex $c 0] + [lindex $c 6])  /2.0]
  set p1y [expr ([lindex $c 1] + [lindex $c 7])  /2.0]
  set p3x [expr ([lindex $c 4] + [lindex $c 10]) /2.0]
  set p3y [expr ([lindex $c 5] + [lindex $c 11]) /2.0]
  set hx  [expr [lindex $c 0] - [lindex $c 6] ]          
  set hy  [expr [lindex $c 1] - [lindex $c 7] ]          
  set w   [expr sqrt($hx*$hx +$hy*$hy)]
   # puts stdout $c
   puts stdout [format "width = %5.3f  mm" $w]
  set size1 [expr sqrt( ($p1x-$p3x)*($p1x-$p3x) + ($p1y-$p3y)*($p1y-$p3y) ) +$w]
   # oversize flag 
   set CtL 0

  # first test
  if {$size1>20.0} { 
     puts stdout [format "Warning: CURVED slit %1d size=%5.3fmm > 20mm" $my $size1]
                    set CtL 1
     } else {
     # puts stdout "GO on!! $size mm <= 20mm"
     }

  # since the bezier curve does not match P2, we use the half length
  set p2x [expr ([lindex $c 2] + [lindex $c 8])  /2.0]
  set p2y [expr ([lindex $c 3] + [lindex $c 9])  /2.0]
  set size2 [expr sqrt(($p2x-$p3x)*($p2x-$p3x)+($p2y-$p3y)*($p2y-$p3y))/2.0+$w]
  # second test 
  if {$size2>20.0} { 
     puts stdout [format "Warning: CURVED slit %1d size=%5.3fmm > 20mm" $my $size2]
                    set CtL 1 
     } else {
     # puts stdout "GO on!! $size mm <= 20mm"
     }
  set size3 [expr sqrt(($p2x-$p1x)*($p2x-$p1x)+($p2y-$p1y)*($p2y-$p1y))/2.0+$w]
  # third test test 
  if {$size3>20.0} { 
     puts stdout [format "Warning: CURVED slit %1d size=%5.3fmm > 20mm" $my $size3]
                    set CtL 1 
     } else {
     # puts stdout "GO on!! $size mm <= 20mm"
     }
  
   puts stdout [format "curve (1-3/1-2/2-3) = (%5.3f/%5.3f/%5.3f) mm" $size1 $size2 $size3 ]

  if {$CtL} {
     info_dialog "CURVED slit \#$my\n too large !\n\n Shrink \#$my !\n" .skycat1
     } 
  
     return $CtL
}


#-----------------------------
public method delete_slit_mxu { w sx sy } {
#------------------------------
# external configuration of an individual MXU slit width, length and posang  
     # puts stdout "We delete individual MXU slit"
     $m_image convert coords $sx $sy screen x y canvas 
     set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
     set my [string range $curtag  3 end]
     # puts stdout ">[$w find closest $x $y]< \
     #           >[$w gettags [$w find closest $x $y]]< \
     #          curtag= >$curtag<  my= >$my<"

#- There fix ref slits that cannot be deleted -#
if {$my <= $nf_refslit} {
    puts "Fixed mxu referece slit ($my) : Cannot be deleted !"
    return
}

 set wid .skycat1.mxu_save   
 catch {destroy $wid}
 set w [DialogWidget $wid \
     -text "You De-Select MXU slit #$my" \
     -bitmap warning \
     -default -1 \
     -buttons {OK Cancel} ]
  if { [$w activate] } return

C_mxus$my erase   
C_mxus$my configure -ON 0

# for statuspanel
set last_action "Delete MXU \#$my"
catch {SP update}
}


#-----------------------------
public method ref_slit_mxu { w sx sy on } {
#------------------------------
# external configuration of an individual MXU 
#    slit   : assign as refernce slit or not  
     $m_image convert coords $sx $sy screen x y canvas 
     set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
     set my [string range $curtag  3 end]

     # circle slits are no longer permitted to be used as
     # reference slits (WH 8/2000)

     # a) it must be a straight slit
     if {[string match *STRAIGHT* [C_mxus$my cget -shape] ]} {
     # b) not more than 10 ref-slits allowed
     # set nors [fors::MXU_SLIT::nor]
     # the ::nor does not work, since we have to delete 
     # refslits when noms = 0, meaning when the mask is 
     # reset 
     set nors [arsc]
         # there is also the Method arsc 
         # which finds the number of ref slits in mxu class
     # puts stdout "nors=>$nors$ on=>$on<"
     if {$nors>=10 && $on} {
        puts stdout "Warning: Not more than 10 ref-slits allowed"
        } else {
        # do not try to conig th slit as a ref-0slit in case 
        # it is already a refslit
        set already [C_mxus$my cget -ref_slit]
        # puts stdout "already = $already"
        if {$already==$on} {
           puts stdout "Warning : (ref)slit is already"
           } else { 
           # now ...
           C_mxus$my configure -ref_slit  $on
           C_mxus$my plot_label e w 
          }
        }
       

     # statuspanel
     if {$on} { 
     set last_action "Reference Slits + \#$my"
     incr sp_rsc 1
     } else { 
     set last_action "Reference Slits - \#$my"
     incr sp_rsc -1 
     } 
     # puts stdout "nor=>$nors< -> [fors::MXU_SLIT::nor]";
     catch {SP update}
     } else {
     info_dialog "CURVED and CIRCLE slits \n cannot be reference slits" .skycat1
     puts stdout "CURVED and CIRCLE slits cannot be reference slits" .skycat1
     }

     }


#-----------------------------
public method indi_slit_mxu { w sx sy } {
#------------------------------
puts "indi_slit_mxu"
# external configuration of an individual MXU 
#    slit   : width, length and posang  
#    circle : radius
#    curved : ???
     # puts stdout "We change individual MXU slit"
     $m_image convert coords $sx $sy screen x y canvas 
     set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
     set my [string range $curtag  3 end]
#    puts stdout ">[$w find closest $x $y]< \
#              >[$w gettags [$w find closest $x $y]]< \
#             curtag= >$curtag<  my= >$my<"
     set shape [C_mxus$my cget -shape]

if {$my <= $nf_refslit} {
    puts "Fixed mxu referece slit ($my) : Cannot be changed !"
    return
}

global .mxw 

     switch -exact [C_mxus$my cget -shape] {
     STRAIGHT {

if [catch {wm deiconify .mxw}] {
   EntryForm .mxw \
   -title "Edit MXU Slit #$my for [C_mxus$my cget -target_name]" \
   -labels {Width/'' Length/'' Inclin/} \
   -command "fors::FORS::my_mask config_mxu $my" \
   -values  "[C_mxus$my cget -slit_width] \
             [C_mxus$my cget -slit_length] \
             [C_mxus$my cget -pos_ang]"
   } else {
   .mxw configure -values "[C_mxus$my cget -slit_width] \
                           [C_mxus$my cget -slit_length] \
                           [C_mxus$my cget -pos_ang]"
   } 

          }
   CURVED { 

if [catch {wm deiconify .mxw}] {
   EntryForm .mxw \
   -title "Edit MXU Curve #$my for [C_mxus$my cget -target_name]" \
   -labels {Width/''} \
   -command "fors::FORS::my_mask config_mxu $my" \
   -values  "[C_mxus$my cget -slit_width]" 
   } else {
   .mxw configure -values "[C_mxus$my cget -slit_width]"
   } 
          # move the Bezier point like the mask
          # 1. make slit $my active (enable )
          # 2. move and update simulatneously
          # 3. button release (disable)
          }
   CIRCLE {

if [catch {wm deiconify .mxw}] {
   EntryForm .mxw \
   -title "Edit MXU Circle #$my for [C_mxus$my cget -target_name]" \
   -labels {Diameter/''} \
   -command "fors::FORS::my_mask config_mxu $my" \
   -values  "[C_mxus$my cget -slit_width]" 
   } else {
   .mxw configure -values "[C_mxus$my cget -slit_width]"
   } 

          }
  }

set last_action "Configure MXU  \#$my"
catch {SP update}

}

#----------------------------------------
public method config_mxu { my wilepo } {
#----------------------------------------
    # store new slit width 
    # puts stdout ">$my< >$wilepo<"
    # my = current mxu slit number
    # wilepo = list of width/'' length'' slitposang/deg
    set wi [lindex $wilepo 0]
    set le [lindex $wilepo 1]
    set po [lindex $wilepo 2]

    # in case of circle diameter
    # puts stdout ">$wi< >$le<"
    if {$po==""} { set po 0.0 }
    if {$le==""} { set le $wi }

    
    set l_size [expr sqrt($wi*$wi+$le*$le)  / cos($po/180.0*3.1415) ]

    if {$l_size > 40.0} {
       # 20mm is the upper limit to automatic mask manufacuring 
       # see VIRMOS-MMU documents 
       puts stdout [format "Error: Slit size %5.3f'' exceeds 20mm (40'') in focf" $l_size]
       puts stdout "MXU slit setup Canceled"
       return 
       }

switch -exact -- [C_mxus$my cget -shape] {
    CIRCLE {
    C_mxus$my configure -slit_width  [lindex $wilepo 0]
    C_mxus$my configure -slit_length [lindex $wilepo 0]
           }
    STRAIGHT {
    C_mxus$my configure -slit_width  [lindex $wilepo 0]
    C_mxus$my configure -slit_length [lindex $wilepo 1]
    C_mxus$my configure -pos_ang     [lindex $wilepo 2]
             }
    CURVED {
    C_mxus$my configure -slit_width  [lindex $wilepo 0]
           }
    }

    C_mxus$my plot_m 
    if {$label} { C_mxus$my plot_label e w }

 if {[string match *CURVED* [C_mxus$my cget -shape]]} {
    update_bez_oc $my
    set a [size_CS $my] }

    }

#----------------------------------------
public method config_slit { my width } {
#----------------------------------------
    # apply only if MOS slit width is 
    # larger than 0.2 arcsec = 0.1 mm focal plane = 1 pixel
    if {$width>=0.2} {
    # store new slit width 
    set mos($my,b) [format "%3.1f" $width ]
    # set mos array 
    $m_image convert coords $targw($my,a) $targw($my,d) "wcs $EQX" sx sy screen
    catch {$this move_slit $m_canvas $sx $sy 1 0 }
    # plot new slit width 
    plot_mask
    } else { puts stdout "Error: slit width not accepted" 
    }
    }

#-----------------------------
public method wave_length { } {
#------------------------------
# external configuration of the wavelength range  
# Cancel button -> destroy
global .wal

if [catch {wm deiconify .wal}] {
   EntryForm .wal \
   -title "grism : [$this cget -grism] \n \
           filter: [$this cget -filter]\n \
           range : $A_disp($good,min) nm < \
           $A_disp($good,cen) nm < $A_disp($good,max) nm\n \
           Enter New Wavelength Intervall :" \
   -labels {Min Max} \
   -command "fors::FORS::my_mask wave_config_mask" \
   -values  "$L_MIN $L_MAX" 
   } else { 
   .wal configure -values "$L_MIN  $L_MAX"
   } 

}

#-----------------------------
public method edit_ref_targ  { c } {
#------------------------------
# wrapper to allow external access to C_ref_targ
# called by menu bars, defined in fors1_menu.tcl

# statuspanel: update statuspanel only for 'add' and 'purge' and 'reset' 

set res 0

puts "EDIT_REF_TARG .."

switch -exact -- $c {
    show { C_ref_targ show  }
   reset { C_ref_targ reset 
           set sp_nort 0
	   set last_action "Reset Reference Targets"
           catch {SP update}
         } 
     add { set stop 1
           set slit_flag 0
           if {[C_ref_targ cget -mos10ali]} { 
               set local ON }  else { set local OFF }

           if {[string match *mos [$this cget -modus] ] || 
               [string match echs [$this cget -modus] ] } {             
              #set parent [utilNamespaceTail $this]              
               
              set wid .skycat1.corr            
              catch {destroy $wid}
              
             
              set w [util::DialogWidget $wid \
                         -text "MOS 10 slit alignment : $local" \
                         -bitmap warning \
                         -buttons {ON OFF} ]
             
               
               if { [$w activate] } { C_ref_targ configure -mos10ali 0
                                        set slit_flag 0
                               } else { C_ref_targ configure -mos10ali 1
                                        set slit_flag 1
                               }                          
           # for status panel
           set sp_mtali [C_ref_targ cget -mos10ali]          
           catch {SP update}



              }

           plot_ref_star_area $slit_flag
           
           C_ref_targ add 

           # for statuspanel
           set sp_nort  [C_ref_targ nort]
	   set last_action "Add Reference Target"
           catch {SP update}

           set stop 0  
           C_ref_targ plot $mos(0,r) 

         $m_canvas delete rtz 
         set reftarg_zone 0

         }

   purge { C_ref_targ purge 
           C_ref_targ plot $mos(0,r) 

           # for status panel
           set sp_nort  [C_ref_targ nort]
	   set last_action "Purge Reference Targets"
           catch {SP update}
         }

  verify {
           # plot the forbidden zone 
           if {[C_ref_targ cget -mos10ali]} {
	       set slit_flag 1
	   } else {
	       set slit_flag 0
	   }
           plot_ref_star_area $slit_flag
           
           # reftarg_zone is set to 1 in plot_ref_star_area

           # call reftarg verify 
           set res [C_ref_targ verify]

           # delete the forbidden zone 
	   $m_canvas delete rtz 
	   set reftarg_zone 0

	   # for status panel
	   set sp_nort [C_ref_targ nort]
	   catch {SP update}
	   
       }

   verify_load {
           # plot the forbidden zone 
           if {[C_ref_targ cget -mos10ali]} {
	       set slit_flag 1
	   } else {
	       set slit_flag 0
	   }
           plot_ref_star_area $slit_flag
           
           # reftarg_zone is set to 1 in plot_ref_star_area

           # call reftarg verify 
           set res [ C_ref_targ verify 1 ]

           # delete the forbidden zone 
	   $m_canvas delete rtz 
	   set reftarg_zone 0

	   # for status panel
	   set sp_nort [C_ref_targ nort]
	   catch {SP update}
	   
       }

   plot { C_ref_targ plot $mos(0,r) }

   mtali  {
          # switch the mos10ali flag 
          set sp_mtali [C_ref_targ cget -mos10ali]
          if {$sp_mtali } { set new_stat 0}
          if {!$sp_mtali} { set new_stat 1}
          C_ref_targ configure -mos10ali $new_stat

          # for status panel
          set sp_mtali [C_ref_targ cget -mos10ali]
          catch {SP update}
          }

   }

return $res
}

#-----------------------------
public method select_mxu_name  { } {
#------------------------------
# Select MASK_ID and proposal ID fro MXU
# Cancel button -> destroy
global .mas

if [catch {wm deiconify .mas}] {
   EntryForm .mas \
   -title "MASK NA --- MASK ID" \
   -labels {MASK_NA MASK_ID} \
   -command  "fors::MXU_SLIT::set_MASK_NAID" \
   -values  [fors::MXU_SLIT::get_MASK_NAID] 
   } else {
   .mas configure -values [fors::MXU_SLIT::get_MASK_NAID]
   }
   tkwait window .mas

return [fors::MXU_SLIT::get_MASK_NAID] 
}

#-----------------------------
public method select_set_dir  { } {
#------------------------------
# external configuration set_dir
# Cancel button -> destroy
global .sd 
puts stdout "old set_dir = >$set_dir<" 

if [catch {wm deiconify .sd}] {
   EntryForm .sd  \
   -title "Directory of setup files" \
   -labels {Directory} \
   -command  "$this configure -set_dir" \
   -values  $set_dir 
   } else {
   .sd configure -values $set_dir 
   }
   tkwait window .sd 

set last_action "Set SetUpDir"
catch {SP update}
}
#-----------------------------
public method get_calib_passwd  { } {
#------------------------------
# external configuration set_dir
# Cancel button -> destroy
global .cpwd 

set passwd cloud

# science masks or sometimes called sky masks 
# are user defined masks van be saved as gerber 
  # format to be manufactored by the mmu of the 
# VIRMOS consortium. Every fims user is pemitted  to 
  # save his own created mask in gerber format in order 
# to be manufacured by the mmu. 
  # standard masks or sometimes called technical masks 
# have a longerlifetime than a science maks, they
  append passwd s; # can be used more often by 
# different astromomers. To provide the fims-maintainer 
  # the exclusive permission to 
# a) define new standard masks 
# b) save standard masks in gerber format
# c) manufacture new standard masks
# d) declare the new standard mask in the 
#    fims/src/standard_mask_list
  # we introduce the fims maintenance password 
  # fims setups with 900000<maskid<900100 require 
  # the password to be saved in gerber format

# on odd days the password is different
set cu_day [exec date -u +%2d]
if {[expr ($cu_day/2) * 2] != $cu_day} {
   # today is unfortunately an odd day
   # we change the password 
   set passwd hokuspokus
   }


if [catch {wm deiconify .cpwd}] {
   EntryForm .cpwd  \
   -title "Maintenance Password" \
   -labels {Password} \
   -command  "$this configure -calib_passwd" \
   -values  ""
   } else {
   .cpwd configure -values "" 

   }
   tkwait window .cpwd 

# puts stdout "user input= >$calib_passwd< passwd= >$passwd<"
return [string match $passwd $calib_passwd]
}

#-----------------------------
public method select_filename  { } {
#------------------------------
# external configuration of the wavelength range
# Cancel button -> destroy
global .waf
# puts stdout "new filename = [$this cget -file_name]" 

if [catch {wm deiconify .waf}] {
   EntryForm .waf \
   -title "ProjectName (8 chars)" \
   -labels {ProjectName} \
   -command  "$this configure -file_name" \
   -values  [$this cget -file_name]
   } else {
   .waf configure -values [$this cget -file_name] 
   }
   tkwait window .waf

# puts stdout "new filename = [$this cget -file_name]" 
set last_action "Set ProjectName"
catch {SP update}
}


#-----------------------------
private method select_cat { sc } {
#------------------------------
# show the filenames instead of the internal 
# widget names

 set allnames ""
  foreach w $sc {
          set filename [lindex [$w info variable instance_idx_ ] 4 ]
          set filename [split $filename ","]
          set filename [lindex $filename 0 ]
          if {[string match */* $filename]} {
              set filename [split $filename "/" ]
              set ll [llength $filename]
              set filename [lindex $filename [expr $ll-1] ]
             } else {
              set filename "{$filename}"
             }
          set allnames "$allnames $filename" 
          }

# show the List dialog

        set wid .skycat1.ld            
        catch {destroy $wid}
        set ld_flag [ ListDialog $wid \
            -text "Select catalog :" \
            -choice $allnames \
            -modal 0 ]
     #choice should be the tail of the file name

# and return the internal widget name again
     set s [$ld_flag activate]
     puts stdout "selected : >$s<"
   foreach w $sc {
       set test [lindex [$w info variable instance_idx_ ] 4 ]
       if { [string match *$s* $test ] } { return $w }
       }

     }

# ------------------------------
private method load_BPM { } {
# -----------------------------
set filename [format "F%1dBadPixelMap.asc" [string index [$this cget -instrument] 4] ]
set myfile $int_dir/lib/fors/$filename
# puts stdout $myfile

     if {[file exists $myfile] & [file readable $myfile]} {
        set fileId [open $myfile r ]
        puts $lfId "FORS::load_BPM  $fileId"
        # puts stdout "FORS::load_BPM  $fileId"

	# read FORS CCD reference pixel 
	gets $fileId line_org
	set rpx [lindex $line_org 0]; set rpy [lindex $line_org 1] 
        # puts stdout "$rpx $rpy"

       if {[string match *HR* [$this cget -collimat] ]} {
          set ls $CCDscale_highres
          } else {
          set ls $CCDscale_standard
          }

        set i 0
        # as long as there are lines in the file
        while {[gets $fileId line_org] >= 0} {
              if {[string match #* $line_org]} {
                 # ignore comment lines
                 # puts stdout $line_org
                 } else {
                 # now no comment lines 
                 incr i +1
		 # badpixel - reference pixel to mm focal plane 
		 # (0.2 ''/pixel) / (1.89 ''/mm) (SR)
		 # (0.1 ''/pixel) / (1.89 ''/mm) (HR)
                 set BPM($i,xmin) [expr ([lindex $line_org 0]-$rpx) * $ls/$focscale ]
                 set BPM($i,xmax) [expr ([lindex $line_org 1]-$rpx) * $ls/$focscale ]
                 set BPM($i,ymin) [expr ([lindex $line_org 2]-$rpy) * -$ls/$focscale ]
                 set BPM($i,ymax) [expr ([lindex $line_org 3]-$rpy) * -$ls/$focscale ]
		  # puts stdout [format "%2d %8.3f %8.3f %8.3f %8.3f" \
                  #       $i  $BPM($i,xmin) $BPM($i,xmax) \
                  #           $BPM($i,ymin) $BPM($i,ymax) ]
                 }
             }
        }
    set BPM_counter $i

# puts stdout "load $BPM_counter bad pixel areas"
}



# ------------------------------
private method plot_BPM { } {
# -----------------------------
# plot BPM
$m_canvas delete -tag BPM 
      for {set i 1} {$i <= $BPM_counter} {incr i 1} {
      $m_canvas create poly $BPM($i,x1) $BPM($i,y1) \
                            $BPM($i,x2) $BPM($i,y2) \
                            $BPM($i,x3) $BPM($i,y3) \
                            $BPM($i,x4) $BPM($i,y4) \
                            $BPM($i,x1) $BPM($i,y1) \
                 -outline white -width 1 -fill "" \
                 -smooth false -stipple "" -tag { A BPM }
                      # tag A to move with the mask 
       }
}

# ------------------------------
private method update_BPM { } {
# -----------------------------
# convert badpixelmask from mm ff to canvas

      for {set i 1} {$i <= $BPM_counter} {incr i 1} {
      set a [fors::CONV_MC::m2c $BPM($i,xmin) $BPM($i,ymin) ]
      set BPM($i,x1) [lindex $a 0]; set BPM($i,y1) [lindex $a 1]

      set a [fors::CONV_MC::m2c $BPM($i,xmin) $BPM($i,ymax) ]
      set BPM($i,x2) [lindex $a 0]; set BPM($i,y2) [lindex $a 1]

      set a [fors::CONV_MC::m2c $BPM($i,xmax) $BPM($i,ymax) ]
      set BPM($i,x3) [lindex $a 0]; set BPM($i,y3) [lindex $a 1]

      set a [fors::CONV_MC::m2c $BPM($i,xmax) $BPM($i,ymin) ]
      set BPM($i,x4) [lindex $a 0]; set BPM($i,y4) [lindex $a 1]
      }
plot_BPM
}


# ------------------------------
private method test_2 { } {
# -----------------------------
puts stdout "flip rxy = [$m_image rotate] \
                  [$m_image flip x] \
                  [$m_image flip y]"
}

#-----------------------------
public method test_optic_distor_FORS { } {
#------------------------------
# handles empirical optical distortion of FORS
# polynomic functions 
puts stdout "test optical distortion"

puts stdout "5 input distortion polynoms"
puts stdout "slit p0 p1 p2 p3 p4"
for {set i 1 } { $i <= 19 } { incr i 1 } {
    set y [expr $i * 1.0] 
    puts stdout [ format "%2d %7.6f %7.6f %7.6f %7.6f %7.6f"  \
                $i [p0 $y] [p1 $y] [p2 $y] [p3 $y] [p4 $y] ]
    }

puts stdout "\n\n"
puts stdout "slit -100 -75 -50  -25 0   25   50   75   100"
for {set i 1 } { $i <= 19 } { incr i 1 } {
    set y [expr $i * 1.0]; # slit number  
    puts stdout \
    [ format "%2d %7.6f %7.6f %7.6f %7.6f %7.6f %7.6f %7.6f %7.6f %7.6f"  \
    $i [od_F -100 $y] \
       [od_F -75  $y] \
       [od_F -50  $y] \
       [od_F -25  $y] \
       [od_F   0  $y] \
       [od_F  25  $y] \
       [od_F  50  $y] \
       [od_F  75  $y] \
       [od_F 100  $y] ]
    }

# for {set i -120 } { $i <= 120 } { incr i 10 } {
#     set x [ expr $i * 1.0 ]
#     set y 1.0
#     set a [$this optic_distor_FORS $x $y]
#     }


puts stdout "y=  -100  -80   -60   ...   60   80   100"
 for {set i -100 } { $i <= 100 } { incr i 50 } {
     set x [expr 1.0*$i ]
     set plot_string  [format " %5.2f" $x ] 
     for { set j 1 } { $j <=19 } { incr j 1 } {
         set y [expr 1.0*$j ]
         set test($j) [$this od_F $x $y]
         # puts stdout $test($j)
         append plot_string [format " %8.4f" $test($j) ]
         }
         puts stdout $plot_string
      }
puts stdout "\n\n"

puts stdout "y=  -100  -80   -60   ...   60   80   100"
 for {set i 40 } { $i <= 60 } { incr i 1 } {
     set x [expr 1.0*$i ]
     set plot_string  [format " %5.2f" $x ] 
     for { set j 40} { $j < 60 } { incr j 1 } {
         set y [expr 1.0*$j ]
         set test($j) [$this od_F $x $y]
         # puts stdout $test($j)
         append plot_string [format " %6.3f" $test($j) ]
         }
         puts stdout $plot_string
      }

# puts stdout "y=  -120  -110  -100  ...  +100  +110  +120"
# for {set i -120 } { $i <= 120 } { incr i 10 } {
#     set x [expr 1.0*$i ]
#     set plot_string  [format " %5.2f" $x ] 
#     for { set j -5 } { $j < 6 } { incr j 1 } {
#         set y [expr 20.0*$j ]
#         set test($j) [$this optic_distor_FORS $x $y]
#         # puts stdout $test($j)
#         append plot_string [format " %6.3f" $test($j) ]
#         }
#         puts stdout $plot_string
#      }


test_ablage 

puts stdout " ablage_SR (1462.32)= 4.40 = [ablage_SR 1462.32]" 
puts stdout " ablage_HR (1460.60)= 22.35 = [ablage_HR 1460.60]" 
puts stdout "distortion finished"
}


#-----------------------------
private method od_F { x y } {
#------------------------------
# handles empirical optical distortion of FORS1
# polynomic functions 
# p0 at x = -100
# p1 at x = -50
# p2 at x = 0
# p3 at x = +50
# p4 at x = +100
# function  argument is the y-value (slit-number)
# x in mm focal plane 
# y in slit number [1..19] 

set ip 5; set im 4 
set xp 100; set xm 50
set flag 1 
for {set i 0} { $i<5 } {incr i 1 } {
    set xt [expr -100.0 + $i * 50.0]
    if {$flag} {
               if {$x < $xt} { 
                   set ip $i; set im [expr $i-1]
                   set xp $xt; set xm [expr $xp-50.0]   
                   set flag 0
                  }
               }
    }
# puts stdout "$xm < $x $xp, $im < $ip"

switch -exact -- $ip  {
   0 { #extrapolate to minimum 
       set pp [ p1 $y ]
       set pm [ p0 $y ] 
       set p [expr ($pm*($xp-$x)-$pp*($xm-$x))/($xp-$xm) ]
       #puts stdout "v x=$x,y=$y,pa=$pa,pb=$pb,xi=$xi,p=$p"
     }
   5 { # extrapolate to maximum 
       set pp [ p4 $y ]
       set pm [ p3 $y ]
       set p [expr $pm + ($pp-$pm)*($x-$xm)/($xp-$xm) ]
       #puts stdout "^ x=$x,y=$y,pa=$pa,pb=$pb,xi=$i,p=$p"
     }
   1 -
   2 -
   3 -
   4 { # interpolate 
       set pp [ p$ip $y ]
       set pm [ p$im $y ]
       set p [expr $pm + ($pp-$pm)*($x-$xm)/($xp-$xm) ]
       #puts stdout "I x=$x y=$y index=$index ,xa=$xa, pb=$pb pa=$pa p=$p,xi=$xi"
     }
   }
return $p
}

#-----------------------------
private method p0 { y } {
#-----------------------------
# at x=-100
return [expr (-952.965+0.516767*$y-0.027334*$y*$y)/(-950.522) ]
}
#-----------------------------
private method p1 { y } {
#-----------------------------
# at x = -50; y is slit number !!! 
return [expr (-475.824+0.259736*$y-0.013703*$y*$y)/(-474.593) ]
}
#-----------------------------
private method p2 { y } {
#-----------------------------
return 1.0
}
#-----------------------------
private method p3 { y } {
#-----------------------------
# at x = +50  
return [expr (475.543-0.228404*$y+0.012263*$y*$y)/474.479  ]
}
#-----------------------------
private method p4 { y } {
#-----------------------------
# at x = +100
return [expr (951.932-0.476773*$y+0.026347*$y*$y)/949.775 ]
}
#-----------------------------
private method ablage_SR { r } {
#-----------------------------
# ZEMAX radial optical correction
# r = radius in pixel from reference pixel 

set pszx [$m_image fits get "HIERARCH ESO DET CHIP1 PSZX"]

if {$pszx==""} {
    if {[string compare [$this cget -instrument] FORS2]==0} {  
	set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"]
	if {$binx == 1} {
	    set pszx 15
	} elseif {$binx == 2} {
	    set pszx 30
	} else {
	    set pszx 15
	}
    } else {
	set pszx 24
    }
} else {
    set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"]
    set pszx [expr {$pszx * $binx}]
}

puts "PSZX: $pszx"
set r [expr ($r*$pszx)/24]

# AAG - PPRS 10127
    set dist1 [$m_image fits get "HIERARCH ESO INS IMAGE DISTOR1"]
    set dist2 [$m_image fits get "HIERARCH ESO INS IMAGE DISTOR2"]
    set dist3 [$m_image fits get "HIERARCH ESO INS IMAGE DISTOR3"]
    set chipName [$m_image fits get "HIERARCH ESO DET CHIP1 ID"]
    if {[string match "*SITE*" $chipName]} {
	puts stdout "DISTORTION: SITE ccd, using hardcoded values"
	return [expr $r*$r*$r*2.113310E-9 - 2.157780E-6*$r*$r + 5.367470E-4*$r]
    }
    #TSz - more clear screen outputs
    if {($dist1 != "") && ($dist2 != "") && ($dist3 != "")} {
	puts stdout "DISTORTION: $chipName ccd, using fits keyword values"
	return [expr {$dist1*$r + $dist2*$r*$r + $dist3*$r*$r*$r}]
    } else {
	if {[string compare [$this cget -instrument] FORS2]==0} {
	    puts stdout "DISTORTION: $chipName  ccd, using FORS2 hardcoded values"
	    return [expr $r*$r*$r*2.113310E-9 - 2.157780E-6*$r*$r + 5.367470E-4*$r]
	} else {
    	    puts stdout "DISTORTION: $chipName  ccd, using FORS1 hardcoded values"
    	    return [expr $r*$r*$r*2.050E-9 - 1.178E-6*$r*$r + 3.458E-4*$r ]
	}
    }
puts "ablage_SR"
#
}
#-----------------------------
private method ablage_HR { r } {
#-----------------------------
# ZEMAX radial optical correction
# r = radius in pixel from reference pixel 

set pszx [$m_image fits get "HIERARCH ESO DET CHIP1 PSZX"]
puts "PSZX: $pszx"

if {$pszx==""} {
    if {[string compare [$this cget -instrument] FORS2]==0} {  
	set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"]
	if {$binx == 1} {
	    set pszx 15
	} elseif {$binx == 2} {
	    set pszx 30
	} else {
	    set pszx 15
	} 
    } else {
	set pszx 24
    }
} else {
    set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"]
    set pszx [expr {$pszx * $binx}]
}
   
set r [expr ($r*$pszx)/24]

# AAG - PPRS 10127
    set dist1 [$m_image fits get "HIERARCH ESO INS IMAGE DISTOR1"]
    set dist2 [$m_image fits get "HIERARCH ESO INS IMAGE DISTOR2"]
    set dist3 [$m_image fits get "HIERARCH ESO INS IMAGE DISTOR3"]
    set chipName [$m_image fits get "HIERARCH ESO DET CHIP1 ID"]
    if {[string match "SITE" $chipName]} {
	puts stdout "DISTORTION: SITE ccd, using hardcoded values"
	return [expr 8.540E-9*$r*$r*$r - 2.464E-6*$r*$r + 6.815E-4*$r ]
    }
    if {($dist1 != "") && ($dist2 != "") && ($dist3 != "")} {
	puts stdout "DISTORTION: $chipName ccd, using fits keyword values"
	return [expr {$dist1*$r + $dist2*$r*$r + $dist3*$r*$r*$r}]
    } else {
	if {[string compare [$this cget -instrument] FORS2]==0} {
	    puts stdout "DISTORTION: $chipName ccd, using FORS2 hardcoded values"
	    return [expr 8.540E-9*$r*$r*$r - 2.464E-6*$r*$r + 6.815E-4*$r ]
	} else {
    	    puts stdout "DISTORTION: $chipName ccd, using FORS1 hardcoded values"
    	    return [expr 9.515E-9*$r*$r*$r - 3.605E-6*$r*$r + 1.001E-3*$r ]
	}
    } 
puts "ablage_HR"
#
}
#-----------------------------
private method test_ablage {  } {
#-----------------------------
puts stdout "\n\n reference pixel at $fr_x $fr_y\ntest ablage SR"
for {set j 0} {$j<=10} {incr j 1} {
    set y [expr $j*200]
    set y_r  [expr ($y-$fr_y)*($y-$fr_y)]
    set strang ""
    for {set i 0} {$i<=10} {incr i 1} {
        set x [expr $i*200]
        set x_r  [expr ($x-$fr_x)*($x-$fr_x)]
        set r [expr sqrt($x_r+$y_r) ]
        set ab [ablage_SR  $r]
        set delx [expr $ab/sqrt(1.0+$y_r/$x_r)]
       
        append strang [ format "%5.2f " $delx]
        }
        puts stdout $strang
     }
puts stdout "\n\ntest ablage HR"
for {set j 0} {$j<=10} {incr j 1} {
    set y [expr $j*200]
    set y_r  [expr ($y-$fr_y)*($y-$fr_y)]
    set strang ""
    for {set i 0} {$i<=10} {incr i 1} {
        set x [expr $i*200]
        set x_r  [expr ($x-$fr_x)*($x-$fr_x)]

        set r [expr sqrt($x_r+$y_r) ]
        set ab [ablage_HR  $r]
        set delx [expr $ab/sqrt(1.0+$y_r/$x_r)]

        append strang [ format "%5.2f " $delx]
        }
        puts stdout $strang
     }

}


#-------------------------------------
public method show_status {} {
#-------------------------------------
   big_status_panel

# if {$label} {
#    big_status_panel
#    } else {
#    small_status_panel
#    }

}


#-------------------------------------
private method small_status_panel {} {
#-------------------------------------
     set text ""
     # basic instrumental setup 
     append text  [format "% 9s : % 10s\n" "INST" [$this cget -instrument] ]
     append text  [format "% 9s : % 10s\n" "MODE" [$this cget -modus] ]
     append text  [format "% 9s : % 10s\n" "MASK" $mask ]
     append text  [format "% 9s : % 10s\n" "COLL"   [$this cget -collimat] ]
     append text  [format "% 9s : % 10s\n" "GRISM"  [$this cget -grism] ]
     append text  [format "% 9s : % 10s\n" "FILTER" [$this cget -filter] ]
     append text  [format "% 9s : % 10s\n" "WIDTH"  [$this cget -slit_width] ]
     append text  [format "% 9s : % 10s\n" "LENGTH" [$this cget -slit_length] ]

     if {[string match *mxu* [$this cget -modus]]} {
        set noms [fors::MXU_SLIT::nos]
        if {$noms>0} { set sp_rsc 0
           for {set i 1} {$i <= $noms} { incr i 1} { 
               if {[C_mxus$i cget -ON]} {
               if {[C_mxus$i cget -ref_slit]} { incr sp_rsc 1
               } } }
        set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
     append text  [format "% 9s : % 10s\n" "MASK_NA"  [lindex $PM 0] ]
     append text  [format "% 9s : % 10s\n" "MASK_ID"  [lindex $PM 1] ]
     append text  [format "% 9s : % 10s\n" "TechMask" [fors::MXU_SLIT::get_mmff] ]
     append text  [format "% 9s : % 10d\n" "X_REFSLIT" $sp_rsc ]
               }

     append text  [format "% 9s : % 10s\n" "X_TYPE"  [$this cget -mxu_shape]  ]
     append text  [format "% 9s : % 10d\n" "X_NUMBER" $noms]
        }

      # other flags 

     append text  [format "% 9s : % 10s\n" "CEN/SLIT" [$this cget -pics] ]

     if {[C_ref_targ cget -mos10ali]} { set local ON } else { set local OFF }

     append text  [format "% 9s : % 2d\n" "MOS10ALI" $local ]
     append text  [format "% 9s : % 10s\n" "RefTarg"  [C_ref_targ nort] ]
     append text  [format "% 9s : % 10s\n" "PROGRAM"  [$this cget -file_name] ]
     append text  [format "% 9s : % 10s\n" "FITS"  [$this cget -file_name] ]
     append text  [format "% 9s : % 10s\n" "loaded" $last_load_fn ]
     append text  [format "% 9s : % 10s\n" "saved"  $last_save_fn ]

     info_dialog $text .skycat1
}

#-------------------------------------
private method big_status_panel {} {
#-------------------------------------
     fors::StatusPanel SP -fors_class $this     
     catch {SP update} 
     }


#-------------------------------------
private method open_log {} {
#-------------------------------------

     # close the log file for the constructor and open 
     # a new log-file for the user commands 
     #
     close $lfId
     set mod [$this cget -modus][string index [$this cget -instrument] 4]
     set log_file $log_dir
     append log_file "/$mod.$exposure_counter.log"
     set lfId [open $log_file w]
     }


# this migth become an external library

#-------------------------------------
private method ldelete_all { w_list value} {
#-------------------------------------
        # modified ldelete
        # delete all list elements by value (single character)
        set ix [lsearch -exact $w_list $value]
        set new $w_list
        
        while {$ix>=0} {
               set new [lreplace $w_list $ix $ix] 
               set ix [lsearch -exact $new $value]
               set w_list $new 
               }
        return $new 
        }      

#-------------------------------------
private method ldelete { list value} {
#-------------------------------------
        # taken from Welch, p34
        # delete one list element by value 
        set ix [lsearch -exact $list $value]
            if {$ix>=0} {
               return [lreplace $list $ix $ix] 
               } else {
               return $list
               }
        }      

#-------------------------------------
private method string_exchange { input old new } {
#-------------------------------------
        # exchange all characters in string 'old' by 'new' 
        # works only correctly if old and new are single chars

        set output ""
        set l [string length $input]
      
        for { set i 0 } { $i < $l } { incr i 1 } {
            set c [string index $input $i]
            if {$c != $old} { 
               append output $c } else { append output $new
               }
            }
 
        return $output
        }

#-------------------------------------
private method string_delete  { input value } {
#-------------------------------------
        # delete all characters of 'value' in string 'input' 
        # works only correctly if old and new are single chars

        set output ""
        set l [string length $input]
      
        for { set i 0 } { $i < $l } { incr i 1 } {
            set c [string index $input $i]
            if {$c != $value} { append output $c }
            }
 
        return $output
        }


#--------------------------
proc FORSInit { g_can g_ima menubar modus control su BPM} {
#--------------------------
     # some remarks on the arguments
     # g_can canvas name 
     # g_ima = image name
     # modus = mos pmos img ipol lss hits
     # control=0 delete current instance 
     #        =1 create the only instance 
     # su=0/1 super user privileges
     # menubar widget to dis/en able menubar buttons

     # case A create the only instance 
     if {$control & !$inst_counter} {

     # -instrument either FORS1 or FORS2 

     # create an instance of FORS
     global ::my_mask
     fors::FORS my_mask $g_can $g_ima $menubar \
          -instrument [string toupper [lindex [split $menubar .] 3] ]\
          -modus $modus \
          -rotangle 5 \
          -width [$g_ima width] \
          -height [$g_ima height] \
          -label $su \
          -badpixelmap $BPM \
          -pics 0 \
          -graytype 0 \
          -slit_width 1.0 \
          -slit_length real \
          -long_width 0.5 \
          -filter none \
          -grism none \
          -colorset Red/Yellow

      incr inst_counter 1

      # disable 'FORS2'-button when 'FORS1'-button is pressed
      set other_menubar [split $menubar .] 
      set FORS_number [string index [lindex $other_menubar 3] 4]
      if {$FORS_number==1} { set FORS_number "fors2" }
      if {$FORS_number==2} { set FORS_number "fors1" }
      set other_menubar [lreplace $other_menubar 3 3 $FORS_number]
      set other_menubar [join $other_menubar "."]
      catch {$other_menubar entryconfigure 0 -state disabled }


# puts stdout "Number of instances: $inst_counter"
#   FORS my_mask $thecanvas  -rotangle 5  -ccx $ccx -ccy $ccy \
#    -collimat standard -graytype   0
# for reasons of debugging
#   my_mask plot_mask  
#   my_mask b/boinind_mos 
#   my_mask help
# puts stdout "my_mask isa FORS = [my_mask isa FORS]"
# puts stdout "my_mask cget -width = [my_mask cget -width]"
# puts stdout "my_mask cget -height = [my_mask cget -height]"
# puts stdout "my_mask cget -rotangle= [my_mask cget -rotangle]"
# puts stdout "my_mask cget -collimat = [my_mask cget -collimat]"
# puts stdout "my_mask cget -graytype = [my_mask cget -graytype]"
# puts stdout "my_mask cget -modus = [my_mask cget -modus]"
# puts stdout "my_mask info class    = [my_mask info class]"
# puts stdout "my_mask info inherit  = [my_mask info inherit]"
# puts stdout "my_mask info heritage = [my_mask info heritage]"
# puts stdout "my_mask info function = [my_mask info function]"
# puts stdout "my_mask info variable = [my_mask info variable]"
# puts stdout "my_mask itcl::find objects -class FORS = \
#              [my_mask itcl::find objects -class FORS]"
# puts stdout "my_mask info function help = \
#              [my_mask info function help]"
# puts stdout "my_mask info variable ccx = \
#              [my_mask info variable ccx]"
# puts stdout "my_mask info variable exposure_counter  = \
#              [my_mask info variable exposure_counter]"
# puts stdout "my_mask info variable inst_counter  = \
#              [my_mask info variable inst_counter]"
# puts stdout "my_mask cget -collimat = \
#              [my_mask cget -collimat]"
# my_mask configure -collimat highres
# my_mask plot_mask  
# puts stdout "my_mask cget -collimat = \
#            [my_mask cget -collimat]"

  # case B delete the only instance 
  } elseif {!$control & $inst_counter} {

  # delete the only instance
  # puts stdout "trying to delete the instance"
  # puts stdout "Number of instances: $inst_counter"

  if {[string compare [itcl::find objects -class FORS] my_mask] == 0} {
     # puts stdout "my_mask does exist"
     } else {
     # puts stdout "my_mask does not exist"
     }

  delete object my_mask
  #delete my_mask
  incr inst_counter -1

      # enable 'FORS2'-button when 'FORS1'-button is pressed
      set other_menubar [split $menubar .] 
      set FORS_number [string index [lindex $other_menubar 3] 4]
      if {$FORS_number==1} { set FORS_number "fors2" }
      if {$FORS_number==2} { set FORS_number "fors1" }
      set other_menubar [lreplace $other_menubar 3 3 $FORS_number]
      set other_menubar [join $other_menubar "."]
      catch {$other_menubar entryconfigure 0 -state normal}

  if {[string compare [itcl::find objects -class FORS] my_mask] == 0} {
     # puts stdout "my_mask does exist"
     } else {
     # puts stdout "my_mask does not exist"
     }

  # puts stdout "Number of instances: $inst_counter"
  #unset my_mask
  } else {
    if {$control} { 
       # puts stdout "Warning : cannot create more than one Mask"
       } else {
       # puts stdout "Warning : cannot delete $inst_counter Masks"
       }
    }

    }


# -------------------
# common variables 
# -------------------
     # number of class instances, only one (mask) permitted
     common inst_counter 0
      
# ------------------ 
#  public variables 
# --------------------

      # --------------------------
      public variable badpixelmap 0 {
      # --------------------------
	     puts stdout "BPM: [$this cget -badpixelmap]"

	     if {$badpixelmap} {
                # load and update BPM
		load_BPM 
                if {$mask_initialized} {
         fors::CONV_MC::coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
             $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
             $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
                   }
                update_BPM 
		} else {
                # purge BPM
		$m_canvas delete -tag BPM 
		}
	     # puts stdout $badpixelmap
	     }

      # --------------------------
      public variable instrument FORS1
      # --------------------------

      # --------------------------
      public variable graytype 0 { 
      # --------------------------
             if {$mask_initialized} { update_and_replot } else { update_color }
             }

      public variable rotangle 5; # rot step 

      # check this and throw out 
      public variable width 200 
      public variable height 200

      public variable file_name "" {
             
             set file_name [string_delete $file_name " " ]
             set file_name [string_delete $file_name "." ]
             set file_name [string_delete $file_name "," ]
             set file_name [string_delete $file_name "*" ]
             set file_name [string_delete $file_name "-" ]
             set file_name [string_delete $file_name "+" ]
             set file_name [string_delete $file_name "_" ]
             set file_name [string_delete $file_name "=" ]
             set file_name [string_delete $file_name ")" ]
             set file_name [string_delete $file_name "(" ]
             set file_name [string_delete $file_name "\[" ]
             set file_name [string_delete $file_name "\]" ]
             set file_name [string range $file_name 0 7]
             puts $lfId "new file name $file_name"

             if {[string match mxu [$this cget -modus] ]} {
                if {![fors::MXU_SLIT::get_mmff]} {
                   update_mask_na
                   }
                }
             }

      public variable calib_passwd ""

      # ---------------------------------
      public variable collimat none {
      # ---------------------------------
         if {$mask_initialized} { 

           if {$badpixelmap} { load_BPM; 
         fors::CONV_MC::coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
             $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
             $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
               update_BPM  }

           # change mask for ipol/hr
           if {[string compare [$this cget -modus] "ipol"] == 0} {
            if {[string match *HR* [$this cget -collimat] ]} {
                  set mask pol 
                  puts stdout "from MASK=mos to MASK=$mask"
                  } else { 
                  set mask mos 
                  puts stdout "from MASK=pol to MASK=$mask"
                  }
            reset_mask
            } else {
            # modus != ipol
            # 24/01/2000 special request by hboehnha
            # hboehnha wants this only for SR/HR-change  
              if {[string match *img* [$this cget -modus]] ||
                  [string match *mos* [$this cget -modus]]} {
                  reset_mask
                  } else {
                  erase; plot_mask 
                  }
            }

          # enable hr for pinhole in hits
          if {[string compare [$this cget -modus] "hits"]==0} {
             if {[string match *SR* [$this cget -collimat] ]} {
                $m_mb.layout entryconfigure 4 -state normal
                } else {
                $m_mb.layout entryconfigure 4 -state disabled
                }
             }

          # pmos/open -> ipol mask 
          if {[string compare [$this cget -modus] "pmos"]==0} {
             if {[string compare [$this cget -slit_width] "open"]==0} {
                if {[string match *HR* [$this cget -collimat] ]} {
                $m_mb.layout entryconfigure 1 -state disabled
                $m_mb.layout entryconfigure 2 -state disabled
                reset_mask
                } else {
                $m_mb.layout entryconfigure 1 -state normal
                $m_mb.layout entryconfigure 2 -state normal
                reset_mask
                }
             }
            }
           } else {
           # mask not initialized = first coll by constructor
           }
      
         # now independent of mask_initialized or not 


         puts stdout "colli: [$this cget -collimat]"
         puts $lfId   "colli: [$this cget -collimat]"
	 set last_action "Select Collimator"
         catch {SP update}
         }

      # -------------------------
      public variable modus mos {
      # -------------------------
         set mask mos; # default mask for default modus

         set coll [$this cget -collimat]; set new_coll $coll

         # PMOS in highres collimator physically not possible
         # mode can be : mos, pmos, ipol, img, lss, hits, mxu
         if {[string compare [$this cget -modus] "pmos"] == 0} {
         if {[string compare [$this cget -slit_width] "open"]} {
         if {[string match *HR* $coll ] } {
            set new_coll [lindex [split $co_range " "] 0]
            }
            }
            }

         # HITS in highres collimator not sensful
         #  note : 9/7/5/3/BigPH/2/4/6/8
         # use BigPH when central (smallest) slit is open
         # BigPH can be used with highres collimator
         if {[string compare [$this cget -modus] "hits"] == 0} {
         if {[string compare [$this cget -long_width] "0.3"]} {
         if {[string match *HR* $coll ] } {
            set new_coll [lindex [split $co_range " "] 0]
            }
            }
            }

         # update 
         if {[string compare $coll $new_coll]} {
             $this configure -collimat $new_coll
            }
 
         # mask can be : mos, pol, lss
         if {[string compare [$this cget -modus] "ipol"] == 0} {
         if {[string match *HR* [$this cget -collimat] ] } {
            set mask pol } }

         # slit less spectroscopy in pmos 
         if {[string compare [$this cget -modus] "pmos"] == 0} {
         if {[string match *open* [$this cget -slit_width] ] } {
         if {[string match *HR* $coll ] } {
            set mask pol } } }

         if {[string compare [$this cget -modus] "lss"] == 0} {
            set mask lss }

         if {[string compare [$this cget -modus] "hits"] == 0} {
            set mask hits }

         # this is absolutely so-called 'Schweinkram'
         # we mis-use the menu buttons. 
         # we redefine the LSS slit_width button for 
         # MXU purposes

         if {[string compare [$this cget -modus] "mxu"] == 0} {
            set mask mxu 

	    # disable HR collimator 
	    $m_mb.layout entryconfigure 6 -state disabled

	    # rename HITS slit to MXU slit type 
            $m_mb.layout entryconfigure 3 -label "MXU slit type" \
                         -state normal 

            $m_mb.layout.ls entryconfigure 0 -label "STRAIGHT" \
                                             -state normal \
        -command {catch {::fors::FORS::my_mask configure -mxu_shape STRAIGHT} }

            $m_mb.layout.ls entryconfigure 1 -label "CIRCLE" \
                                             -state normal \
        -command {catch {::fors::FORS::my_mask configure -mxu_shape CIRCLE} }

            $m_mb.layout.ls entryconfigure 2 -label "CURVED" \
                                             -state normal \
        -command {catch {::fors::FORS::my_mask configure -mxu_shape CURVED} }

            $m_mb.layout.ls entryconfigure 3 -label "none" \
                                             -state disabled \
        -command {catch {::fors::FORS::my_mask configure -mxu_shape none} }
                                    
            $m_mb.layout.ls entryconfigure 4 -label "none" \
                                             -state disabled \
        -command {catch {::fors::FORS::my_mask configure -mxu_shape none} }

            $m_mb.layout.ls entryconfigure 5 -label "none" \
                                             -state disabled \
        -command {catch {::fors::FORS::my_mask configure -mxu_shape none} }

            $m_mb.layout.ls entryconfigure 6 -label "none" \
                                             -state disabled \
        -command {catch {::fors::FORS::my_mask configure -mxu_shape none} }

            $m_mb.layout.ls entryconfigure 7 -label "none" \
                                             -state disabled \
        -command {catch {::fors::FORS::my_mask configure -mxu_shape none} }

            $m_mb.layout.ls entryconfigure 8 -label "none" \
                                             -state disabled \
        -command {catch {::fors::FORS::my_mask configure -mxu_shape none} }

            $m_mb.layout.ls entryconfigure 9 -label "none" \
                                             -state disabled \
        -command {catch {::fors::FORS::my_mask configure -mxu_shape none} }


            # default is STRAIGHT
            $m_mb.layout.ls invoke 0

            } else {
            $m_mb.layout entryconfigure 3 -label "Slit Width LSS" 
            # --------------------index 3--LSS SLIT WIDTH
           set l_counter 0
           foreach i {all 0.3 0.4 0.5 0.7 1.0 1.3 1.6 2.0 2.5} {
               $m_mb.layout.ls entryconfigure $l_counter -label "  $i ''" \
               -variable ls -value $i \
               -state normal \
               -command {catch {::fors::FORS::my_mask configure -long_width $ls}
                        }
               incr l_counter
               }

            }

        puts stdout "modus: [$this cget -modus] with mask $mask"
        puts $lfId   "modus: [$this cget -modus] with mask $mask"
        catch {SP update}
        }

      # ---------------------------
      public variable label 1 {
      # ---------------------------
             puts stdout "label: [$this cget -label]"
             puts $lfId   "label: [$this cget -label]"

             if {$mask_initialized} {
                if {[string compare [$this cget -modus] mxu]==0} {
                   fors::MXU_SLIT::label $label
                }

             erase 
             plot_mask 
             if {$badpixelmap} { update_BPM }
             # because of the brutal erase
                }
             }

      # ----------------------------
      public variable fors_frame 1 {
      # ----------------------------
             # puts stdout "FORS frame flag: [$this cget -fors_frame]"
             puts $lfId   "FORS frame flag: [$this cget -fors_frame]"

             if {$mask_initialized} {
             set_scale $fors_frame 
             reset_mask
             } else {
             # do not reset when called by the constructor
             set_scale $fors_frame
             }
             catch {SP update}
             }

      # ----------------------------
      public variable mxu_shape STRAIGHT  {
      # ----------------------------
           puts $lfId   "MXU aperture type: [$this cget -mxu_shape]"
	   set last_action "Select MXU Slit Type"
           catch  {SP update}
           }

      # ----------------------------
      public variable pics 0 {
      # ----------------------------
           catch {SP update}
           # the center-slit option calls the 
           # pickObject widget
           puts $lfId   "center/slit flag : [$this cget -pics]"
           if {$mask_initialized} { update_and_replot }
           if {$pics==1} {center_slit} 
            }

      # -----------------------------
      public variable slit_width  1.0 {
      # -----------------------------
             if {$mask_initialized} {

                # we need to know if the slit_width changed from
                # open to numerical or vice versa 
                set new_open_flag [string match *open* $slit_width]
                # puts stdout "old_open_flag = $old_open_flag"
                # puts stdout "new_open_flag = $new_open_flag"

                # puts stdout "mask_init = $mask_initialized"
                if {[string compare [$this cget -modus] pmos]==0} {
                   if {[string match *SR* [$this cget -collimat] ]} {
                      $m_mb.layout entryconfigure 1 -state normal
                      } else {
                      $m_mb.layout entryconfigure 1 -state disabled
                      }
                  if {[string compare [$this cget -slit_width] open]==0} {
                     # allow collimator change
                     $m_mb.layout entryconfigure 6 -state normal
                     } else {
                     # disable collimator change 
                     $m_mb.layout entryconfigure 6 -state disabled
                     }
                  }

                # the Save button 
                if {[string compare [$this cget -slit_width] open]==0} {
                if {[string compare [$this cget -modus] pmos]==0} {
                     $m_mb entryconfigure 6 \
                     -label "Save 'pims.$exposure_counter.targ'"
                    } else {
                     $m_mb entryconfigure 6 \
                     -label "Save 'ims.$exposure_counter.targ'"
                    }
                } else {
                     $m_mb entryconfigure 6 \
                     -label "Save '[$this cget -modus].$exposure_counter.targ'"
               } 

                   
       if {$new_open_flag != $old_open_flag} { 
          puts stdout "change open/numeric: reset mask"
          reset_mask } else {

        switch -exact -- [$this cget -modus] {
        echs - 
        mos -
        pmos {
          for {set i 1} {$i < 20} {incr i 1} {
              # set the new slit width 
              set mos($i,b) [$this cget -slit_width]
              # if slit is positioned
              if {$targw($i,f) != 0} {
                 $m_image convert coords $targw($i,a) $targw($i,d) \
                     "wcs $EQX" sx sy screen
                 # moves the slit again to the same position with the 
                 # new slit width 
                 catch {$this move_slit $m_canvas $sx $sy 1 0 }
                 }
               }
          plot_mask
             }
          mxu {
              # NumberOfMxuSlits
              set noms [fors::MXU_SLIT::nos]
              if {$noms>0} {
              for {set i 1} {$i<=$noms} { incr i 1} {
                  if {$i > $nf_refslit} {
                  if {[C_mxus$i cget -ON]} {
                      # puts stdout ">>> $i <<<"
                      C_mxus$i configure -slit_width $slit_width
                      C_mxus$i plot_m
                      if {$label } { C_mxus$i plot_label e w }
                      }
		  }
                  }
                  }
              }
          default reset_mask
              }

           }
              set old_open_flag $new_open_flag
              # puts stdout "old_open_flag = $old_open_flag"
              # puts stdout "new_open_flag = $new_open_flag"

              }

             puts stdout "width: [$this cget -slit_width]"
             puts $lfId   "width: [$this cget -slit_width]"
	     set last_action "Select Slit Width"
             catch {SP update}
             }

      # ---------------------------
      public variable slit_length real {
      # ---------------------------
        switch -exact -- [$this cget -modus] {
        mos -
        pmos {
         if {[string compare [$this cget -slit_length] "real"]} {
            # configure non-real MOS slit_length 
            adjust_slit_length
            reset_mask 
            } else {
            # configure 'real' MOS slit_length 
            if {$mask_initialized} {
               # not called by constructor 
               load_instrument; # again
               # balde should not exceed focal field
               set mosslitmin(1)  $YSRMIN
               set mosslitmax(19) $YSRMAX
               reset_mask
               } 
             }
            puts stdout "length: [$this cget -slit_length]"
            }
       mxu {
              set noms [fors::MXU_SLIT::nos]
              if {$noms>0} {
              for {set i 1} {$i<=$noms} { incr i 1} {
		  if {$i > $nf_refslit} {
                  if {[C_mxus$i cget -ON]} {
                     C_mxus$i configure -slit_length $slit_length
                     C_mxus$i plot_m
                     if {$label} {C_mxus$i plot_label e w }
                     }
		 }
                  }
               }
           }
           }
	 set last_action "Select Slit Length"
       catch {SP update}
       }

      # ---------------------------
      public variable colorset    Red/Yellow
      # ---------------------------

      # -------------------------------
      public variable long_width 0.5 {
      # -------------------------------
      set doit 0; set lm [$this cget -modus]
      if {$mask_initialized} { 
         if {[string compare $lm  "lss"]==0} { set doit 1 }
         if {[string compare $lm "hits"]==0} { set doit 1 }
          } else { set doit 1 }

     
      if {$doit} {
         # long_number is internal slit counter
         switch -exact -- $long_width {
            all {set long_number 0 }
            PH  {set long_number 1 }
            0.3 {set long_number 1 }
            0.4 {set long_number 2 }
            0.5 {set long_number 3 }
            0.7 {set long_number 4 }
            1.0 {set long_number 5 }
            1.3 {set long_number 6 }
            1.6 {set long_number 7 }
            2.0 {set long_number 8 }
            2.5 {set long_number 9 }
        default {puts stdout "Warning: LSS $long_width" }
             }

        # allow coll=highres for hits-pinhole
        if {[string compare $lm "hits"]==0} {
           if {$long_number==1} {
              # pinhole in use 
              # enable collimator
              $m_mb.layout entryconfigure 6 -state normal
              } else {
              # no pinhole in use 
              $m_mb.layout entryconfigure 6 -state disabled
              }
            }
             if {$mask_initialized} { reset_mask }
          }
         puts stdout "lslit: [$this cget -long_width]"
         puts $lfId   "lslit: [$this cget -long_width]"
	 set last_action "Select LongSlit"
        catch {SP update}
        }

      # -------------------------------
      public variable filter none {
      # -------------------------------
             if {[string match *s* [$this cget -modus] ] ||
                 [string match *mxu* [$this cget -modus] ] } {
             # search in the dispersion table
             set prio_flag 0; set good 0
             # 0 = not filter, 1 = a filter, 2 = best filter
             for {set i 1} { $i<=$A_disp(0)} { incr i 1 } {
               if {[string compare $grism $A_disp($i,grism)]==0} {
                  if {$prio_flag==0} {
                     set good $i
                     # lassign $pat filter disp lamc ins_lmin ins_lmax
                     if {[string compare $filter $A_disp($i,filter)]==0} {
                        set prio_flag 2 } else { set prio_flag 1 }
                     # puts "$i $good $prio_flag"
                   }
                   # if there are further filters in table 
                   # select
                   if {$prio_flag==1}  {
                      # filter 'none' matches better than the wrong filter
                      if {[string compare $A_disp($i,filter) "none"]==0} {
                         set good $i 
                         # lassign $pat filter disp lamc ins_lmin ins_lmax
                         set prio_flag 1
                         # puts "$i $good $prio_flag"
                       }
                       # if the filter matches -> overwrite
                       if {[string compare $A_disp($i,filter) $filter]==0} {
                          set good $i
                          # lassign $pat filter disp lamc ins_lmin ins_lmax
                          set prio_flag 2
                          # puts "$i $good $prio_flag"
                       }
                    }
                # flag==2 is already the best case, do not overwrite
                 }
                 }
             
                          
             if {$prio_flag<2} { set warn ">> OS filter not found" } else {
                                 set warn ">> OS filter found ok" }
             puts stdout [format "filter: %12s %2d %2d %20s" \
               [$this cget -filter] $good $prio_flag $warn]
             # specify the wavelength limits
             set L_MIN [expr ( $A_disp($good,cen)+$A_disp($good,min) ) * 0.5 ]
             set L_MAX [expr ( $A_disp($good,cen)+$A_disp($good,max) ) * 0.5 ]
             wave_limit
             wave_config_mask "$L_MIN $L_MAX grism"

             if {[string compare [$this cget -modus] mxu]==0} {
             set ldisp [expr $CCDscale_standard/$focscale/$A_disp($good,disp)/$PIX]
             set pos [lsearch -exact $filt_range [$this cget -filter] ]
             set l_lam0 [lindex $lamcent_range $pos]
             set l_fwhm [lindex $fwhm_range $pos]

             # grism-disp[mm/nm]  grism-lam0[nm]  lam0-of-filt[nm]  fwhm-of-filt[nm]
             set 4val [list $ldisp [expr $A_disp($good,cen)*1.0] $l_lam0 $l_fwhm ]

             # puts stdout "mxu-disp: $4val"
             fors::MXU_SLIT::disp $4val
             }

             # if {$mask_initialized} { reset_mask }
             }

         puts $lfId   "filter: [$this cget -filter]"
	 set last_action "Select Filter"
         catch {SP update}
             }
             
      # ----------------------------------
      public variable grism none {
      # ----------------------------------
             switch -exact -- [$this cget -modus] {
            echs { # do not call echs_pos when mask does not yet 
                   # exist (not when called from constructor)
                   if {$mask_initialized} { 
                      # puts stdout ">>!!! echs_pos called from config -grism"
                      echs_pos } }
             mos -
            pmos -
             lss -
            hits -
             mxu {
                
             # search in the dispersion table 
             set prio_flag 0; set good 0
             # 0 = not filter, 1 = a filter, 2 = best filter
             for {set i 1} { $i<=$A_disp(0)} { incr i 1 } {
               if {[string compare $grism $A_disp($i,grism)]==0} {
                  if {$prio_flag==0} {
                     set good $i 
                     # lassign $pat filter disp lamc ins_lmin ins_lmax
                     if {[string compare $filter $A_disp($i,filter)]==0} {
                        set prio_flag 2 } else { set prio_flag 1 }
                     # puts "$i $good $prio_flag"
                   }
                   # if there are further filters in table 
                   # select
                   if {$prio_flag==1}  {
                      # filter 'none' matches better than the wrong filter
                      if {[string compare $A_disp($i,filter) "none"]==0} {
                         set good $i 
                         # lassign $pat filter disp lamc ins_lmin ins_lmax
                         set prio_flag 1
                         # puts "$i $good $prio_flag"
                       }
                       # if the filter matches -> overwrite
                       if {[string compare $A_disp($i,filter) $filter]==0} {
                          set good $i
                          # lassign $pat filter disp lamc ins_lmin ins_lmax
                          set prio_flag 2
                          # puts "$i $good $prio_flag"
                       }
                    }
                # flag==2 is already the best case, do not overwrite
                 }
                 }
             
                        
       switch -exact -- $prio_flag {
          0 { puts stdout "Error  : Filter-Grism mismatch"       }
          1 { puts stdout "Warning: Filter does not match Grism" }
          2 { puts stdout "Ok     : Filter does match Grism"     }
    default { puts stdout "Error  : Filter-Grism mismatch"       }
           }


             if {$prio_flag<2} { set warn ">> filter not found" } else {
                                 set warn "" }
             puts stdout [format "grism : %12s %2d %2d %20s" \
               [$this cget -grism] $good $prio_flag $warn]

             # specify the wavelength limits
             set L_MIN [expr ( $A_disp($good,cen)+$A_disp($good,min) ) * 0.5 ]
             set L_MAX [expr ( $A_disp($good,cen)+$A_disp($good,max) ) * 0.5 ]
             wave_limit

             wave_config_mask "$L_MIN $L_MAX grism"

             }

             }


         puts $lfId   "grism: [$this cget -grism]"
	 set last_action "Select Grism"
         catch {SP update}

         if {[string match *GRIS_1400V+18* [$this cget -grism]]} {
            info_dialog "Note: Spectra of  GRIS_1400V+18\n\
                         are offset by 111.4 pixel towards +Y\n\
                         (toward lower slit numbers=TOP)\n" .skycat1
                         
            }
	 if {[string match *GRIS_600RI+19* [$this cget -grism]]} {
            info_dialog "Note: Spectra of  GRIS_600RI+19\n\
                         are offset by 272.4 pixel toward +Y\n\
                         (toward lower slit numbers=TOP)\n" .skycat1

            }
             }

      private variable lochost $env(HOST); # fims install dir
      public variable int_dir  $env(FIMSROOT); # fims install dir
      public variable isf_dir  $env(ISF_DIR); # load FORS1.isf / FORS2.isf 
      public variable log_dir  $env(LOG_DIR); # write log files 
      public variable set_dir  $env(SET_DIR); # write setup files 
      public variable prep_dir $env(PREP_DIR);# load fits files 

# ----------------
# private variables
# ----------------

      private variable F_N mycopy.targ 
      # versions
      private variable f_v
      private variable s_v
      # logfile name
      private variable lfId
      # current catalog name 
      private variable CatFileName no_catalog
      # local name for canvas and image
      private variable m_image 
      private variable m_canvas
      private variable m_mb
      # scale (zoom) of the world/canvas
      private variable canvscale
      # equinox
      private variable EQX
      # boundary errors in slits_lengths  [mm]
      private variable s_slit 4
      # the actual position of the mask in world units
      private variable world_x 
      private variable world_y
      # the initial position of the mask in world units
      private variable wx
      private variable wy
      # the cordinates of the MOS mask, a big matrix 
      private variable mos
      # the BadPixelMap
      private variable BPM
      private variable BPM_counter
      # the cordinates of the reference objects 
      private variable ref
      # the grism-OS-filter-dispersion-array 
      private variable A_disp
      private variable good
      # the world coordinates of the 19 positioned slits
      private variable targw
      # absolute angle of the mask in rad with respect to WCS
      private variable abs_ang_rad 0.0
      # size of the ccd
      # these coordinates remain constant for both collimators
      private variable x_ccd 300
      # the  center of the mask (telescope pointing position on canvas)
      private variable ccx
      private variable ccy
      # initial canvas psoiton of mask = canvas center
      private variable fr_x
      private variable fr_y 
      # scale pixel/''
      # here a very preliminary and incorrect formular
      private variable worldscale 1.7
      # old position for shifting
      private variable oldpos
      private variable op

      # the length of the spectrum on the CCD in arcsec 
      private variable speclen 50.0
      private variable specmin 50.0
      private variable specmax 50.0
      private variable ccdm_specmin 50.0
      private variable ccdm_specmax 50.0

      # user defined wavelength range 
      private variable L_MIN 400.0
      private variable L_MAX 600.0
 
      # FORS configuration constants
      private variable XSRMAX
      private variable YSRMAX
      private variable XHRMAX
      private variable YHRMAX
      private variable XSRMIN
      private variable YSRMIN
      private variable XHRMIN
      private variable YHRMIN
      private variable XBLADEA
      private variable XBLADEB
      private variable mosslitmin
      private variable mosslitmax  

      private variable longslitxmin
      private variable longslitymin
      private variable longslitxmax
      private variable longslitymax

      private variable htslitxmin
      private variable htslitymin
      private variable htslitxmax
      private variable htslitymax
      private variable mxureftp
      private variable mxurefsx
      private variable mxurefsy
      private variable mxurefwd
      private variable mxurefle
      private variable mxurefro

      private variable pinhxcent 
      private variable pinhycent

      private variable polarimslitxmin
      private variable polarimslitymin
      private variable polarimslitxmax
      private variable polarimslitymax

      private variable long_number 
      private variable mask
      private variable slit_coll
      private variable nf_refslit
      private variable co_range
      private variable gris_range
      private variable filt_range
      private variable osfi_range
      private variable fwhm_range
      private variable lamcent_range
      private variable mxu_refslit_range
      private variable slit_range
      private variable htslit_range
      private variable ph_range
      private variable echelle_range
      private variable exposure_counter 1
      private variable trapa gray25; # transparency slit blades 
      private variable disp_string
      private variable NOG
 
      # VLT-U1 focal scale in arcsec/mm
      # there is rho=0.5280 theta -1.1288E-6 theta*theta
      # we use  1/0.5280 [arcsec/mm]
      # x [mm] *$focscale = y [arcsec]  (mtoas)
      private variable focscale

      # The values, received from Paranal 
      # the focal length of UT1 = 108827.0mm
      # the focal length of UT2 = 108518.8mm
      # focscale=arctan(1/fl)* 180/pi * 3600 

      private variable focscale_UT1 1.901679  
      private variable focscale_UT2 1.9007288
      # still the nominal values corresponding
      # to focal length of 108827.0mm
      private variable focscale_UT3 1.9007288
      private variable focscale_UT4 1.9007288

      # scale on the CCD  [mm/arcsec]
      # y [arcsec] * CCDscale = x [mm] (astom)
      # this has to be modified:
      # take the Focus value from FORS1.isf
      private variable CCDscale_standard 0.2
      private variable CCDscale_highres  0.1

      # CCD dimensions
      private variable CCDMASTER_XSRMIN
      private variable CCDMASTER_XSRMAX
      private variable CCDSLAVE_XSRMIN
      private variable CCDSLAVE_XSRMAX    
      private variable CCDMASTER_YSRMIN
      private variable CCDMASTER_YSRMAX
      private variable CCDSLAVE_YSRMIN
      private variable CCDSLAVE_YSRMAX 
      private variable CCDGAP_SR
      private variable CCDMASTER_XHRMIN
      private variable CCDMASTER_XHRMAX
      private variable CCDSLAVE_XHRMIN
      private variable CCDSLAVE_XHRMAX    
      private variable CCDMASTER_YHRMIN
      private variable CCDMASTER_YHRMAX
      private variable CCDSLAVE_YHRMIN
      private variable CCDSLAVE_YHRMAX    
      private variable CCDGAP_HR
      private variable ccd_frame
      private variable FIELDSTOP_XMIN
      private variable FIELDSTOP_XMAX
      private variable FIELDSTOP_YMIN
      private variable FIELDSTOP_YMAX
     
     
      # pixelsize in mm
      private variable PIX 0.024 
      private variable warn_flag 0

      # numeric arcmin_to_degree
      private variable atd 2.7777778E-4
      private variable trigo 1
      # numeric canvas_to_mm
      private variable ctom
      private variable ccdm_ctom
      private variable ccds_ctom
      # fill colors
      private variable color1 red
      private variable color2 yellow
      # outline colors
      private variable color3 red
      private variable color4 yellow
 
      # bonsai matrix elements for mm-wcs
      private variable bonsai_flag
      # maximum permitted number of mxu slits
      # AAG PPRS 10127
      private variable mnomxus 1000 

      private variable lss_ref_ra
      private variable lss_ref_de
      private variable lss_sli_ra
      private variable lss_sli_de
      private variable fits_x_scale
      private variable fits_y_scale
      private variable WP
      private variable man_WP 0.0
      private variable CSO
      private variable CRO
      private variable PAM 0
      private variable tcs_pam 0
      private variable reftarg_zone 0
      
      private variable mask_initialized 0
      private variable mountangle 180
      private variable stop 0
      private variable dith_num 1
      private variable echs_flag 0 
      private variable dire_flag 0 

      private variable new_open_flag 0
      private variable old_open_flag 0
      
      private variable last_action  ""
      private variable last_save_fn ""
      private variable last_load_fn ""
     
      private variable sp_mtali 0;  # MOS10ali       only for statuspanel
      private variable sp_nort "";   # NumOfRefTarg   only for statuspanel
      private variable sp_rsc 0;     # RefSlitCounter only for statuspanel 
      private variable sp_mid "";    # MaskId         only for statuspanel 
      private variable sp_instrume ""; # underlying fits image 
      private variable sp_origin   ""; # underlying fits image 
	 }
     # end of the class FORS

 }
# end of the namespace

# ___oOo___

